'From Squeak3.4gamma of ''7 January 2003'' [latest update: #5169] on 16 March 2003 at 8:51:32 pm'!
"Change Set:		SyntaxFixes-ajh
Date:			16 March 2003
Author:			Anthony Hannan <ajh18@cornell.edu>

The closure compiler (the SmaCC SqueakParser) is a little less forgiving then the previous one with respect to syntax.  For example, a Smalltalk sentence should not end with two periods.  The previous parser allowed this, but the closure compiler does not.  This changeset changes those methods in the 3.4 image that contained these anomolies.  The semantics are unaltered.  Most of the problems were extra periods, the other most common was missing quotes from non-binary special symbols.  For example, #) should be #')'.
"!

!AliceWorld methodsFor: 'temporary' stamp: 'ajh 3/15/2003 02:12'!
makeActorFrom: filename
	"Creates a new actor using the specification from the given file"

	| aFile words line startSubstr index parent name texture meshFile matrix baseActor newActor protoClass actorClass fileVersion |

	myUndoStack closeStack.
	words _ (filename findTokens: #'.').

	((words last) = 'mdl') ifTrue: [
			aFile _ (CrLfFileStream readOnlyFileNamed: filename) ascii.

			"First see if we need to create a prototype class for this model"
			(ActorPrototypeClasses includesKey: (aFile localName))
					ifTrue: [ protoClass _ ActorPrototypeClasses at: (aFile localName) ]
					ifFalse: [
							"Make a new prototype class for this model"
							protoClass _ (WonderlandActor newUniqueClassInstVars: ''
															classInstVars: '').
							ActorPrototypeClasses at: (aFile localName) put: protoClass.
							].

			"Check what version this mdl file is"
			line _ aFile upTo: (Character cr).
			line _ aFile upTo: (Character cr).
			line _ aFile upTo: (Character cr).

			((line truncateTo: 7) = 'version')
					ifTrue: [ fileVersion _ 1 ]
					ifFalse: [ fileVersion _ 0 ].

			[ line _ aFile upTo: (Character cr).
			  (aFile atEnd) ifTrue: [ true ]
			  			ifFalse: [ words _ line findTokens: '='.
								  false ]
			] whileFalse: [

					"See if we're creating a new object"
					(((words size) > 1) and: [ ((words at: 2) beginsWith: ' _MakeObject')
											or: [ (words at: 2) beginsWith: ' Alice.MakeObject' ] ])

						ifTrue: [
						(fileVersion = 0)
							ifTrue: [
								words _ line findTokens: #,.
								parent _ (words at: 2) withBlanksTrimmed.
								name _ (((words at: 3) withBlanksTrimmed)
													findBetweenSubStrs: '"') at: 1.
									]
							ifFalse: [
								name _ (words at: 1) truncateTo: (((words at: 1) size) - 1).
								parent _ ((words at: 3) findTokens: #,) at: 1.
									].

						"Now pull in the texture to use"
						startSubstr _ name , '.SetTexture'.
						[(line _ aFile upTo: (Character cr)) beginsWith: startSubstr] whileFalse: [].

						texture _ (line findBetweenSubStrs: '"') at: 2.
						texture _ (aFile directory pathName), FileDirectory slash, texture.

						"Read the composite matrix to use"
						startSubstr _ name , '._SetLocalTransformation'.
						[(line _ aFile upTo: (Character cr)) beginsWith: startSubstr] whileFalse: [].

						matrix _ B3DMatrix4x4 new.
						words _ line findBetweenSubStrs: ',()'.
						words removeAllSuchThat: [:str | str = ' '].

						index _ words size.
						4 to: 1 by: -1 do: [:i | 
							4 to: 1 by: -1 do: [:j | matrix at: i at: j put: 
										((words at: index) withBlanksTrimmed) asNumber.
										   		index _ index - 1. ]. ].

						1 to: 4 do: [:i | index _ matrix at: i at: 4.
									   matrix at: i at: 4 put: (matrix at: 4 at: i).
									   matrix at: 4 at: i put: index. ].

						matrix a14: (matrix a14 negated).

						"Read the mesh file to use"
						startSubstr _ 'LoadGeometry'.
						[(line _ aFile upTo: (Character cr)) beginsWith: startSubstr] whileFalse: [].

						meshFile _ (line findBetweenSubStrs: '"') at: 2.
						meshFile _ (aFile directory pathName), FileDirectory slash, meshFile.

						"Now build the actor name"
						words _ name findTokens: '.'.
						name _ words last.
						name at: 1 put: ((name at: 1) asLowercase).

						"Now build the parent name"
						parent _ parent copyReplaceAll: '.' with: ' '.

						"Now create the object"
						(parent = 'None') ifTrue: [
								actorClass _ protoClass newUniqueClassInstVars: '' classInstVars: ''.
								baseActor _ actorClass createFor: self.
								actorClassList addLast: actorClass.
								baseActor setName: name.
								baseActor setTexture: texture.
								baseActor loadMeshFromFile: meshFile.
								baseActor setComposite: matrix.
												]  "end base actor creation"

										ifFalse: [
								actorClass _ WonderlandActor newUniqueClassInstVars: ''
																classInstVars: ''.
								newActor _ actorClass createFor: self.
								actorClassList addLast: actorClass.
								newActor setName: name.
								parent _ (baseActor getChildNamed: parent).
								newActor reparentTo: parent.
								newActor becomePart.
								newActor setTexture: texture.
								newActor loadMeshFromFile: meshFile.
								newActor setComposite: matrix.
												]. "end new actor with parent"

															]. "end MakeObject parsing"

											]. "end file parsing"

			aFile close.
			myUndoStack openStack.

			"Ensure that the new actor's name is unique"
			name _ self uniqueNameFrom: (baseActor getName).
			baseActor setName: name.
			myNamespace at: name put: baseActor.

			"Add an undo item to undo the creation of this object"
			myUndoStack push: (UndoAction new: [ baseActor removeFromScene.
													myNamespace removeKey: name ifAbsent: [].
												  ] ).

			^ baseActor.
								]. " end mdl file parsing"
! !


!B3DTransformerPlugin methodsFor: 'primitives' stamp: 'ajh 3/15/2003 12:36'!
b3dOrthoNormInverseMatrix
	| srcOop dstOop src dst x y z rx ry rz |
	self export: true.
	self var: #src declareC:'float *src'.
	self var: #dst declareC:'float *dst'.
	self var: #x declareC:'double x'.
	self var: #y declareC:'double y'.
	self var: #z declareC:'double z'.
	self var: #rx declareC:'double rx'.
	self var: #ry declareC:'double ry'.
	self var: #rz declareC:'double rz'.

	interpreterProxy methodArgumentCount = 0
		ifFalse:[^interpreterProxy primitiveFail].
	srcOop := interpreterProxy stackObjectValue: 0.
	interpreterProxy failed ifTrue:[^nil].
	((interpreterProxy isWords: srcOop) and:[(interpreterProxy slotSizeOf: srcOop) = 16])
		ifFalse:[^interpreterProxy primitiveFail].
	dstOop := interpreterProxy clone: srcOop.
	"reload srcOop in case of GC"
	srcOop := interpreterProxy stackObjectValue: 0.
	src := interpreterProxy firstIndexableField: srcOop.
	dst := interpreterProxy firstIndexableField: dstOop.

	"Transpose upper 3x3 matrix"
	"dst at: 0 put: (src at: 0)."	dst at: 1 put: (src at: 4). 	dst at: 2 put: (src at: 8). 
	dst at: 4 put: (src at: 1). 	"dst at: 5 put: (src at: 5)."	dst at: 6 put: (src at: 9). 
	dst at: 8 put: (src at: 2). 	dst at: 9 put: (src at: 6). 	"dst at: 10 put: (src at: 10)."

	"Compute inverse translation vector"
	x := src at: 3.
	y := src at: 7.
	z := src at: 11.
	rx := (x * (dst at: 0)) + (y * (dst at: 1)) + (z * (dst at: 2)).
	ry := (x * (dst at: 4)) + (y * (dst at: 5)) + (z * (dst at: 6)).
	rz := (x * (dst at: 8)) + (y * (dst at: 9)) + (z * (dst at: 10)).

	dst at: 3 put: (self cCoerce: 0.0-rx to: 'float').
	dst at: 7 put: (self cCoerce: 0.0-ry to: 'float').
	dst at: 11 put: (self cCoerce: 0.0-rz to: 'float').

	interpreterProxy pop: 1.
	interpreterProxy push: dstOop.
! !


!BalloonEnginePlugin methodsFor: 'private' stamp: 'ajh 3/15/2003 12:38'!
shortRunLengthAt: i from: runArray
	"Return the run-length value from the given ShortRunArray."
	^((self cCoerce: runArray to:'int *') at: i) bitShift: -16! !


!CRDictionary class methodsFor: 'exported instances' stamp: 'ajh 3/15/2003 14:08'!
exportedName: aSymbol
	"Returns the instance with the given exported name.
	For a fast lookup, the instances with an exported name are stored in a dictionary
	of the class (ExportedInstanceDictionary)"

	aSymbol isEmptyOrNil ifTrue: [^ nil].
	^ ExportedInstanceDictionary at: aSymbol ifAbsent: [].! !


!CRFeature methodsFor: 'accessing' stamp: 'ajh 3/15/2003 14:08'!
asMorphSize: sizeNumber border: aNumberOrPoint relative: relativeBoolean properties: aCRDisplayProperties showPoints: pointsBoolean orientation: aSymbol 
	"Return a morph representing this feature"

	| morph subMorph |
	morph _ AlignmentMorph newColumn color: Color transparent.
	pointsBoolean
		ifTrue: 
			[subMorph _ AlignmentMorph new color: Color transparent.
			morph addMorphBack: subMorph.
			subMorph listDirection: aSymbol.
			subMorph addMorphBack: (self
					capturedPointsMorphSize: sizeNumber
					border: aNumberOrPoint
					relative: relativeBoolean
					properties: aCRDisplayProperties)]
		ifFalse: [subMorph _ morph].
	subMorph addMorphFront: (self
			featureMorphSize: sizeNumber
			border: aNumberOrPoint
			relative: relativeBoolean
			properties: aCRDisplayProperties).
	morph addMorphFront: self textMorph.
	^ morph.! !


!CRGesture methodsFor: 'accessing' stamp: 'ajh 3/15/2003 14:08'!
distance
	"The distance from the captured feature to the result feature at the iterator position"

	^ self distanceAt: self lookupIndex! !

!CRGesture methodsFor: 'accessing' stamp: 'ajh 3/15/2003 14:08'!
evaluateCode
	"Eveluate the code at the iterator position of the result with THIS INSTANCE AS A
	RECEIVER. (Only if the character at the iterator position is really code)"

	^ self evaluateCodeAt: self lookupIndex
! !


!CRStrokeFeature methodsFor: 'private comparing' stamp: 'ajh 3/15/2003 14:09'!
sameClassAcuteAngleDistance: aCRFeature maxNormDist: maxNumber
	"This mainly heuristic method measures the distance between two features in terms of
	acute angles and their positions"

	| myVal otherVal factor diff minCount significantCount result |

	"The position is only considered if the acute angle count is at least this value
	(An acute angle count of 100 means that there is one completely acute angle)." 
	minCount _ 50.

	myVal _ self acuteAngleCount.
	otherVal _ aCRFeature acuteAngleCount.
	significantCount _ myVal > otherVal ifTrue: [myVal] ifFalse: [otherVal].	"max"
	diff _ (myVal - otherVal).
	diff < 0 ifTrue: [diff _ 0 - diff].		"abs"
	factor _ 500 * diff * diff // 40000.	"500 * diff * diff // (200 * 200)"

	significantCount > minCount ifTrue: 
		[myVal _ self acuteAnglePosition.
		otherVal _ aCRFeature acuteAnglePosition.
		diff _ (myVal - otherVal).
		diff < 0 ifTrue: [diff _ 0 - diff].		"abs"
		factor _ factor + (500 * diff * diff // 1600)].

	result _ maxNumber * factor // 1000.
	^ maxNumber > result
		ifTrue: [result]
		ifFalse: [maxNumber].	"min"! !


!CRStrokeFeature class methodsFor: 'primitive test' stamp: 'ajh 3/15/2003 14:09'!
testPrimitiveDictionaryName: aDictSymbol forReference: forReferenceBoolean detailed: detailedBoolean
	"CRStrokeFeature primitiveTestDictionaryName: #PrimTest1 forReference: true.	"

	"Method to test a primitive for CRStrokeFeature>>sameClassAbsoluteStrokeDistance:forReference:.
	See comment of method CRStrokeFeature class>>testPrimitive:original:dictionary:relDiffLimit:"

	| primSelector origSelector dict relDiffLimit |

	"Selectors for the original and the primitive methods"
	primSelector _ #primSameClassAbsoluteStrokeDistance:forReference:.
	origSelector _ #origSameClassAbsoluteStrokeDistance:forReference:.

	"This boolean value is used as the second argument of the test method.
	The test should run successfully with both true and false"

	Transcript cr; show: 'Testing primitive with dictionary: ', aDictSymbol, ', for reference: ', forReferenceBoolean printString, '...'.

	"Use the CRStrokeFeatures in this dictionary as test cases"
	"Note: You can use the method #exportedName: instead of #name: to refer to the exported
	name of a dictionary"
	dict _ CRDictionary name: aDictSymbol.
	dict isNil ifTrue: [self error: 'Dictionary ', aDictSymbol, ' is not available!!'].

	"If the relative difference between primitive and original is bigger than this value, an
	exception is raised"
	relDiffLimit _ 0.

	dict fillFeaturesCacheIfNotFull.
	self
		testPrimitive: [:feature1 :feature2 | feature1 perform: primSelector with: feature2 with: forReferenceBoolean]
		original: [:feature1 :feature2 | feature1 perform: origSelector with: feature2 with: forReferenceBoolean]
		dictionary: dict
		relDiffLimit: relDiffLimit
		detailed: detailedBoolean
	! !


!ChangeSet methodsFor: 'accessing' stamp: 'ajh 3/15/2003 14:11'!
editPostscript
	"edit the receiver's postscript, in a separate window.  "

	| deps found |
	self assurePostscriptExists.
	deps _ postscript dependents select:
		[:m | (m isKindOf: SystemWindow) or:
			[m isKindOf: StandardSystemView]].
	deps size > 0 ifTrue:
		[Smalltalk isMorphic
			ifTrue:
				[found _ deps detect: [:obj | (obj isKindOf: SystemWindow) and: [obj world == self currentWorld]] ifNone: [nil].
				found ifNotNil:  [^ found activate]]
			ifFalse:
				[found _ deps detect: [:obj | (obj isKindOf: StandardSystemView) and: [ScheduledControllers scheduledControllers includes: obj controller]] ifNone: [nil].
				found ifNotNil: [^ ScheduledControllers activateController: found controller]].
		self inform:
'Caution -- there', (deps size isOrAreStringWith: 'other window'), '
already open on this postscript elsewhere'].

	postscript openLabel: 'Postscript for ChangeSet named ', name! !

!ChangeSet methodsFor: 'fileIn/Out' stamp: 'ajh 3/15/2003 14:11'!
objectForDataStream: refStrm
	"I am about to be written on an object file.  Write a path to me in the other system instead."

	refStrm projectChangeSet == self ifTrue: [^ self].

	"try to write reference for me"
	^ DiskProxy 
		global: #ChangeSorter
		selector: #existingOrNewChangeSetNamed: 
		args: (Array with: self name)
"===
	refStrm replace: self with: nil.
	^ nil
==="
! !


!Date methodsFor: 'inquiries' stamp: 'ajh 3/15/2003 14:15'!
daylightSavingsInEffect
	"Return true if DST is observed at or after 2am on this day"
	self dayMonthYearDo: 
		[:day :month :year |
		(month < 4 or: [month > 10]) ifTrue: [^ false].  "False November through March"
		(month > 4 and: [month < 10]) ifTrue: [^ true].  "True May through September"
		month = 4
		ifTrue:	["It's April -- true on first Sunday or later"
				day >= 7 ifTrue: [^ true].  "Must be after"
				^ day > (self weekdayIndex \\ 7)]
		ifFalse: ["It's October -- false on last Sunday or later"
				day <= 24 ifTrue: [^ true].  "Must be before"
				^ day <= (24 + (self weekdayIndex \\ 7))]]! !


!Decompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/15/2003 14:15'!
case: dist
	"statements = keyStmts CascadeFlag keyValueBlock ... keyStmts"

	| nextCase end thenJump stmtStream elements b node cases otherBlock |
	nextCase _ pc + dist.
	end _ limit.
	"Now add CascadeFlag & keyValueBlock to statements"
	statements addLast: stack removeLast.
	stack addLast: CaseFlag. "set for next pop"
	statements addLast: (self blockForCaseTo: nextCase).
	stack last == CaseFlag
		ifTrue: "Last case"
			["ensure jump is within block (in case thenExpr returns wierdly I guess)"
			stack removeLast. "get rid of CaseFlag"
			thenJump _ exit <= end ifTrue: [exit] ifFalse: [nextCase].
			stmtStream _ ReadStream on: (self popTo: stack removeLast).
			elements _ OrderedCollection new.
			b _ OrderedCollection new.
			[stmtStream atEnd] whileFalse:
				[(node _ stmtStream next) == CascadeFlag
					ifTrue:
						[elements addLast: (constructor
							codeMessage: (constructor codeBlock: b returns: false)
							selector: (constructor codeSelector: #-> code: #macro)
							arguments: (Array with: stmtStream next)).
						 b _ OrderedCollection new]
					ifFalse: [b addLast: node]].
			b size > 0 ifTrue: [self error: 'Bad cases'].
			cases _ constructor codeBrace: elements.
			otherBlock _ self blockTo: thenJump.
			stack addLast:
				(constructor
					codeMessage: stack removeLast
					selector: (constructor codeSelector: #caseOf:otherwise: code: #macro)
					arguments: (Array with: cases with: otherBlock))]! !


!FXBlt methodsFor: 'accessing' stamp: 'ajh 3/15/2003 14:18'!
colorMap: map
	"See last part of BitBlt comment. 6/18/96 tk"
	colorMap _ map.
	"As long as we need to fix problems with colorMaps"
	map ifNotNil:[
		(map isKindOf: ColorMap) ifFalse:[
			map size < 256 ifTrue:[
				colorMap _ ColorMap shifts: nil masks: nil colors: map.
			] ifFalse:[ sourceForm 
					ifNil:["Can't fix -- ignore" colorMap _ nil]
					ifNotNil:[colorMap _ sourceForm colormapIfNeededFor: destForm]]]].
! !

!FXBlt methodsFor: 'private' stamp: 'ajh 3/15/2003 14:18'!
recover
	"Recover after a BitBlt operation has failed. Return true if the
	copyBits operation should be tried again."

	"As long as we need to fix problems with colorMaps"
	colorMap ifNotNil:[
		(colorMap isKindOf: ColorMap) ifFalse:[
			colorMap size < 256 ifTrue:[
				colorMap _ ColorMap shifts: nil masks: nil colors: colorMap.
			] ifFalse:[ sourceForm 
					ifNil:["Can't fix -- ignore" colorMap _ nil]
					ifNotNil:[colorMap _ sourceForm colormapIfNeededFor: destForm]].
		^true "try again"]].

	"Check for compressed source, destination or halftone forms"
	((sourceForm isKindOf: Form) and: [sourceForm unhibernate])
		ifTrue: [^true]. "try again"
	((destForm isKindOf: Form) and: [destForm unhibernate])
		ifTrue: [^true]. "try again"
	((halftoneForm isKindOf: Form) and: [halftoneForm unhibernate])
		ifTrue: [^true]. "try again"

	^false "unable to recover"! !


!FilePlugin methodsFor: 'directory primitives' stamp: 'ajh 3/15/2003 14:19'!
makeDirEntryName: entryName size: entryNameSize
	createDate: createDate modDate: modifiedDate
	isDir: dirFlag fileSize: fileSize

	| modDateOop createDateOop nameString results stringPtr fileSizeOop |
	self var: 'entryName' declareC: 'char *entryName'.
	self var: 'stringPtr' declareC:'char *stringPtr'.
	self var: 'fileSize' declareC:'squeakFileOffsetType fileSize'.

	"allocate storage for results, remapping newly allocated
	 oops in case GC happens during allocation"
	interpreterProxy pushRemappableOop:
		(interpreterProxy instantiateClass: (interpreterProxy classArray) indexableSize: 5).
	interpreterProxy pushRemappableOop:
		(interpreterProxy instantiateClass: (interpreterProxy classString) indexableSize: entryNameSize).
	interpreterProxy pushRemappableOop: 
		(interpreterProxy positive32BitIntegerFor: createDate).
	interpreterProxy pushRemappableOop: 
		(interpreterProxy positive32BitIntegerFor: modifiedDate).
	interpreterProxy pushRemappableOop:
		(interpreterProxy positive64BitIntegerFor: fileSize).

	fileSizeOop   _ interpreterProxy popRemappableOop.
	modDateOop   _ interpreterProxy popRemappableOop.
	createDateOop _ interpreterProxy popRemappableOop.
	nameString    _ interpreterProxy popRemappableOop.
	results         _ interpreterProxy popRemappableOop.

	"copy name into Smalltalk string"
	stringPtr _ interpreterProxy firstIndexableField: nameString.
	0 to: entryNameSize - 1 do: [ :i |
		stringPtr at: i put: (entryName at: i).
	].

	interpreterProxy storePointer: 0 ofObject: results withValue: nameString.
	interpreterProxy storePointer: 1 ofObject: results withValue: createDateOop.
	interpreterProxy storePointer: 2 ofObject: results withValue: modDateOop.
	dirFlag
		ifTrue: [ interpreterProxy storePointer: 3 ofObject: results withValue: interpreterProxy trueObject ]
		ifFalse: [ interpreterProxy storePointer: 3 ofObject: results withValue: interpreterProxy falseObject ].
	interpreterProxy storePointer: 4 ofObject: results withValue: fileSizeOop.
	^ results! !


!FormEditor methodsFor: 'editing tools' stamp: 'ajh 3/15/2003 14:22'!
curve
	"Conic-section specified by three points designated by: first point--press 
	red button second point--release red button third point--click red button. 
	The resultant curve on the display is displayed according to the current 
	form and mode."

	| firstPoint secondPoint thirdPoint curve |
	"sensor noButtonPressed ifTrue: [^self]."
	firstPoint _ self cursorPoint.
	form
		displayOn: Display
		at: firstPoint
		clippingBox: view insetDisplayBox
		rule: (Display depth > 1 ifTrue: [Form paint]
										ifFalse: [mode])
		fillColor: color.
	secondPoint _ self trackFormUntil: [sensor noButtonPressed].
	form
		displayOn: Display
		at: secondPoint
		clippingBox: view insetDisplayBox
		rule: Form reverse
		fillColor: color.
	thirdPoint _ self trackFormUntil: [sensor redButtonPressed].
	form
		displayOn: Display
		at: thirdPoint
		clippingBox: view insetDisplayBox
		rule: (Display depth > 1 ifTrue: [Form paint]
										ifFalse: [mode])
		fillColor: color.
	form
		displayOn: Display
		at: secondPoint
		clippingBox: view insetDisplayBox
		rule: Form reverse
		fillColor: color.
	curve _ CurveFitter new.
	curve firstPoint: firstPoint.
	curve secondPoint: secondPoint.
	curve thirdPoint: thirdPoint.
	curve form: form.
	curve
		displayOn: Display
		at: 0 @ 0
		clippingBox: view insetDisplayBox
		rule: (Display depth > 1 ifTrue: [Form paint]
										ifFalse: [mode])
		fillColor: color.
	sensor waitNoButton! !


!HTTPLoader methodsFor: 'private' stamp: 'ajh 3/15/2003 14:23'!
removeProcess: downloadProcess
	downloads remove: downloadProcess ifAbsent: []! !


!HTTPSocket methodsFor: 'as yet unclassified' stamp: 'ajh 3/15/2003 14:23'!
header: headerText
	"set the headers.  Then getHeader: can be used"

	"divide into basic lines"
	| lines foldedLines i statusLine |
	lines _ headerText findTokens: (String with: Character cr  with: Character linefeed).
	statusLine _ lines first.
	lines _ lines copyFrom: 2 to: lines size.

	"parse the status (pretty trivial right now)"
	responseCode _ (statusLine findTokens: ' ') second.

	"fold lines that start with spaces into the previous line"
	foldedLines _ OrderedCollection new.
	lines do: [ :line |
		line first isSeparator ifTrue: [
			foldedLines at: foldedLines size  put: (foldedLines last, line) ]
		ifFalse: [ foldedLines add: line ] ].

	"make a dictionary mapping headers to header contents"
	headers _ Dictionary new.
	foldedLines do: [ :line |
		i _ line indexOf: $:.
		i > 0 ifTrue: [
			headers 
			at: (line copyFrom: 1 to: i-1) asLowercase 
			put: (line copyFrom: i+1 to: line size) withBlanksTrimmed ] ].
! !


!MethodFinder methodsFor: 'initialize' stamp: 'ajh 3/15/2003 14:33'!
copy: mthFinder addArg: aConstant
	| more |
	"Copy inputs and answers, add an additional data argument to the inputs.  The same constant for every example"

	more _ Array with: aConstant.
	data _ mthFinder data collect: [:argList | argList, more].
	answers _ mthFinder answers.
	self load: nil.
! !

!MethodFinder methodsFor: 'find a constant' stamp: 'ajh 3/15/2003 14:33'!
constEquiv
	| const subTest got jj |
	"See if (data1 = C) or (data1 ~= C) is the answer"

	"quick test"
	((answers at: 1) class superclass == Boolean) ifFalse: [^ false].
	2 to: answers size do: [:ii | 
		((answers at: ii) class superclass == Boolean) ifFalse: [^ false]].

	const _ (thisData at: 1) at: 1.
	got _ (subTest _ MethodFinder new copy: self addArg: const) 
				searchForOne isEmpty not.
	got ifFalse: ["try other polarity for ~~ "
		(jj _ answers indexOf: (answers at: 1) not) > 0 ifTrue: [
		const _ (thisData at: jj) at: 1.
		got _ (subTest _ MethodFinder new copy: self addArg: const) 
				searchForOne isEmpty not]]. 
	got ifFalse: [^ false]. 

	"replace data2 with const in expressions"
	subTest expressions do: [:exp |
		expressions add: (exp copyReplaceAll: 'data2' with: const printString)].
	selector addAll: subTest selectors.
	^ true! !


!Morph methodsFor: 'naming' stamp: 'ajh 3/15/2003 02:16'!
renameTo: aName
	"Set Player name in costume.  Update Viewers.  Fix all tiles (old style).  fix References.  New tiles: recompile, and recreate open scripts.  If coming in from disk, and have name conflict, References will already have new name."

	| aPresenter putInViewer aPasteUp renderer oldKey assoc classes oldName |
	oldName _ self knownName.
	(renderer _ self topRendererOrSelf) setNameTo: aName.
	putInViewer _ false.
	((aPresenter _ self presenter) isNil or: [renderer player isNil]) ifFalse:
		[putInViewer _ aPresenter currentlyViewing: renderer player.
		putInViewer ifTrue:
			[renderer player viewerFlapTab hibernate]].  "empty it temporarily"
	(aPasteUp _ self topPasteUp) ifNotNil:
		[aPasteUp allTileScriptingElements do: [:m | m bringUpToDate]].

	"Fix References dictionary.  See restoreReferences to know why oldKey is 
		already aName, but oldName is the old name."
	oldKey _ References keyAtIdentityValue: renderer player ifAbsent: [nil].
	oldKey ifNotNil:
		[assoc _ References associationAt: oldKey.
		oldKey = aName ifFalse:
			["normal rename"
			assoc key: (renderer player uniqueNameForReferenceFrom: aName).
			References rehash]].

	putInViewer ifTrue: [aPresenter viewMorph: self].	"recreate my viewer"
	oldKey ifNil: [^ aName].

	"Force strings in tiles to be remade with new name.  New tiles only."
	Preferences universalTiles ifFalse: [^ aName].
	classes _ (Smalltalk allCallsOn: assoc) collect: [ :each | each classSymbol].
	(classes asSet) do: [:clsName |
		(Smalltalk at: clsName) replaceSilently: oldName to: aName].
		"replace in text body of all methods.  Can be wrong!!"
	"Redo the tiles that are showing.  This is also done in caller in unhibernate."
	aPasteUp ifNotNil: [
		aPasteUp allTileScriptingElements do: [:mm | "just ScriptEditorMorphs"
			(mm isKindOf: ScriptEditorMorph) ifTrue:
				[((mm playerScripted class compiledMethodAt: mm scriptName) hasLiteral: assoc)
					ifTrue: [mm hibernate; unhibernate]]]].
	^ aName! !


!CategoryViewer methodsFor: 'entries' stamp: 'ajh 3/15/2003 14:10'!
addIsOverColorDetailTo: aRow
	| clrTile readout aTile |
	aRow addMorphBack: (Morph new color: self color; extent: 2@10).  " spacer"
	aRow addMorphBack: (clrTile _ Color blue newTileMorphRepresentative).
	aRow addMorphBack: (AlignmentMorph new beTransparent).  "flexible spacer"
	readout _ UpdatingStringMorphWithArgument new
			target: scriptedPlayer; getSelector: #seesColor:; growable: false; putSelector: nil;
			argumentTarget: clrTile colorSwatch argumentGetSelector: #color.
	readout useDefaultFormat.
	aTile _ StringReadoutTile new typeColor: Color lightGray lighter.
	aTile addMorphBack: readout.
	aRow addMorphBack: aTile.
	aTile setLiteralTo: (scriptedPlayer seesColor: clrTile colorSwatch color) printString width: 30! !

!CategoryViewer methodsFor: 'entries' stamp: 'ajh 3/15/2003 14:10'!
addOverlapsDetailTo: aRow
	"Disreputable magic: add necessary items to a viewer row abuilding for the overlaps phrase"

	aRow addMorphBack: (Morph new color: self color; extent: 2@10).  " spacer"
	aRow addMorphBack:  self tileForSelf.
	aRow addMorphBack: (AlignmentMorph new beTransparent).  "flexible spacer"

! !

!CategoryViewer methodsFor: 'entries' stamp: 'ajh 3/15/2003 14:10'!
addTouchesADetailTo: aRow
	| clrTile readout aTile |
	aRow addMorphBack: (Morph new color: self color; extent: 2@10).  " spacer"
	aRow addMorphBack: (clrTile _ self tileForSelf).
	aRow addMorphBack: (AlignmentMorph new beTransparent).  "flexible spacer"

	"readout _ UpdatingStringMorphWithArgument new
			target: scriptedPlayer; getSelector: #seesColor:; growable: false; putSelector: nil;
			argumentTarget: clrTile colorSwatch argumentGetSelector: #color.
	readout useDefaultFormat.
	aTile _ StringReadoutTile new typeColor: Color lightGray lighter.
	aTile addMorphBack: readout.
	aRow addMorphBack: aTile.
	aTile setLiteralTo: (scriptedPlayer seesColor: clrTile colorSwatch color) printString width: 30"! !

!CategoryViewer methodsFor: 'entries' stamp: 'ajh 3/15/2003 14:10'!
infoButtonFor: aScriptOrSlotSymbol
	"Answer a fully-formed morph that will serve as the 'info button' alongside an entry corresponding to the given slot or script symbol.  If no such button is appropriate, answer a transparent graphic that fills the same space."

	| aButton |
	(self wantsRowMenuFor: aScriptOrSlotSymbol) ifFalse:
		["Fill the space with sweet nothing, since there is no meaningful menu to offer"
		aButton _ RectangleMorph new beTransparent extent: (17@20).
		aButton borderWidth: 0.
		^ aButton].

	aButton _ IconicButton new labelGraphic: Cursor menu.
	aButton target: scriptedPlayer;
		actionSelector: #infoFor:inViewer:;
		arguments: (Array with:aScriptOrSlotSymbol with: self);
		color: Color transparent;
		borderWidth: 0;
		shedSelvedge;
		actWhen: #buttonDown.
	aButton setBalloonText: 'Press here to get a menu'.
	^ aButton! !


!IndentingListItemMorph methodsFor: 'mouse events' stamp: 'ajh 3/15/2003 14:25'!
acceptDroppingMorph: toDrop event: evt 
	complexContents acceptDroppingObject: toDrop complexContents.
	toDrop delete.
	self clearDropHighlighting.! !


!MagnifierMorph methodsFor: 'menu' stamp: 'ajh 3/15/2003 14:31'!
chooseMagnification
	| result |
	result _ (SelectionMenu selections: #(1.5 2 4 8))
		startUpWithCaption: 'Choose magnification
(currently ', magnification printString, ')'.
	(result == nil or: [result = magnification]) ifTrue: [^ self].
	magnification _ result.
	self extent: self extent. "round to new magnification"
	self changed. "redraw even if extent wasn't changed"! !


!FishEyeMorph methodsFor: 'initialization' stamp: 'ajh 3/15/2003 14:20'!
transformX: aFloatArray
	| focus gridNum2 subArray dMaxX |

	focus _ srcExtent x asFloat / 2.

	gridNum2 _ (aFloatArray findFirst: [:x | x > focus]) - 1.

	dMaxX _ 0.0 - focus.
	subArray _ self g: (aFloatArray copyFrom: 1 to: gridNum2) max: dMaxX focus: focus.

	aFloatArray replaceFrom: 1 to: gridNum2 with: subArray startingAt: 1.


	dMaxX _ focus.    " = (size - focus)"
	subArray _ self g: (aFloatArray copyFrom: gridNum2 + 1 to: gridNum x + 1)
		max: dMaxX focus: focus.

	aFloatArray replaceFrom: gridNum2 + 1 to: gridNum x + 1 with: subArray startingAt: 1.
! !

!FishEyeMorph methodsFor: 'initialization' stamp: 'ajh 3/15/2003 14:20'!
transformY: aFloatArray
	| focus subArray dMaxY |

	focus _ srcExtent y asFloat / 2.

	(aFloatArray at: 1) <= focus ifTrue: [
		dMaxY _ 0.0 - focus.
	] ifFalse: [
		dMaxY _ focus.    " = (size - focus)"
	].
		
	subArray _ self g: (aFloatArray copyFrom: 1 to: gridNum x + 1) max: dMaxY focus: focus.

	aFloatArray replaceFrom: 1 to: gridNum x + 1 with: subArray startingAt: 1.

! !


!ObjectMemory methodsFor: 'gc -- mark and sweep' stamp: 'ajh 3/15/2003 14:36'!
sweepPhase
	"Sweep memory from youngStart through the end of memory. Free all inaccessible objects and coalesce adjacent free chunks. Clear the mark bits of accessible objects. Compute the starting point for the first pass of incremental compaction (compStart). Return the number of surviving objects."
	"Details: Each time a non-free object is encountered, decrement the number of available forward table entries. If all entries are spoken for (i.e., entriesAvailable reaches zero), set compStart to the last free chunk before that object or, if there is no free chunk before the given object, the first free chunk after it. Thus, at the end of the sweep phase, compStart through compEnd spans the highest collection of non-free objects that can be accomodated by the forwarding table. This information is used by the first pass of incremental compaction to ensure that space is initially freed at the end of memory. Note that there should always be at least one free chunk--the one at the end of the heap."

	| entriesAvailable survivors freeChunk firstFree oop oopHeader oopHeaderType hdrBytes oopSize freeChunkSize |
	self inline: false.
	entriesAvailable _ self fwdTableInit: 8.  "Two-word blocks"
	survivors _ 0.
	freeChunk _ nil.
	firstFree _ nil.  "will be updated later"
	oop _ self oopFromChunk: youngStart.
	[oop < endOfMemory] whileTrue: [
		"get oop's header, header type, size, and header size"
		oopHeader _ self baseHeader: oop.
		oopHeaderType _ oopHeader bitAnd: TypeMask.
		(oopHeaderType = HeaderTypeShort) ifTrue: [
			oopSize _ oopHeader bitAnd: SizeMask.
			hdrBytes _ 0.
		] ifFalse: [
			(oopHeaderType = HeaderTypeClass) ifTrue: [
				oopSize _ oopHeader bitAnd: SizeMask.
				hdrBytes _ 4.
			] ifFalse: [
				(oopHeaderType = HeaderTypeSizeAndClass) ifTrue: [
					oopSize _ (self sizeHeader: oop) bitAnd: AllButTypeMask.
					hdrBytes _ 8.
				] ifFalse: [  "free chunk"
					oopSize _ oopHeader bitAnd: AllButTypeMask.
					hdrBytes _ 0.
				].
			].
		].

		(oopHeader bitAnd: MarkBit) = 0 ifTrue: ["object is not marked; free it"
			"<-- Finalization support: We need to mark each oop chunk as free -->"
			self longAt: oop - hdrBytes put: HeaderTypeFree.
			freeChunk ~= nil ifTrue: [
				"enlarge current free chunk to include this oop"
				freeChunkSize _ freeChunkSize + oopSize + hdrBytes.
			] ifFalse: [
				"start a new free chunk"
				freeChunk _ oop - hdrBytes.  "chunk may start 4 or 8 bytes before oop"
				freeChunkSize _ oopSize + (oop - freeChunk).  "adjust size for possible extra header bytes"
				firstFree = nil ifTrue: [ firstFree _ freeChunk ].
			].
		] ifFalse: [
			"object is marked; clear its mark bit and possibly adjust the compaction start"
			self longAt: oop put: (oopHeader bitAnd: AllButMarkBit).
			"<-- Finalization support: Check if we're running about a weak class -->"
			(self isWeak: oop) ifTrue:[
				self finalizeReference: oop.
			].
			entriesAvailable > 0 ifTrue: [
				entriesAvailable _ entriesAvailable - 1.
			] ifFalse: [
				"start compaction at the last free chunk before this object"
				firstFree _ freeChunk.
			].
			freeChunk ~= nil ifTrue: [
				"record the size of the last free chunk"
				self longAt: freeChunk
					    put: ((freeChunkSize bitAnd: AllButTypeMask) bitOr: HeaderTypeFree).
			].
			freeChunk _ nil.
			survivors _ survivors + 1.
		].
		oop _ self oopFromChunk: (oop + oopSize).  "get next oop"
	].

	freeChunk ~= nil ifTrue: [
		"record size of final free chunk"
		self longAt: freeChunk
			    put: ((freeChunkSize bitAnd: AllButTypeMask) bitOr: HeaderTypeFree).
	].

	oop = endOfMemory
		ifFalse: [ self error: 'sweep failed to find exact end of memory' ].

	firstFree = nil
		ifTrue: [ self error: 'expected to find at least one free object' ]
		ifFalse: [ compStart _ firstFree ].

	displayBits = 0 ifFalse: [
		"TPR: clear mark bit of Acorn's displayBits object, which may lie outside object space"
		oopHeader _ self baseHeader: displayBits.
		self longAt: displayBits put: (oopHeader bitAnd: AllButMarkBit)].	

	^ survivors
! !


!Interpreter methodsFor: 'image save/restore' stamp: 'ajh 3/15/2003 14:26'!
readImageFromFile: f HeapSize: desiredHeapSize StartingAt: imageOffset
	"Read an image from the given file stream, allocating the given amount of memory to its object heap. Fail if the image has an unknown format or requires more than the given amount of memory."
	"Details: This method detects when the image was stored on a machine with the opposite byte ordering from this machine and swaps the bytes automatically. Furthermore, it allows the header information to start 512 bytes into the file, since some file transfer programs for the Macintosh apparently prepend a Mac-specific header of this size. Note that this same 512 bytes of prefix area could also be used to store an exec command on Unix systems, allowing one to launch Smalltalk by invoking the image name as a command."
	"This code is based on C code by Ian Piumarta and Smalltalk code by Tim Rowledge. Many thanks to both of you!!!!"

	| swapBytes headerStart headerSize dataSize oldBaseAddr minimumMemory memStart bytesRead bytesToShift heapSize |
	self var: #f declareC: 'sqImageFile f'.
	self var: #headerStart declareC: 'squeakFileOffsetType headerStart'.
	self var: #dataSize declareC: 'size_t dataSize'.
	self var: #imageOffset declareC: 'squeakFileOffsetType imageOffset'.

	swapBytes _ self checkImageVersionFrom: f startingAt: imageOffset.
	headerStart _ (self sqImageFilePosition: f) - 4.  "record header start position"

	headerSize			_ self getLongFromFile: f swap: swapBytes.
	dataSize				_ self getLongFromFile: f swap: swapBytes.
	oldBaseAddr			_ self getLongFromFile: f swap: swapBytes.
	specialObjectsOop	_ self getLongFromFile: f swap: swapBytes.
	lastHash			_ self getLongFromFile: f swap: swapBytes.
	savedWindowSize	_ self getLongFromFile: f swap: swapBytes.
	fullScreenFlag		_ self getLongFromFile: f swap: swapBytes.
	extraVMMemory		_ self getLongFromFile: f swap: swapBytes.

	lastHash = 0 ifTrue: [
		"lastHash wasn't stored (e.g. by the cloner); use 999 as the seed"
		lastHash _ 999].

	"decrease Squeak object heap to leave extra memory for the VM"
	heapSize _ self cCode: 'reserveExtraCHeapBytes(desiredHeapSize, extraVMMemory)'.

	"compare memory requirements with availability"
	minimumMemory _ dataSize + 100000.  "need at least 100K of breathing room"
	heapSize < minimumMemory ifTrue: [
		GenerateBrowserPlugin
			ifTrue: [
				self plugInNotifyUser: 'The amount of memory specified by the ''memory'' EMBED tag is not enough for the installed Squeak image file.'.
				^ nil]
			ifFalse: [self error: 'Insufficient memory for this image']].

	"allocate a contiguous block of memory for the Squeak heap"
	memory _ self cCode: '(unsigned char *) sqAllocateMemory(minimumMemory, heapSize)'.
	memory = nil ifTrue: [
		GenerateBrowserPlugin
			ifTrue: [
				self plugInNotifyUser: 'There is not enough memory to give Squeak the amount specified by the ''memory'' EMBED tag.'.
				^ nil]
			ifFalse: [self error: 'Failed to allocate memory for the heap']].

	memStart _ self startOfMemory.
	memoryLimit _ (memStart + heapSize) - 24.  "decrease memoryLimit a tad for safety"
	endOfMemory _ memStart + dataSize.

	"position file after the header"
	self sqImageFile: f Seek: headerStart + headerSize.

	"read in the image in bulk, then swap the bytes if necessary"
	bytesRead _ self cCode: 'sqImageFileRead(memory, sizeof(unsigned char), dataSize, f)'.
	bytesRead ~= dataSize ifTrue: [
		GenerateBrowserPlugin
			ifTrue: [
				self plugInNotifyUser: 'Squeak had problems reading its image file.'.
				self plugInShutdown.
				^ nil]
			ifFalse: [self error: 'Read failed or premature end of image file']].
	swapBytes ifTrue: [self reverseBytesInImage].

	"compute difference between old and new memory base addresses"
	bytesToShift _ memStart - oldBaseAddr.
	self initializeInterpreter: bytesToShift.  "adjusts all oops to new location"
	^ dataSize
! !

!Interpreter methodsFor: 'image segment in/out' stamp: 'ajh 3/15/2003 14:28'!
primitiveStoreImageSegment
	"This primitive is called from Squeak as...
		<imageSegment> storeSegmentFor: arrayOfRoots into: aWordArray outPointers: anArray."

"This primitive will store a binary image segment (in the same format as the Squeak image file) of the receiver and every object in its proper tree of subParts (ie, that is not refered to from anywhere else outside the tree).  All pointers from within the tree to objects outside the tree will be copied into the array of outpointers.  In their place in the image segment will be an oop equal to the offset in the outPointer array (the first would be 4). but with the high bit set."

"The primitive expects the array and wordArray to be more than adequately long.  In this case it returns normally, and truncates the two arrays to exactly the right size.  To simplify truncation, both incoming arrays are required to be 256 bytes or more long (ie with 3-word headers).  If either array is too small, the primitive will fail, but in no other case.

During operation of the primitive, it is necessary to convert from both internal and external oops to their mapped values.  To make this fast, the headers of the original objects in question are replaced by the mapped values (and this is noted by adding the forbidden XX header type).  Tables are kept of both kinds of oops, as well as of the original headers for restoration.

To be specific, there are two similar two-part tables, the outpointer array, and one in the upper fifth of the segmentWordArray.  Each grows oops from the bottom up, and preserved headers from halfway up.

In case of either success or failure, the headers must be restored.  In the event of primitive failure, the table of outpointers must also be nilled out (since the garbage in the high half will not have been discarded."

	| outPointerArray segmentWordArray savedYoungStart lastOut lastIn firstIn lastSeg endSeg segOop fieldPtr fieldOop mapOop doingClass lastPtr extraSize hdrTypeBits arrayOfRoots hdrBaseIn hdrBaseOut header firstOut versionOffset |

	outPointerArray _ self stackValue: 0.
	segmentWordArray _ self stackValue: 1.
	arrayOfRoots _ self stackValue: 2.

	"Essential type checks"
	((self formatOf: arrayOfRoots) = 2				"Must be indexable pointers"
		and: [(self formatOf: outPointerArray) = 2				"Must be indexable pointers"
		and: [(self formatOf: segmentWordArray) = 6]])	"Must be indexable words"
		ifFalse: [^ self primitiveFail].
	((self headerType: outPointerArray) = HeaderTypeSizeAndClass	"Must be 3-word header"
		and: [(self headerType: segmentWordArray) = HeaderTypeSizeAndClass])	"Must be 3-word header"
		ifFalse: [^ self primitiveFail].

	DoAssertionChecks ifTrue: [self verifyCleanHeaders].
	"Use the top half of outPointers for saved headers."
	firstOut _ outPointerArray + BaseHeaderSize.
	lastOut _ firstOut - 4.
	hdrBaseOut _ outPointerArray + ((self lastPointerOf: outPointerArray) // 8 * 4). "top half"

	lastSeg _ segmentWordArray.
	endSeg _ segmentWordArray + (self sizeBitsOf: segmentWordArray) - 4.

	"Write a version number for byte order and version check"
	versionOffset _ 4.
	lastSeg _ lastSeg + versionOffset.
	lastSeg > endSeg ifTrue: [^ self primitiveFail].
	self longAt: lastSeg put: self imageSegmentVersion.

	"Allocate top 1/8 of segment for table of internal oops and saved headers"
	firstIn _ endSeg - ((self sizeBitsOf: segmentWordArray) // 32 * 4).  "Take 1/8 of seg"
	lastIn _ firstIn - 4.
	hdrBaseIn _ firstIn + ((self sizeBitsOf: segmentWordArray) // 64 * 4). "top half"

	"First mark the rootArray and all root objects."
	self longAt: arrayOfRoots put: ((self longAt: arrayOfRoots) bitOr: MarkBit).
	lastPtr _ arrayOfRoots + (self lastPointerOf: arrayOfRoots).
	fieldPtr _ arrayOfRoots + BaseHeaderSize.
	[fieldPtr <= lastPtr] whileTrue:
		[fieldOop _ self longAt: fieldPtr.
		(self isIntegerObject: fieldOop) ifFalse:
			[self longAt: fieldOop put: ((self longAt: fieldOop) bitOr: MarkBit)].
		fieldPtr _ fieldPtr + 4].

	"Then do a mark pass over all objects.  This will stop at our marked roots,
	thus leaving our segment unmarked in their shadow."
	savedYoungStart _ youngStart.
	youngStart _ self startOfMemory.  "process all of memory"
		"clear the recycled context lists"
		freeContexts _ NilContext.
		freeLargeContexts _ NilContext.
	self markAndTraceInterpreterOops.	"and special objects array"
	youngStart _ savedYoungStart.
	
	"Finally unmark the rootArray and all root objects."
	self longAt: arrayOfRoots put: ((self longAt: arrayOfRoots) bitAnd: AllButMarkBit).
	fieldPtr _ arrayOfRoots + BaseHeaderSize.
	[fieldPtr <= lastPtr] whileTrue:
		[fieldOop _ self longAt: fieldPtr.
		(self isIntegerObject: fieldOop) ifFalse:
			[self longAt: fieldOop put: ((self longAt: fieldOop) bitAnd: AllButMarkBit)].
		fieldPtr _ fieldPtr + 4].

	"All external objects, and only they, are now marked.
	Copy the array of roots into the segment, and forward its oop."
	lastIn _ lastIn + 4.
	lastIn >= hdrBaseIn ifTrue: [successFlag _ false].
	lastSeg _ self copyObj: arrayOfRoots toSegment: segmentWordArray addr: lastSeg stopAt: firstIn saveOopAt: lastIn headerAt: hdrBaseIn + (lastIn - firstIn).
	successFlag ifFalse:
		[lastIn _ lastIn - 4.
		self restoreHeadersFrom: firstIn to: lastIn from: hdrBaseIn and: firstOut to: lastOut from: hdrBaseOut.
		^ self primitiveFailAfterCleanup: outPointerArray].

	"Now run through the segment fixing up all the pointers.
	Note that more objects will be added to the segment as we make our way along."
	segOop _ self oopFromChunk: segmentWordArray + versionOffset + BaseHeaderSize.
	[segOop <= lastSeg] whileTrue:
		[(self headerType: segOop) <= 1
			ifTrue: ["This object has a class field (type=0 or 1) -- start with that."
					fieldPtr _ segOop - 4.  doingClass _ true]
			ifFalse: ["No class field -- start with first data field"
					fieldPtr _ segOop + BaseHeaderSize.  doingClass _ false].
		lastPtr _ segOop + (self lastPointerOf: segOop).	"last field"

		"Go through all oops, remapping them..."
		[fieldPtr > lastPtr] whileFalse:
			["Examine each pointer field"
			fieldOop _ self longAt: fieldPtr.
			doingClass ifTrue:
				[hdrTypeBits _ fieldOop bitAnd: TypeMask.
				fieldOop _ fieldOop - hdrTypeBits].
			(self isIntegerObject: fieldOop)
				ifTrue: ["Just an integer -- nothing to do"
						fieldPtr _ fieldPtr + 4]
				ifFalse:
				[header _ self longAt: fieldOop.
				(header bitAnd: TypeMask) = HeaderTypeFree
					ifTrue: ["Has already been forwarded -- this is the link"
							mapOop _ header bitAnd: AllButTypeMask]
					ifFalse:
					[((self longAt: fieldOop) bitAnd: MarkBit) = 0
						ifTrue:
							["Points to an unmarked obj -- an internal pointer.
							Copy the object into the segment, and forward its oop."
							lastIn _ lastIn + 4.
							lastIn >= hdrBaseIn ifTrue: [successFlag _ false].
							lastSeg _ self copyObj: fieldOop toSegment: segmentWordArray addr: lastSeg stopAt: firstIn saveOopAt: lastIn headerAt: hdrBaseIn + (lastIn - firstIn).
							successFlag ifFalse:
								["Out of space in segment"
								lastIn _ lastIn - 4.
								self restoreHeadersFrom: firstIn to: lastIn from: hdrBaseIn and: firstOut to: lastOut from: hdrBaseOut.
								^ self primitiveFailAfterCleanup: outPointerArray].
							mapOop _ (self longAt: fieldOop) bitAnd: AllButTypeMask]
						ifFalse:
							["Points to a marked obj -- an external pointer.
							Map it as a tagged index in outPointers, and forward its oop."
							lastOut _ lastOut + 4.
							lastOut >= hdrBaseOut ifTrue:
								["Out of space in outPointerArray"
								lastOut _ lastOut - 4.
								self restoreHeadersFrom: firstIn to: lastIn from: hdrBaseIn and: firstOut to: lastOut from: hdrBaseOut.
								^ self primitiveFailAfterCleanup: outPointerArray].
							mapOop _ lastOut - outPointerArray bitOr: 16r80000000.
							self forward: fieldOop to: mapOop
								savingOopAt: lastOut andHeaderAt: hdrBaseOut + (lastOut - firstOut)]].
					"Replace the oop by its mapped value"
					doingClass
						ifTrue: [self longAt: fieldPtr put: mapOop + hdrTypeBits.
								fieldPtr _ fieldPtr + 8.
								doingClass _ false]
						ifFalse: [self longAt: fieldPtr put: mapOop.
								fieldPtr _ fieldPtr + 4].
]].
		segOop _ self objectAfter: segOop].

	self restoreHeadersFrom: firstIn to: lastIn from: hdrBaseIn and: firstOut to: lastOut from: hdrBaseOut.

	"Truncate the outPointerArray..."
	((outPointerArray + (self lastPointerOf: outPointerArray) - lastOut) < 12
		or: [(endSeg - lastSeg) < 12]) ifTrue:
			["Not enough room to insert simple 3-word headers"
			^ self primitiveFailAfterCleanup: outPointerArray].
	extraSize _ self extraHeaderBytes: segmentWordArray.
	hdrTypeBits _ self headerType: segmentWordArray.
	"Copy the 3-word wordArray header to establish a free chunk."
	self transfer: 3
		from: segmentWordArray - extraSize
		to: lastOut+4.
	"Adjust the size of the original as well as the free chunk."
	self longAt: lastOut+4
		put: outPointerArray + (self lastPointerOf: outPointerArray) - lastOut - extraSize + hdrTypeBits.
	self longAt: outPointerArray-extraSize
		put: lastOut - firstOut + 8 + hdrTypeBits.
	"Note that pointers have been stored into roots table"
	self beRootIfOld: outPointerArray.

	"Truncate the image segment..."
	"Copy the 3-word wordArray header to establish a free chunk."
	self transfer: 3
		from: segmentWordArray - extraSize
		to: lastSeg+4.
	"Adjust the size of the original as well as the free chunk."
	self longAt: segmentWordArray-extraSize
		put: lastSeg - segmentWordArray + BaseHeaderSize + hdrTypeBits.
	self longAt: lastSeg+4
		put: endSeg - lastSeg - extraSize + hdrTypeBits.

	DoAssertionChecks ifTrue: [self verifyCleanHeaders].
	self pop: 3.  "...leaving the reciever on the stack as return value"
! !


!InterpreterSimulator methodsFor: 'file primitives' stamp: 'ajh 3/15/2003 14:29'!
makeDirEntryName: entryName size: entryNameSize
	createDate: createDate modDate: modifiedDate
	isDir: dirFlag fileSize: fileSize

	| modDateOop createDateOop nameString results |
	self var: 'entryName' declareC: 'char *entryName'.

	"allocate storage for results, remapping newly allocated
	 oops in case GC happens during allocation"
	self pushRemappableOop:
		(self instantiateClass: (self splObj: ClassArray) indexableSize: 5).
	self pushRemappableOop:
		(self instantiateClass: (self splObj: ClassString) indexableSize: entryNameSize).
	self pushRemappableOop: (self positive32BitIntegerFor: createDate).
	self pushRemappableOop: (self positive32BitIntegerFor: modifiedDate).

	modDateOop   _ self popRemappableOop.
	createDateOop _ self popRemappableOop.
	nameString    _ self popRemappableOop.
	results         _ self popRemappableOop.

	1 to: entryNameSize do: [ :i |
		self storeByte: i-1 ofObject: nameString withValue: (entryName at: i) asciiValue.
	].

	self storePointer: 0 ofObject: results withValue: nameString.
	self storePointer: 1 ofObject: results withValue: createDateOop.
	self storePointer: 2 ofObject: results withValue: modDateOop.
	dirFlag
		ifTrue: [ self storePointer: 3 ofObject: results withValue: trueObj ]
		ifFalse: [ self storePointer: 3 ofObject: results withValue: falseObj ].
	self storePointer: 4 ofObject: results
		withValue: (self integerObjectOf: fileSize).
	^ results
! !

!InterpreterSimulator methodsFor: 'float primitives' stamp: 'ajh 3/15/2003 14:29'!
storeFloatAt: floatBitsAddress from: aFloat

	self longAt: floatBitsAddress put: (aFloat at: 1).
	self longAt: floatBitsAddress+4 put: (aFloat at: 2).
! !


!PCXReadWriter methodsFor: 'private-decoding' stamp: 'ajh 3/15/2003 14:42'!
readHeader

	| xMin xMax yMin yMax |
	self next.	"skip over manufacturer field"
	version _ self next.
	encoding _ self next.
	bitsPerPixel _ self next.
	xMin _ self nextWord.
	yMin _ self nextWord.
	xMax _ self nextWord.
	yMax _ self nextWord.
	width _ xMax - xMin + 1.
	height _ yMax - yMin + 1.
	self next: 4. "skip over device resolution"
	self next: 49. "skip over EGA color palette"
	colorPlanes _ self next.
	rowByteSize _ self nextWord.
	isGrayScale _ (self next: 2) = 2.
	self next: 58. "skip over filler"



! !


!PaintBoxMorph methodsFor: 'initialization' stamp: 'ajh 3/15/2003 14:43'!
loadoffImage: fileName
	"Read in and convert the background image for the paintBox.  All
buttons off.  A .bmp 24-bit image."
	"	Prototype loadoffImage: 'roundedPalette3.bmp'	"

	| pic16Bit blt type getBounds |
	type _ 'bmp'.  " gif or bmp  "
	getBounds _ 'fromPic'.	"fromUser = draw out rect of paintbox on image"
		"fromOB = just read in new bits, keep same size and place as last time."
		"fromPic = picture is just the PaintBox, use its bounds"
type = 'gif' ifTrue: [
	pic16Bit "really 8" _ GIFReadWriter formFromFileNamed: fileName.
	getBounds = 'fromUser' ifTrue: ["Just first time, collect the bounds"
			pic16Bit display.
			OriginalBounds _ Rectangle fromUser].
	getBounds = 'fromPic' ifTrue: [OriginalBounds _ pic16Bit boundingBox].
	].
		"Use OriginalBounds as it was last time"
type = 'bmp' ifTrue: [
	pic16Bit _ (Form fromBMPFileNamed: fileName) asFormOfDepth: 16.
	getBounds = 'fromUser' ifTrue: ["Just first time, collect the bounds"
			pic16Bit display.
			OriginalBounds _ Rectangle fromUser].
		"Use OriginalBounds as it was last time"
	(getBounds = 'fromPic') ifTrue: [OriginalBounds _ pic16Bit boundingBox].
	AllOffImage _ Form extent: OriginalBounds extent depth: 16.
	].

type = 'gif' ifTrue: [
	AllOffImage _ ColorForm extent: OriginalBounds extent depth: 8.
	AllOffImage colors: pic16Bit colors].

	blt _ BitBlt current toForm: AllOffImage.
	blt sourceForm: pic16Bit; combinationRule: Form over;
		sourceRect: OriginalBounds; destOrigin: 0@0; copyBits.

type = 'bmp' ifTrue: [AllOffImage mapColor: Color transparent to: Color black].
	self image: AllOffImage.
	self invalidRect: bounds.

	! !


!PenPointRecorder methodsFor: 'line drawing' stamp: 'ajh 3/15/2003 14:55'!
drawFrom: p1 to: p2
	"Overridden to skip drawing but track bounds of the region traversed."

	points ifNil: [points _ OrderedCollection with: p1].
	points addLast: p2! !


!PianoRollScoreMorph methodsFor: 'submorphs-add/remove' stamp: 'ajh 3/15/2003 14:56'!
privateRemoveMorph: aMorph
	| trackSize |
	trackSize _ score ambientTrack size.
	score removeAmbientEventWithMorph: aMorph.
	trackSize = score ambientTrack size ifFalse:
		["Update duration if we removed an event"
		scorePlayer updateDuration].
	^ super privateRemoveMorph: aMorph! !


!PluggableMessageCategoryListMorph methodsFor: 'as yet unclassified' stamp: 'ajh 3/15/2003 15:52'!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel keystroke: keyActionSel getRawListSelector: getRawSel

	self model: anObject.
	getListSelector _ getListSel.
	getIndexSelector _ getSelectionSel.
	setIndexSelector _ setSelectionSel.
	getMenuSelector _ getMenuSel.
	keystrokeActionSelector _ keyActionSel.
	autoDeselect _ true.
	self borderWidth: 1.
	getRawListSelector _ getRawSel.
	self list: self getList.
	self selectionIndex: self getCurrentSelectionIndex.
	self initForKeystrokes! !


!PluggableMessageCategoryListMorph class methodsFor: 'as yet unclassified' stamp: 'ajh 3/15/2003 15:52'!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel keystroke: keyActionSel getRawListSelector: getRawSel

	^ self new on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel keystroke: keyActionSel getRawListSelector: getRawSel! !


!PostscriptCanvas methodsFor: 'morph drawing' stamp: 'ajh 3/15/2003 15:53'!
setupGStateForMorph:aMorph
! !

!PostscriptCanvas methodsFor: 'drawing' stamp: 'ajh 3/15/2003 15:53'!
drawRectangle: r
		color: color
		borderWidth: borderWidth
		borderColor: borderColor

	^self frameAndFillRectangle: r fillColor: color borderWidth: borderWidth borderColor: borderColor .
! !


!Preferences class methodsFor: 'misc' stamp: 'ajh 3/15/2003 16:27'!
browseThemes
	"Open up a message-category browser on the theme-defining methods"

	| aBrowser |
	aBrowser _ Browser new setClass: Preferences class selector: #outOfTheBox.
	aBrowser messageCategoryListIndex: ((Preferences class organization categories indexOf: 'themes' ifAbsent: [^ self inform: 'no themes found']) + 1).
	Browser openBrowserView: (aBrowser openMessageCatEditString: nil)
		label: 'Preference themes'

	"Preferences browseThemes"! !


!ProjectSorterMorph methodsFor: 'as yet unclassified' stamp: 'ajh 3/15/2003 16:29'!
navigator: aThreadNavigator listOfPages: listOfPages

	| morphsForPageSorter pixelsAvailable pixelsNeeded scale |

	"a bit of fudging to try to outguess the layout mechanism and get best possible scale"
	pixelsAvailable _ Display extent - 130.
	pixelsAvailable _ pixelsAvailable x * pixelsAvailable y.
	pixelsNeeded _ 100@75.
	pixelsNeeded _ pixelsNeeded x * pixelsNeeded y  * listOfPages size.
	scale _ (pixelsAvailable / pixelsNeeded min: 1) sqrt.
	sizeOfEachMorph _ (100@75 * scale) rounded.

	morphsForPageSorter _ self morphsForMyContentsFrom: listOfPages sizedTo: sizeOfEachMorph.
	morphsForPageSorter _ morphsForPageSorter reject: [ :each | each isNil].
	self changeExtent: Display extent.

	self
		book: aThreadNavigator 
		morphsToSort: morphsForPageSorter.
	pageHolder 
		cursor: aThreadNavigator currentIndex;
		fullBounds;
		hResizing: #rigid.

! !


!Scanner methodsFor: 'multi-character scans' stamp: 'ajh 3/15/2003 16:31'!
xColon		"Allow := for assignment by converting to #_ "
	aheadChar = $= ifTrue:
		[self step.
		tokenType _ #leftArrow.
		self step.
		^ token _ #'_'].
	"Otherwise, just do what normal scan of colon would do"
	tokenType _ #colon.
	^ token _ self step asSymbol! !


!Parser methodsFor: 'primitives' stamp: 'ajh 3/15/2003 14:48'!
externalFunctionDeclaration
	"Parse the function declaration for a call to an external library."
	| descriptorClass callType retType externalName args argType module fn |
	descriptorClass _ Smalltalk at: #ExternalFunction ifAbsent:[nil].
	descriptorClass == nil ifTrue:[^0].
	callType _ descriptorClass callingConventionFor: here.
	callType == nil ifTrue:[^0].
	"Parse return type"
	self advance.
	retType _ self externalType: descriptorClass.
	retType == nil ifTrue:[^self expected:'return type'].
	"Parse function name or index"
	externalName _ here.
	(self match: #string) 
		ifTrue:[externalName _ externalName asSymbol]
		ifFalse:[(self match:#number) ifFalse:[^self expected:'function name or index']].
	(self matchToken:'(' asSymbol) ifFalse:[^self expected:'argument list'].
	args _ WriteStream on: Array new.
	[here == #')'] whileFalse:[
		argType _ self externalType: descriptorClass.
		argType == nil ifTrue:[^self expected:'argument'].
		argType isVoid & argType isPointerType not ifFalse:[args nextPut: argType].
	].
	(self matchToken:')' asSymbol) ifFalse:[^self expected:')'].
	(self matchToken: 'module:') ifTrue:[
		module _ here.
		(self match: #string) ifFalse:[^self expected: 'String'].
		module _ module asSymbol].
	Smalltalk at: #ExternalLibraryFunction ifPresent:[:xfn|
		fn _ xfn name: externalName 
				module: module 
				callType: callType
				returnType: retType
				argumentTypes: args contents.
		self allocateLiteral: fn.
	].
	^120! !


!ScreenController methodsFor: 'menu messages' stamp: 'ajh 3/15/2003 16:31'!
propagateChanges
	"The changes made in this isolated project will be propagated to projects above."

	CurrentProjectRefactoring currentPropagateChanges! !


!SecureHashAlgorithm methodsFor: 'public' stamp: 'ajh 3/15/2003 16:32'!
hashInteger: aPositiveInteger seed: seedInteger
	"Hash the given positive integer. The integer to be hashed should have 512 or fewer bits. This entry point is used in the production of random numbers"

	| buffer dstIndex |
	"Initialize totalA through totalE to their seed values."
	totalA _ ThirtyTwoBitRegister new
		load: ((seedInteger bitShift: -128) bitAnd: 16rFFFFFFFF).
	totalB _ ThirtyTwoBitRegister new
		load: ((seedInteger bitShift: -96) bitAnd: 16rFFFFFFFF).
	totalC _ ThirtyTwoBitRegister new
		load: ((seedInteger bitShift: -64) bitAnd: 16rFFFFFFFF).
	totalD _ ThirtyTwoBitRegister new
		load: ((seedInteger bitShift: -32) bitAnd: 16rFFFFFFFF).
	totalE _ ThirtyTwoBitRegister new
		load: (seedInteger bitAnd: 16rFFFFFFFF).
	self initializeTotalsArray.

	"pad integer with zeros"
	buffer _ ByteArray new: 64.
	dstIndex _ 0.
	aPositiveInteger digitLength to: 1 by: -1 do: [:i |
		buffer at: (dstIndex _ dstIndex + 1) put: (aPositiveInteger digitAt: i)].

	"process that one block"
	self processBuffer: buffer.

	^ self finalHash
! !


!SelectionMorph methodsFor: 'halo commands' stamp: 'ajh 3/15/2003 16:32'!
doDup: evt fromHalo: halo handle: dupHandle

	selectedItems _ selectedItems collect: [:m | m duplicate].
	selectedItems do: [:m | self owner addMorph: m].
	dupDelta == nil
		ifTrue: ["First duplicate operation -- note starting location"
				dupLoc _ self position.
				evt hand grabMorph: self.
				halo removeAllHandlesBut: dupHandle]
		ifFalse: ["Subsequent duplicate does not grab, but only moves me and my morphs"
				dupLoc _ nil.
				self position: self position + dupDelta]
! !


!SelectorBrowser methodsFor: 'as yet unclassified' stamp: 'ajh 3/15/2003 16:33'!
markMatchingClasses
	"If an example is used, mark classes matching the example instance with an asterisk."

	| unmarkedClassList firstPartOfSelector receiverString receiver |

	self flag: #mref.	"allows for old-fashioned style"

	"Only 'example' queries can be marked."
	(contents asString includes: $.) ifFalse: [^ self].

	unmarkedClassList _ classList copy.

	"Get the receiver object of the selected statement in the message list."
	firstPartOfSelector _ (Scanner new scanTokens: (selectorList at: selectorIndex)) second.
	receiverString _ (ReadStream on: (selectorList at: selectorIndex))
						upToAll: firstPartOfSelector.
	receiver _ Compiler evaluate: receiverString.

	unmarkedClassList do: [ :classAndMethod | | class |
		(classAndMethod isKindOf: MethodReference) ifTrue: [
			(receiver isKindOf: classAndMethod actualClass) ifTrue: [
				classAndMethod stringVersion: '*', classAndMethod stringVersion.
			]
		] ifFalse: [
			class _ Compiler evaluate:
					((ReadStream on: classAndMethod) upToAll: firstPartOfSelector).
			(receiver isKindOf: class) ifTrue: [
				classList add: '*', classAndMethod.
				classList remove: classAndMethod
			]
		].
	].
! !

!SelectorBrowser methodsFor: 'as yet unclassified' stamp: 'ajh 3/15/2003 16:33'!
selectedMessageName
	"Answer the name of the currently selected message."

	| example tokens |
	selectorIndex = 0 ifTrue: [^nil].
	example _ selectorList at: selectorIndex.
	tokens _ Scanner new scanTokens: example.
	tokens size = 1 ifTrue: [^ tokens first].
	tokens first == #'^' ifTrue: [^ nil].
	(tokens second includes: $:) ifTrue: [^ example findSelector].
	Symbol hasInterned: tokens second ifTrue: [:aSymbol | ^ aSymbol].
	^ nil! !


!ServerDirectory methodsFor: 'dis/connect' stamp: 'ajh 3/15/2003 16:34'!
openNoDataFTP
	"Open a connection to the directory and server I hold.  Return a FTPSocket.  No dataPort is opened.  When you are all done, be sure to tell the socket to QUIT, and then destroy it."

	| so rr serverIP what |
	Socket initializeNetwork.
	socket ifNotNil: [socket isValid 
			ifTrue: [^ socket]	"already open"
			ifFalse: [socket _ nil]].

	Cursor wait showWhile: [
		serverIP _ NetNameResolver addressForName: server timeout: 20.
		serverIP ifNil: [
			^ 'Could not resolve the server named: ', server].
		so _ FTPSocket new.
		so portNum: 21.
		so connectTo: serverIP port: 21.  "21 is for the control connection"
		so waitForConnectionUntil: FTPSocket standardDeadline.
		].

	"Transcript cr; show: 'ftp: ', server; cr."
	(rr _ so lookFor: '220 ') == true ifFalse: [^ rr].	"220 para1 Microsoft FTP Service"
	[	"repeat both USER and PASS since some servers require it"
		so sendCommand: 'USER ', user.
		(rr _ so lookFor: '331 ') == true ifFalse: [^ rr].	"331 Password required"
		so sendCommand: 'PASS ', self password.		"will ask user, if needed"
		 (rr _ so lookSoftlyFor: '230 ') == true
	] 	"230 User logged in"
		whileFalse: [
			rr first == $5 ifFalse: [^ rr].	"timeout"
			passwordHolder _ nil.
			what _ (PopUpMenu labels: 'enter password\give up' withCRs) 
				startUpWithCaption: 'Would you like to try another password?'.
			what = 1 ifFalse: [so destroy.  ^ rr]].
	(rr _ self changeWorkingDirectory: so) == true ifFalse: [^rr].
	"Need to ask for name of directory to make sure?"
	"socket _ so."	"If user wants to keep connnection open, he must store socket"
	^ so! !


!StrikeFont methodsFor: 'file in/out' stamp: 'ajh 3/15/2003 16:37'!
newFromStrike: fileName
	"Build an instance from the strike font file name. The '.strike' extension
	is optional."

	| strike startName raster16 |
	name _ fileName copyUpTo: $..	"assumes extension (if any) is '.strike'"
	strike _ FileStream readOnlyFileNamed: name, '.strike.'.
	strike binary.

	"strip off direcory name if any"
	startName _ name size.
	[startName > 0 and: [((name at: startName) ~= $>) & ((name at: startName) ~= $])]]
		whileTrue: [startName _ startName - 1].
	name _ name copyFrom: startName+1 to: name size.

	type			_		strike nextWord.		"type is ignored now -- simplest
												assumed.  Kept here to make
												writing and consistency more
												straightforward."
	minAscii		_		strike nextWord.
	maxAscii		_		strike nextWord.
	maxWidth		_		strike nextWord.
	strikeLength	_		strike nextWord.
	ascent			_		strike nextWord.
	descent			_		strike nextWord.
	"xOffset			_"		strike nextWord. 	
	raster16			_		strike nextWord.	
	superscript		_		ascent - descent // 3.	
	subscript		_		descent - ascent // 3.	
	emphasis		_		0.
	glyphs			_	Form extent: (raster16 * 16) @ (self height)  
							offset: 0@0.
		glyphs bits fromByteStream: strike.

	xTable _ (Array new: maxAscii + 3) atAllPut: 0.
	(minAscii + 1 to: maxAscii + 3) do:
		[:index | xTable at: index put: strike nextWord].

	"Set up space character"
	((xTable at: (Space asciiValue + 2))  = 0 or:
			[(xTable at: (Space asciiValue + 2)) = (xTable at: (Space asciiValue + 1))])
		ifTrue:	[(Space asciiValue + 2) to: xTable size do:
					[:index | xTable at: index put: ((xTable at: index) + DefaultSpace)]].
	strike close.
	characterToGlyphMap _ nil.! !


!String methodsFor: 'accessing' stamp: 'ajh 3/15/2003 16:37'!
lineCount
	"Answer the number of lines represented by the receiver, where every cr adds one line.  5/10/96 sw"

	| cr count |
	cr _ Character cr.
	count _ 1  min: self size.
	1 to: self size do:
		[:i | (self at: i) == cr ifTrue: [count _ count + 1]].
	^ count

"
'Fred
the
Bear' lineCount
"! !


!SuperSwikiServer methodsFor: 'testing' stamp: 'ajh 3/15/2003 16:38'!
queryProjects: criteria
	| result |
	"SuperSwikiServer defaultSuperSwiki queryProjects: #('submittedBy: mir' )"
	result _ self sendToSwikiProjectServer: {
		'action: findproject'.
	}  , criteria.
	(result beginsWith: 'OK') ifFalse: [^self inform: result printString].
	^self parseQueryResult: (ReadStream on: result).
! !

!SuperSwikiServer methodsFor: 'for real' stamp: 'ajh 3/15/2003 16:38'!
matchingEntries: criteria
	| result |
	eToyUserListUrl ifNil:[^self entries].
	result _ self sendToSwikiProjectServer: {
		'action: listmatchingprojects'.
	}  , criteria.
	(result beginsWith: 'OK')
		ifFalse: [^self entries]. "If command not supported"
	^self parseListEntries: result! !


!SystemWindow methodsFor: 'menu' stamp: 'ajh 3/16/2003 14:03'!
addCustomMenuItems: aCustomMenu hand: aHandMorph
	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
"template..."
	aCustomMenu addLine.
	aCustomMenu add: 'edit label...' action: #relabel.
! !


!TextContainer methodsFor: 'container protocol' stamp: 'ajh 3/16/2003 14:05'!
rectanglesAt: lineY height: lineHeight
	"Return a list of rectangles that are at least minWidth wide
	in the specified horizontal strip of the shadowForm.
	Cache the results for later retrieval if the owner does not change."
	| hProfile rects thisWidth thisX count pair outerWidth lineRect lineForm |
	pair _ Array with: lineY with: lineHeight.
	rects _ rectangleCache at: pair ifAbsent: [nil].
	rects ifNotNil: [^ rects].

	outerWidth _ minWidth + (2*OuterMargin).
	self shadowForm.  "Compute the shape"
	lineRect _ 0@(lineY - shadowForm offset y)
					extent: shadowForm width@lineHeight.
	lineForm _ shadowForm copy: lineRect.

	"Check for a full line -- frequent case"
	(lineForm tallyPixelValues at: 2) = lineRect area
	ifTrue:
		[rects _ Array with: (shadowForm offset x@lineY extent: lineRect extent)]
	ifFalse:
		["No such luck -- scan the horizontal profile for segments of minWidth"
		hProfile _ lineForm xTallyPixelValue: 1 orNot: false.
		rects _ OrderedCollection new.
		thisWidth _ 0.  thisX _ 0.
		1 to: hProfile size do:
			[:i | count _ hProfile at: i.
			count >= lineHeight ifTrue:
				[thisWidth _ thisWidth + 1]
				ifFalse:
				[thisWidth >= outerWidth ifTrue:
					[rects addLast: ((thisX + shadowForm offset x)@lineY
									extent: thisWidth@lineHeight)].
				thisWidth _ 0. thisX _ i]].
		thisWidth >= outerWidth ifTrue:
				[rects addLast: ((thisX + shadowForm offset x)@lineY
									extent: thisWidth@lineHeight)]].
	rects _ rects collect: [:r | r insetBy: OuterMargin@0].
	rectangleCache at: pair put: rects.
	^ rects! !


!TextFieldMorph class methodsFor: 'instance creation' stamp: 'ajh 3/16/2003 14:05'!
authoringPrototype 
	"Answer an instance of the receiver that can serve as a prototype for authoring"

	| proto |
	proto _ super authoringPrototype.
	proto setProperty: #shared toValue: true.
	proto extent: 170 @ 30.
	proto color: Color veryLightGray lighter.
	proto contents: 'on a clear day you can...'.
	^ proto
! !


!TheWorldMenu methodsFor: 'commands' stamp: 'ajh 3/16/2003 14:06'!
propagateChanges
        "The changes made in this isolated project will be propagated to projects above."

        self projectForMyWorld propagateChanges.! !


!Utilities class methodsFor: 'miscellaneous' stamp: 'ajh 3/16/2003 14:07'!
methodDiffFor: aString class: aClass selector: aSelector prettyDiffs: prettyDiffBoolean
	"Return a string comprising a source-code diff between an existing method and the source-code in aString.  DO prettyDiff if prettyDiffBoolean is true."

	^ (aClass notNil and: [aClass includesSelector: aSelector])
		ifTrue:
			[TextDiffBuilder
				buildDisplayPatchFrom: (aClass sourceCodeAt: aSelector)
				to: aString
				inClass: aClass
				prettyDiffs: prettyDiffBoolean]
		ifFalse:
			[aString copy]! !


!Wonderland methodsFor: 'creating' stamp: 'ajh 3/16/2003 14:09'!
makeActorFrom: filename
	"Creates a new actor using the specification from the given file"

	| aFile words line startSubstr index parent name texture meshFile matrix baseActor newActor protoClass actorClass fileVersion pos |
	myUndoStack closeStack.

	words _ (filename findTokens: #'.').
	((words last) = 'mdl') ifFalse: [^self].
	aFile _ (CrLfFileStream readOnlyFileNamed: filename) ascii.

	"First see if we need to create a prototype class for this model"
	(ActorPrototypeClasses includesKey: (aFile localName))
		ifTrue: [ protoClass _ ActorPrototypeClasses at: (aFile localName) ]
		ifFalse: ["Make a new prototype class for this model"
				protoClass _ (WonderlandActor newUniqueClassInstVars: ''
															classInstVars: '').
				ActorPrototypeClasses at: (aFile localName) put: protoClass].

	"Check what version this mdl file is"
	line _ aFile upTo: (Character cr).
	line _ aFile upTo: (Character cr).
	line _ aFile upTo: (Character cr).

	((line truncateTo: 7) = 'version')
		ifTrue: [ fileVersion _ 1 ]
		ifFalse: [ fileVersion _ 0 ].

	[ line _ aFile upTo: (Character cr).
	aFile atEnd] whileFalse:[
		words _ line findTokens: '='.
		"See if we're creating a new object"
		(((words size) > 1) and: [ ((words at: 2) beginsWith: ' _MakeObject')
			or: [ (words at: 2) beginsWith: ' Alice.MakeObject' ] ]) ifTrue: [
			(fileVersion = 0) ifTrue: [
				words _ line findTokens: #,.
				parent _ (words at: 2) withBlanksTrimmed.
				name _ (((words at: 3) withBlanksTrimmed) findBetweenSubStrs: '"') at: 1.
			] ifFalse: [
				name _ (words at: 1) truncateTo: (((words at: 1) size) - 1).
				parent _ ((words at: 3) findTokens: #,) at: 1.
			].

			"Now pull in the texture to use"
			startSubstr _ name , '.SetTexture'.
			pos _ aFile position.
			texture := nil.
			[aFile atEnd or:[texture notNil]] whileFalse:[
				line _ aFile upTo: (Character cr).
				(line beginsWith: startSubstr) ifTrue:[
					texture _ (line findBetweenSubStrs: '"') at: 2.
					texture _ (aFile directory pathName), FileDirectory slash, texture]].
			texture ifNil:[aFile position: pos].

			"Read the composite matrix to use"
			startSubstr _ name , '._SetLocalTransformation'.
			pos _ aFile position.
			matrix := nil.
			[aFile atEnd or:[matrix notNil]] whileFalse:[
				line _ aFile upTo: (Character cr).
				(line beginsWith: startSubstr) ifTrue:[
					matrix _ B3DMatrix4x4 new.
					words _ line findBetweenSubStrs: ',()'.
					words removeAllSuchThat: [:str | str = ' '].
					index _ words size.
					4 to: 1 by: -1 do: [:i | 
						4 to: 1 by: -1 do: [:j | matrix at: i at: j put: 
									((words at: index) withBlanksTrimmed) asNumber.
									   		index _ index - 1. ]. ].

					1 to: 4 do: [:i | index _ matrix at: i at: 4.
								   matrix at: i at: 4 put: (matrix at: 4 at: i).
								   matrix at: 4 at: i put: index. ].

					matrix a14: (matrix a14 negated).
				].
			].
			matrix ifNil:[aFile position: pos].

			"Read the mesh file to use"
			startSubstr _ 'LoadGeometry'.
			pos _ aFile position.
			meshFile := nil.
			[aFile atEnd or:[meshFile notNil]] whileFalse:[
				line _ aFile upTo: (Character cr).
				(line beginsWith: startSubstr) ifTrue:[
					meshFile _ (line findBetweenSubStrs: '"') at: 2.
					meshFile _ (aFile directory pathName), FileDirectory slash, meshFile.
				].
			].
			meshFile ifNil:[aFile position: pos].

			"Now build the actor name"
			words _ name findTokens: '.'.
			name _ words last.
			name at: 1 put: ((name at: 1) asLowercase).

			"Now build the parent name"
			parent _ parent copyReplaceAll: '.' with: ' '.

			"Now create the object"
			(parent = 'None') ifTrue: [
				actorClass _ protoClass newUniqueClassInstVars: '' classInstVars: ''.
				baseActor _ actorClass createFor: self.
				actorClassList addLast: actorClass.
				baseActor setName: name.
				texture ifNotNil:[baseActor setTexture: texture].
				meshFile ifNotNil:[baseActor loadMeshFromFile: meshFile].
				matrix ifNotNil:[baseActor setComposite: matrix].
				"end base actor creation"
			] ifFalse: [
				actorClass _ WonderlandActor newUniqueClassInstVars: ''
											classInstVars: ''.
				newActor _ actorClass createFor: self.
				actorClassList addLast: actorClass.
				newActor setName: name.
				parent _ (baseActor getChildNamed: parent).
				newActor reparentTo: parent.
				newActor becomePart.
				texture ifNotNil:[newActor setTexture: texture].
				meshFile ifNotNil:[newActor loadMeshFromFile: meshFile].
				matrix ifNotNil:[newActor setComposite: matrix].
				"end new actor with parent"
			].
		"end MakeObject parsing"
		].

	]. "end file parsing"

	aFile close.
	myUndoStack openStack.

	"Ensure that the new actor's name is unique"
	name _ self uniqueNameFrom: (baseActor getName).
	baseActor setName: name.
	myNamespace at: name put: baseActor.
	scriptEditor updateActorBrowser.

	"Add an undo item to undo the creation of this object"
	myUndoStack push: (UndoAction new: 
		[ baseActor removeFromScene.
		myNamespace removeKey: name ifAbsent: [].
		scriptEditor updateActorBrowser.  ] ).
	^ baseActor.
! !


!WonderlandActor methodsFor: 'private' stamp: 'ajh 3/16/2003 14:09'!
loadMeshFromFile: meshFile
	"Load this object's mesh from the specified file"

	| dict words |

	dict _ myWonderland getSharedMeshDict.

	myMesh _ dict at: meshFile
				ifAbsent: [
							words _ (meshFile findTokens: #'.').

							((words last) = 'vfb') ifTrue: [
								myMesh _ WonderlandMesh fromVFBFile: meshFile ].

							((words last) = 'obj') ifTrue: [
								myMesh _ WonderlandMesh fromOBJFile: meshFile ].

							dict at: meshFile put: myMesh.
						].
! !

