'From Squeak3.5 of ''11 April 2003'' [latest update: #5180] on 4 June 2003 at 3:41:18 pm'!
"Change Set:		Compiler-ajh
Date:			23 May 2003
Author:			Anthony Hannan <ajh18@cornell.edu>

This adds the closure compiler.  It uses SmaCC and the RB AST.  SmaCC-Development, and Closures-ajh must be filed in before this.  To use it set Preferences compileBlocksAsClosures to true."!

SystemOrganization addCategory: 'Compiler-Syntax'!
SystemOrganization addCategory: 'Compiler-Semantics'!
SystemOrganization addCategory: 'Compiler-IR'!
SystemOrganization addCategory: 'Compiler-Bytecodes'!

Object subclass: #BytecodeGenerator
	instanceVariableNames: 'seqOrder orderSeq seqBytes jumps literals lastLiteral currentSeqId currentSeqNum bytes lastSpecialReturn instrMaps instrMap maxTemp stacks stack primNum numArgs '
	classVariableNames: 'BytecodeTable Bytecodes SpecialConstants SpecialSelectors '
	poolDictionaries: ''
	category: 'Compiler-Bytecodes'!

!BytecodeGenerator commentStamp: 'ajh 5/23/2003 10:59' prior: 0!
I generate bytecodes in response to 'instructions' messages being sent to me.  I rewrite jumps at the end so their jump offsets are correct (see #bytecodes).  For example, to create a compiled method that compares first instVar to first arg and returns 'yes' or 'no' (same example as in IRBuilder), do:

	BytecodeGenerator new
		numArgs: 1;
		pushInstVar: 1;
		pushTemp: 1;
		send: #>;
		if: false goto: #else;
		pushLiteral: 'yes';
		returnTop;
		label: #else;
		pushLiteral: 'no';
		returnTop;
		compiledMethod

You can send #ir to the compiledMethod to decompile to its IRMethod, and you can send #methodNode to either to decompile to its parse tree.
!


!Compiler commentStamp: 'ajh 3/26/2003 00:37' prior: 0!
The compiler accepts Smalltalk source code and compiles it with respect to a given class. The user of the compiler may supply a context so that temporary variables are accessible during compilation. If there is an error, a requestor (usually a kind of StringHolderController) is sent the message notify:at:in: so that the error message can be displayed. If there is no error, then the result of compilation is a MethodNode, which is the root of a parse tree whose nodes are kinds of ParseNodes. The parse tree can be sent messages to (1) generate code for a CompiledMethod (this is done for compiling methods or evaluating expressions); (2) pretty-print the code (for formatting); or (3) produce a map from object code back to source code (used by debugger program-counter selection).

----

Closure Compiler
by Anthony Hannan 3/13/2003

The Compiler class (like before) serves as the interface to the compiler.  If Preferences compileBlocksAsClosures is true it will run the new closure compiler.  The closure compiler classes reside in their own system category called 'Compiler', which is broken down into four minor categories: 'Compiler-Syntax', 'Compiler-Semantics', 'Compiler-IR', and 'Compiler-Bytecodes'.  The closure compiler also uses classes in 'SmaCC-Runtime' and the Refactory abstract syntax tree in 'Refactory-Parser'.

Like most compilers, the closure compiler translates source text into bytecodes in a sequence of phases or transformations.

Phase 1 - Scan and parse text into abstract syntax tree (AST)

SqueakScanner and SqueakParser in 'Compiler-Syntax' transform a method or do-it text into a Refactory abstract syntax tree.  SqueakScanner and SqueakParser classes were automatically generated using the SmaCC parser generator from a token and grammar specification derived from the StScanner and StParser specifications that comes with SmaCC.

Phase 2 - Verify and annotate AST (semantic analysis)

SemanticChecker in 'Compiler-Semantics' binds RBVariableNodes in the AST to temp, instance, or pool (class, pool, or global) SemVars, raising a notifier if no match is found.  The lookup is performed on a chain of SemScopes, one for the class, method and each nested closure.  Var usage (read/write/capture-in-closure) is tracked using a FiniteAutomaton.  The final usage state is used to determine unused temps and escaping temps (temps whose storage need to reside independent of the context since a block may assign to it after the context is gone).

Phase 3 - Translate AST to intermediate representation (IR)

SemanticTranslator in 'Compiler-Semantics' visits each node in the AST and invokes the appropriate instruction message on IRBuilder in 'Compiler-IR' resulting in an IRMethod.  An IRMethod consists of IRInstructions grouped by IRSequence (basic block).  Each IRInstruction represents a simple stack instruction like pushTemp: or send:.

Phase 4 - Optimize IR

Two simple optimization are applied to the IRMethod: absorbJumpsToSingleInstrs and absorbConstantConditionalJumps. The first eliminates jumps to returns and returns directly.  The second converts chained constant jumps created by and:/or: to single jumps directly to the target code.

Phase 5 - Translate IR to bytecodes (CompiledMethod)

IRTranslator in 'Compiler-IR' visits each IR instruction and invokes the appropriate bytecode message on BytecodeGenerator in 'Compiler-Bytecodes'.  Certain combinations of IR instructions (like storeTemp: and popTop) are mapped to a single bytecode instructions.
!

Object subclass: #Decompiler2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Bytecodes'!

!Decompiler2 commentStamp: 'ajh 3/25/2003 00:28' prior: 0!
This mirrors the old Decompiler interface delegating the real work to BytecodeDecompiler then IRDecompiler.!

Object subclass: #FiniteAutomaton
	instanceVariableNames: 'states actions transitions '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!FiniteAutomaton commentStamp: 'ajh 3/25/2003 00:44' prior: 0!
I am a deterministic finite automaton (or finite state machine), with states, input actions, and transition rules.  See users for an example.!

Object subclass: #IRBuilder
	instanceVariableNames: 'ir tempMap jumpBackTargetStacks jumpAheadStacks currentSequence sourceMapNodes sourceMapByteIndex '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRBuilder commentStamp: 'ajh 5/23/2003 11:03' prior: 0!
I provide a simple interface for constructing an IRMethod.  For example, to create an ir method that compares first instVar to first arg and returns 'yes' or 'no' (same example as in BytecodeGenerator), do:

	IRBuilder new
		rargs: #(self a);		"receiver and args names"
		addTemps: #(z);		"extra temps (not used here)"
		pushTemp: #self;
		getField: 1;
		pushTemp: #a;
		send: #>;
		jumpAheadTo: #else if: false;
		pushLiteral: 'yes';
		returnTop;
		jumpAheadTarget: #else;
		pushLiteral: 'no';
		returnTop;
		ir

Sending #compiledMethod to an ir method will generate its compiledMethod.  Sending #methodNode to it will decompile to its parse tree.
!

Link subclass: #IRInstruction
	instanceVariableNames: 'sourceNode bytecodeIndex '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRInstruction commentStamp: 'ajh 3/24/2003 23:38' prior: 0!
I am an instruction in the IR (intermediate representation) language.  The IR serves as the intermediary between the Smalltalk language and the bytecode language.  It is easier to optimize and translate to/from this language than it is to optimize/translate directly from Smalltalk to bytecodes.  The IR is generic and simple consisting of just twelve instructions.  They are:

	goto: labelNum
	if: boolean goto: labelNum1 otherwise: labelNum2
	label: labelNum
	popTop
	pushDup
	pushLiteral: object
	pushTemp: tempIndex
	remoteReturn
	returnTop
	send: selector
	send: selector toSuperOf: behavior
	storeTemp: tempIndex

Each instruction is reified as an instance of one of my eight subclasses and grouped by basic block (IRSequence) into an IRMethod.  IRInterpreter visits each instruction in a IRMethod responding to the above instruction messages sent to it.
!

IRInstruction subclass: #IRConstant
	instanceVariableNames: 'constant '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRConstant commentStamp: 'ajh 3/24/2003 23:56' prior: 0!
Instruction "pushLiteral: object"!

IRInstruction subclass: #IRDup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRDup commentStamp: 'ajh 3/24/2003 23:56' prior: 0!
Instruction "pushDup"!

Object subclass: #IRInterpreter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRInterpreter commentStamp: 'ajh 3/24/2003 23:55' prior: 0!
I visit each IRInstruction in an IRMethod in order.  Each instruction sends its instruction message to me upon being visited.  See my 'instructions' method category for complete list of instructions.  Subclasses should override them.!

IRInterpreter subclass: #IRDecompiler
	instanceVariableNames: 'stack sp scope currentInstr valueLabelMap '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRDecompiler commentStamp: 'ajh 3/25/2003 00:37' prior: 0!
I interpret IRMethod instructions and generate a Smalltalk abstract syntax tree rooted at a RBMethodNode.

This is implemented like a shift-reduce parser.  Each instruction either causes a node to be pushed on the stack (shift), or causes one or more nodes to be popped and combined into a single node which is push back on the stack (reduce).  Most reduction is done at the "label: labelNum" instruction where it tries to reduce jump structures into control messages like #ifTrue:, whileFalse:, etc.

Several pseudo nodes (RBPseudoNode and subclasses) are used to represent basic instructions that have not been reduced to real AST nodes yet.
!

IRInstruction subclass: #IRJump
	instanceVariableNames: 'destination '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRJump commentStamp: 'ajh 3/24/2003 23:56' prior: 0!
Instruction "goto: labelNum"!

IRJump subclass: #IRJumpIf
	instanceVariableNames: 'boolean otherwise '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRJumpIf commentStamp: 'ajh 3/24/2003 23:56' prior: 0!
Instruction "if: boolean goto: labelNum1 otherwise: labelNum2"!

Object subclass: #IRMethod
	instanceVariableNames: 'startSequence primitiveNode rargKeys otherTempKeys compiledMethod '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRMethod commentStamp: 'ajh 5/23/2003 11:08' prior: 0!
I am a method in the IR (intermediate representation) language consisting of IRInstructions grouped by IRSequence (basic block).  The IRSequences form a control graph (therefore I only have to hold onto the starting sequence).  #compiledMethod will convert me to a CompiledMethod.  #methodNode will convert me back to a parse tree.
!

IRInstruction subclass: #IRPop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRPop commentStamp: 'ajh 3/24/2003 23:57' prior: 0!
Instruction "popTop"!

IRInterpreter subclass: #IRPrinter
	instanceVariableNames: 'stream indent '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRPrinter commentStamp: 'ajh 3/25/2003 00:22' prior: 0!
I interpret IRMethod instructions and write them out to a print stream.!

IRInstruction subclass: #IRReturn
	instanceVariableNames: 'isRemote '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRReturn commentStamp: 'ajh 3/24/2003 23:57' prior: 0!
Instruction "returnTop" or "remoteReturn"!

IRInstruction subclass: #IRSend
	instanceVariableNames: 'selector superOf '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRSend commentStamp: 'ajh 3/24/2003 23:57' prior: 0!
Instruction "send: selector" or "send: selector toSuperOf: behavior"!

LinkedList subclass: #IRSequence
	instanceVariableNames: 'orderNumber '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRSequence commentStamp: 'ajh 3/14/2003 00:33' prior: 0!
I hold a sequence of IRInstructions where only the last instruction jumps or returns, the rest are guaranteed to execute in order (ie. a basic block).!

IRInstruction subclass: #IRTemp
	instanceVariableNames: 'number isStore '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRTemp commentStamp: 'ajh 3/24/2003 23:58' prior: 0!
Instruction "pushTemp: tempIndex" or "storeTemp: tempIndex"!

IRInterpreter subclass: #IRTranslator
	instanceVariableNames: 'pending gen currentInstr '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-IR'!

!IRTranslator commentStamp: 'ajh 3/25/2003 00:26' prior: 0!
I interpret IRMethod instructions, sending the appropriate bytecode messages to my BytecodeGenerator (gen).  I hold some messages back in pending awaiting certain sequences of them that can be consolidated into single bytecode instructions, otherwise the pending messages are executed in order as if they were executed when they first appeared.!

InstructionStream subclass: #BytecodeDecompiler
	instanceVariableNames: 'irBuilder '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Bytecodes'!

!BytecodeDecompiler commentStamp: 'ajh 3/25/2003 00:26' prior: 0!
I interpret bytecode instructions, sending the appropriate instruction messages to my IRBuilder, resulting in an IRMethod.!

Set subclass: #LiteralSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Bytecodes'!

!LiteralSet commentStamp: 'ajh 3/25/2003 00:33' prior: 0!
Holds a unique set of literals.  Literal objects are equal if they are #= plus they are the same class.  This set uses this rule for finding elements.

Example:
	Set new add: 'anthony'; add: #anthony; size  "= 1"
	LiteralSet new add: 'anthony'; add: #anthony; size  "= 2"
!

OrderedCollection subclass: #LiteralList
	instanceVariableNames: 'equalitySet '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Bytecodes'!

!LiteralList commentStamp: 'ajh 3/25/2003 00:31' prior: 0!
Holds a unique ordered collection of literals!

Object subclass: #Parser2
	instanceVariableNames: 'source requestor scope doitFlag failBlock endOfMethodPattern '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Syntax'!

!Parser2 commentStamp: 'ajh 3/24/2003 21:36' prior: 0!
This mirrors the old Parser protocol but delegates to the real parser which is SqueakParser.!

Object subclass: #PrimitiveNode
	instanceVariableNames: 'primitiveNum spec '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Syntax'!

!PrimitiveNode commentStamp: 'ajh 3/24/2003 21:35' prior: 0!
I represent a primitive.  I am more than just a number if I am a named primitive.

Structure:

 num	<Integer>	Primitive number.
 spec	<Object>		Stored in first literal when num is 117 or 120.
!

Error subclass: #PrimitiveParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Syntax'!

!PrimitiveParseError commentStamp: 'ajh 3/24/2003 18:21' prior: 0!
I get signalled if there is a parsing error while parsing the primitive text between angle brackets <...>!

Scanner subclass: #PrimitiveParser
	instanceVariableNames: 'here hereType hereMark hereEnd prevMark prevEnd startPosition spec '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Syntax'!

!PrimitiveParser commentStamp: 'ajh 3/24/2003 18:23' prior: 0!
I parse the the primitive text between angle brackets.  I am basically a copy of the original Parser but with only the primitive parse methods retained.!

RBValueNode subclass: #RBArrayNode
	instanceVariableNames: 'leftBrace rightBrace statements '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-SqueakRB-ParserExtensions'!
RBValueNode subclass: #RBBlockNode
	instanceVariableNames: 'left right colons body arguments bar ir scope '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-Parser'!
RBProgramNode subclass: #RBDoItNode
	instanceVariableNames: 'body source tags scope ir '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-SqueakRB-ParserExtensions'!
RBValueNode subclass: #RBLiteralNode
	instanceVariableNames: 'token value '
	classVariableNames: 'Null '
	poolDictionaries: ''
	category: 'Refactory-Parser'!
RBProgramNode subclass: #RBMethodNode
	instanceVariableNames: 'selector selectorParts body source arguments tags primitiveNode ir scope '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-Parser'!
RBProgramNodeVisitor subclass: #RBFormatter
	instanceVariableNames: 'codeStream lineStart firstLineLength tabs positionDelta '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-Parser'!
RBProgramNode subclass: #RBPseudoNode
	instanceVariableNames: 'mapInstr '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-SqueakRB-ParserExtensions'!
RBPseudoNode subclass: #RBPseudoDupNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-SqueakRB-ParserExtensions'!
RBPseudoNode subclass: #RBPseudoGotoNode
	instanceVariableNames: 'destination forValue '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-SqueakRB-ParserExtensions'!
RBPseudoNode subclass: #RBPseudoIfNode
	instanceVariableNames: 'boolean destination otherwise '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-SqueakRB-ParserExtensions'!
RBPseudoNode subclass: #RBPseudoLabelNode
	instanceVariableNames: 'destination '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-SqueakRB-ParserExtensions'!
RBPseudoNode subclass: #RBPseudoPopNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-SqueakRB-ParserExtensions'!
RBPseudoNode subclass: #RBPseudoSendNode
	instanceVariableNames: 'selector '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-SqueakRB-ParserExtensions'!
RBProgramNode subclass: #RBReturnNode
	instanceVariableNames: 'return value homeBinding '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-Parser'!
RBValueNode subclass: #RBVariableNode
	instanceVariableNames: 'token binding '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-Parser'!
Object subclass: #SemScope
	instanceVariableNames: 'outerScope '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemScope commentStamp: 'ajh 3/24/2003 21:53' prior: 0!
I am a symbol table where variable names are associated with SemVars.  Each context (method/closure) get a fresh scope that inherits from its outer scope.!

SemScope subclass: #SemClassScope
	instanceVariableNames: 'class '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemClassScope commentStamp: 'ajh 3/24/2003 21:44' prior: 0!
I include all variable enries for class, pool and global vars.!

SemScope subclass: #SemContextScope
	instanceVariableNames: 'vars thisContextVar receiverVar isInlined '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemContextScope commentStamp: 'ajh 3/24/2003 21:51' prior: 0!
I hold all temp and field vars for a method node or non-inlined block node (closure).  Field vars are either inst vars in the case of method node or free vars (outer temp references) in the case of block node.!

SemScope subclass: #SemNilScope
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemNilScope commentStamp: 'ajh 3/24/2003 21:52' prior: 0!
When the class is not known, use me to at least find global vars.!

Object subclass: #SemVar
	instanceVariableNames: 'name usage '
	classVariableNames: 'UsageAutomaton '
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemVar commentStamp: 'ajh 3/24/2003 22:38' prior: 0!
I am an entry in a SemScope that gets associated with variable nodes of the same name.  There a three different subclasses of vars: temp vars, field vars, and pool/global vars.!

SemVar subclass: #SemFieldVar
	instanceVariableNames: 'receiverVar index outerVar '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemFieldVar commentStamp: 'ajh 3/24/2003 21:52' prior: 0!
I'm a field var of some receiverVar.  if receiverVar is 'self' than I am an instance variable.  If receiverVar is a closure than I am a free variable (outer temp reference) and my outerVar is the outer reference var.!

SemVar subclass: #SemPoolVar
	instanceVariableNames: 'assoc '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemPoolVar commentStamp: 'ajh 3/24/2003 21:53' prior: 0!
I am a global, pool, or class variable.!

SemVar subclass: #SemTempVar
	instanceVariableNames: 'index scope '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemTempVar commentStamp: 'ajh 3/24/2003 21:54' prior: 0!
I am an argument or temporary variable of a method or block.!

RBProgramNodeVisitor subclass: #SemanticChecker
	instanceVariableNames: 'scope '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemanticChecker commentStamp: 'ajh 3/24/2003 21:58' prior: 0!
I visit each node in the abstract syntax tree and declare and lookup variable names in the given scope binding the variable nodes to the scope entries.  The scope starts out as a class (or nil) scope but grows and shrinks as method/blocks are entered and left.  Each method and block node remembers the scope it was associated with.

I am analogous to the "type checker" in a traditional compiler.
!

Error subclass: #SemanticError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemanticError commentStamp: 'ajh 3/24/2003 22:31' prior: 0!
If a variable is not found or some other problem occurs during checking (SemanticChecker) than a particular subclass instance of me is signal.  Many are benign and continue if not handled.!

RBProgramNodeVisitor subclass: #SemanticTranslator
	instanceVariableNames: 'methodBuilder effectTranslator valueTranslator '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemanticTranslator commentStamp: 'ajh 3/24/2003 22:19' prior: 0!
I visit an abstract syntax tree and generate IR (intermediate representation) instructions for each node by sending the appropriate message to my methodBuilder (an IRBuilder).  I hold onto my two subclasses one for generating instructions for value, the other for generating instructions for effect.!

SemanticTranslator subclass: #SemanticTranslatorForEffect
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemanticTranslatorForEffect commentStamp: 'ajh 3/24/2003 22:20' prior: 0!
I override some methods in my super to generate instructions for effect only.!

SemanticTranslator subclass: #SemanticTranslatorForValue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!SemanticTranslatorForValue commentStamp: 'ajh 3/24/2003 22:20' prior: 0!
I override some methods in my super to generate instructions for effect and value only.!

SemanticError subclass: #ShadowVariableError
	instanceVariableNames: 'variableNode shadowedVar '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!ShadowVariableError commentStamp: 'ajh 3/24/2003 22:21' prior: 0!
I get signalled when a variable in a block or method scope shadows a variable of the same name in an outer scope.  The default action is to allow it.!

SmaCCParser subclass: #SqueakParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Syntax'!

!SqueakParser commentStamp: 'ajh 3/24/2003 18:24' prior: 0!
I parse Smalltalk text into a Refactory abstract syntax tree, whose root is RBMethodNode or RBDoItNode.  Methods under 'generated-*' categories were automatically generated using SmaCC.!

SmaCCScanner subclass: #SqueakScanner
	instanceVariableNames: 'comments '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Syntax'!

!SqueakScanner commentStamp: 'ajh 3/24/2003 21:31' prior: 0!
I parse Smalltalk text into tokens which are used by the SqueakParser.  Methods under 'generated-*' categories were automatically generated using SmaCC.!

Object subclass: #StackCount
	instanceVariableNames: 'start position length '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Bytecodes'!

!StackCount commentStamp: 'ajh 3/25/2003 00:34' prior: 0!
This keeps track of the stack count for the BytecodeGenerator.!

RBProgramNodeVisitor subclass: #TNodeTranslator
	instanceVariableNames: 'tMethodClass '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Syntax'!

!TNodeTranslator commentStamp: 'ajh 3/24/2003 21:34' prior: 0!
I visit an abstract syntax tree rooted at RBMethodNode and generate a C-translator abstract systax tree.  This is used when translating VM Slang/Smalltalk into C.!

SemanticError subclass: #UndeclaredSelectorError
	instanceVariableNames: 'messageNode '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!UndeclaredSelectorError commentStamp: 'ajh 3/24/2003 22:23' prior: 0!
I get signalled when no selector is defined for a message send, indicating a possible mispelling.  My default action is to create a new selector.!

SemanticError subclass: #UndeclaredVariableError
	instanceVariableNames: 'variableNode '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!UndeclaredVariableError commentStamp: 'ajh 3/24/2003 22:33' prior: 0!
I get signalled when an unknown variable it references.  My default action is to raise an error.!

SemanticError subclass: #UninitializedVariableError
	instanceVariableNames: 'variableNode '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!UninitializedVariableError commentStamp: 'ajh 3/24/2003 22:34' prior: 0!
I get signalled when a temporary variable is used before it is assigned to.  My default action is to allow it since all temps get initialized to nil.!

SemanticError subclass: #UnusedVariableError
	instanceVariableNames: 'variableNode '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Compiler-Semantics'!

!UnusedVariableError commentStamp: 'ajh 3/24/2003 22:35' prior: 0!
I get signalled when a temporary variable is declared but never used.  My default action is to allow it since it is benign.!


!Object methodsFor: 'evaluating' stamp: 'ajh 3/18/2003 17:11'!
doItScope

	^ self class parseScope newMethodScope! !


!Behavior methodsFor: 'accessing' stamp: 'ajh 3/13/2003 21:13'!
decompilerClass
	"Answer a decompiler class appropriate for compiled methods of this class."

	^ Preferences compileBlocksAsClosures
		ifTrue: [Decompiler2]
		ifFalse: [Decompiler]! !

!Behavior methodsFor: 'compiling' stamp: 'ajh 2/26/2003 20:00'!
parseScope

	^ SemClassScope new class: self! !


!BlockClosure methodsFor: 'inspecting' stamp: 'ajh 3/19/2003 00:50'!
doItScope

	| scope names i |
	scope _ SemContextScope new initClosureReceiver.
	names _ self closureVarNames.
	scope fieldVarNames: names startingAt: self class instSize + 1.
	scope fieldVars with: self do: [:var :value |
		(value isMemberOf: SharedTempHolder) ifTrue: [var markEscape]].
	i _ names indexOf: 'self'.
	scope outerScope: (i = 0
		ifTrue: [nil parseScope]
		ifFalse: [(self at: i) doItScope]).
	^ scope! !


!BytecodeGenerator methodsFor: 'initialize' stamp: 'ajh 5/22/2003 17:11'!
initialize

	literals _ LiteralList new.
	"The following dicts are keyed by sequence id given by client in label: (and gotos)."
	seqOrder _ IdentityDictionary new.  "seqId -> seq order num"
	seqBytes _ IdentityDictionary new.  "seqId -> seq bytecodes"
	jumps _ IdentityDictionary new.  "seqId -> last jump instr"
	instrMaps _ IdentityDictionary new.  "seqId -> (clientInstr -> bytecode pos)"
	stacks _ IdentityDictionary new.  "seqId -> stackCount"
	maxTemp _ 0.
	primNum _ 0.
	numArgs _ 0.
	currentSeqNum _ 0.
	orderSeq _ OrderedCollection new.  "reverse map of seqOrder"

	"starting label in case one is not provided by client"
	self label: self newDummySeqId.
! !

!BytecodeGenerator methodsFor: 'initialize' stamp: 'ajh 3/13/2003 18:21'!
numArgs: n

	numArgs _ n! !

!BytecodeGenerator methodsFor: 'initialize' stamp: 'ajh 3/13/2003 18:21'!
primitiveNode: aPrimitiveNode

	literals isEmpty ifFalse: [self error: 'init prim before adding instructions'].
	aPrimitiveNode spec ifNotNil: [literals add: aPrimitiveNode spec].
	primNum _ aPrimitiveNode num.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 5/22/2003 12:22'!
goto: seqId

	stacks at: seqId put: (stack linkTo: (stacks at: seqId ifAbsentPut: [nil])).

	self saveLastJump: (Message
		selector: #from:goto:
		arguments: {currentSeqId. seqId}).
	self from: currentSeqId goto: seqId.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 5/22/2003 13:26'!
if: bool goto: seqId

	| otherwiseSeqId |
	otherwiseSeqId _ self newDummySeqId.
	self if: bool goto: seqId otherwise: otherwiseSeqId.
	self label: otherwiseSeqId.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 5/22/2003 12:26'!
if: bool goto: seqId1 otherwise: seqId2

	stack pop.
	stacks at: seqId1 put: (stack linkTo: (stacks at: seqId1 ifAbsentPut: [nil])).
	stacks at: seqId2 put: (stack linkTo: (stacks at: seqId2 ifAbsentPut: [nil])).

	self saveLastJump: (Message
		selector: #from:if:goto:otherwise:
		arguments: {currentSeqId. bool. seqId1. seqId2}).
	self from: currentSeqId if: bool goto: seqId1 otherwise: seqId2.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 5/22/2003 14:16'!
label: seqId

	(currentSeqId notNil and: [(jumps at: currentSeqId) isNil]) ifTrue: [
		"make previous implicit goto explicit"
		self goto: seqId.
	].

	lastSpecialReturn _ nil.
	currentSeqId _ seqId.
	currentSeqNum _ currentSeqNum + 1.
	seqOrder at: seqId put: currentSeqNum.
	orderSeq at: currentSeqNum ifAbsentPut: [seqId].
	bytes _ seqBytes at: seqId ifAbsentPut: [OrderedCollection new].
	jumps at: seqId ifAbsentPut: [nil].
	instrMap _ instrMaps at: seqId ifAbsentPut: [OrderedCollection new].
	stack _ stacks at: seqId ifAbsentPut: [StackCount new].
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/13/2003 17:48'!
popTop

	stack pop.

	self nextPut: (Bytecodes at: #popStackBytecode).
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/13/2003 17:48'!
pushDup

	stack push.

	self nextPut: (Bytecodes at: #duplicateTopBytecode).
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/13/2003 17:49'!
pushInstVar: instVarIndex

	| interval |
	stack push.

	interval _ Bytecodes at: #pushReceiverVariableBytecode.
	instVarIndex <= interval size ifTrue: [
		^ self nextPut: (interval at: instVarIndex).
	].
	instVarIndex <= 64 ifTrue: [
		self nextPut: (Bytecodes at: #extendedPushBytecode).
		^ self nextPut: (0 "instVar" << 6) + instVarIndex - 1.
	].
	instVarIndex <= 256 ifFalse: [self error: 'can''t reference more than 256 inst vars'].
	self nextPut: (Bytecodes at: #doubleExtendedDoAnythingBytecode).
	self nextPut: 2 "pushInstVar" << 5.
	self nextPut: instVarIndex - 1.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/13/2003 17:50'!
pushLiteral: object

	| index interval |
	stack push.

	(index _ SpecialConstants identityIndexOf: object) > 0 ifTrue: [
		^ self nextPut: (Bytecodes at: #pushConstantTrueBytecode) + index - 1].

	index _ self addLiteral: object.
	interval _ Bytecodes at: #pushLiteralConstantBytecode.
	(index <= interval size) ifTrue: [
		^ self nextPut: (interval at: index)
	].
	index <= 64 ifTrue: [
		self nextPut: (Bytecodes at: #extendedPushBytecode).
		^ self nextPut: 2 "lit constant" << 6 + index - 1
	].
	index > 256 ifTrue: [self error: 'too many literals (>256)'].
	self nextPut: (Bytecodes at: #doubleExtendedDoAnythingBytecode).
	self nextPut: 3 "lit constant" << 5.
	self nextPut: index - 1.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/13/2003 17:50'!
pushReceiver

	stack push.

	self nextPut: (Bytecodes at: #pushReceiverBytecode)! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/13/2003 17:51'!
pushTemp: index

	| interval |
	stack push.
	maxTemp _ index max: maxTemp.

	interval _ Bytecodes at: #pushTemporaryVariableBytecode.
	index <= interval size ifTrue: [
		^ self nextPut: (interval at: index).
	].
	index <= 64 ifFalse: [self error: 'too many temp vars (>64)'].
	self nextPut: (Bytecodes at: #extendedPushBytecode).
	self nextPut: (1 "temp" << 6) + index - 1.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/13/2003 17:51'!
pushThisContext

	stack push.

	self nextPut: (Bytecodes at: #pushActiveContextBytecode).
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 5/22/2003 14:01'!
remoteReturn

	self saveLastJump: #return.

	self send: #remoteReturnTo:.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 5/22/2003 14:02'!
returnConstant: obj

	self saveLastJump: #return.

	bytes size = 0 ifTrue: [
		lastSpecialReturn _ Message selector: #returnConstant: argument: obj].

	obj caseOf: {
		[true] -> [self nextPut: (Bytecodes at: #returnTrue)].
		[false] -> [self nextPut: (Bytecodes at: #returnFalse)].
		[nil] -> [self nextPut: (Bytecodes at: #returnNil)]
	} otherwise: [
		self pushLiteral: obj.
		self returnTop.
	]
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 5/22/2003 14:02'!
returnInstVar: index

	self saveLastJump: #return.

	bytes size = 0 ifTrue: [
		lastSpecialReturn _ Message selector: #returnInstVar: argument: index].

	self pushInstVar: index.
	self returnTop.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 5/22/2003 14:02'!
returnReceiver

	self saveLastJump: #return.

	bytes size = 0 ifTrue: [
		lastSpecialReturn _ Message selector: #returnReceiver].

	self nextPut: (Bytecodes at: #returnReceiver).
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 5/22/2003 14:02'!
returnTop

	self saveLastJump: #return.

	self nextPut: (Bytecodes at: #returnTopFromMethod).
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/16/2003 14:43'!
send: selector

	| index nArgs |
	nArgs _ selector numArgs.
	stack pop: nArgs.

	SpecialSelectors at: selector ifPresent: [:i |
		^ self nextPut: (Bytecodes at: #bytecodePrimAdd) + i].

	index _ self addLiteral: selector.
	(index <= 16 and: [nArgs <= 2]) ifTrue: [
		"short send"
		^ self nextPut: (Bytecodes at: #sendLiteralSelectorBytecode) first + (nArgs * 16) + index - 1
	].
	(index <= 32 and: [nArgs <= 7]) ifTrue: [
		"extended (2-byte) send"
		self nextPut: (Bytecodes at: #singleExtendedSendBytecode).
		^ self nextPut: nArgs * 32 + index - 1
	].
	(index <= 64 and: [nArgs <= 3]) ifTrue: [
		"new extended (2-byte)"
		self nextPut: (Bytecodes at: #secondExtendedSendBytecode).
		^ self nextPut: nArgs * 64 + index - 1
	].
	"long (3-byte) send"
	self nextPut: (Bytecodes at: #doubleExtendedDoAnythingBytecode).
	self nextPut: nArgs.
	self nextPut: index - 1.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/16/2003 14:43'!
send: selector toSuperOf: behavior

	| index nArgs |
	nArgs _ selector numArgs.
	stack pop: nArgs.

	self addLastLiteral: behavior holder.
	index _ self addLiteral: selector.
	(index <= 32 and: [nArgs <= 7]) ifTrue: [
		"extended (2-byte) send"
		self nextPut: (Bytecodes at: #singleExtendedSuperBytecode).
		^ self nextPut: nArgs * 32 + index - 1
	].
	"long (3-byte) send"
	self nextPut: (Bytecodes at: #doubleExtendedDoAnythingBytecode).
	self nextPut: 1 << 5 "super" + nArgs.
	self nextPut: index - 1.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/9/2003 20:36'!
storeInstVar: index

	index <= 64 ifTrue: [
		self nextPut: (Bytecodes at: #extendedStoreBytecode).
		^ self nextPut: (0 "instVar" << 6) + index - 1.
	].
	index <= 256 ifFalse: [self error: 'can''t reference more than 256 inst vars'].
	self nextPut: (Bytecodes at: #doubleExtendedDoAnythingBytecode).
	self nextPut: 5 "storeInstVar" << 5.
	self nextPut: index - 1.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/13/2003 18:00'!
storePopInstVar: index

	| interval |
	stack pop.

	interval _ Bytecodes at: #storeAndPopReceiverVariableBytecode.
	index <= interval size ifTrue: [
		^ self nextPut: (interval at: index)
	].
	index <= 64 ifTrue: [
		self nextPut: (Bytecodes at: #extendedStoreAndPopBytecode).
		^ self nextPut: (0 "instVar" << 6) + index - 1.
	].
	index <= 256 ifFalse: [
		self error: 'can''t reference more than 256 inst vars'].
	self nextPut: (Bytecodes at: #doubleExtendedDoAnythingBytecode).
	self nextPut: 6 "storePopInstVar" << 5.
	self nextPut: index - 1.
! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/13/2003 18:01'!
storePopTemp: index

	| interval |
	stack pop.
	maxTemp _ index max: maxTemp.

	interval _ Bytecodes at: #storeAndPopTemporaryVariableBytecode.
	index <= interval size ifTrue: [
		^ self nextPut: (interval at: index)
	].
	index <= 64 ifTrue: [
		self nextPut: (Bytecodes at: #extendedStoreAndPopBytecode).
		^ self nextPut: (1 "temp" << 6) + index - 1.
	].
	self error: 'too many temps (>64)'! !

!BytecodeGenerator methodsFor: 'instructions' stamp: 'ajh 3/13/2003 18:01'!
storeTemp: index

	maxTemp _ index max: maxTemp.

	index <= 64 ifFalse: [self error: 'too many temps (>64)'].
	self nextPut: (Bytecodes at: #extendedStoreBytecode).
	self nextPut: (1 "temp" << 6) + index - 1.
! !

!BytecodeGenerator methodsFor: 'private' stamp: 'ajh 3/6/2003 22:48'!
addLastLiteral: object

	lastLiteral ifNil: [^ lastLiteral _ object].
	(lastLiteral literalEqual: object)
		ifFalse: [self error: 'there can only be one last literal'].! !

!BytecodeGenerator methodsFor: 'private' stamp: 'ajh 3/8/2003 20:56'!
addLiteral: object

	literals add: object.
	^ literals indexOf: object! !

!BytecodeGenerator methodsFor: 'private' stamp: 'ajh 5/22/2003 13:00'!
from: fromSeqId goto: toSeqId

	| distance from to |
	from _ seqOrder at: fromSeqId.
	to _ seqOrder at: toSeqId ifAbsent: [^ self].
	from + 1 = to ifTrue: [^ self].  "fall through, no jump needed"

	from < to ifTrue: [ "jump forward"
		distance _ (from + 1 to: to - 1) inject: 0 into: [:size :i | 
				size + (seqBytes at: (orderSeq at: i)) size].
		self jumpForward: distance.
	] ifFalse: [ "jump backward"
		distance _ ((to to: from - 1) inject: 0 into: [:size :i |
				size + (seqBytes at: (orderSeq at: i)) size])
			+ bytes size.
		self jumpBackward: distance.
	].
! !

!BytecodeGenerator methodsFor: 'private' stamp: 'ajh 5/22/2003 13:22'!
from: fromSeqId if: bool goto: toSeqId otherwise: otherwiseSeqId

	| distance from to otherwise |
	from _ seqOrder at: fromSeqId.
	to _ seqOrder at: toSeqId ifAbsent: [^ self jump: 0 if: bool].  "not done yet"
	otherwise _ seqOrder at: otherwiseSeqId ifAbsent: [^ self jump: 0 if: bool].  "not done yet"
	from < to ifFalse: [self errorConditionalJumpBackwards].
	from + 1 = otherwise ifFalse: [self errorFallThroughSequenceNotNext].
	distance _ (from + 1 to: to - 1)
		inject: 0
		into: [:size :i | size + (seqBytes at: (orderSeq at: i)) size].
	self jump: distance if: bool.
! !

!BytecodeGenerator methodsFor: 'private' stamp: 'ajh 3/24/2003 17:48'!
jump: distance if: condition

	| hi |
	distance = 0 ifTrue: [
		"jumps to fall through, no-op"
		^ self nextPut: (Bytecodes at: #popStackBytecode)].
	condition ifTrue: [
		hi _ distance // 256.
		hi < 8 ifFalse: [self error: 'true jump too big'].
		self nextPut: (Bytecodes at: #longJumpIfTrue) first + hi.
		self nextPut: distance \\ 256.
	] ifFalse: [
		distance <= 8 ifTrue: [
			self nextPut: (Bytecodes at: #shortConditionalJump) first + distance - 1.
		] ifFalse: [
			hi _ distance // 256.
			hi < 8 ifFalse: [self error: 'false jump too big'].
			self nextPut: (Bytecodes at: #longJumpIfFalse) first + hi.
			self nextPut: distance \\ 256.
		].
	]	
! !

!BytecodeGenerator methodsFor: 'private' stamp: 'ajh 3/24/2003 17:46'!
jumpBackward: distance

	| dist |
	distance = 0 ifTrue: [^ self].  "no-op"
	dist _ 1024 - distance - 2.
	dist < 0 ifTrue: [self error: 'back jump to big'].
	self nextPut: (Bytecodes at: #longUnconditionalJump) first + (dist // 256).
	self nextPut: dist \\ 256.
! !

!BytecodeGenerator methodsFor: 'private' stamp: 'ajh 3/24/2003 17:46'!
jumpForward: distance

	distance = 0 ifTrue: [^ self].  "no-op"
	distance <= 8 ifTrue: [
		self nextPut: (Bytecodes at: #shortUnconditionalJump) first + distance - 1.
	] ifFalse: [
		distance > 1023 ifTrue: [self error: 'forward jump too big'].
		self nextPut: (Bytecodes at: #longUnconditionalJump) first + (distance // 256) + 4.
		self nextPut: distance \\ 256.
	].
! !

!BytecodeGenerator methodsFor: 'private' stamp: 'ajh 5/22/2003 13:28'!
newDummySeqId

	^ Object new! !

!BytecodeGenerator methodsFor: 'private' stamp: 'ajh 3/13/2003 13:00'!
nextPut: byte

	bytes add: byte! !

!BytecodeGenerator methodsFor: 'private' stamp: 'ajh 5/22/2003 12:23'!
saveLastJump: message

	jumps at: currentSeqId put: {bytes size. message}.
! !

!BytecodeGenerator methodsFor: 'private' stamp: 'ajh 5/22/2003 14:04'!
updateJump: seqId
	"Recalculate final jump bytecodes.  Return true if jump bytecodes SIZE has changed, otherwise return false"

	| pair s1 |
	pair _ jumps at: seqId.
	pair last == #return ifTrue: [^ false].  "no jump, a return"
	bytes _ seqBytes at: seqId.
	s1 _ bytes size.
	bytes popTo: pair first.
	pair last sendTo: self.
	^ s1 ~= bytes size! !

!BytecodeGenerator methodsFor: 'results' stamp: 'ajh 5/22/2003 13:06'!
bytecodes

	| stream |
	[ orderSeq
		inject: false
		into: [:changed :seqId | (self updateJump: seqId) | changed]
	] whileTrue.

	stream _ (ByteArray new: 100) writeStream.
	orderSeq do: [:seqId |
		(instrMaps at: seqId) do: [:assoc |
			assoc key "instr" bytecodeIndex: stream position + assoc value.
		].
		stream nextPutAll: (seqBytes at: seqId).
	].
	^ stream contents! !

!BytecodeGenerator methodsFor: 'results' stamp: 'ajh 5/23/2003 10:48'!
compiledMethod

	^ self compiledMethodWith: #(0)! !

!BytecodeGenerator methodsFor: 'results' stamp: 'ajh 3/13/2003 18:27'!
compiledMethodWith: trailer

	^ (CompiledMethod
		primitive: (self primNum > 0
			ifTrue: [self primNum]
			ifFalse: [self quickMethodPrim])
		numArgs: self numArgs
		numTemps: (self numTemps max: self numArgs)
		stackSize: self stackSize
		literals: self literals
		bytecodes: self bytecodes
		trailer: trailer)
		isClosureCompiled: true! !

!BytecodeGenerator methodsFor: 'results' stamp: 'ajh 3/16/2003 13:57'!
literals

	^ lastLiteral
		ifNil: [literals]
		ifNotNil: [literals asArray copyWith: lastLiteral]! !

!BytecodeGenerator methodsFor: 'results' stamp: 'ajh 3/13/2003 18:27'!
numArgs

	^ numArgs! !

!BytecodeGenerator methodsFor: 'results' stamp: 'ajh 3/13/2003 18:03'!
numTemps

	^ maxTemp! !

!BytecodeGenerator methodsFor: 'results' stamp: 'ajh 3/13/2003 18:27'!
primNum

	^ primNum! !

!BytecodeGenerator methodsFor: 'results' stamp: 'ajh 3/13/2003 18:28'!
quickMethodPrim

	| i |
	self numArgs = 0 ifFalse: [^ 0].
	lastSpecialReturn ifNil: [^ 0].
	seqBytes size = 1 ifFalse: [^ 0].
	^ lastSpecialReturn selector caseOf: {
		[#returnReceiver] -> [256].
		[#returnConstant:] -> [
			(i _ SpecialConstants indexOf: lastSpecialReturn argument) > 0
				ifTrue: [256 + i] ifFalse: [0]].
		[#returnInstVar:] -> [263 + lastSpecialReturn argument]
	  }! !

!BytecodeGenerator methodsFor: 'results' stamp: 'ajh 3/13/2003 18:03'!
stackSize

	^ (stacks collect: [:s | s length]) max! !

!BytecodeGenerator methodsFor: 'mapping' stamp: 'ajh 3/13/2003 13:00'!
mapBytesTo: instr
	"Associate next byte with instr"

	instrMap add: instr -> (bytes size + 1)! !


!ClassDescription methodsFor: 'compiling' stamp: 'ajh 3/11/2003 19:13'!
compile: code notifying: requestor trailer: bytes 
		ifFail: failBlock
		elseSetSelectorAndNode: selAndNodeBlock
	"Intercept this message in order to remember system changes.
	 5/15/96 sw: modified so that if the class does not wish its methods logged in the changes file, then they also won't be accumulated in the current change set.
	7/12/96 sw: use wantsChangeSetLogging to determine whether to put in change set"

	| methodNode selector newMethod priorMethodOrNil |
	methodNode _ self compilerClass new
				compile: code
				in: self
				notifying: requestor
				ifFail: failBlock.
	selector _ methodNode selector.
	selAndNodeBlock value: selector value: methodNode.
	requestor ifNotNil:
		["Note this change for recent submissions list"
		Utilities noteMethodSubmission: selector forClass: self].
	Preferences compileBlocksAsClosures ifFalse: [
		methodNode encoder requestor: requestor].  "Why was this not preserved?"
	newMethod _ methodNode generate: bytes.
	priorMethodOrNil _ (self methodDict includesKey: selector)
		ifTrue: [self compiledMethodAt: selector]
		ifFalse: [nil].
	Smalltalk changes noteNewMethod: newMethod forClass: self
		selector: selector priorMethod: priorMethodOrNil.
	self addSelector: selector withMethod: newMethod.
	^ newMethod! !

!ClassDescription methodsFor: 'fileIn/Out' stamp: 'ajh 3/15/2003 16:02'!
printMethodChunk: selector withPreamble: doPreamble on: outStream
		moveSource: moveSource toFile: fileIndex
	"Copy the source code for the method associated with selector onto the fileStream.  If moveSource true, then also set the source code pointer of the method."
	| preamble method oldPos newPos sourceFile endPos |
	doPreamble 
		ifTrue: [preamble _ self name , ' methodsFor: ' ,
					(self organization categoryOfElement: selector) asString printString]
		ifFalse: [preamble _ ''].
	method _ self methodDict at: selector.
	((method fileIndex = 0
		or: [(SourceFiles at: method fileIndex) == nil])
		or: [(oldPos _ method filePosition) = 0])
		ifTrue:
		["The source code is not accessible.  We must decompile..."
		preamble size > 0 ifTrue: [outStream cr; nextPut: $!!; nextChunkPut: preamble; cr].
		outStream nextChunkPut: (method decompilerClass new decompile: selector
											in: self method: method) decompileString]
		ifFalse:
		[sourceFile _ SourceFiles at: method fileIndex.
		sourceFile position: oldPos.
		preamble size > 0 ifTrue:    "Copy the preamble"
			[outStream copyPreamble: preamble from: sourceFile].
		"Copy the method chunk"
		newPos _ outStream position.
		outStream copyMethodChunkFrom: sourceFile.
		sourceFile skipSeparators.      "The following chunk may have ]style["
		sourceFile peek == $] ifTrue: [
			outStream cr; copyMethodChunkFrom: sourceFile].
		moveSource ifTrue:    "Set the new method source pointer"
			[endPos _ outStream position.
			method checkOKToAdd: endPos - newPos at: newPos.
			method setSourcePosition: newPos inFile: fileIndex]].
	preamble size > 0 ifTrue: [outStream nextChunkPut: ' '].
	^ outStream cr! !


!BytecodeGenerator class methodsFor: 'initialize' stamp: 'ajh 3/6/2003 22:44'!
bytecodeTableFrom: specArray
	"SpecArray is an array of either
	(index selector) or
	(index1 index2 selector)."

	| contiguous |
	Bytecodes _ IdentityDictionary new: 256.
	BytecodeTable _ Array new: 256.
	contiguous _ 0.
	specArray do: [ :spec |
		(spec at: 1) = contiguous ifFalse: [self error: 'Non-contiguous table entry'].
		spec size = 2 ifTrue: [
			Bytecodes at: (spec at: 2) put: (spec at: 1).
			BytecodeTable at: (spec at: 1) + 1 put: (spec at: 2).
			contiguous _ contiguous + 1.
		] ifFalse: [
			spec size = 3 ifFalse: [self error: 'bad spec size'].
			Bytecodes at: (spec at: 3) put: ((spec at: 1) to: (spec at: 2)).
			(spec at: 1) to: (spec at: 2) do: [ :i |
				BytecodeTable at: i + 1 put: (spec at: 3).
			].
			contiguous _ contiguous + ((spec at: 2) - (spec at: 1)) + 1.
		].
	].
	^ BytecodeTable! !

!BytecodeGenerator class methodsFor: 'initialize' stamp: 'ajh 3/6/2003 22:40'!
initialize

	self initializeBytecodeTable.
	self initializeSpecialSelectors.
	self initializeSpecialConstants.
! !

!BytecodeGenerator class methodsFor: 'initialize' stamp: 'ajh 3/6/2003 22:42'!
initializeBytecodeTable
	"BytecodeWriteStream initialize"
	"Defines all the bytecode instructions for the Compiler and the Interpreter.
	The following bytecode tuple format is:

	#(bytecode bytecodeSelector)

	bytecodeSelector is the method in the Interpreter that gets executed for the given bytecode.
	Common Send selector position within the specialSelectorsArray is hard code in the Interpreter, see senders of Interpreter specialSelector:."

	^ self bytecodeTableFrom:
	#(
		(  0  15 pushReceiverVariableBytecode)
		( 16  31 pushTemporaryVariableBytecode)
		( 32  63 pushLiteralConstantBytecode)
		( 64  95 pushLiteralVariableBytecode)
		( 96 103 storeAndPopReceiverVariableBytecode)
		(104 111 storeAndPopTemporaryVariableBytecode)
		(112 pushReceiverBytecode)
		(113 pushConstantTrueBytecode)
		(114 pushConstantFalseBytecode)
		(115 pushConstantNilBytecode)
		(116 pushConstantMinusOneBytecode)
		(117 pushConstantZeroBytecode)
		(118 pushConstantOneBytecode)
		(119 pushConstantTwoBytecode)
		(120 returnReceiver)
		(121 returnTrue)
		(122 returnFalse)
		(123 returnNil)
		(124 returnTopFromMethod)
		(125 returnTopFromBlock)
		(126 unknownBytecode)
		(127 unknownBytecode)
		(128 extendedPushBytecode)
		(129 extendedStoreBytecode)
		(130 extendedStoreAndPopBytecode)
		(131 singleExtendedSendBytecode)
		(132 doubleExtendedDoAnythingBytecode)
		(133 singleExtendedSuperBytecode)
		(134 secondExtendedSendBytecode)
		(135 popStackBytecode)
		(136 duplicateTopBytecode)
		(137 pushActiveContextBytecode)
		(138 143 experimentalBytecode)
		(144 151 shortUnconditionalJump)
		(152 159 shortConditionalJump)
		(160 167 longUnconditionalJump)
		(168 171 longJumpIfTrue)
		(172 175 longJumpIfFalse)

		"176-191 were sendArithmeticSelectorBytecode"
		(176 bytecodePrimAdd)
		(177 bytecodePrimSubtract)
		(178 bytecodePrimLessThan)
		(179 bytecodePrimGreaterThan)
		(180 bytecodePrimLessOrEqual)
		(181 bytecodePrimGreaterOrEqual)
		(182 bytecodePrimEqual)
		(183 bytecodePrimNotEqual)
		(184 bytecodePrimMultiply)
		(185 bytecodePrimDivide)
		(186 bytecodePrimMod)
		(187 bytecodePrimMakePoint)
		(188 bytecodePrimBitShift)
		(189 bytecodePrimDiv)
		(190 bytecodePrimBitAnd)
		(191 bytecodePrimBitOr)	

		"192-207 were sendCommonSelectorBytecode"
		(192 bytecodePrimAt)
		(193 bytecodePrimAtPut)
		(194 bytecodePrimSize)
		(195 bytecodePrimNext)
		(196 bytecodePrimNextPut)
		(197 bytecodePrimAtEnd)
		(198 bytecodePrimEquivalent)
		(199 bytecodePrimClass)
		(200 bytecodePrimBlockCopy)
		(201 bytecodePrimValue)
		(202 bytecodePrimValueWithArg)
		(203 bytecodePrimDo)
		(204 bytecodePrimNew)
		(205 bytecodePrimNewWithArg)
		(206 bytecodePrimPointX)
		(207 bytecodePrimPointY)

		(208 255 sendLiteralSelectorBytecode)
	)
! !

!BytecodeGenerator class methodsFor: 'initialize' stamp: 'ajh 3/6/2003 22:45'!
initializeSpecialConstants

	SpecialConstants _ {true. false. nil. -1. 0. 1. 2}! !

!BytecodeGenerator class methodsFor: 'initialize' stamp: 'ajh 3/6/2003 22:40'!
initializeSpecialSelectors
	"Create a map from specialSelector -> bytecode offset from sendAdd (the first one)"

	| array |
	SpecialSelectors _ IdentityDictionary new.
	array _ self specialSelectorsArray.  "Smalltalk specialObjectsArray at: 24"
	1 to: array size by: 2 "skip numArgs" do: [:i |
		SpecialSelectors at: (array at: i) put: i - 1 / 2].
! !

!BytecodeGenerator class methodsFor: 'initialize' stamp: 'ajh 3/15/2003 15:43'!
specialConstants

	^ SpecialConstants! !

!BytecodeGenerator class methodsFor: 'initialize' stamp: 'ajh 3/6/2003 22:45'!
specialSelectorsArray

	^ #(#+ 1 #- 1 #< 1 #> 1 #<= 1 #>= 1 #= 1 #~= 1 #* 1 #/ 1 #\\ 1 #@ 1 #bitShift: 1 #// 1 #bitAnd: 1 #bitOr: 1 #at: 1 #at:put: 2 #size 0 #next 0 #nextPut: 1 #atEnd 0 #== 1 #class 0 #blockCopy: 1 #value 0 #value: 1 #do: 1 #new 0 #new: 1 #x 0 #y 0)! !

!BytecodeGenerator class methodsFor: 'instance creation' stamp: 'ajh 3/9/2003 21:50'!
new

	^ super new initialize! !


!CodeHolder methodsFor: 'message list' stamp: 'ajh 3/15/2003 16:03'!
decompiledSourceIntoContents
	"Obtain a source string by decompiling the method's code, and place that source string into my contents.  Get temps from source file"

	| tempNames class selector method |
	class _ self selectedClassOrMetaClass.
	selector _ self selectedMessageName.
	method _ class compiledMethodAt: selector ifAbsent: [^ ''].	"method deleted while in another project"
	currentCompiledMethod _ method.

	tempNames _ (class compilerClass new
					parse: method getSourceFromFile asString in: class notifying: nil)
					tempNames.
	contents _ ((method decompilerClass new withTempNames: tempNames)
			decompile: selector in: class method: method) decompileString.
	contents _ contents asText makeSelectorBoldIn: class.
	^ contents copy! !

!CodeHolder methodsFor: 'message list' stamp: 'ajh 3/15/2003 16:03'!
selectedMessage
	"Answer a copy of the source code for the selected message.  This generic version is probably actually never reached, since every subclass probably reimplements and does not send to super.  In time, ideally, most, or all, reimplementors would vanish and all would defer instead to a universal version right here.  Everything in good time."

	| class selector method tempNames |
	contents == nil ifFalse: [^ contents copy].
	self showingDecompile ifTrue:
		[^ self decompiledSourceIntoContents].

	class _ self selectedClassOrMetaClass.
	(class isNil or: [(selector _ self selectedMessageName) isNil]) ifTrue: [^ ''].
	method _ class compiledMethodAt: selector ifAbsent: [^ ''].	"method deleted while in another project"
	currentCompiledMethod _ method.

	(Sensor controlKeyPressed
		or: [method fileIndex > 0 and: [(SourceFiles at: method fileIndex) == nil]])
		ifTrue:
		["Emergency or no source file -- decompile without temp names"
		contents _ (method decompilerClass new decompile: selector in: class method: method)
			decompileString.
		contents _ contents asText makeSelectorBoldIn: class.
		^ contents copy].

	Sensor leftShiftDown ifTrue:
		["Special request to decompile -- get temps from source file"
		tempNames _ (class compilerClass new
						parse: method getSourceFromFile asString in: class notifying: nil)
						tempNames.
		contents _ ((method decompilerClass new withTempNames: tempNames)
				decompile: selector in: class method: method) decompileString.
		contents _ contents asText makeSelectorBoldIn: class.
		^ contents copy].

	self showComment
		ifFalse:
			[contents _ self sourceStringPrettifiedAndDiffed]
		ifTrue:
			[contents _ self commentContents].
	^ contents _ contents copy asText makeSelectorBoldIn: class! !


!Browser methodsFor: 'message list' stamp: 'ajh 3/15/2003 16:02'!
selectedMessage
	"Answer a copy of the source code for the selected message."

	| class selector method |
	contents == nil ifFalse: [^ contents copy].
	self showingDecompile ifTrue:
		[^ self decompiledSourceIntoContents].

	class _ self selectedClassOrMetaClass.
	selector _ self selectedMessageName.
	method _ class compiledMethodAt: selector ifAbsent: [^ ''].	"method deleted while in another project"
	currentCompiledMethod _ method.

	(Sensor controlKeyPressed
		or: [method fileIndex > 0 and: [(SourceFiles at: method fileIndex) == nil]])
		ifTrue:
		["Emergency or no source file -- decompile without temp names"
		contents _ (method decompilerClass new decompile: selector in: class method: method)
			decompileString.
		contents _ contents asText makeSelectorBoldIn: class.
		^ contents copy].

	Sensor leftShiftDown ifTrue:
		["Special request to decompile, old and dubious interface"
		^ self decompiledSourceIntoContents].

	self showingDocumentation
		ifFalse:
			[contents _ self sourceStringPrettifiedAndDiffed]
		ifTrue:
			[contents _ self commentContents].
	^ contents _ contents copy asText makeSelectorBoldIn: class! !


!CompiledMethod methodsFor: 'printing' stamp: 'ajh 3/15/2003 16:01'!
decompileString
	| clAndSel cl sel |
	clAndSel _ self who.
	clAndSel = #(unknown unknown)
		ifTrue:
			[cl _ Object.
			sel _ #xxxUnknown.
			self numArgs >= 1
				ifTrue:
					[sel _ sel , ':'.
					2 to: self numArgs do: [:i | sel _ sel , 'with:'].
					sel _ sel asSymbol]]
		ifFalse:
			[cl _ clAndSel first.
			sel _ clAndSel last].
	^ (self decompilerClass new
			decompile: sel in: cl method: self) decompileString! !

!CompiledMethod methodsFor: 'literals' stamp: 'ajh 5/30/2003 01:31'!
allEmbeddedBlockMethods

	| set |
	set _ OrderedCollection new.
	1 to: self numLiterals do: [:i |  | lit |
		lit _ self literalAt: i.
		(lit isKindOf: CompiledMethod) ifTrue: [
			set add: lit.
			set addAll: lit allEmbeddedBlockMethods.
		] ifFalse: [(lit isKindOf: BlockClosure) ifTrue: [
			set add: lit method.
			self addAll: lit method allEmbeddedBlockMethods
		]].
	].
	^ set! !

!CompiledMethod methodsFor: 'literals' stamp: 'ajh 5/30/2003 01:31'!
embeddedBlockMethods

	| set |
	set _ OrderedCollection new.
	1 to: self numLiterals do: [:i |  | lit |
		lit _ self literalAt: i.
		(lit isKindOf: CompiledMethod) ifTrue: [
			set add: lit.
		] ifFalse: [(lit isKindOf: BlockClosure) ifTrue: [
			set add: lit method.
		]].
	].
	^ set! !

!CompiledMethod methodsFor: 'scanning' stamp: 'ajh 5/30/2003 01:36'!
readsField: varIndex 
	"Answer whether the receiver loads the instance variable indexed by the argument."

	self allEmbeddedBlockMethods do: [:meth |
		| scanner catcher instr scan |
		scanner _ meth scanner.
		catcher _ Message catcher.
		[scanner atEnd] whileFalse: [
			instr _ scanner interpretNextInstructionFor: catcher.
			(instr selector = #send:super:numArgs: and: [instr argument = #getInstVar:]) ifTrue: [
				scan _ scanner copy.
				scan pc: scan previousPc. scan pc: scan previousPc.
				instr _ scan interpretNextInstructionFor: catcher.
				(instr selector = #pushConstant: and: [instr argument = varIndex])
					ifTrue: [^ true]
			].
		].
	].
	self isReturnField ifTrue: [^ self returnField + 1 = varIndex].
	varIndex <= 16 ifTrue: [^ self scanFor: varIndex - 1].
	varIndex <= 64 ifTrue: [^ self scanLongLoad: varIndex - 1].
	^ self scanVeryLongLoad: 64 offset: varIndex - 1! !

!CompiledMethod methodsFor: 'scanning' stamp: 'ajh 5/30/2003 01:18'!
scanner

	^ InstructionStream on: self! !

!CompiledMethod methodsFor: 'scanning' stamp: 'ajh 5/30/2003 01:36'!
writesField: field 
	"Answer whether the receiver stores into the instance variable indexed by the argument."

	self allEmbeddedBlockMethods do: [:meth |
		| scanner catcher instr scan |
		scanner _ meth scanner.
		catcher _ Message catcher.
		[scanner atEnd] whileFalse: [
			instr _ scanner interpretNextInstructionFor: catcher.
			(instr selector = #send:super:numArgs: and: [instr argument = #storeIn:instVar:]) ifTrue: [
				scan _ scanner copy.
				scan pc: scan previousPc. scan pc: scan previousPc.
				instr _ scan interpretNextInstructionFor: catcher.
				(instr selector = #pushConstant: and: [instr argument = field])
					ifTrue: [^ true]
			].
		].
	].
	self isQuick ifTrue: [^ false].
	field <= 8 ifTrue:
		[^ (self scanFor: 96 + field - 1) or: [self scanLongStore: field - 1]].
	field <= 64 ifTrue:
		[^ self scanLongStore: field - 1].
	^ self scanVeryLongStore: 160 offset: field - 1! !

!CompiledMethod methodsFor: 'source code management' stamp: 'ajh 3/15/2003 16:02'!
getSourceFor: selector in: class
	"Retrieve or reconstruct the source code for this method."
	| source flagByte |
	flagByte _ self last.
	(flagByte = 0
		or: [flagByte = 251 "some source-less methods have flag = 251, rest = 0"
			and: [((1 to: 3) collect: [:i | self at: self size - i]) = #(0 0 0)]])
		ifTrue:
		["No source pointer -- decompile without temp names"
		^ (self decompilerClass new decompile: selector in: class method: self)
			decompileString].
	flagByte < 252 ifTrue:
		["Magic sources -- decompile with temp names"
		^ ((self decompilerClass new withTempNames: self tempNames)
				decompile: selector in: class method: self)
			decompileString].

	"Situation normal;  read the sourceCode from the file"
	(source _ self getSourceFromFile) == nil ifFalse: [^ source].

	"Something really wrong -- decompile blind (no temps)"
	^ (self decompilerClass new decompile: selector in: class method: self)
			decompileString! !

!CompiledMethod methodsFor: 'decompiling' stamp: 'ajh 5/28/2003 01:10'!
blockNodeIn: homeMethodNode
	"Return the block node for self"

	homeMethodNode ifNil: [
		^ self decompilerClass new decompileBlock: self].

	homeMethodNode ir compiledMethod.  "generate method"
	homeMethodNode nodesDo: [:node |
		(node isBlock and:
		 [node scope isInlined not and:
		  [node ir compiledMethod = self]])
			ifTrue: [
				BlockNodeCache _ self -> node.
				^ node]
	].
	self errorNodeNotFound! !

!CompiledMethod methodsFor: 'decompiling' stamp: 'ajh 3/13/2003 21:13'!
decompilerClass

	^ self isClosureCompiled
		ifTrue: [Decompiler2]
		ifFalse: [Decompiler]! !

!CompiledMethod methodsFor: 'decompiling' stamp: 'ajh 3/2/2003 13:51'!
ir

	^ BytecodeDecompiler new decompile: self! !

!CompiledMethod methodsFor: 'decompiling' stamp: 'ajh 3/3/2003 12:02'!
methodNodeDecompileClass: aClass selector: selector
	"Return the parse tree that represents self"

	| source |
	^ (source _ self getSourceFromFile)
		ifNil: [self decompileClass: aClass selector: selector]
		ifNotNil: [self parserClass new parse: source class: (self sourceClass ifNil: [aClass])]! !


!CompiledMethod class methodsFor: 'instance creation' stamp: 'ajh 3/9/2003 15:09'!
primitive: primNum numArgs: numArgs numTemps: numTemps stackSize: stackSize literals: literals bytecodes: bytecodes trailer: trailerBytes
	"Create method with given attributes.  numTemps includes numArgs.  stackSize does not include numTemps."

	| compiledMethod |
	compiledMethod _ self
		newBytes: bytecodes size
		trailerBytes: trailerBytes 
		nArgs: numArgs
		nTemps: numTemps
		nStack: stackSize
		nLits: literals size
		primitive: primNum.
	(WriteStream with: compiledMethod)
		position: compiledMethod initialPC - 1;
		nextPutAll: bytecodes.
	literals withIndexDo: [:obj :i | compiledMethod literalAt: i put: obj].
	^ compiledMethod! !


!Compiler methodsFor: 'public access' stamp: 'ajh 3/19/2003 01:12'!
compileDoIt: textOrStream in: aContext to: receiver notifying: aRequestor ifFail: failBlock
	"Compiles the sourceStream into a parse tree, then generates code into a method.  This method is then wrapped in a block with the receiver or context as the sole free variable which the method refers to.  If requestor is not nil, then it will receive a notify:at: message if there is a compile error, followed by the failBlock being executed."

	| scope parser blockNode method |
	scope _ aContext
		ifNotNil: [aContext doItScope]
		ifNil: [receiver class parseScope newMethodScope].
	parser _ self parserClass new.
	blockNode _ parser
			parse: textOrStream readStream
			class: scope
			noPattern: true
			notifying: aRequestor
			ifFail: [^ failBlock value].
	method _ blockNode generate: #(0).
	"cacheDoItNode ifTrue: [
		method cacheDoItBlockNode: blockNode.
		method cacheHomeMethod: nil.
	]."
	^ MethodClosure new
		method: method;
		receiver: (aContext ifNil: [receiver]);
		yourself! !

!Compiler methodsFor: 'public access' stamp: 'ajh 9/14/2002 22:00'!
evaluate2: textOrStream in: aContext to: receiver notifying: aRequestor ifFail: failBlock

	^ (self compileDoIt: textOrStream in: aContext to: receiver notifying: aRequestor ifFail: [^ failBlock value]) value! !

!Compiler methodsFor: 'public access' stamp: 'ajh 1/27/2003 20:35'!
evaluate: textOrStream in: aContext to: receiver notifying: aRequestor ifFail: failBlock
	"Compiles the sourceStream into a parse tree, then generates code into a 
	method. This method is then installed in the receiver's class so that it 
	can be invoked. In other words, if receiver is not nil, then the text can 
	refer to instance variables of that receiver (the Inspector uses this). If 
	aContext is not nil, the text can refer to temporaries in that context (the 
	Debugger uses this). If aRequestor is not nil, then it will receive a 
	notify:at: message before the attempt to evaluate is aborted. Finally, the 
	compiled method is invoked from here as DoIt or (in the case of 
	evaluation in aContext) DoItIn:. The method is subsequently removed 
	from the class, but this will not get done if the invocation causes an 
	error which is terminated. Such garbage can be removed by executing: 
	Smalltalk allBehaviorsDo: [:cl | cl removeSelector: #DoIt; removeSelector: 
	#DoItIn:]."

	| methodNode method value |
	Preferences compileBlocksAsClosures ifTrue: [
		cacheDoItNode _ true.
		^ self evaluate2: textOrStream in: aContext to: receiver notifying: aRequestor ifFail: failBlock].
	class _ (aContext == nil ifTrue: [receiver] ifFalse: [aContext receiver]) class.
	self from: textOrStream class: class context: aContext notifying: aRequestor.
	methodNode _ self translate: sourceStream noPattern: true ifFail:
		[^failBlock value].
	method _ methodNode generate: #(0 0 0 0).
	self interactive ifTrue:
		[method _ method copyWithTempNames: methodNode tempNames].
	context == nil
		ifTrue: [class addSelector: #DoIt withMethod: method.
				value _ receiver DoIt.
				InMidstOfFileinNotification signal ifFalse: [
					class removeSelectorSimply: #DoIt.
				].
				^value]
		ifFalse: [class addSelector: #DoItIn: withMethod: method.
				value _ receiver DoItIn: context.
				InMidstOfFileinNotification signal ifFalse: [
					class removeSelectorSimply: #DoItIn:.
				].
				^value]! !

!Compiler methodsFor: 'private' stamp: 'ajh 3/6/2003 01:04'!
dialectParserClass

	^ self parserClass = Parser2
		ifTrue: [self toDo]
		ifFalse: [DialectParser]! !


!Compiler class methodsFor: 'accessing' stamp: 'ajh 1/20/2002 18:57'!
parserClass
	"Return a parser class to use for parsing method headers."

	^ Preferences compileBlocksAsClosures
		ifTrue: [Parser2]
		ifFalse: [Parser]! !

!Compiler class methodsFor: 'as yet unclassified' stamp: 'ajh 3/26/2003 00:39'!
initialize

	Preferences
		addPreference: #compileBlocksAsClosures
		category: #general
		default: false
		balloonHelp: 'If true, blocks ([...]) will be compiled as BlockClosures instead of BlockContexts.  BlockClosures are independent of their home context but are a little slower (for now).  Only newly compiled methods will be affected by this preference.'
! !


!Debugger methodsFor: 'code pane' stamp: 'ajh 3/15/2003 01:34'!
pcRange
	"Answer the indices in the source code for the method corresponding to 
	the selected context's program counter value."

	| i pc end |
	(selectingPC and: [contextStackIndex ~= 0])
		ifFalse: [^1 to: 0].
	sourceMap == nil ifTrue:
		[sourceMap _ theMethodNode sourceMap.
		tempNames _ theMethodNode tempNames.
		self selectedContext method cacheTempNames: tempNames].
	sourceMap size = 0 ifTrue: [^1 to: 0].

	(theMethodNode isKindOf: RBProgramNode) ifTrue: [
		pc _ contextStackIndex = 1
			ifTrue: [self selectedContext pc]
			ifFalse: [self selectedContext previousPc].
		i _ sourceMap reverseDetectIndex: [:pcRange | pcRange key <= pc] ifNone: [^ 1 to: 0].
		^ (sourceMap at: i) value
	].

	pc_ self selectedContext pc -
		(("externalInterrupt" true and: [contextStackIndex=1])
			ifTrue: [1]
			ifFalse: [2]).
	i _ sourceMap indexForInserting: (Association key: pc value: nil).
	i < 1 ifTrue: [^1 to: 0].
	i > sourceMap size
		ifTrue:
			[end _ sourceMap inject: 0 into:
				[:prev :this | prev max: this value last].
			^ end+1 to: end].
	^(sourceMap at: i) value! !


!Decompiler2 methodsFor: 'as yet unclassified' stamp: 'ajh 3/13/2003 21:09'!
decompile: aMethod
	"Answer a MethodNode that is the root of the parse tree for the argument, aMethod.  Selector will be 'unknown' and inst vars will be 'instVarN'.  selector and class can be placed in resulting MethodNode later"

	^self
		decompile: nil
		in: nil
		method: aMethod
! !

!Decompiler2 methodsFor: 'as yet unclassified' stamp: 'ajh 3/13/2003 21:09'!
decompile: aSelector in: aClass 
	"See Decompiler|decompile:in:method:. The method is found by looking up 
	the message, aSelector, in the method dictionary of the class, aClass."

	^self
		decompile: aSelector
		in: aClass
		method: (aClass compiledMethodAt: aSelector)! !

!Decompiler2 methodsFor: 'as yet unclassified' stamp: 'ajh 3/13/2003 21:11'!
decompile: selector in: aClass method: aMethod
	"Answer a MethodNode that is the root of the parse tree for the 
	argument, aMethod, which is the CompiledMethod associated with the 
	message, aSelector. Variables are determined with respect to the 
	argument, aClass."

	| ird methodNode |
	ird _ IRDecompiler new.
	ird scope: aClass parseScope newMethodScope.
	methodNode _ ird decompileIR: aMethod ir.
	selector ifNotNil: [methodNode selector: selector].
	^ methodNode! !

!Decompiler2 methodsFor: 'as yet unclassified' stamp: 'ajh 3/18/2003 10:51'!
decompileBlock: aMethod
	"Answer a BlockNode"

	| ird |
	ird _ IRDecompiler new.
	ird scope: nil parseScope newBlockScope.
	^ (ird decompileIR: aMethod ir) asBlock! !

!Decompiler2 methodsFor: 'as yet unclassified' stamp: 'ajh 3/15/2003 15:57'!
withTempNames: tempNameArray
! !


!FiniteAutomaton methodsFor: 'initializing' stamp: 'ajh 9/22/2002 17:22'!
actions: actionsArray transitions: transitionTable

	actions _ IdentityDictionary new.
	actionsArray withIndexDo: [:action :i |
		actions at: action put: i].

	states _ Array new: transitionTable size.
	transitions _ IdentityDictionary new.
	transitionTable withIndexDo: [:transitionPair :i |
		states at: i put: transitionPair first.
		transitions at: transitionPair first "from state" 
				 put: transitionPair second "to states indexed by action index"].
! !

!FiniteAutomaton methodsFor: 'accessing' stamp: 'ajh 9/22/2002 17:22'!
higherState: state1 or: state2

	^ (states indexOf: state1) >= (states indexOf: state2)
		ifTrue: [state1]
		ifFalse: [state2]! !

!FiniteAutomaton methodsFor: 'accessing' stamp: 'ajh 9/22/2002 17:53'!
state: state transition: action
	"Return new state after action transition from state"

	^ (transitions at: state) at: (actions at: action)! !


!IRBuilder methodsFor: 'initialize' stamp: 'ajh 3/10/2003 14:06'!
addTemp: tempName

	self addTemps: {tempName}! !

!IRBuilder methodsFor: 'initialize' stamp: 'ajh 3/17/2003 11:07'!
addTemps: tempKeys

	| otherKeys i |
	otherKeys _ ir otherTempKeys.
	i _ ir numArgs + otherKeys size.
	tempKeys do: [:key |
		tempMap at: key put: (i _ i + 1)].
	ir otherTempKeys: otherKeys, tempKeys.
! !

!IRBuilder methodsFor: 'initialize' stamp: 'ajh 3/10/2003 14:10'!
primitiveNode: primNode

	ir primitiveNode: primNode! !

!IRBuilder methodsFor: 'initialize' stamp: 'ajh 3/17/2003 11:12'!
rargs: tempKeys

	tempKeys withIndexDo: [:key :i |
		tempMap at: key put: i - 1].
	ir rargKeys: tempKeys.
! !

!IRBuilder methodsFor: 'initialize' stamp: 'ajh 3/17/2003 11:06'!
thisContext: tempKey

	tempMap at: tempKey put: -1! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/11/2003 10:58'!
getField: instVarIndex
	"Receiver must be on top"

	self pushLiteral: instVarIndex.
	self send: #getInstVar:.
! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 14:30'!
jumpAheadTarget: labelSymbol
	"Pop latest jumpAheadTo: with this labelSymbol and have it point to this new instruction sequence"

	| jumpInstr |
	self startNewSequence.
	jumpInstr _ (jumpAheadStacks at: labelSymbol ifAbsent: [self error: 'Missing jumpAheadTo: ', labelSymbol printString]) pop.
	jumpInstr destination: currentSequence.
! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 14:24'!
jumpAheadTo: labelSymbol
	"Jump to the sequence that will be created when jumpAheadTarget: labelSymbol is sent to self.  This is and its corresponding target is only good for one use.  Other jumpAheadTo: with the same label will be put on a stack and superceed existing ones until its jumpAheadTarget: is called."

	"jumpAheadTarget: label will pop this and replace destination with its basic block"
	(jumpAheadStacks at: labelSymbol ifAbsentPut: [OrderedCollection new])
		push: (self add: IRJump new).
	self startNewSequence.
! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 14:30'!
jumpAheadTo: labelSymbol if: boolean
	"Conditional jump to the sequence that will be created when jumpAheadTarget: labelSymbol is sent to self.  This and its corresponding target is only good for one use.  Other jumpAheadTo:... with the same label will be put on a stack and superceed existing ones until its jumpAheadTarget: is called."

	| instr |
	"jumpAheadTarget: label will pop this and replace destination with its basic block"
	(jumpAheadStacks at: labelSymbol ifAbsentPut: [OrderedCollection new])
		push: (instr _ self add: (IRJumpIf new boolean: boolean)).
	self startNewSequence.
	instr otherwise: currentSequence.
! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 14:31'!
jumpBackTarget: labelSymbol
	"Remember this basic block for a future jumpBackTo: labelSymbol.  Stack up remembered targets with same name and remove them from stack for each jumpBackTo: called with same name."

	self startNewSequence.
	(jumpBackTargetStacks at: labelSymbol ifAbsentPut: [OrderedCollection new])
		push: currentSequence.
! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 14:32'!
jumpBackTo: labelSymbol
	"Pop last remembered position with this label and write an unconditional jump to it"

	| sequence |
	sequence _ (jumpBackTargetStacks at: labelSymbol ifAbsent: [self error: 'Missing jumpBackTarget: ', labelSymbol printString]) pop.
	self add: (IRJump new destination: sequence).
	self startNewSequence.
! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 01:09'!
popTop

	self add: IRInstruction popTop! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 01:09'!
pushDup

	self add: IRInstruction pushDup! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 01:09'!
pushLiteral: object

	self add: (IRInstruction pushLiteral: object)! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 14:52'!
pushReceiver

	self add: (IRInstruction pushTemp: 0)! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/17/2003 11:06'!
pushTemp: key

	| index |
	index _ tempMap at: key.
	self add: (IRInstruction pushTemp: index)! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 14:52'!
pushThisContext

	self add: (IRInstruction pushTemp: -1)! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/15/2003 01:55'!
remoteReturn

	self add: IRInstruction remoteReturn.
	self startNewSequence.
! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/15/2003 01:55'!
returnTop

	self add: IRInstruction returnTop.
	self startNewSequence.
! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 01:09'!
send: selector

	self add: (IRInstruction send: selector)! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 01:09'!
send: selector toSuperOf: behavior

	self add: (IRInstruction send: selector toSuperOf: behavior)! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/10/2003 15:03'!
setField: instVarIndex
	"receiver must be on top with new field value underneath"

	self pushLiteral: instVarIndex.
	self send: #storeIn:instVar:.
! !

!IRBuilder methodsFor: 'instructions' stamp: 'ajh 3/17/2003 11:06'!
storeTemp: key

	| index |
	index _ tempMap at: key.
	self add: (IRInstruction storeTemp: index)! !

!IRBuilder methodsFor: 'private' stamp: 'ajh 3/13/2003 13:20'!
add: instr

	"Associate instr with current parse node or byte range"
	instr sourceNode: self sourceNode.
	instr bytecodeIndex: self sourceByteIndex.
	^ currentSequence add: instr! !

!IRBuilder methodsFor: 'private' stamp: 'ajh 3/17/2003 11:06'!
initialize

	ir _ IRMethod new.
	tempMap _ Dictionary new.
	jumpAheadStacks _ IdentityDictionary new.
	jumpBackTargetStacks _ IdentityDictionary new.
	sourceMapNodes _ OrderedCollection new.  "stack"

	"Leave an empty sequence up front (guaranteed not to be in loop)"
	ir startSequence: (IRSequence new orderNumber: 0).
	currentSequence _ IRSequence new orderNumber: 1.
	ir startSequence add: (IRJump new destination: currentSequence).
! !

!IRBuilder methodsFor: 'private' stamp: 'ajh 3/10/2003 17:45'!
startNewSequence
	"End current instruction sequence and start a new sequence to add instructions to.  If ending block just falls through to new block then add an explicit jump to it so they stay linked"

	| newSequence |
	currentSequence isEmpty ifTrue: [^ self].  "block is still empty, continue using it"
	newSequence _ IRSequence new
		orderNumber: currentSequence orderNumber + 1.
	currentSequence last isJumpOrReturn ifFalse: [
		self add: (IRJump new destination: newSequence)].
	currentSequence _ newSequence.
! !

!IRBuilder methodsFor: 'mapping' stamp: 'ajh 3/13/2003 13:17'!
mapToByteIndex: index
	"decompiling"

	sourceMapByteIndex _ index! !

!IRBuilder methodsFor: 'mapping' stamp: 'ajh 3/10/2003 17:42'!
mapToNode: object
	"new instructions will be associated with object"

	sourceMapNodes push: object! !

!IRBuilder methodsFor: 'mapping' stamp: 'ajh 3/10/2003 17:44'!
popMap

	sourceMapNodes pop! !

!IRBuilder methodsFor: 'mapping' stamp: 'ajh 3/13/2003 13:19'!
sourceByteIndex
	"decompiling"

	^ sourceMapByteIndex! !

!IRBuilder methodsFor: 'mapping' stamp: 'ajh 3/10/2003 17:46'!
sourceNode

	^ sourceMapNodes isEmpty
		ifTrue: [nil]
		ifFalse: [sourceMapNodes top]! !

!IRBuilder methodsFor: 'results' stamp: 'ajh 3/10/2003 15:51'!
ir

	^ ir! !

!IRBuilder methodsFor: 'decompiling' stamp: 'ajh 3/10/2003 17:48'!
addJumpBackTarget: label to: sequence

	(jumpBackTargetStacks at: label ifAbsentPut: [OrderedCollection new])
		push: sequence! !

!IRBuilder methodsFor: 'decompiling' stamp: 'ajh 3/21/2003 01:48'!
testJumpAheadTarget: label

	jumpAheadStacks at: label ifPresent: [:stack |
		[stack isEmpty] whileFalse: [self jumpAheadTarget: label]
	]! !


!IRBuilder class methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 17:56'!
new

	^ super new initialize! !


!IRInstruction methodsFor: 'accessing' stamp: 'ajh 3/6/2003 14:32'!
executeOn: interpreter
	"Send approriate message to interpreter"

	self subclassResponsibility! !

!IRInstruction methodsFor: 'accessing' stamp: 'ajh 3/6/2003 14:32'!
successorSequences
	"sent to last instruction in sequence which is expected to be a jump and return instruction"

	^ #()! !

!IRInstruction methodsFor: 'testing' stamp: 'ajh 3/10/2003 16:12'!
isConstant: valueTest

	^ false! !

!IRInstruction methodsFor: 'testing' stamp: 'ajh 3/11/2003 00:19'!
isGoto
	"is unconditional jump"

	^ false! !

!IRInstruction methodsFor: 'testing' stamp: 'ajh 3/11/2003 00:21'!
isIf

	^ false! !

!IRInstruction methodsFor: 'testing' stamp: 'ajh 3/11/2003 00:22'!
isJump
	"goto or if"

	^ false! !

!IRInstruction methodsFor: 'testing' stamp: 'ajh 3/10/2003 16:13'!
isJumpOrReturn

	^ self isJump or: [self isReturn]! !

!IRInstruction methodsFor: 'testing' stamp: 'ajh 3/10/2003 16:09'!
isReturn

	^ false! !

!IRInstruction methodsFor: 'mapping' stamp: 'ajh 3/13/2003 13:07'!
bytecodeIndex

	^ bytecodeIndex! !

!IRInstruction methodsFor: 'mapping' stamp: 'ajh 3/13/2003 13:14'!
bytecodeIndex: index

	bytecodeIndex _ index! !

!IRInstruction methodsFor: 'mapping' stamp: 'ajh 3/6/2003 14:32'!
sourceNode

	^ sourceNode
	! !

!IRInstruction methodsFor: 'mapping' stamp: 'ajh 3/6/2003 14:32'!
sourceNode: parseNode

	sourceNode _ parseNode
	! !


!IRConstant methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:43'!
constant

	^ constant! !

!IRConstant methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:43'!
constant: object

	constant _ object! !

!IRConstant methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:46'!
executeOn: interpreter

	^ interpreter pushLiteral: constant! !

!IRConstant methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 16:12'!
isConstant: valueTest

	^ valueTest value: constant! !


!IRDup methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:46'!
executeOn: interpreter

	^ interpreter pushDup! !


!IRInstruction class methodsFor: 'instance creation' stamp: 'ajh 3/10/2003 01:07'!
goto: seq

	^ IRJump new
		destination: seq! !

!IRInstruction class methodsFor: 'instance creation' stamp: 'ajh 3/10/2003 01:07'!
if: bool goto: seq1 otherwise: seq2

	^ IRJumpIf new
		boolean: bool;
		destination: seq1;
		otherwise: seq2! !

!IRInstruction class methodsFor: 'instance creation' stamp: 'ajh 3/10/2003 01:07'!
popTop

	^ IRPop new! !

!IRInstruction class methodsFor: 'instance creation' stamp: 'ajh 3/10/2003 01:08'!
pushDup

	^ IRDup new! !

!IRInstruction class methodsFor: 'instance creation' stamp: 'ajh 3/10/2003 01:08'!
pushLiteral: object

	^ IRConstant new
		constant: object! !

!IRInstruction class methodsFor: 'instance creation' stamp: 'ajh 3/12/2003 12:16'!
pushTemp: index

	^ IRTemp new
		number: index;
		isStore: false! !

!IRInstruction class methodsFor: 'instance creation' stamp: 'ajh 3/10/2003 01:08'!
remoteReturn

	^ IRReturn new
		isRemote: true! !

!IRInstruction class methodsFor: 'instance creation' stamp: 'ajh 3/10/2003 01:08'!
returnTop

	^ IRReturn new
		isRemote: false! !

!IRInstruction class methodsFor: 'instance creation' stamp: 'ajh 3/10/2003 01:08'!
send: selector

	^ IRSend new
		selector: selector! !

!IRInstruction class methodsFor: 'instance creation' stamp: 'ajh 3/10/2003 01:08'!
send: selector toSuperOf: behavior

	^ IRSend new
		selector: selector;
		superOf: behavior! !

!IRInstruction class methodsFor: 'instance creation' stamp: 'ajh 3/12/2003 12:16'!
storeTemp: index

	^ IRTemp new
		number: index;
		isStore: true! !


!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 13:26'!
goto: seqNum! !

!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 13:26'!
if: bool goto: seqNum1 otherwise: seqNum2! !

!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 13:25'!
label: seqNum! !

!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 14:11'!
popTop! !

!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 14:12'!
pushDup! !

!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/2/2003 16:08'!
pushLiteral: object! !

!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/2/2003 16:09'!
pushTemp: index! !

!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/2/2003 16:10'!
remoteReturn! !

!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/2/2003 16:08'!
returnTop! !

!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/2/2003 16:11'!
send: selector! !

!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/2/2003 16:11'!
send: selector toSuperOf: behavior! !

!IRInterpreter methodsFor: 'instructions' stamp: 'ajh 3/2/2003 16:11'!
storeTemp: index! !

!IRInterpreter methodsFor: 'interpret' stamp: 'ajh 3/9/2003 12:33'!
interpret: ir

	self interpretAll: ir allSequences! !

!IRInterpreter methodsFor: 'interpret' stamp: 'ajh 3/10/2003 23:30'!
interpretAll: irSequences

	irSequences do: [:seq | self interpretSequence: seq]! !

!IRInterpreter methodsFor: 'interpret' stamp: 'ajh 3/6/2003 15:31'!
interpretInstruction: irInstruction

	irInstruction executeOn: self! !

!IRInterpreter methodsFor: 'interpret' stamp: 'ajh 3/9/2003 13:20'!
interpretSequence: instructionSequence

	self label: instructionSequence orderNumber.
	instructionSequence do: [:instr | self interpretInstruction: instr].
! !


!IRDecompiler methodsFor: 'init' stamp: 'ajh 3/24/2003 01:22'!
decompileIR: ir

	| sequenceNode temps args goto seq value |
	1 to: ir numArgs do: [:i | scope tempVarAt: i].
	self interpret: ir.
	self label: #return.  "final label to return"

	self Label: #return.
	goto _ self Goto.
	value _ self ValueOrNone.
	seq _ self Sequence.
	sp = 1 ifFalse: [self patternError].

	value ifNotNil: [seq addNode: value].
	sequenceNode _ (self newBlock: seq return: goto) body.

	temps _ scope tempVars asArray.
	args _ temps first: ir numArgs.
	ir rargKeys: (args copyWithFirst: scope receiverVar).
	args _ args collect: [:var | self newVar: var].
	temps _ temps allButFirst: ir numArgs.
	ir otherTempKeys: temps.
	sequenceNode temporaries: (temps collect: [:var | self newVar: var]).

	^ RBMethodNode new
		selectorParts: (self newSelectorParts: ir numArgs dummySelector);
		arguments: args;
		body: sequenceNode;
		privIR: ir;
		primitiveNode: ir primitiveNode;
		scope: scope! !

!IRDecompiler methodsFor: 'init' stamp: 'ajh 3/3/2003 15:15'!
scope: aSemScope

	scope _ aSemScope! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/23/2003 12:47'!
goto: seqNum

	self stackPush: (RBPseudoGotoNode new destination: seqNum).
! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/20/2003 17:21'!
if: bool goto: seqNum1 otherwise: seqNum2

	self stackPush: (RBPseudoIfNode new
		boolean: bool;
		destination: seqNum1;
		otherwise: seqNum2)! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/25/2003 00:04'!
label: seqNum

	stack isEmpty ifTrue: [  "start"
		^ stack push: (RBPseudoLabelNode new destination: seqNum)].

	"Reduce jump structures to one of the following if possible"
	[	(self endIfNil: seqNum) or: [
		 (self endAndOr: seqNum) or: [
		  (self endAndOr2: seqNum) or: [
		   (self endIfThen: seqNum) or: [
		    (self endIfThenElse: seqNum) or: [
		     (self endCase: seqNum) or: [
		      (self endToDo: seqNum) or: [
		       (self endWhile: seqNum)]]]]]]]
	] whileTrue.

	stack push: (RBPseudoLabelNode new destination: seqNum).
! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/24/2003 16:01'!
popTop

	| value |
	stack top ifNil: [^ stack pop].  "pop no-op from #simplifyTempAssign:"
	stack top isValue ifTrue: [
		(stack top: 2) isSequence ifTrue: [
			value _ stack pop.
			^ stack top addNode: value.
		] ifFalse: [(stack top: 2) isPseudo ifTrue: [
			value _ stack pop.
			^ stack push: (RBSequenceNode statements: {value}).
		]].
	].
	stack push: RBPseudoPopNode new
! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/9/2003 14:12'!
pushDup

	stack push: RBPseudoDupNode new! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/24/2003 21:14'!
pushLiteral: object

	object isBlock ifTrue: [^ self block: object method capturedVars: #()].
	self stackPush: (RBLiteralNode
		literalToken: (SmaCCToken value: object name)
		value: object).
! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/17/2003 10:15'!
pushTemp: tempIndex

	| var |
	var _ scope tempVarAt: tempIndex.
	self stackPush: (self newVar: var)! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/20/2003 18:17'!
remoteReturn

	stack pop.  "pop home context free var"
	self goto: #return.
! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/20/2003 17:26'!
returnTop

	self goto: #return.
! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/24/2003 15:45'!
send: selector

	| args rcvr |
	args _ OrderedCollection new.
	[	selector numArgs timesRepeat: [args addFirst: self Value].
		rcvr _ self Value.
	] on: Abort do: [
		self stackPush: (RBPseudoSendNode new selector: selector).
		^ self cascade
	].

	(rcvr isLiteral and: [selector beginsWith: #createBlockt:]) ifTrue: [
		^ self block: rcvr value capturedVars: args].
	(rcvr isLiteral and: [selector == #createBlockWithVars:]) ifTrue: [
		^ self block: rcvr value capturedVars: (
			args first "brace stream cascade" messages allButLast "braceArray"
				collect: [:m | m arguments first])
	].

	self stackPush: (self simplify: (RBMessageNode new
		receiver: rcvr
		selectorParts: (self newSelectorParts: selector)
		arguments: args)).
! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/24/2003 00:45'!
send: selector toSuperOf: behavior

	| args rcvr |
	args _ OrderedCollection new.
	selector numArgs timesRepeat: [args addFirst: self Value].
	rcvr _ self Value.
	(rcvr isVariable and: [rcvr name = 'self']) ifFalse: [self patternError].

	rcvr identifierToken: (SmaCCToken value: 'super').
	self stackPush: (RBMessageNode new
		receiver: rcvr
		selectorParts: (self newSelectorParts: selector)
		arguments: args).
! !

!IRDecompiler methodsFor: 'instructions' stamp: 'ajh 3/24/2003 15:55'!
storeTemp: tempIndex

	| var |
	var _ scope tempVarAt: tempIndex.
	self stackPush: (self simplifyTempAssign:
		(RBAssignmentNode variable: (self newVar: var) value: self Value)).
! !

!IRDecompiler methodsFor: 'priv instructions' stamp: 'ajh 3/21/2003 00:47'!
block: method capturedVars: varNodes

	| vars |
	vars _ varNodes collect: [:v |
		v isVariable ifTrue: [v binding]
		  ifFalse: [
			(v isMessage and: [v selector = #contextTag and: [v receiver isVariable]])
				ifFalse: [self patternError].
			v receiver binding]
	].
	self stackPush: (IRDecompiler new
		scope: (scope newBlockScope capturedVars: vars);
		decompileIR: method ir)
		asBlock! !

!IRDecompiler methodsFor: 'priv instructions' stamp: 'ajh 3/24/2003 01:54'!
cascade

	| messages selector args rcvr |
	messages _ OrderedCollection new.
	[	"last message"
		selector _ self Send selector.
		args _ OrderedCollection new.
		selector numArgs timesRepeat: [args addFirst: self Value].
		messages addFirst: selector -> args.

		"rest of messages"
		[(rcvr _ self ValueOrNone) isNil] whileTrue: [
			self Pop.
			selector _ self Send selector.
			args _ OrderedCollection new.
			selector numArgs timesRepeat: [args addFirst: self Value].
			self Dup.
			messages addFirst: selector -> args.
		].
	] on: Abort do: [^ false].

	messages _ messages collect: [:assoc |
		RBMessageNode
			receiver: rcvr
			selector: assoc key
			arguments: assoc value].
	self stackPush: (RBCascadeNode messages: messages).
! !

!IRDecompiler methodsFor: 'priv instructions' stamp: 'ajh 3/24/2003 13:29'!
endAndOr2: seqNum

	| goto seq p if2 test else o if1 seqValue elseTest |
	[
		goto _ self Goto.
		seqValue _ self ValueOrNone.
		seq _ self Sequence.
		p _ self Label destination.
		if2 _ self IfGoto: seqNum otherwise: p.
		elseTest _ self Value.
		else _ self SequenceBackTo: goto destination.
		o _ self Label destination.
		o = goto destination ifTrue: [self abort].
		if1 _ self IfGoto: seqNum otherwise: o.
		test _ self Value.
	] on: Abort do: [^ false].

	if1 boolean = if2 boolean ifFalse: [
		test _ RBMessageNode
			receiver: test selector: #not arguments: #().
	].
	self stackPush: (RBMessageNode
		receiver: test
		selector: (if2 boolean ifTrue: [#or:] ifFalse: [#and:])
		arguments: {self newBlock: (else addNode: elseTest) return: nil}).
	stack push: if2.
	self label: p.
	stack push: seq.
	seqValue ifNotNil: [stack push: seqValue].
	stack push: goto.
	^ true
! !

!IRDecompiler methodsFor: 'priv instructions' stamp: 'ajh 3/24/2003 12:39'!
endAndOr: seqNum

	| o test branches if body block sel1 sel2 if2 |
	branches _ OrderedCollection new.
	[
		(if2 _ self If) otherwise = seqNum ifFalse: [self abort].
		[	test _ self Value.
			body _ self Sequence.
			branches add: {body. test}.
			o _ self Label destination.
			(if _ self If) otherwise = o ifFalse: [self abort].
			if destination = seqNum
		] whileFalse: [
			if boolean = if2 boolean ifFalse: [self abort].
			if destination = if2 destination ifFalse: [self abort].
		].
		if boolean = if2 boolean ifTrue: [self abort].
		test _ self Value.
	] on: Abort do: [^ false].

	if boolean
		ifTrue: [sel1 _ #or:. sel2 _ #and:]
		ifFalse: [sel1 _ #and:. sel2 _ #or:].
	block _ self newBlock: (branches first first addNode: branches first second).
	branches allButFirstDo: [:pair |
		block _ self newBlock: (pair first addNode: (RBMessageNode
				receiver: pair second
				selector: sel2
				arguments: {block})).
	].
	self stackPush: (RBMessageNode
		receiver: test
		selector: sel1
		arguments: {block}).
	stack push: if2.
	^ true! !

!IRDecompiler methodsFor: 'priv instructions' stamp: 'ajh 3/24/2003 00:07'!
endCase: seqNum

	| otherwiseGoto goto node otherwiseValue otherwiseSeq n branchValue branchSeq f caseValue caseSeq rcvr branches message |
	branches _ OrderedCollection new.
	[	"otherwise"
		otherwiseGoto _ self GotoOrReturn: seqNum.
		node _ self stackDown.
		(node isPop or: [node isPseudoSend]) ifTrue: [
			node isPop ifTrue: [node _ self Send].
			node selector == #caseError ifFalse: [self abort].
		] ifFalse: [
			sp _ sp + 1.  "stackUp"
			otherwiseValue _ self ValueOrNone.
			otherwiseSeq _ self Sequence.
		].
		n _ self Label destination.

		"last case branch"
		goto _ self GotoOrReturn: seqNum.
		branchValue _ self ValueOrNone.
		branchSeq _ self Sequence.
		f _ self Label destination.
		"last case"
		self IfGoto: n otherwise: f.
		self Send selector == #= ifFalse: [self abort].
		caseValue _ self Value.
		caseSeq _ self Sequence.
		otherwiseSeq ifNil: [self Dup].
		branches addFirst: ({caseSeq. caseValue} -> {branchSeq. branchValue. goto}).

		[(rcvr _ self ValueOrNone) isNil] whileTrue: [
			"case branch"
			n _ self Label destination.
			goto _ self GotoOrReturn: seqNum.
			branchValue _ self ValueOrNone.
			branchSeq _ self Sequence.
			self Pop.
			f _ self Label destination.
			"case"
			self IfGoto: n otherwise: f.
			self Send selector == #= ifFalse: [self abort].
			caseValue _ self Value.
			caseSeq _ self Sequence.
			self Dup.
			branches addFirst: ({caseSeq. caseValue} -> {branchSeq. branchValue. goto}).
		].
	] on: Abort do: [^ false].

	branches _ branches collect: [:assoc |
		assoc key second
			ifNotNil: [assoc key first addNode: assoc key second].
		assoc value second
			ifNotNil: [assoc value first addNode: assoc value second].
		RBMessageNode
			receiver: (self newBlock: assoc key first return: nil)
			selector: #->
			arguments:
				{self newBlock: assoc value first return: assoc value third}
	].
	message _ otherwiseSeq
		ifNil: [
			RBMessageNode
				receiver: rcvr
				selector: #caseOf:
				arguments: {RBArrayNode statements: branches}]
		ifNotNil: [
			otherwiseValue
				ifNotNil: [otherwiseSeq addNode: otherwiseValue].
			RBMessageNode
				receiver: rcvr
				selector: #caseOf:otherwise:
				arguments: 
					{RBArrayNode statements: branches.
					self newBlock: otherwiseSeq return: otherwiseGoto}.
		].
	self stackPush: message.
	branchValue ifNil: [self popTop].
	self goto: seqNum.
	^ true! !

!IRDecompiler methodsFor: 'priv instructions' stamp: 'ajh 3/24/2003 02:39'!
endIfNil: seqNum

	| goto branch o if rcvr value |
	[
		goto _ self GotoOrReturn: seqNum.
		value _ self Value.
		branch _ self Sequence.
		self Pop.
		o _ self Label destination.
		if _ self IfGoto: seqNum otherwise: o.
		self Send selector == #== ifFalse: [self abort].
		(self Value isLiteral: [:v | v isNil]) ifFalse: [self abort].
		self Dup.
		rcvr _ self Value.
	] on: Abort do: [^ false].

	branch addNode: value.
	self stackPush: (RBMessageNode
		receiver: rcvr
		selector: (if boolean ifTrue: [#ifNotNil:] ifFalse: [#ifNil:])
		arguments: {self newBlock: branch return: goto}).
	self goto: seqNum.
	^ true! !

!IRDecompiler methodsFor: 'priv instructions' stamp: 'ajh 3/24/2003 01:46'!
endIfThen: seqNum

	| goto branch o if test value |
	[
		goto _ self Goto.
		(goto destination == seqNum or: [self isExplicitReturn: goto]) ifFalse: [self abort].
		goto isRet ifTrue: [value _ self Value].
		branch _ self Sequence.
		o _ self Label destination.
		if _ self IfGoto: seqNum otherwise: o.
		test _ self Value.
	] on: Abort do: [^ false].

	value ifNotNil: [branch addNode: value].
	self stackPush: (self simplify: (RBMessageNode
		receiver: test
		selector: (if boolean ifTrue: [#ifFalse:] ifFalse: [#ifTrue:])
		arguments: {self newBlock: branch return: goto})).
	self popTop.
	self goto: seqNum.
	^ true! !

!IRDecompiler methodsFor: 'priv instructions' stamp: 'ajh 3/24/2003 19:11'!
endIfThenElse: seqNum

	| goto2 else d goto1 then o if test value2 value1 |
	[
		goto2 _ self Goto.
		value2 _ self ValueOrNone.
		else _ self Sequence.
		d _ self Label destination.
		goto1 _ self Goto.
		((self isExplicitReturn: goto2) or: [goto2 destination == goto1 destination]) ifFalse: [self abort].
		value1 _ self ValueOrNone.
		then _ self Sequence.
		o _ self Label destination.
		if _ self IfGoto: d otherwise: o.
		test _ self Value.
	] on: Abort do: [^ false].

	value2 ifNotNil: [else addNode: value2].
	value1 ifNotNil: [then addNode: value1].
	self stackPush: (self simplify: (else statements isEmpty
		ifTrue: [RBMessageNode
			receiver: test
			selector: (if boolean ifTrue: [#ifFalse:] ifFalse: [#ifTrue:])
			arguments: {self newBlock: then return: goto1}]
		ifFalse: [RBMessageNode
			receiver: test
			selector: (if boolean
				ifTrue: [#ifFalse:ifTrue:]
				ifFalse: [#ifTrue:ifFalse:])
			arguments: {
				self newBlock: then return: goto1.
				self newBlock: else return: goto2}])).
	value1 ifNil: [self popTop].
	self goto: goto1 destination.
	(else statements isEmpty and:
	 [stack anySatisfy: [:n | n isIf and: [n destination = d]]]
	) ifTrue: [
		self label: d.
		self goto: goto2 destination.
	].
	^ true! !

!IRDecompiler methodsFor: 'priv instructions' stamp: 'ajh 3/24/2003 02:04'!
endToDo: seqNum

	| start limit incr iter step loopBlock o if test limitExpr init |
	[
		start _ self Goto destination.
		limit _ self Value.
		incr _ self Assignment.
		iter _ incr variable.
		(incr value isMessage and:
		 [incr value selector == #+ and:
		  [incr value receiver isVariable and: 
		   [incr value receiver binding == iter binding]]]
		) ifFalse: [self abort].
		step _ incr value arguments first.
		loopBlock _ self Sequence.
		o _ self Label destination.
		if _ self IfGoto: seqNum otherwise: o.
		test _ self Value.
		(test isMessage and:
		 [(test selector == #<= or: [test selector == #>=]) and:
		  [(valueLabelMap at: test arguments first ifAbsent: [self abort]) destination = start]]
		) ifFalse: [self abort].
		limitExpr _ test arguments first.
		limitExpr isAssignment ifTrue: [
			limitExpr variable binding == limit binding ifFalse: [self abort].
			limitExpr _ limitExpr value.
		].
		init _ test receiver.
		(init isAssignment and: [init variable binding == iter binding])
			ifFalse: [self abort].
	] on: Abort do: [^ false].

	loopBlock _ self newBlock: loopBlock return: nil.
	loopBlock arguments: {iter}.
	self stackPush: ((step isLiteral: [:c | c = 1])
		ifTrue: [RBMessageNode
				receiver: init value
				selector: #to:do:
				arguments: {limitExpr. loopBlock}]
		ifFalse: [RBMessageNode
				receiver: init value
				selector: #to:by:do:
				arguments: {limitExpr. step. loopBlock}]).
	self popTop.
	self goto: seqNum.
	^ true
! !

!IRDecompiler methodsFor: 'priv instructions' stamp: 'ajh 3/24/2003 02:18'!
endWhile: seqNum

	| start loopBlock if test sequence o |
	[
		start _ self Goto destination.
		loopBlock _ self Sequence.
		o _ self Label destination.
		if _ self IfGoto: seqNum otherwise: o.
		test _ self Value.
		sequence _ self SequenceBackTo: start.
		self Label: start.
		sp _ sp + 1.  "stackUp"
	] on: Abort do: [^ false].

	self stackPush: (self simplify: (RBMessageNode
		receiver: (self newBlock: (sequence addNode: test) return: nil)
		selector: (if boolean ifTrue: [#whileFalse:] ifFalse: [#whileTrue:])
		arguments: {self newBlock: loopBlock return: nil})).
	self popTop.
	self goto: seqNum.
	^ true
! !

!IRDecompiler methodsFor: 'interpret' stamp: 'ajh 3/13/2003 13:48'!
interpretInstruction: irInstruction

	currentInstr _ irInstruction.
	super interpretInstruction: irInstruction.
! !

!IRDecompiler methodsFor: 'interpret' stamp: 'ajh 3/20/2003 23:37'!
interpretSequence: instructionSequence

	super interpretSequence: instructionSequence.
	currentInstr _ nil.
! !

!IRDecompiler methodsFor: 'private' stamp: 'ajh 3/23/2003 21:17'!
initialize

	stack _ OrderedCollection new.
	scope _ nil parseScope newMethodScope.  "in case never set"
	valueLabelMap _ IdentityDictionary new.
! !

!IRDecompiler methodsFor: 'private' stamp: 'ajh 3/22/2003 11:03'!
isExplicitReturn: goto

	^ goto isRet and:
	  [goto mapInstr notNil and:
	   [goto mapInstr isRemote or: [scope isBlockScope not]]]! !

!IRDecompiler methodsFor: 'private' stamp: 'ajh 3/20/2003 23:21'!
mapNode: node

	currentInstr ifNil: [^ self].
	node isPseudo
		ifTrue: [node mapInstr: currentInstr]
		ifFalse: [currentInstr sourceNode: node]! !

!IRDecompiler methodsFor: 'private' stamp: 'ajh 3/24/2003 12:07'!
newBlock: sequence

	^ self newBlock: sequence return: nil! !

!IRDecompiler methodsFor: 'private' stamp: 'ajh 3/24/2003 01:24'!
newBlock: sequence return: goto

	| statements ret |
	statements _ sequence statements.
	(goto notNil and: [self isExplicitReturn: goto]) ifTrue: [
		ret _ RBReturnNode value: statements last.
		goto mapInstr sourceNode: ret.
		statements atLast: 1 put: ret.
	].
	sequence statements: statements.
	^ RBBlockNode body: sequence! !

!IRDecompiler methodsFor: 'private' stamp: 'ajh 3/13/2003 14:10'!
newSelectorParts: selector

	^ selector keywords collect: [:word |
		SmaCCToken value: word]! !

!IRDecompiler methodsFor: 'private' stamp: 'ajh 3/17/2003 10:14'!
newVar: semVar

	^ RBVariableNode new
		identifierToken: (SmaCCToken value: semVar name);
		binding: semVar! !

!IRDecompiler methodsFor: 'private' stamp: 'ajh 3/24/2003 15:54'!
simplify: mess
	"mess is a messageNode.  If it is a message created by the compiler convert it back to its normal form"

	| rcvr sc |
	(mess selector == #poolValue and: [mess receiver isLiteral]) ifTrue: [
		^ self newVar: (SemPoolVar newFor: mess receiver value)
	].

	(mess selector = #setInPool: and: [mess arguments first isLiteral]) ifTrue: [
		^ RBAssignmentNode
			variable: (self newVar: (SemPoolVar newFor: mess arguments first value))
			value: mess receiver
	].

	(mess selector = #getInstVar: and:
	 [mess arguments first isLiteral and:
	  [mess receiver isVariable]]) ifTrue: [
		rcvr _ mess receiver binding.
		rcvr == scope receiverVar ifTrue: [
			^ self newVar: (scope receiverVarAt: mess arguments first value)].
		(rcvr isFieldVar and: [rcvr originalTempVar notNil]) ifTrue: [
			sc _ rcvr originalTempVar scope.
			rcvr = scope receiverVar ifTrue: [
				^ self newVar: (sc receiverVarAt: mess arguments first value)]].
	].

	(mess selector = #storeIn:instVar: and:
	 [mess arguments last isLiteral and:
	  [mess arguments first isVariable]]) ifTrue: [
		rcvr _ mess arguments first binding.
		rcvr == scope receiverVar ifTrue: [
			^ RBAssignmentNode
				variable: (self newVar: (scope receiverVarAt: mess arguments last value))
				value: mess receiver
		].
		(rcvr isFieldVar and: [rcvr originalTempVar notNil]) ifTrue: [
			sc _ rcvr originalTempVar scope.
			rcvr = scope receiverVar ifTrue: [
				^ RBAssignmentNode
					variable: (self newVar: (sc receiverVarAt: mess arguments last value))
					value: mess receiver]
		].
	].

	(mess selector = #tempHolderValue and: [mess receiver isVariable]) ifTrue: [
		^ mess receiver
	].

	(mess selector = #storeInTempHolder: and: [mess arguments first isVariable]) ifTrue: [
		^ RBAssignmentNode
			variable: mess arguments first
			value: mess receiver
	].

	^ mess! !

!IRDecompiler methodsFor: 'private' stamp: 'ajh 3/24/2003 16:01'!
simplifyTempAssign: assignment
	"If it is a assignment created by the compiler convert it back to its normal form"

	| mess |
	((mess _ assignment value) isMessage and: 
	 [mess selector = #wrapInTempHolder and:
	  [mess receiver isLiteral: [:v | v isNil]]]
	) ifTrue: [
		^ nil  "no-op"
	].

	^ assignment! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/22/2003 17:03'!
Assignment

	| node |
	(node _ self stackDown) isAssignment ifTrue: [^ node].
	self abort! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/23/2003 22:34'!
Dup

	| node |
	(node _ self stackDown) isDup ifTrue: [^ node].
	self abort! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/22/2003 16:33'!
Goto

	| node |
	(node _ self stackDown) isGoto ifTrue: [^ node].
	self abort! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/23/2003 12:57'!
Goto: seqNum

	| goto |
	(goto _ self Goto) destination = seqNum ifTrue: [^ goto].
	self abort! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/23/2003 23:17'!
GotoOrReturn: seqNum

	| goto |
	goto _ self Goto.
	(goto destination = seqNum or: [goto isRet]) ifTrue: [^ goto].
	self abort! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/22/2003 16:33'!
If

	| node |
	(node _ self stackDown) isIf ifTrue: [^ node].
	self abort! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/22/2003 17:07'!
IfGoto: seqNum otherwise: seqNum2

	| if |
	((if _ self If) destination = seqNum and: [if otherwise = seqNum2])
		ifTrue: [^ if].
	self abort! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/22/2003 16:33'!
Label

	| node |
	(node _ self stackDown) isLabel ifTrue: [^ node].
	self abort! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/22/2003 16:34'!
Label: seqNum

	| label |
	(label _ self Label) destination = seqNum ifTrue: [^ label].
	self abort! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/22/2003 17:36'!
Pop

	| node |
	(node _ self stackDown) isPop ifTrue: [^ node].
	self abort! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/23/2003 22:32'!
Send

	| node |
	(node _ self stackDown) isPseudoSend ifTrue: [^ node].
	self abort! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/24/2003 01:08'!
Sequence

	| node seq i goto |
	seq _ RBSequenceNode statements: #().
	i _ sp ifNil: [sp _ stack size].
	[	node _ stack at: i.
		node isSequence ifTrue: [
			seq addNodesFirst: node statements.
			node _ stack at: (i _ i - 1).
		].
		"include sequences before fall-through gotos"
		(node isLabel and: [i > 1 "not first"]) ifFalse: [
			sp _ i.
			^ seq].
		goto _ stack at: (i _ i - 1).
		goto isGoto and: [goto destination = node destination]
	] whileTrue: [i _ i - 1].
	sp _ i + 1.  "points to label"
	^ seq! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/24/2003 01:08'!
SequenceBackTo: labelNum
	"Return previous sequence stopping at non-sequence or label: num, whichever comes first"

	| node seq i goto |
	seq _ RBSequenceNode statements: #().
	i _ sp ifNil: [sp _ stack size].
	[	node _ stack at: i.
		node isSequence ifTrue: [
			seq addNodesFirst: node statements.
			node _ stack at: (i _ i - 1).
		].
		"includes sequences back to labelNum"
		(node isLabel and: [i > 1 "not first"]) ifFalse: [
			sp _ i.
			^ seq].
		node destination = labelNum ifTrue: [
			sp _ i.
			^ seq].
		goto _ stack at: (i _ i - 1).
		goto isGoto and: [goto destination = node destination]
	] whileTrue: [i _ i - 1].
	sp _ i + 1.  "points to label"
	^ seq! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/23/2003 21:44'!
Value

	| node |
	node _ self ValueOrNone.
	node ifNil: [self abort].
	^ node! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/23/2003 21:43'!
ValueOrNone

	| node i label |
	i _ sp ifNil: [sp _ stack size].
	[	node _ stack at: i.
		node isValue ifTrue: [
			label ifNotNil: [valueLabelMap at: node put: label].
			sp _ i - 1.
			^ node].
		"test for value before fall-through goto"
		(node isLabel and: [i > 1 "not first"]) ifFalse: [^ nil].
		label _ node.
		node _ stack at: (i _ i - 1).
		node isGoto and: [node destination = label destination]
	] whileTrue: [i _ i - 1].
	^ nil! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/24/2003 00:59'!
abort

	| spWas |
	spWas _ sp.
	sp _ nil.
	Abort signal! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/24/2003 01:14'!
fixStack

	sp ifNotNil: [stack popTo: sp].
	sp _ nil.
! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/22/2003 19:24'!
stackDown

	| node |
	sp ifNil: [sp _ stack size].
	sp = 0 ifTrue: [self abort].
	node _ stack at: sp.
	sp _ sp - 1.
	^ node! !

!IRDecompiler methodsFor: 'stack' stamp: 'ajh 3/24/2003 16:00'!
stackPush: node

	self fixStack.
	stack push: node.
	node ifNil: [^ self].  "no op"
	self mapNode: node.
! !


!IRDecompiler class methodsFor: 'as yet unclassified' stamp: 'ajh 3/3/2003 12:26'!
new

	^ super new initialize! !


!IRJump methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 23:08'!
destination

	^ destination! !

!IRJump methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 23:08'!
destination: sequence

	destination _ sequence! !

!IRJump methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:47'!
executeOn: interpreter

	^ interpreter goto: destination orderNumber! !

!IRJump methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 00:19'!
isGoto
	"is unconditional jump"

	^ true! !

!IRJump methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 00:22'!
isJump
	"goto or if"

	^ true! !

!IRJump methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 00:02'!
successorSequences

	^ {destination}! !


!IRJumpIf methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:43'!
boolean

	^ boolean! !

!IRJumpIf methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:43'!
boolean: bool

	boolean _ bool! !

!IRJumpIf methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:47'!
executeOn: interpreter

	^ interpreter if: boolean goto: destination orderNumber otherwise: otherwise orderNumber! !

!IRJumpIf methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 00:19'!
isGoto
	"is unconditional jump"

	^ false! !

!IRJumpIf methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 00:21'!
isIf

	^ true! !

!IRJumpIf methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:43'!
otherwise

	^ otherwise! !

!IRJumpIf methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:44'!
otherwise: sequence

	otherwise _ sequence! !

!IRJumpIf methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 00:02'!
successorSequences

	^ {destination. otherwise}! !


!IRMethod methodsFor: 'initialize' stamp: 'ajh 3/17/2003 11:07'!
initialize

	primitiveNode _ PrimitiveNode null.
	rargKeys _ #(self).
	otherTempKeys _ #().
! !

!IRMethod methodsFor: 'initialize' stamp: 'ajh 3/17/2003 11:04'!
otherTempKeys: tempKeys

	otherTempKeys _ tempKeys! !

!IRMethod methodsFor: 'initialize' stamp: 'ajh 3/10/2003 23:08'!
primitiveNode: aPrimitiveNode

	primitiveNode _ aPrimitiveNode! !

!IRMethod methodsFor: 'initialize' stamp: 'ajh 3/17/2003 11:12'!
rargKeys: tempKeys

	rargKeys _ tempKeys! !

!IRMethod methodsFor: 'initialize' stamp: 'ajh 3/10/2003 17:52'!
startSequence: irSequence

	startSequence _ irSequence! !

!IRMethod methodsFor: 'accessing' stamp: 'ajh 3/9/2003 15:35'!
allSequences

	^ startSequence withAllSuccessors! !

!IRMethod methodsFor: 'accessing' stamp: 'ajh 3/17/2003 11:11'!
numArgs

	^ rargKeys size - 1! !

!IRMethod methodsFor: 'accessing' stamp: 'ajh 3/17/2003 11:07'!
otherTempKeys

	^ otherTempKeys! !

!IRMethod methodsFor: 'accessing' stamp: 'ajh 3/10/2003 18:10'!
primitiveNode

	^ primitiveNode! !

!IRMethod methodsFor: 'accessing' stamp: 'ajh 3/17/2003 11:13'!
rargKeys

	^ rargKeys! !

!IRMethod methodsFor: 'accessing' stamp: 'ajh 3/10/2003 17:53'!
startSequence

	^ startSequence! !

!IRMethod methodsFor: 'accessing' stamp: 'ajh 3/17/2003 11:07'!
tempKeys

	^ rargKeys allButFirst, otherTempKeys! !

!IRMethod methodsFor: 'printing' stamp: 'ajh 3/9/2003 15:53'!
longPrintOn: stream

	IRPrinter new
		indent: 0;
		stream: stream;
		interpret: self! !

!IRMethod methodsFor: 'optimizing' stamp: 'ajh 3/10/2003 15:45'!
absorbConstantConditionalJumps

	startSequence absorbConstantConditionalJumps: IdentitySet new! !

!IRMethod methodsFor: 'optimizing' stamp: 'ajh 3/10/2003 15:45'!
absorbJumpsToSingleInstrs

	startSequence absorbJumpToSingleInstr: IdentitySet new! !

!IRMethod methodsFor: 'optimizing' stamp: 'ajh 3/18/2003 19:05'!
optimize

	self removeEmptyStart.
	self absorbJumpsToSingleInstrs.  "do before next to get in right form"
	self absorbConstantConditionalJumps.
	self absorbJumpsToSingleInstrs.  "do again since new opportunities may have arised after last step" 
! !

!IRMethod methodsFor: 'optimizing' stamp: 'ajh 3/18/2003 19:25'!
removeEmptyStart

	startSequence size = 1 ifTrue: [
		"startSeq is just unconditional jump, forget it"
		startSequence _ startSequence last destination].
! !

!IRMethod methodsFor: 'translating' stamp: 'ajh 3/10/2003 15:55'!
compiledMethod

	^ compiledMethod ifNil: [self compiledMethodWith: #(0)]! !

!IRMethod methodsFor: 'translating' stamp: 'ajh 3/15/2003 02:02'!
compiledMethodWith: trailer

	^ compiledMethod _ IRTranslator new
		interpret: self;
		compiledMethodWith: trailer! !

!IRMethod methodsFor: 'translating' stamp: 'ajh 3/10/2003 15:54'!
privCompiledMethod: aCompiledMethod

	compiledMethod _ aCompiledMethod! !

!IRMethod methodsFor: 'mapping' stamp: 'ajh 3/19/2003 13:38'!
sourceMap
	"Return a mapping from bytecode pcs to source code ranges"

	| start map |
	"Besides getting start position, make sure bytecodeIndices are filled in"
	start _ self compiledMethod initialPC - 1.  
	map _ OrderedCollection new.
	self allSequences do: [:seq |
		seq do: [:instr |  | node |
			((node _ instr sourceNode) notNil and: 
			 [node debugHighlightStart notNil and:
			  [node debugHighlightStop notNil and:
			   [instr bytecodeIndex notNil]]]) ifTrue: [
				map add:
					instr bytecodeIndex + start
						-> (node debugHighlightStart to: node debugHighlightStop)]
		]
	].
	^ map! !

!IRMethod methodsFor: 'decompiling' stamp: 'ajh 3/15/2003 00:48'!
methodNode

	^ IRDecompiler new decompileIR: self! !


!IRMethod class methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 23:17'!
new

	^ super new initialize! !


!IRPop methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:47'!
executeOn: interpreter

	^ interpreter popTop! !


!IRPrinter methodsFor: 'initialize' stamp: 'ajh 3/9/2003 15:49'!
indent: tabs

	indent _ tabs! !

!IRPrinter methodsFor: 'initialize' stamp: 'ajh 3/9/2003 15:50'!
stream: stringWriteStream

	stream _ stringWriteStream! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 15:41'!
goto: seqNum

	stream nextPutAll: 'goto: '.
	seqNum printOn: stream.
! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 15:42'!
if: bool goto: seqNum1 otherwise: seqNum2

	stream nextPutAll: 'if: '.
	bool printOn: stream.
	stream nextPutAll: ' goto: '.
	seqNum1 printOn: stream.
	stream nextPutAll: ' else: '.
	seqNum2 printOn: stream.
! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/11/2003 00:36'!
label: seqNum

	"add tab and cr since this does not get called within interpretInstruction:"
	stream cr.  "extra cr just to space out sequences"
	indent timesRepeat: [stream tab].
	stream nextPutAll: 'label: '.
	seqNum printOn: stream.
	stream cr.
! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 15:42'!
popTop

	stream nextPutAll: 'popTop'! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 15:42'!
pushDup

	stream nextPutAll: 'pushDup'! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/21/2003 01:49'!
pushLiteral: object

	stream nextPutAll: 'pushLiteral: '.
	object isVariableBinding ifTrue: [^ stream nextPutAll: object key].
	object printOn: stream.

	((object isKindOf: BlockClosure) or: [object isKindOf: CompiledMethod])
		ifTrue: [
			IRPrinter new
				indent: indent + 1;
				stream: stream;
				interpret: object method ir removeEmptyStart].
! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/10/2003 14:12'!
pushTemp: index

	stream nextPutAll: 'pushTemp: '.
	index printOn: stream.
	index = 0 ifTrue: [stream nextPutAll: ' "receiver"'].
	index = -1 ifTrue: [stream nextPutAll: ' "thisContext"'].
! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 15:43'!
remoteReturn

	stream nextPutAll: 'remoteReturn'.
! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 15:43'!
returnTop

	stream nextPutAll: 'returnTop'.
! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 15:44'!
send: selector

	stream nextPutAll: 'send: '.
	selector printOn: stream.
! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 15:44'!
send: selector toSuperOf: behavior

	stream nextPutAll: 'send: '.
	selector printOn: stream.
	stream nextPutAll: ' toSuperOf: '.
	behavior printOn: stream.
! !

!IRPrinter methodsFor: 'instructions' stamp: 'ajh 3/9/2003 15:44'!
storeTemp: index

	stream nextPutAll: 'storeTemp: '.
	index printOn: stream.
! !

!IRPrinter methodsFor: 'interpret' stamp: 'ajh 3/9/2003 15:48'!
interpretInstruction: irInstruction

	indent timesRepeat: [stream tab].
	super interpretInstruction: irInstruction.
	stream cr.
! !


!IRReturn methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:48'!
executeOn: interpreter

	^ isRemote
		ifTrue: [interpreter remoteReturn]
		ifFalse: [interpreter returnTop]! !

!IRReturn methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:44'!
isRemote

	^ isRemote! !

!IRReturn methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:44'!
isRemote: boolean

	isRemote _ boolean! !

!IRReturn methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 16:10'!
isReturn

	^ true! !


!IRSend methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:48'!
executeOn: interpreter

	^ superOf
		ifNil: [interpreter send: selector]
		ifNotNil: [interpreter send: selector toSuperOf: superOf]! !

!IRSend methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:44'!
selector

	^ selector! !

!IRSend methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:44'!
selector: symbol

	selector _ symbol! !

!IRSend methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:45'!
superOf

	^ superOf! !

!IRSend methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:45'!
superOf: behavior

	superOf _ behavior! !


!IRSequence methodsFor: 'comparing' stamp: 'ajh 3/11/2003 00:29'!
= other
	"Override collection equal with identity equal"

	^ self == other! !

!IRSequence methodsFor: 'comparing' stamp: 'ajh 6/18/2002 15:09'!
hash

	^ self identityHash! !

!IRSequence methodsFor: 'successor sequences' stamp: 'ajh 3/19/2003 22:22'!
nextSequence

	| sequences i |
	sequences _ self withAllSuccessors.
	i _ sequences detectIndex: [:seq | seq orderNumber = self orderNumber].
	i = sequences size ifTrue: [^ nil].
	^ sequences at: i + 1! !

!IRSequence methodsFor: 'successor sequences' stamp: 'ajh 3/19/2003 01:24'!
successorSequences

	self isEmpty ifTrue: [^ #()].
	^ self last successorSequences! !

!IRSequence methodsFor: 'successor sequences' stamp: 'ajh 9/26/2002 16:14'!
withAllSuccessors
	"Return me and all my successors sorted by sequence orderNumber"

	| list |
	list _ OrderedCollection new: 20.
	self withAllSuccessorsDo: [:seq | list add: seq].
	^ list asSortedCollection: [:x :y | x orderNumber <= y orderNumber]! !

!IRSequence methodsFor: 'successor sequences' stamp: 'ajh 7/18/2002 01:37'!
withAllSuccessorsDo: block
	"Iterate over me and all my successors only once"

	self withAllSuccessorsDo: block alreadySeen: IdentitySet new! !

!IRSequence methodsFor: 'successor sequences' stamp: 'ajh 3/6/2003 01:31'!
withAllSuccessorsDo: block alreadySeen: set
	"Iterate over me and all my successors only once"

	(set includes: self) ifTrue: [^ self].
	set add: self.
	block value: self.
	self successorSequences do: [:seq |
		seq ifNotNil: [seq withAllSuccessorsDo: block alreadySeen: set]].
! !

!IRSequence methodsFor: 'optimizing' stamp: 'ajh 3/13/2003 00:26'!
absorbConstantConditionalJumps: alreadySeen
	"Collapse sequences that look like:
			[if] goto s1
			...
		 s1:	pushConst: true/false
			goto s2
		 s2:	if true/false goto s3 else s4
	into:
			[if] goto s3/s4

	These sequences are produced by and:/or: messages"

	| seq bool if |
	(alreadySeen includes: self) ifTrue: [^ self].
	alreadySeen add: self.

	[(seq _ self successorSequences) size > 0  "not return"
	  and: [(seq _ seq first "destination") size = 2
	   and: [(seq first isConstant: [:obj | (bool _ obj) isKindOf: Boolean])
	    and: [seq last isGoto
	     and: [(if _ seq last destination first) isIf]]]]
	] whileTrue: [ "absorb"
		self last destination: (bool == if boolean
			ifTrue: [if destination]
			ifFalse: [if otherwise]).
	].

	self successorSequences do: [:instrs | instrs absorbConstantConditionalJumps: alreadySeen].
! !

!IRSequence methodsFor: 'optimizing' stamp: 'ajh 3/20/2003 00:10'!
absorbJumpToSingleInstr: alreadySeen
	"Collapse jumps to single return instructions into caller"

	| seqs seq |
	(alreadySeen includes: self) ifTrue: [^ self].
	alreadySeen add: self.

	[ (seqs _ self successorSequences) size = 1  "unconditional jump..." 
	   and: [(seq _ seqs first) size = 1  "...to single instruction..."
	    and: [seq successorSequences size < 2]]  "...but don't collapse conditional jumps so their otherwiseSequences can stay right after them"
	] whileTrue: [ "replace goto with single instruction"
		self removeLast.
		seq do: [:instr | self add: instr copy].
	].

	seqs do: [:instrs | instrs absorbJumpToSingleInstr: alreadySeen].
! !

!IRSequence methodsFor: 'printing' stamp: 'ajh 3/11/2003 00:57'!
longPrintOn: stream

	[IRPrinter new
		indent: 0;
		stream: stream;
		interpretSequence: self
	] onDNU: #orderNumber do: [:ex | ex resume: ex receiver]! !

!IRSequence methodsFor: 'printing' stamp: 'ajh 3/11/2003 00:25'!
printOn: stream

	stream nextPutAll: 'an '.
	self class printOn: stream.
	stream space.
	stream nextPut: $(.
	self orderNumber printOn: stream.
	stream nextPut: $).
! !

!IRSequence methodsFor: 'manipulating' stamp: 'ajh 3/19/2003 22:25'!
splitAfter: instruction

	| newSeq instr next |
	next _ self nextSequence.
	next _ next
		ifNil: [self orderNumber + 1]
		ifNotNil: [(next orderNumber + self orderNumber) / 2].
	newSeq _ IRSequence new orderNumber: next.
	instr _ instruction.
	[(instr _ instr nextLink) isNil] whileFalse: [newSeq add: instr].
	instruction nextLink: nil.
	lastLink _ instruction.
	self add: (IRJump new destination: newSeq).
	^ newSeq! !

!IRSequence methodsFor: 'initializing' stamp: 'ajh 7/18/2002 01:23'!
orderNumber: n
	"Sequences are sorted by this number"

	orderNumber _ n! !

!IRSequence methodsFor: 'accessing' stamp: 'ajh 7/18/2002 01:23'!
orderNumber
	"Sequences are sorted by this number"

	^ orderNumber! !


!IRTemp methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 12:18'!
executeOn: interpreter

	^ isStore
		ifTrue: [interpreter storeTemp: number]
		ifFalse: [interpreter pushTemp: number]! !

!IRTemp methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:45'!
isStore

	^ isStore! !

!IRTemp methodsFor: 'as yet unclassified' stamp: 'ajh 3/10/2003 00:45'!
isStore: boolean

	isStore _ boolean! !

!IRTemp methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 12:17'!
number

	^ number! !

!IRTemp methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 12:16'!
number: n

	number _ n! !


!IRTranslator methodsFor: 'initialize' stamp: 'ajh 3/9/2003 22:02'!
initialize

	gen _ BytecodeGenerator new.
! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/9/2003 22:09'!
goto: seqNum

	self doPending.
	gen goto: seqNum.
! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/9/2003 22:09'!
if: bool goto: seqNum1 otherwise: seqNum2

	self doPending.
	gen if: bool goto: seqNum1 otherwise: seqNum2.
! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/9/2003 22:10'!
label: seqNum

	pending _ OrderedCollection new.
	gen label: seqNum.
! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/9/2003 22:35'!
popTop

	"if last was storeTemp or storeInstVar then convert to storePopTemp or storePopInstVar"
	#storeTemp: == self pendingSelector ifTrue: [
		^ self pendingSelector: #storePopTemp:].
	#storeInstVar: == self pendingSelector ifTrue: [
		^ self pendingSelector: #storePopInstVar:].

	"otherwise do normal pop"
	self doPending.
	gen popTop.
! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/9/2003 22:10'!
pushDup

	self doPending.
	gen pushDup.
! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/12/2003 11:58'!
pushLiteral: object

	self addPending: (Message selector: #pushLiteral: argument: object)! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/9/2003 23:39'!
pushTemp: index

	index = 0 ifTrue: [
		^ self addPending: (Message selector: #pushReceiver)].
	(self pendingMatches: {
		[:m | m selector == #storePopTemp: and: [m argument = index]]}
	) ifTrue: [
		^ self pendingSelector: #storeTemp:].
	self doPending.
	index = -1 ifTrue: [^ gen pushThisContext].
	gen pushTemp: index.
! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/9/2003 22:12'!
remoteReturn

	self doPending.
	gen remoteReturn.
! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/10/2003 16:27'!
returnTop

	#pushReceiver == self pendingSelector ifTrue: [
		self pendingSelector: #returnReceiver.
		^ self doPending
	].
	#pushLiteral: == self pendingSelector ifTrue: [
		self pendingSelector: #returnConstant:.
		^ self doPending
	].
	#pushInstVar: == self pendingSelector ifTrue: [
		self pendingSelector: #returnInstVar:.
		^ self doPending
	].
	self doPending.
	gen returnTop.
! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/13/2003 04:26'!
send: selector

	"If get/set inst var, access it directly"
	| index |
	((#(getInstVar: #storeIn:instVar:) identityIncludes: selector) and:
	 [self pendingMatches: {
		[:m | m selector == #pushReceiver].
		[:m | m selector == #pushLiteral: and: [m argument isInteger]]}]
	) ifTrue: [
		index _ self popPending argument.
		self popPending.  "pop pushReceiver"
		self addPending: (Message
			selector: (selector == #getInstVar:
				ifTrue: [#pushInstVar:] ifFalse: [#storeInstVar:])
			argument: index).
		(self pendingMatches: {
			[:m | m selector == #storePopInstVar: and: [m argument = index]].
			[:m | m selector == #pushInstVar: and: [m argument = index]]}
		) ifTrue: [
			self popPending.
			self pendingSelector: #storeInstVar:.
		].
		^ self
	].

	"otherwise do normal send"
	self doPending.
	gen send: selector.
! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/9/2003 22:14'!
send: selector toSuperOf: behavior

	self doPending.
	gen send: selector toSuperOf: behavior.
! !

!IRTranslator methodsFor: 'instructions' stamp: 'ajh 3/12/2003 11:58'!
storeTemp: index

	self addPending: (Message selector: #storeTemp: argument: index)! !

!IRTranslator methodsFor: 'interpret' stamp: 'ajh 3/15/2003 02:02'!
interpret: ir

	ir optimize.
	gen primitiveNode: ir primitiveNode.
	gen numArgs: ir numArgs.
	super interpret: ir.
! !

!IRTranslator methodsFor: 'interpret' stamp: 'ajh 3/13/2003 18:07'!
interpretAll: irSequences

	irSequences withIndexDo: [:seq :i | seq orderNumber: i].
	super interpretAll: irSequences.
! !

!IRTranslator methodsFor: 'interpret' stamp: 'ajh 3/13/2003 04:50'!
interpretInstruction: irInstruction

	currentInstr _ irInstruction.
	super interpretInstruction: irInstruction.
! !

!IRTranslator methodsFor: 'priv pending' stamp: 'ajh 3/13/2003 04:30'!
addPending: message

	pending push: currentInstr -> message! !

!IRTranslator methodsFor: 'priv pending' stamp: 'ajh 3/13/2003 04:49'!
doPending
	"execute pending instructions"

	| assoc |
	[pending isEmpty] whileFalse: [
		assoc _ pending removeFirst.
		gen mapBytesTo: assoc key "instr".
		assoc value "message" sendTo: gen.
	].
	gen mapBytesTo: currentInstr.
! !

!IRTranslator methodsFor: 'priv pending' stamp: 'ajh 3/13/2003 04:32'!
pendingMatches: blocks

	^ (pending collect: [:assoc | assoc value "message"])
		endSatisfies: blocks! !

!IRTranslator methodsFor: 'priv pending' stamp: 'ajh 3/13/2003 04:33'!
pendingSelector

	pending isEmpty ifTrue: [^ nil].
	^ pending top value "message" selector! !

!IRTranslator methodsFor: 'priv pending' stamp: 'ajh 3/13/2003 04:33'!
pendingSelector: selector

	pending top value "message" setSelector: selector! !

!IRTranslator methodsFor: 'priv pending' stamp: 'ajh 3/13/2003 04:34'!
popPending

	^ pending pop value "message"! !

!IRTranslator methodsFor: 'results' stamp: 'ajh 3/13/2003 18:28'!
compiledMethodWith: trailer

	^ gen compiledMethodWith: trailer! !


!IRTranslator class methodsFor: 'instance creation' stamp: 'ajh 3/9/2003 22:02'!
new

	^ super new initialize! !


!InstructionStream methodsFor: 'decoding' stamp: 'ajh 3/2/2003 14:06'!
interpret

	[self atEnd] whileFalse: [self interpretNextInstructionFor: self]! !

!InstructionStream methodsFor: 'scanning' stamp: 'ajh 5/30/2003 01:34'!
previousPc

	| currentPc dummy prevPc |
	currentPc _ pc.
	pc _ self method initialPC.
	dummy _ Message catcher.
	[pc = currentPc] whileFalse: [
		prevPc _ pc.
		self interpretNextInstructionFor: dummy.
	].
	^ prevPc! !


!BlockContext methodsFor: 'private-debugger' stamp: 'ajh 3/18/2003 17:15'!
doItScope

	^ self home doItScope! !


!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 03:41'!
blockReturnTop

	irBuilder returnTop! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/10/2003 14:53'!
doDup

	irBuilder pushDup! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/10/2003 14:55'!
doPop

	irBuilder popTop! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/13/2003 13:19'!
interpretNextInstructionFor: client

	| byteIndex |
	byteIndex _ pc - self method initialPC + 1.
	irBuilder mapToByteIndex: byteIndex.
	irBuilder testJumpAheadTarget: byteIndex.
	super interpretNextInstructionFor: client.
! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/19/2003 01:26'!
jump: dist

	| index seq instr newSeq |
	index _ pc + dist - self method initialPC + 1.
	dist >= 0 ifTrue: [
		"jump forward"
		^ irBuilder jumpAheadTo: index].

	"jump backward"
	seq _ irBuilder ir allSequences
		reverseDetect: [:s | s notEmpty and: [s first bytecodeIndex <= index]].
	seq first bytecodeIndex = index ifTrue: [
		newSeq _ seq.
	] ifFalse: [
		instr _ seq detect: [:i | i nextLink bytecodeIndex = index].
		newSeq _ seq splitAfter: instr.
	].
	irBuilder addJumpBackTarget: index to: newSeq.
	irBuilder jumpBackTo: index.
! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 13:59'!
jump: dist if: bool

	| index |
	index _ pc + dist - self method initialPC + 1.
	dist >= 0 ifTrue: [
		"jump forward"
		^ irBuilder jumpAheadTo: index if: bool].

	self error: 'can only conditional jump forward'! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:34'!
methodReturnConstant: value

	self
		pushConstant: value;
		methodReturnTop! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:33'!
methodReturnReceiver

	self
		pushReceiver;
		methodReturnTop! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:31'!
methodReturnTop

	irBuilder returnTop! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:33'!
popIntoReceiverVariable: offset

	self
		storeIntoReceiverVariable: offset;
		doPop! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:34'!
popIntoTemporaryVariable: offset

	self
		storeIntoTemporaryVariable: offset;
		doPop! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:34'!
pushActiveContext

	irBuilder pushThisContext! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:35'!
pushConstant: value

	irBuilder pushLiteral: value! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/15/2003 15:49'!
pushLiteralVariable: assoc

	self pushConstant: assoc.
	self send: #poolValue super: false numArgs: 0.
! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:35'!
pushReceiver

	irBuilder pushReceiver! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:36'!
pushReceiverVariable: offset

	self pushReceiver.
	irBuilder getField: offset + 1.
! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:36'!
pushTemporaryVariable: offset

	irBuilder pushTemp: offset + 1! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 23:49'!
send: selector super: superFlag numArgs: numArgs

	selector == #remoteReturnTo: ifTrue: [^ irBuilder remoteReturn].
	superFlag
		ifTrue: [irBuilder send: selector toSuperOf: self method literals last]
		ifFalse: [irBuilder send: selector]! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:39'!
storeIntoReceiverVariable: offset

	self pushReceiver.
	irBuilder setField: offset + 1.
! !

!BytecodeDecompiler methodsFor: 'instruction decoding' stamp: 'ajh 3/2/2003 11:39'!
storeIntoTemporaryVariable: offset

	irBuilder storeTemp: offset + 1! !

!BytecodeDecompiler methodsFor: 'public access' stamp: 'ajh 3/21/2003 01:50'!
decompile: aCompiledMethod

	| ir |
	self method: aCompiledMethod pc: aCompiledMethod initialPC.
	irBuilder _ IRBuilder new.
	irBuilder primitiveNode: aCompiledMethod primitiveNode.
	irBuilder rargs: (0 to: aCompiledMethod numArgs).
	irBuilder addTemps: (aCompiledMethod numArgs + 1 to: aCompiledMethod numTemps).
	aCompiledMethod isQuick
		ifTrue: [self quickMethod]
		ifFalse: [self interpret].
	ir _ irBuilder ir.
	ir privCompiledMethod: aCompiledMethod.
	^ ir! !

!BytecodeDecompiler methodsFor: 'private' stamp: 'ajh 3/15/2003 15:43'!
quickMethod

	self method primitive = 256 ifTrue: [
		^ self methodReturnReceiver
	].
	self method isReturnSpecial ifTrue: [
		^ self methodReturnConstant: (BytecodeGenerator specialConstants at: self method primitive - 256)
	].
	self method isReturnField ifTrue: [
		self pushReceiverVariable: self method returnField.
		^ self methodReturnTop
	].
	self halt: 'quick method inconsistency'! !


!LiteralSet methodsFor: 'as yet unclassified' stamp: 'ajh 12/9/2001 16:03'!
add: newObject
	"Include newObject as one of the receiver's elements.  If equivalent is already present don't add and return equivalent object"

	| index |
	newObject ifNil: [self error: 'Sets cannot meaningfully contain nil as an element'].
	index _ self findElementOrNil: newObject.
	^ (array at: index) 
		ifNil: [self atNewIndex: index put: newObject. newObject]
		ifNotNil: [array at: index]! !

!LiteralSet methodsFor: 'as yet unclassified' stamp: 'ajh 2/2/2002 19:16'!
scanFor: anObject
	"Scan the key array for the first slot containing either a nil (indicating an empty slot) or an element that matches anObject. Answer the index of that slot or zero if no slot is found. This method will be overridden in various subclasses that have different interpretations for matching elements."
	| element start finish |
	start _ (anObject hash \\ array size) + 1.
	finish _ array size.

	"Search from (hash mod size) to the end."
	start to: finish do:
		[:index | ((element _ array at: index) == nil
					or: [element literalEqual: anObject])
					ifTrue: [^ index ]].

	"Search from 1 to where we started."
	1 to: start-1 do:
		[:index | ((element _ array at: index) == nil
					or: [element literalEqual: anObject])
					ifTrue: [^ index ]].

	^ 0  "No match AND no empty slot"! !


!MethodContext methodsFor: 'closure support' stamp: 'ajh 3/24/2003 20:54'!
doItScope

	| outerScope rcvr scope i |
	outerScope _ self isClosureContext
		ifTrue: [self receiver doItScope]
		ifFalse: [self mclass parseScope newMethodScope].
	rcvr _ outerScope receiverVar name.
	scope _ SemContextScope new initContextReceiver.
	scope fieldVarNames: (self tempNames copyWithFirst: rcvr) startingAt: self class instSize.
		"above assumes MethodContext-receiver is last inst var"
	i _ 0.
	scope fieldVars allButFirstDo: [:var |
		((self temp: (i _ i + 1)) isMemberOf: SharedTempHolder) ifTrue: [var markEscape]].
	scope outerScope: outerScope.
	^ scope! !


!OrderedCollection methodsFor: 'adding' stamp: 'ajh 5/22/2003 12:03'!
at: index ifAbsentPut: block
	"Return value at index, however, if value does not exist (nil or out of bounds) then add block's value at index (growing self if necessary)"

	| v |
	index <= self size ifTrue: [
		^ (v _ self at: index)
			ifNotNil: [v]
			ifNil: [self at: index put: block value]
	].
	[self size < index] whileTrue: [self add: nil].
	^ self at: index put: block value! !


!LiteralList methodsFor: 'adding' stamp: 'ajh 3/6/2003 18:00'!
addLast: object
	"Only add if not already in list"

	(equalitySet includes: object) ifTrue: [^ object].
	equalitySet add: object.
	super addLast: object.
	^ object
! !

!LiteralList methodsFor: 'private' stamp: 'ajh 1/21/2003 12:21'!
setCollection: anArray

	super setCollection: anArray.
	equalitySet _ LiteralSet new: anArray size.
! !

!LiteralList methodsFor: 'accessing' stamp: 'ajh 1/21/2003 12:21'!
indexOf: anElement startingAt: start ifAbsent: exceptionBlock

	start to: self size do:
		[:index | ((self at: index) literalEqual: anElement) ifTrue: [^ index]].
	^ exceptionBlock value! !


!ParseNode methodsFor: 'testing' stamp: 'ajh 3/24/2003 16:16'!
isJust: node

	^ false! !

!ParseNode methodsFor: 'converting' stamp: 'ajh 3/24/2003 16:25'!
ifNilReceiver

	^ self! !


!MethodNode methodsFor: 'code generation' stamp: 'ajh 3/24/2003 14:55'!
generate: trailer 

	Preferences compileBlocksAsClosures ifTrue: [
		^ (Parser2
			parse: self decompileString
			class: self encoder classEncoding
		   ) generate: trailer
	].

	^ self generateNative: trailer! !

!MethodNode methodsFor: 'code generation' stamp: 'ajh 3/24/2003 14:52'!
sourceMap
	"Answer a SortedCollection of associations of the form: pc (byte offset in 
	me) -> sourceRange (an Interval) in source text."

	self generateNative: #(0 0 0 0).
	^encoder sourceMap! !


!Parser2 methodsFor: 'public access' stamp: 'ajh 3/11/2003 14:35'!
parse: sourceStreamOrString class: behavior

	^ self parse: sourceStreamOrString readStream class: behavior
		noPattern: false notifying: nil ifFail: [nil]! !

!Parser2 methodsFor: 'public access' stamp: 'ajh 9/14/2002 09:28'!
parse: sourceStream class: behavior noPattern: noPattern context: context notifying: req ifFail: aBlock
	"Backwards compatibilty"

	context ifNotNil: [self error: 'Use Compiler-evaluate... instead'].
	^ self parse: sourceStream class: behavior noPattern: noPattern notifying: req ifFail: aBlock! !

!Parser2 methodsFor: 'public access' stamp: 'ajh 3/12/2003 15:02'!
parse: sourceStream class: parseScope noPattern: doitBool notifying: req ifFail: aBlock
	"Parse sourceStream into a embedded BlockNode if doitFlag is true (no method header) or a MethodNode if doitFlag is false.  Parsing is done with respect to parseScope to find non-local variables.  Errors in parsing are reported to req if not nil followed by executing the fail block."

	| parser tree |
	doitFlag _ doitBool.
	scope _ parseScope parseScope.
	parser _ (req notNil and: [RequestAlternateSyntaxSetting signal and: [(sourceStream isKindOf: FileStream) not]])
		ifTrue: ["Squeak00Parser" self toDo]
		ifFalse: [SqueakParser].
	source _ sourceStream.
	requestor _ req.
	failBlock _ [^ aBlock value].
	tree _ [doitFlag 
		ifTrue: [parser parseDoIt: source]
		ifFalse: [parser parseMethod: source]
	 ] on: SmaCCParserError do: [:ex |
		self notify: ex description at: ex tag position].

	^ [[tree verifyIn: scope
	  ] on: SemanticError do: [:ex |
		ex correctIn: self]
	  ] on: PrimitiveParseError do: [:ex |
		self notify: ex description at: ex tag position]
! !

!Parser2 methodsFor: 'public access' stamp: 'ajh 3/6/2003 00:42'!
parseArgsAndTemps: aString notifying: req 
	"No initialization required.  Parse the argument, aString, notifying req if an error occurs. Otherwise, answer a two-element Array containing Arrays of strings (the argument names and temporary variable names)."

	aString == nil ifTrue: [^ #()].

	^ (self parse: aString readStream class: nil noPattern: false notifying: req ifFail: [^ nil]) tempNames! !

!Parser2 methodsFor: 'public access' stamp: 'ajh 3/11/2003 19:59'!
parseMethodComment: aString setPattern: aBlock
	"Answer the method comment for the argument, aString. Evaluate aBlock with the methodNode containing the selector and arguments"

	| methodNode comments |
	methodNode _ SqueakParser parseMethodPattern: aString.
	endOfMethodPattern _ methodNode methodPatternStop.
	aBlock value: methodNode.
	comments _ methodNode comments.
	comments isEmpty ifTrue: [^ nil].
	^ comments first! !

!Parser2 methodsFor: 'public access' stamp: 'ajh 3/11/2003 20:00'!
parseSelector: aString 
	"Answer the message selector for the argument, aString, which should parse successfully up to the temporary declaration or the end of the method header."

	| methodNode |
	methodNode _ SqueakParser parseMethodPattern: aString.
	endOfMethodPattern _ methodNode methodPatternStop.
	^ methodNode selector! !

!Parser2 methodsFor: 'error handling' stamp: 'ajh 3/6/2003 01:03'!
fail

	^ failBlock value! !

!Parser2 methodsFor: 'error handling' stamp: 'ajh 6/9/2002 19:47'!
interactive

	^ (requestor == nil or: [requestor isKindOf: SyntaxError]) not! !

!Parser2 methodsFor: 'error handling' stamp: 'ajh 9/12/2002 01:10'!
notify: string at: location
	requestor isNil
		ifTrue: [SyntaxError 
					errorInClass: (self classEncoding ifNil: [FakeClassPool])
					withCode: 
						(source contents
							copyReplaceFrom: location
							to: location - 1
							with: string , ' ->')
					doitFlag: doitFlag]
		ifFalse: [requestor
					notify: string , ' ->'
					at: location
					in: source].
	^self fail! !

!Parser2 methodsFor: 'accessing' stamp: 'ajh 3/11/2003 12:16'!
classEncoding

	^ scope classEncoding! !

!Parser2 methodsFor: 'accessing' stamp: 'ajh 3/25/2003 23:45'!
encoder

	^ self! !

!Parser2 methodsFor: 'accessing' stamp: 'ajh 3/10/2003 22:17'!
requestor
	"Return the source code editor"

	^ requestor! !

!Parser2 methodsFor: 'accessing' stamp: 'ajh 6/9/2002 19:47'!
requestor: editor
	"set the source code editor"

	requestor _ editor! !

!Parser2 methodsFor: 'error correction' stamp: 'ajh 3/13/2003 04:22'!
correctSelector: messageNode
	"Correct the proposed selector in messageNode, correcting the original text if such action is indicated.  Fail if the proposedKeyword couldn't be converted into a valid selector.  Spots is an ordered collection of intervals within the test stream of the for each of the keyword parts."

	| alternatives aStream choice userSelection lines firstLine sel spots newSel |

	sel _ messageNode selectorString.
	"If we can't ask the user, assume that the keyword will be defined later"
	self interactive ifFalse: [^ sel asSymbol].

	spots _ messageNode selectorParts collect: [:token | token sourceInterval].
	userSelection _ requestor selectionInterval.
	requestor selectFrom: spots first first to: spots last last.
	requestor select.
	alternatives _ messageNode owningScope possibleSelectorsFor: sel.

	aStream _ WriteStream on: (String new: 200).
	aStream nextPutAll: (sel contractTo: 35); cr.
	firstLine _ 1.
 	alternatives do: [:s | aStream nextPutAll: s; cr].
	aStream nextPutAll: 'cancel'.
	lines _ Array with: firstLine with: (alternatives size + firstLine).
	
	choice _ (PopUpMenu labels: aStream contents lines: lines)
		startUpWithCaption: 
'Unknown selector, please 
confirm, correct, or cancel'.

	(choice = 0) | (choice > (lines at: 2)) ifTrue: [^ self fail].

	requestor deselect.
	requestor selectInvisiblyFrom: userSelection first to: userSelection last.

	choice = 1 ifTrue: [^ sel asSymbol].
	newSel _ alternatives at: choice - 1.
	self substituteSelector: newSel in: messageNode.
	((sel last ~~ $:) and: [newSel last == $:]) ifTrue: [^ self fail].
	^ (messageNode owningScope lookupSelector: newSel)
		ifNil: [self halt: 'chosen selector expected to exist']
! !

!Parser2 methodsFor: 'error correction' stamp: 'ajh 3/20/2003 01:32'!
correctVariable: variableNode
	"Correct the variableNode to a known variable, or declare it as a new
	variable if such action is requested.  We support declaring lowercase
	variables as temps, and uppercase variables as Globals or ClassVars,
	depending on whether the context is nil (class=UndefinedObject)."

	| varName spot alternatives aStream choice userSelection temp binding declareSize |

	varName _ variableNode name.
	"If we can't ask the user for correction, make it undeclared"
	self interactive ifFalse: [
		Transcript show: ' (' , varName , ' is Undeclared) '.
		varName _ varName asSymbol.
		Undeclared at: varName put: nil.
		^ SemPoolVar newFor: (Undeclared associationAt: varName)
	].

	spot _ variableNode sourceInterval.
	temp _ varName first isLowercase.
	"First check to see if the requestor knows anything about the variable"
	(temp and: [(binding _ requestor bindingOf: varName) notNil])
		ifTrue: [^ SemPoolVar newFor: binding].
	userSelection _ requestor selectionInterval.
	requestor selectFrom: spot first to: spot last.
	requestor select.

	alternatives _ variableNode owningScope possibleVarsFor: varName.

	aStream _ WriteStream on: (String new: 200).
	temp ifTrue: [
		declareSize _ 1.
		aStream nextPutAll: 'declare temp'; cr.
	] ifFalse: [
		declareSize _ 1.
		self classEncoding ifNotNil: [
			declareSize _ declareSize + 1.  
			aStream nextPutAll: 'declare class variable'; cr].
		aStream nextPutAll: 'declare global'; cr.
	].
	alternatives do: [:name | aStream nextPutAll: name; cr].
	aStream nextPutAll: 'cancel'.

	choice _ (PopUpMenu labels: aStream contents
				lines: {declareSize. declareSize + alternatives size})
		startUpWithCaption:
(('Unknown variable: ', varName, '
please correct, or cancel:') asText makeBoldFrom: 19 to: 19 + varName size).

	(choice = 0 or: [choice > (declareSize + alternatives size)]) ifTrue: [^ self fail].

	requestor deselect.
	requestor selectInvisiblyFrom: userSelection first to: userSelection last.
	temp ifTrue: [
		choice = 1 ifTrue: [^ self declareTempAndPaste: variableNode]
	] ifFalse: [
		declareSize = 1 ifTrue: [
			choice = 1 ifTrue: [^ self declareGlobal: variableNode].
		] ifFalse: [ "declareSize = 2"
			choice = 1 ifTrue: [^ self declareClassVar: variableNode].
			choice = 2 ifTrue: [^ self declareGlobal: variableNode].
		].
	].

	"Spelling correction"
	varName _ alternatives at: choice - declareSize.
	variableNode changeToken: (SmaCCToken value: varName start: spot first).
	self substituteWord: varName wordInterval: spot offset: 0.
	^ (variableNode owningScope lookupVar: varName)
		ifNil: [self halt: 'var should have been found']! !

!Parser2 methodsFor: 'error correction' stamp: 'ajh 3/20/2003 01:30'!
declareClassVar: variableNode

	self classEncoding theNonMetaClass
		addClassVarName: variableNode name asSymbol.
	^ (variableNode owningScope lookupVar: variableNode name)
		ifNil: [self halt: 'should be found']! !

!Parser2 methodsFor: 'error correction' stamp: 'ajh 3/20/2003 01:30'!
declareGlobal: variableNode

	Smalltalk at: variableNode name asSymbol put: nil.
	^ (variableNode owningScope lookupVar: variableNode name)
		ifNil: [self halt: 'should be found']! !

!Parser2 methodsFor: 'error correction' stamp: 'ajh 3/20/2003 01:31'!
declareTempAndPaste: variableNode

	| name insertion theTextString methodBody pos start c var tempNode prev |

	name _ variableNode name.
	theTextString _ requestor text string.
	methodBody _ variableNode root body.
	methodBody temporaries notEmpty ifTrue: [
  		"Paste it after last temp"
		prev _ pos _ methodBody temporaries last stop + 1.
		(theTextString at: pos) isSeparator ifTrue: [
			insertion _ name, ' '.
			pos _ pos + 1.  start _ pos.
		] ifFalse: [
			insertion _ ' ', name.
			start _ pos + 1.
		].
		self substituteWord: insertion
			wordInterval: (pos to: pos - 1) offset: 0.
	] ifFalse: [
		"No bars - insert some with CR & tab"
		insertion _ '| ', name, ' |', String cr.
		prev _ pos _ methodBody statements first start - 1.
		start _ 0.
		[pos = 0 or: [(c _ theTextString at: pos) = Character cr]] whileFalse: [
			c isSeparator ifTrue: [
				insertion _ c asString, insertion.
				start _ start + 1.
			].
			pos _ pos - 1.
		].
		pos _ pos + 1.
		self substituteWord: insertion
			wordInterval: (pos to: pos - 1) offset: 0.
		start _ pos + 2 + start.
	].
	var _ methodBody owningScope addTemp: name.
	tempNode _ RBVariableNode new
		identifierToken: (SmaCCToken value: name start: start);
		binding: var.
	methodBody root adjustPositionsAfter: prev by: insertion size.
	methodBody temporaries: (methodBody temporaries copyWith: tempNode).
	^ (variableNode owningScope lookupVar: name)
		ifNil: [self halt: 'should have been found']! !

!Parser2 methodsFor: 'error correction' stamp: 'ajh 3/12/2003 14:34'!
queryUninitializedTemp: variableNode

	| varStart varName |
	self interactive ifFalse: [^ self].
	varName _ variableNode name.
	varStart _ variableNode start.
	requestor selectFrom: varStart to: varStart + varName size - 1; select.
	((PopUpMenu labels:
'yes
no') startUpWithCaption:
		((varName , ' appears to be
undefined at this point.
Proceed anyway?') asText makeBoldFrom: 1 to: varName size))
		= 1 ifFalse: [^ self fail]! !

!Parser2 methodsFor: 'error correction' stamp: 'ajh 3/15/2003 13:27'!
removeUnusedTemp: variableNode
	"Removing unused temp, variableNode, if verified by the user"

	| varName interval |
	self interactive ifFalse: [^ self].  "don't remove and continue"
	varName _ variableNode name.
	((PopUpMenu labels: 'yes\no' withCRs) startUpWithCaption:
		((varName , ' appears to be
unused in this method.
OK to remove it?') asText makeBoldFrom: 1 to: varName size))
			= 1 ifFalse: [^ self].  "don't remove and continue"

	"Remove temp then continue"
	interval _ variableNode sourceInterval.
	(requestor text at: interval last + 1) isSeparator ifTrue: [
		interval _ interval first to: interval last + 1].
	self substituteWord: '' wordInterval: interval offset: 0.
	variableNode root adjustPositionsAfter: interval first by: 0 - interval size.
	variableNode parent removeTemporaryNamed: varName.
	^ self! !

!Parser2 methodsFor: 'error correction' stamp: 'ajh 3/17/2003 13:07'!
substituteSelector: newSel in: messageNode
	"Substitute the correctSelector into the (presuamed interactive) receiver."

	| offset newParts |
	newParts _ OrderedCollection new.
	offset _ 0.
	newSel keywords
	  with: (messageNode selectorParts collect: [:tok | tok sourceInterval])
	  do: [:word :interval |
		newParts add: (SmaCCToken value: word start: interval first + offset).
		offset _ self substituteWord: word wordInterval: interval offset: offset.
	  ].
	messageNode changeSelectorParts: newParts asArray.
! !

!Parser2 methodsFor: 'error correction' stamp: 'ajh 3/11/2003 23:22'!
substituteWord: correctWord wordInterval: spot offset: o
	"Substitute the correctSelector into the (presuamed interactive) receiver."

	requestor
		correctFrom: (spot first + o)
		to: (spot last + o)
		with: correctWord.
	^ o + correctWord size - spot size! !

!Parser2 methodsFor: 'error correction' stamp: 'ajh 3/19/2003 16:43'!
variable: varNode shadows: semVar

	self interactive ifFalse: [
		^ Transcript show: '(', varNode name, ' is shadowed)'].
	self notify: 'Name already defined' at: varNode start.
! !

!Parser2 methodsFor: 'old protocol' stamp: 'ajh 3/11/2003 20:01'!
endOfLastToken

	^ endOfMethodPattern! !


!PrimitiveNode methodsFor: 'as yet unclassified' stamp: 'ajh 7/14/2001 12:37'!
num

	^ primitiveNum! !

!PrimitiveNode methodsFor: 'as yet unclassified' stamp: 'ajh 7/14/2001 12:23'!
num: n

	primitiveNum _ n! !

!PrimitiveNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 12:26'!
printOn: aStream

	aStream nextPutAll: 'primitive '; print: primitiveNum! !

!PrimitiveNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/19/2003 22:06'!
printPrimitiveOn: aStream 
	"Print the primitive on aStream"

	| primIndex primDecl |
	primIndex _ primitiveNum.
	primIndex = 0 ifTrue: [^ self].
	primIndex = 120 ifTrue: [
		"External call spec"
		^ aStream print: spec].
	aStream nextPutAll: '<primitive: '.
	primIndex = 117 ifTrue: [
		primDecl _ spec.
		aStream nextPut: $';
			nextPutAll: (primDecl at: 2);
			nextPut: $'.
		(primDecl at: 1) ifNotNil: [
			aStream nextPutAll: ' module: ';
				nextPut: $';
				nextPutAll: (primDecl at: 1);
				nextPut: $'].
	] ifFalse: [aStream print: primIndex].
	aStream nextPut: $>.
	(primIndex ~= 117 and: [primIndex ~= 120]) ifTrue: [
		Smalltalk at: #Interpreter ifPresent: [:cls |
			aStream nextPutAll: ' "', 
				((cls classPool at: #PrimitiveTable) at: primIndex + 1) , '" '
		].
	].
! !

!PrimitiveNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/19/2003 22:02'!
sourceText

	^ String streamContents: [:stream |
		self printPrimitiveOn: stream]! !

!PrimitiveNode methodsFor: 'as yet unclassified' stamp: 'ajh 7/14/2001 12:37'!
spec

	^ spec! !

!PrimitiveNode methodsFor: 'as yet unclassified' stamp: 'ajh 7/14/2001 12:30'!
spec: literal

	spec _ literal! !


!PrimitiveNode class methodsFor: 'as yet unclassified' stamp: 'ajh 7/14/2001 12:47'!
null

	^ self new num: 0! !


!PrimitiveParseError methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 15:26'!
isResumable

	^ true! !


!PrimitiveParser methodsFor: 'public access' stamp: 'ajh 3/11/2003 11:32'!
parse: primitiveString

	| prim |
	super scan: primitiveString readStream.
	prevMark _ hereMark _ mark.
	self advance.

	prim _ self primitive.
	^ PrimitiveNode new
		num: prim;
		spec: spec! !

!PrimitiveParser methodsFor: 'public access' stamp: 'ajh 3/11/2003 18:34'!
startPosition: n
	"Needed only for error reporting"

	startPosition _ n! !

!PrimitiveParser methodsFor: 'scanning' stamp: 'ajh 3/11/2003 11:10'!
advance
	| this |
	prevMark _ hereMark.
	prevEnd _ hereEnd.
	this _ here.
	here _ token.
	hereType _ tokenType.
	hereMark _ mark.
	hereEnd _ source position - (source atEnd ifTrue: [hereChar == 30 asCharacter ifTrue: [0] ifFalse: [1]] ifFalse: [2]).
	self scanToken.
	"Transcript show: 'here: ', here printString, ' mark: ', hereMark printString, ' end: ', hereEnd printString; cr."
	^this! !

!PrimitiveParser methodsFor: 'scanning' stamp: 'ajh 3/11/2003 11:10'!
endOfLastToken

	^ prevEnd ifNil: [mark]! !

!PrimitiveParser methodsFor: 'scanning' stamp: 'ajh 3/11/2003 11:10'!
match: type 
	"Answer with true if next tokens type matches."

	hereType == type
		ifTrue: 
			[self advance.
			^true].
	^false! !

!PrimitiveParser methodsFor: 'scanning' stamp: 'ajh 3/11/2003 11:10'!
matchReturn

	^ self match: #upArrow! !

!PrimitiveParser methodsFor: 'scanning' stamp: 'ajh 3/11/2003 11:10'!
matchToken: thing 
	"Matches the token, not its type."

	here = thing ifTrue: [self advance. ^true].
	^false! !

!PrimitiveParser methodsFor: 'scanning' stamp: 'ajh 3/11/2003 11:10'!
startOfNextToken
	"Return starting position in source of next token."

	hereType == #doIt ifTrue: [^source position + 1].
	^hereMark! !

!PrimitiveParser methodsFor: 'primitives' stamp: 'ajh 3/11/2003 11:32'!
allocateLiteral: lit

	spec _ lit! !

!PrimitiveParser methodsFor: 'primitives' stamp: 'ajh 3/15/2003 16:28'!
externalFunctionDeclaration
	"Parse the function declaration for a call to an external library."
	| descriptorClass callType retType externalName args argType module fn |
	descriptorClass _ Smalltalk at: #ExternalFunction ifAbsent:[nil].
	descriptorClass == nil ifTrue:[^0].
	callType _ descriptorClass callingConventionFor: here.
	callType == nil ifTrue:[^0].
	"Parse return type"
	self advance.
	retType _ self externalType: descriptorClass.
	retType == nil ifTrue:[^self expected:'return type'].
	"Parse function name or index"
	externalName _ here.
	(self match: #string) 
		ifTrue:[externalName _ externalName asSymbol]
		ifFalse:[(self match:#number) ifFalse:[^self expected:'function name or index']].
	(self matchToken:'(' asSymbol) ifFalse:[^self expected:'argument list'].
	args _ WriteStream on: Array new.
	[here == #')'] whileFalse:[
		argType _ self externalType: descriptorClass.
		argType == nil ifTrue:[^self expected:'argument'].
		argType isVoid & argType isPointerType not ifFalse:[args nextPut: argType].
	].
	(self matchToken:')' asSymbol) ifFalse:[^self expected:')'].
	(self matchToken: 'module:') ifTrue:[
		module _ here.
		(self match: #string) ifFalse:[^self expected: 'String'].
		module _ module asSymbol].
	Smalltalk at: #ExternalLibraryFunction ifPresent:[:xfn|
		fn _ xfn name: externalName 
				module: module 
				callType: callType
				returnType: retType
				argumentTypes: args contents.
		self allocateLiteral: fn.
	].
	^120! !

!PrimitiveParser methodsFor: 'primitives' stamp: 'ajh 3/11/2003 11:10'!
externalType: descriptorClass
	"Parse an return an external type"
	| xType |
	xType _ descriptorClass atomicTypeNamed: here.
	xType == nil ifTrue:["Look up from class scope"
		Symbol hasInterned: here ifTrue:[:sym|
			xType _ descriptorClass structTypeNamed: sym]].
	xType == nil ifTrue:[
		"Raise an error if user is there"
		self interactive ifTrue:[^nil].
		"otherwise go over it silently"
		xType _ descriptorClass forceTypeNamed: here].
	self advance.
	(self matchToken:#*)
		ifTrue:[^xType asPointerType]
		ifFalse:[^xType]! !

!PrimitiveParser methodsFor: 'primitives' stamp: 'ajh 3/11/2003 11:10'!
primitive
	| n |
	(self matchToken: #<) ifFalse: [^ 0].
	n _ self primitiveDeclarations.
	(self matchToken: #>) ifFalse: [^ self expected: '>'].
	^ n! !

!PrimitiveParser methodsFor: 'primitives' stamp: 'ajh 3/11/2003 11:10'!
primitiveDeclarations
	| prim module |
	(self matchToken: 'primitive:') ifFalse:[^self externalFunctionDeclaration].
	prim _ here.
	(self match: #number) ifTrue:[^prim].	"Indexed primitives"
	(self match: #string) ifFalse:[^self expected:'Integer or String'].
	(self matchToken: 'module:') ifTrue:[
		module _ here.
		(self match: #string) ifFalse:[^self expected: 'String'].
		module _ module asSymbol].
	(self allocateLiteral: (Array with: module with: prim asSymbol with: 0 with: 0)).
	^117! !

!PrimitiveParser methodsFor: 'error handling' stamp: 'ajh 3/11/2003 15:04'!
expected: aString 
	"Notify a problem at token 'here'."

	tokenType == #doIt ifTrue: [hereMark _ hereMark + 1].
	hereType == #doIt ifTrue: [hereMark _ hereMark + 1].
	^ self notify: aString , ' expected'! !

!PrimitiveParser methodsFor: 'error handling' stamp: 'ajh 3/11/2003 18:36'!
localPosition

	^ hereMark! !

!PrimitiveParser methodsFor: 'error handling' stamp: 'ajh 3/11/2003 15:17'!
notify: aString
	"Notify problem at token at 'self position'."

	^ PrimitiveParseError new
		tag: self;
		signal: aString! !

!PrimitiveParser methodsFor: 'error handling' stamp: 'ajh 3/11/2003 18:37'!
position

	^ self startPosition + self localPosition - 1! !

!PrimitiveParser methodsFor: 'error handling' stamp: 'ajh 3/11/2003 18:34'!
startPosition

	^ startPosition! !


!RBProgramNode methodsFor: 'accessing' stamp: 'ajh 3/17/2003 00:28'!
comment

	^ (comments isNil or: [comments isEmpty])
		ifTrue: [nil]
		ifFalse: [comments first]! !

!RBProgramNode methodsFor: 'accessing' stamp: 'ajh 3/15/2003 15:17'!
debugHighlightStart

	^ self start! !

!RBProgramNode methodsFor: 'accessing' stamp: 'ajh 3/15/2003 15:18'!
debugHighlightStop

	^ self stop! !

!RBProgramNode methodsFor: 'testing' stamp: 'ajh 2/25/2003 14:34'!
isArray
	^false! !

!RBProgramNode methodsFor: 'testing' stamp: 'ajh 3/3/2003 22:28'!
isLiteral: valueTestBlock
	^false! !

!RBProgramNode methodsFor: 'querying' stamp: 'ajh 2/27/2003 22:40'!
root

	^ parent
		ifNil: [self]
		ifNotNil: [parent root]! !

!RBProgramNode methodsFor: 'replacing' stamp: 'ajh 3/13/2003 16:09'!
adjustPositionsAfter: sourcePos by: delta
	"Slide token positions after sourcePos by delta"

	self children do: [:node |
		node adjustPositionsAfter: sourcePos by: delta]! !

!RBProgramNode methodsFor: 'inline' stamp: 'ajh 2/25/2003 19:48'!
isInline

	^ false! !

!RBProgramNode methodsFor: 'semantics' stamp: 'ajh 3/13/2003 04:19'!
owningScope

	^ parent owningScope! !

!RBProgramNode methodsFor: 'decompiling' stamp: 'ajh 3/15/2003 14:57'!
decompileString

	^ self formattedCode! !

!RBProgramNode methodsFor: 'decompiling' stamp: 'ajh 3/4/2003 00:50'!
isCaseBranch

	^ false! !

!RBProgramNode methodsFor: 'decompiling' stamp: 'ajh 3/2/2003 23:22'!
isDup

	^ false! !

!RBProgramNode methodsFor: 'decompiling' stamp: 'ajh 3/2/2003 23:22'!
isGoto

	^ false! !

!RBProgramNode methodsFor: 'decompiling' stamp: 'ajh 3/1/2003 20:12'!
isIf

	^ false! !

!RBProgramNode methodsFor: 'decompiling' stamp: 'ajh 3/2/2003 23:22'!
isLabel

	^ false! !

!RBProgramNode methodsFor: 'decompiling' stamp: 'ajh 3/2/2003 23:22'!
isPop

	^ false! !

!RBProgramNode methodsFor: 'decompiling' stamp: 'ajh 3/3/2003 18:42'!
isPseudo

	^ false! !

!RBProgramNode methodsFor: 'decompiling' stamp: 'ajh 3/23/2003 22:23'!
isPseudoSend

	^ false! !


!RBArrayNode methodsFor: 'visitor' stamp: 'ajh 3/17/2003 00:25'!
acceptVisitor: aProgramNodeVisitor
	^ aProgramNodeVisitor acceptArrayNode: self! !

!RBArrayNode methodsFor: 'testing' stamp: 'ajh 2/25/2003 14:34'!
isArray

	^ true! !

!RBArrayNode methodsFor: 'initialization' stamp: 'ajh 3/4/2003 02:04'!
leftBrace: leftBrace0  rightBrace: rightBrace0  statements: statements0
	self leftBrace: leftBrace0.
	self rightBrace: rightBrace0.
	self statements: statements0.! !

!RBArrayNode methodsFor: 'replacing' stamp: 'ajh 3/19/2003 16:14'!
adjustPositionsAfter: sourcePos by: delta
	"Slide token positions after sourcePos by delta"

	self leftBrace: leftBrace + delta.
	self rightBrace: rightBrace + delta.
	super adjustPositionsAfter: sourcePos by: delta.
! !


!RBArrayNode class methodsFor: 'instance creation' stamp: 'ajh 3/4/2003 02:03'!
statements: statements
	^ self new statements: statements! !


!RBBlockNode methodsFor: 'replacing' stamp: 'ajh 3/19/2003 16:15'!
adjustPositionsAfter: sourcePos by: delta
	"Slide token positions after sourcePos by delta"

	self left: left + delta.
	self right: right + delta.
	super adjustPositionsAfter: sourcePos by: delta.
! !

!RBBlockNode methodsFor: 'printing' stamp: 'ajh 3/17/2003 09:12'!
sourceText

	| text |
	self parent ifNil: [^ self formattedCode asText].
	text _ [self root sourceText]
		on: Error
		do: [^ self formattedCode asText].
	text addAttribute: TextColor gray from: 1 to: self start - 1.
	text addAttribute: TextColor gray from: self stop + 1 to: text size.
	^ text! !

!RBBlockNode methodsFor: 'inline' stamp: 'ajh 3/13/2003 02:43'!
isInlined

	(parent isMessage and: [parent isInlineAndOr]) ifTrue: [^ true].
	(parent isMessage and: [parent isInlineIf]) ifTrue: [^ true].
	(parent isMessage and: [parent isInlineIfNil]) ifTrue: [^ true].
	(parent isMessage and: [parent isInlineToDo]) ifTrue: [^ true].
	(parent isMessage and: [parent isInlineWhile]) ifTrue: [^ true].
	(parent isMessage and: [parent parent isArray and: [parent parent parent isMessage and: [parent parent parent isInlineCase]]]) ifTrue: [^ true].
	(parent isMessage and: [parent isInlineCase]) ifTrue: [^ true].  "otherwise branch"
	^ false! !

!RBBlockNode methodsFor: 'semantics' stamp: 'ajh 5/28/2003 01:24'!
owningScope

	^ scope! !

!RBBlockNode methodsFor: 'semantics' stamp: 'ajh 2/26/2003 15:52'!
scope

	^ scope! !

!RBBlockNode methodsFor: 'semantics' stamp: 'ajh 2/26/2003 15:53'!
scope: aSemClosureScope

	scope _ aSemClosureScope! !

!RBBlockNode methodsFor: 'translate' stamp: 'ajh 3/10/2003 20:23'!
generate: trailer

	^ self generateIR compiledMethodWith: trailer! !

!RBBlockNode methodsFor: 'translate' stamp: 'ajh 3/10/2003 20:24'!
generateIR

	^ ir _ SemanticTranslator new
		translateBlockNode: self;
		ir! !

!RBBlockNode methodsFor: 'translate' stamp: 'ajh 3/10/2003 20:24'!
ir

	^ ir ifNil: [self generateIR]! !

!RBBlockNode methodsFor: 'translate' stamp: 'ajh 3/3/2003 15:31'!
privIR: irMethod

	ir _ irMethod! !

!RBBlockNode methodsFor: 'debugging' stamp: 'ajh 3/18/2003 14:07'!
freeNames
	"All free/closure var names in closure order"

	^ self scope fieldVars collect: [:semVar | semVar asString]! !

!RBBlockNode methodsFor: 'debugging' stamp: 'ajh 2/28/2003 00:18'!
sourceMap
	"Return a mapping from bytecode pcs to source code ranges"

	^ self ir sourceMap! !

!RBBlockNode methodsFor: 'debugging' stamp: 'ajh 3/17/2003 11:11'!
tempNames
	"All temp names in context order"

	^ self ir tempKeys collect: [:semVar | semVar asString]! !


!RBCascadeNode methodsFor: 'accessing' stamp: 'ajh 2/25/2003 01:12'!
receiver

	^ self messages first receiver! !


!RBDoItNode methodsFor: 'comparing' stamp: 'ajh 2/26/2003 18:35'!
= anObject 
	self == anObject ifTrue: [^true].
	self class = anObject class ifFalse: [^false].
	(self body = anObject body) ifFalse: [^false].
	^true! !

!RBDoItNode methodsFor: 'comparing' stamp: 'ajh 2/26/2003 18:35'!
equalTo: anObject withMapping: aDictionary 
	self class = anObject class ifFalse: [^false].
	(self body equalTo: anObject body withMapping: aDictionary) 
		ifFalse: [^false].
	^self primitiveSources = anObject primitiveSources! !

!RBDoItNode methodsFor: 'comparing' stamp: 'ajh 2/26/2003 18:36'!
hash
	^ self body hash! !

!RBDoItNode methodsFor: 'visitor' stamp: 'ajh 2/26/2003 18:31'!
acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptDoItNode: self! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
addNode: aNode 
	^body addNode: aNode! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
addReturn
	body addReturn! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
addSelfReturn
	^body addSelfReturn! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
body
	^body! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
body: stmtsNode 
	body := stmtsNode.
	body parent: self! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:28'!
children
	^ {self body}! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
primitiveSources
	^self tags 
		collect: [:each | self source copyFrom: each first to: each last]! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
source
	^source! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
source: anObject
	source := anObject! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
start
	^1! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
stop
	^source size! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
tags
	^tags isNil ifTrue: [#()] ifFalse: [tags]! !

!RBDoItNode methodsFor: 'accessing' stamp: 'ajh 2/26/2003 18:22'!
tags: aCollectionOfIntervals 
	tags := aCollectionOfIntervals! !

!RBDoItNode methodsFor: 'matching' stamp: 'ajh 2/26/2003 18:24'!
copyInContext: aDictionary 
	^(self class new)
		body: (body copyInContext: aDictionary);
		source: (aDictionary at: '-source-');
		yourself! !

!RBDoItNode methodsFor: 'matching' stamp: 'ajh 2/26/2003 18:28'!
match: aNode inContext: aDictionary 
	self class == aNode class ifFalse: [^false].
	aDictionary at: '-source-' put: aNode source.
	^ body match: aNode body inContext: aDictionary! !

!RBDoItNode methodsFor: 'testing' stamp: 'ajh 2/26/2003 18:22'!
isLast: aNode 
	^body isLast: aNode! !

!RBDoItNode methodsFor: 'testing' stamp: 'ajh 2/26/2003 18:22'!
lastIsReturn
	^body lastIsReturn! !

!RBDoItNode methodsFor: 'testing' stamp: 'ajh 2/26/2003 18:22'!
references: aVariableName 
	^body references: aVariableName! !

!RBDoItNode methodsFor: 'testing' stamp: 'ajh 2/26/2003 18:22'!
uses: aNode 
	^body == aNode and: [aNode lastIsReturn]! !

!RBDoItNode methodsFor: 'copying' stamp: 'ajh 2/26/2003 18:30'!
postCopy
	super postCopy.
	body := body copy.
! !

!RBDoItNode methodsFor: 'printing' stamp: 'ajh 2/26/2003 18:22'!
printOn: aStream
	aStream nextPutAll: self formattedCode! !

!RBDoItNode methodsFor: 'replacing' stamp: 'ajh 2/26/2003 18:57'!
replaceNode: aNode withNode: anotherNode
	aNode == body ifTrue: [self body: anotherNode].
! !

!RBDoItNode methodsFor: 'semantics' stamp: 'ajh 3/13/2003 04:25'!
owningScope

	^ scope! !

!RBDoItNode methodsFor: 'semantics' stamp: 'ajh 3/16/2003 08:33'!
scope

	^ scope ifNil: [
		self verifyIn: nil parseScope.
		scope
	  ]! !

!RBDoItNode methodsFor: 'semantics' stamp: 'ajh 3/17/2003 15:34'!
scope: aSemScope

	scope _ aSemScope! !

!RBDoItNode methodsFor: 'semantics' stamp: 'ajh 3/6/2003 00:56'!
verifyIn: classOrScope
	"Look up vars in classOrScope.  My tree will be annotated with bindings to SemScopes and SemVars."

	SemanticChecker new
		scope: classOrScope parseScope;
		visitNode: self! !

!RBDoItNode methodsFor: 'translate' stamp: 'ajh 3/10/2003 20:23'!
generate: trailer

	^ self generateIR compiledMethodWith: trailer! !

!RBDoItNode methodsFor: 'translate' stamp: 'ajh 3/10/2003 20:23'!
generateIR

	^ ir _ SemanticTranslator new
		visitNode: self;
		ir! !

!RBDoItNode methodsFor: 'translate' stamp: 'ajh 3/10/2003 20:25'!
ir

	^ ir ifNil: [self generateIR]! !


!RBDoItNode class methodsFor: 'instance creation' stamp: 'ajh 3/11/2003 19:08'!
body: aSequenceNode

	^ self new body: aSequenceNode! !


!RBLiteralNode methodsFor: 'testing' stamp: 'ajh 3/3/2003 22:29'!
isLiteral: valueTestBlock

	^ valueTestBlock value: self value! !

!RBLiteralNode methodsFor: 'initialize-release' stamp: 'ajh 2/27/2003 17:23'!
initialize

	value _ Null! !

!RBLiteralNode methodsFor: 'initialize-release' stamp: 'ajh 2/27/2003 17:22'!
value: object

	value _ object! !

!RBLiteralNode methodsFor: 'accessing' stamp: 'ajh 2/27/2003 17:24'!
value

	^ value == Null
		ifTrue: [token realValue]
		ifFalse: [value]! !

!RBLiteralNode methodsFor: 'replacing' stamp: 'ajh 3/13/2003 16:11'!
adjustPositionsAfter: sourcePos by: delta
	"Slide token positions after sourcePos by delta"

	token start > sourcePos ifTrue: [
		token start: token start + delta]! !


!RBLiteralNode class methodsFor: 'instance creation' stamp: 'ajh 2/27/2003 17:23'!
initialize

	Null _ Object new! !

!RBLiteralNode class methodsFor: 'instance creation' stamp: 'ajh 2/27/2003 17:22'!
literalToken: aLiteralToken value: object
	^self new literalToken: aLiteralToken; value: object! !

!RBLiteralNode class methodsFor: 'instance creation' stamp: 'ajh 2/27/2003 17:23'!
new

	^ super new initialize! !


!RBMessageNode methodsFor: 'accessing' stamp: 'ajh 3/15/2003 15:18'!
debugHighlightStart

	^ self selectorParts first start! !

!RBMessageNode methodsFor: 'accessing' stamp: 'ajh 3/15/2003 15:19'!
debugHighlightStop

	^ self stopWithoutParentheses! !

!RBMessageNode methodsFor: 'accessing' stamp: 'ajh 3/11/2003 23:16'!
selectorPartIntervals

	^ selectorParts collect: [:token | token sourceInterval]! !

!RBMessageNode methodsFor: 'accessing' stamp: 'ajh 3/11/2003 22:48'!
selectorString

	| selectorStream |
	selectorStream := WriteStream on: (String new: 50).
	selectorParts do: [:each | selectorStream nextPutAll: each value].
	^ selectorStream contents! !

!RBMessageNode methodsFor: 'private' stamp: 'ajh 3/11/2003 23:40'!
selectorParts: tokenCollection 
	selectorParts := tokenCollection.
	selector _ nil.
! !

!RBMessageNode methodsFor: 'testing' stamp: 'ajh 2/25/2003 17:16'!
lastIsReturn

	^ ((#(ifTrue:ifFalse: ifFalse:ifTrue: ifNil:ifNotNil: ifNotNil:ifNil:) includes: self selector)
		and: [arguments first isBlock and: 
			[arguments first body lastIsReturn
				and: [arguments last isBlock and: [arguments last body lastIsReturn]]]])
	  or: [(#(caseOf: caseOf:otherwise:) includes: self selector)
		and: [arguments first isArray and: [arguments first statements allSatisfy: [:assocMessage | assocMessage arguments first isBlock and: [assocMessage arguments first body lastIsReturn]]]
			and: [selector == #caseOf: or: [arguments second isBlock and: [arguments second body lastIsReturn]]]]]! !

!RBMessageNode methodsFor: 'replacing' stamp: 'ajh 3/13/2003 16:12'!
adjustPositionsAfter: sourcePos by: delta
	"Slide token positions after sourcePos by delta"

	selectorParts do: [:token |
		token start > sourcePos ifTrue: [
			token start: token start + delta]
	].
	super adjustPositionsAfter: sourcePos by: delta.
! !

!RBMessageNode methodsFor: 'replacing' stamp: 'ajh 3/17/2003 13:11'!
changeSelectorParts: tokenCollection

	| root oldToken newToken |
	root _ self root.
	1 to: selectorParts size do: [:i |
		oldToken _ selectorParts at: i.
		newToken _ tokenCollection at: i.
		root adjustPositionsAfter: oldToken stop
			by: newToken stop - oldToken stop.
		selectorParts at: i put: newToken.
	].
	selector _ nil.
! !

!RBMessageNode methodsFor: 'inline' stamp: 'ajh 2/25/2003 19:47'!
isInline

	self isInlineAndOr ifTrue: [^ true].
	self isInlineCase ifTrue: [^ true].
	self isInlineIf ifTrue: [^ true].
	self isInlineIfNil ifTrue: [^ true].
	self isInlineToDo ifTrue: [^ true].
	self isInlineWhile ifTrue: [^ true].
	^ false! !

!RBMessageNode methodsFor: 'inline' stamp: 'ajh 3/14/2003 21:22'!
isInlineAndOr

	self receiver isBlock ifTrue: [^ false].
	self isCascaded ifTrue: [^ false].
	(#(and: or:) includes: self selectorString) ifFalse: [^ false].
	self arguments first isBlock ifFalse: [^ false].
	self arguments first arguments isEmpty ifFalse: [self notify: 'and: (or:) takes zero-arg block'. ^ false].
	^ true! !

!RBMessageNode methodsFor: 'inline' stamp: 'ajh 3/14/2003 21:23'!
isInlineCase

	self isCascaded ifTrue: [^ false].
	(#(caseOf: caseOf:otherwise:) includes: self selectorString) ifFalse: [^ false].
	self arguments size = 2 ifTrue: [  "otherwise block"
		self arguments last isBlock ifFalse: [^ false]].
	self arguments first isArray ifFalse: [^ false].
	self arguments first statements do: [:assoc |
		(assoc isMessage and: [assoc selectorString = #->]) ifFalse: [^ false].
		assoc receiver isBlock ifFalse: [^ false].
		assoc receiver arguments isEmpty ifFalse: [self notify: 'caseOf: takes zero-arg blocks'. ^ false].
		assoc arguments first isBlock ifFalse: [^ false].
		assoc arguments first arguments isEmpty ifFalse: [self notify: 'caseOf: takes zero-arg blocks'].
	].
	^ true! !

!RBMessageNode methodsFor: 'inline' stamp: 'ajh 3/14/2003 21:23'!
isInlineIf

	self receiver isBlock ifTrue: [^ false].
	self isCascaded ifTrue: [^ false].
	(#(ifTrue: ifFalse: ifTrue:ifFalse: ifFalse:ifTrue:) includes: self selectorString) ifFalse: [^ false].
	self arguments do: [:node | node isBlock ifFalse: [^ false]].
	self arguments do: [:block |
		block arguments isEmpty ifFalse: [
			self notify: 'ifTrue:ifFalse: takes zero-arg blocks'.
			^ false
		]
	].
	^ true! !

!RBMessageNode methodsFor: 'inline' stamp: 'ajh 3/14/2003 21:24'!
isInlineIfNil

	| assertNone assertOneOrNone |
	self receiver isBlock ifTrue: [^ false].
	self isCascaded ifTrue: [^ false].
	(#(ifNil: ifNotNil: ifNil:ifNotNil: ifNotNil:ifNil:) includes: self selectorString) ifFalse: [^ false].
	self arguments do: [:node | node isBlock ifFalse: [^ false]].
	assertNone _ [:block | 
		block arguments isEmpty ifFalse: [self notify: 'ifNil: takes zero-arg block'. ^ false]
	].
	assertOneOrNone _ [:block | 
		block arguments size > 1 ifTrue: [self notify: 'ifNotNil: takes zero- or one-arg block'. ^ false]
	].
	self selectorString caseOf: {
		[#ifNil:] 		 	-> [assertNone value: self arguments first].
		[#ifNil:ifNotNil:] 	-> [assertNone value: self arguments first. 
							   assertOneOrNone value: self arguments last].
		[#ifNotNil:] 		  	-> [assertOneOrNone value: self arguments first].
		[#ifNotNil:ifNil:] 	-> [assertOneOrNone value: self arguments first. 
							   assertNone value: self arguments last]
	}.
	^ true! !

!RBMessageNode methodsFor: 'inline' stamp: 'ajh 3/17/2003 09:02'!
isInlineToDo

	| block step |
	self receiver isBlock ifTrue: [^ false].
	self isCascaded ifTrue: [^ false].
	(#(to:do: to:by:do:) includes: self selectorString) ifFalse: [^ false].
	self arguments first isBlock ifTrue: [^ false].
	block _ self arguments last.
	block isBlock ifFalse: [^ false].
	block arguments size = 1 ifFalse: [
		self notify: 'to:do: block must take one arg'. ^ false].
	(ParseTreeSearcher new
		matches: block arguments first name , ' := `@object' do: [:n :a | true];
		executeTree: block initialAnswer: false) ifTrue: [^ false].
	self arguments size = 3 "to:by:do:" ifTrue: [
		step _ self arguments second.
		step isLiteral ifFalse: [^ false].
		step value = 0 ifTrue: [self notify: 'by: step can''t be zero'. ^ false].
	].
	^ true! !

!RBMessageNode methodsFor: 'inline' stamp: 'ajh 3/14/2003 21:25'!
isInlineWhile

	self isCascaded ifTrue: [^ false].
	(#(whileFalse: whileTrue: whileFalse whileTrue) includes: self selectorString) ifFalse: [^ false].
	self receiver isBlock ifFalse: [^ false].
	self receiver arguments isEmpty ifFalse: [self notify: 'while receiver block must have no arguments'. ^ false].
	self arguments isEmpty ifFalse: [
		self arguments first isBlock ifFalse: [^ false].
		self arguments first arguments isEmpty ifFalse: [self notify: 'while takes a zero-arg block as its argument'. ^ false].
	].
	^ true! !


!RBMethodNode methodsFor: 'accessing' stamp: 'ajh 3/11/2003 19:58'!
methodPatternStop

	^ self arguments isEmpty
		ifTrue: [self selectorParts first stop]
		ifFalse: [self arguments last stop]! !

!RBMethodNode methodsFor: 'accessing' stamp: 'ajh 3/19/2003 22:02'!
primitiveSources

	| tgs text |
	tgs _ self tags.
	tgs isEmpty ifTrue: [
		text _ self primitiveText.
		text isEmpty ifTrue: [^ #()].
		^ {text}
	].
	^ tgs collect: [:each | self source copyFrom: each first to: each last]! !

!RBMethodNode methodsFor: 'accessing' stamp: 'ajh 3/11/2003 18:32'!
primitiveStartPosition

	| set |
	set _ self tags.
	set isEmpty ifTrue: [^ 0].
	set size > 1 ifTrue: [self error: 'only one primitive string expected'].
	^ set first first! !

!RBMethodNode methodsFor: 'accessing' stamp: 'ajh 3/19/2003 22:02'!
primitiveText

	| set |
	self tags isEmpty ifTrue: [
		^ self primitiveNode
			ifNil: ['']
			ifNotNil: [self primitiveNode sourceText]
	].
	set _ self primitiveSources.
	set size > 1 ifTrue: [self error: 'only one primitive string expected'].
	^ set first
! !

!RBMethodNode methodsFor: 'copying' stamp: 'ajh 3/3/2003 15:28'!
asBlock

	^ (RBBlockNode
		arguments: arguments
		body: body)
		privIR: ir;
		scope: scope! !

!RBMethodNode methodsFor: 'printing' stamp: 'ajh 2/27/2003 22:44'!
sourceText

	^ (self source ifNil: [self formattedCode]) asText! !

!RBMethodNode methodsFor: 'replacing' stamp: 'ajh 3/13/2003 16:13'!
adjustPositionsAfter: sourcePos by: delta
	"Slide token positions after sourcePos by delta"

	selectorParts do: [:token |
		token start > sourcePos ifTrue: [
			token start: token start + delta]
	].
	super adjustPositionsAfter: sourcePos by: delta.
! !

!RBMethodNode methodsFor: 'semantics' stamp: 'ajh 3/13/2003 04:25'!
owningScope

	^ scope! !

!RBMethodNode methodsFor: 'semantics' stamp: 'ajh 3/11/2003 18:27'!
primitiveNode

	^ primitiveNode ifNil: [PrimitiveNode null]! !

!RBMethodNode methodsFor: 'semantics' stamp: 'ajh 3/11/2003 18:18'!
primitiveNode: aPrimitiveNode

	primitiveNode _ aPrimitiveNode! !

!RBMethodNode methodsFor: 'semantics' stamp: 'ajh 3/16/2003 08:32'!
scope

	^ scope ifNil: [
		self verifyIn: nil parseScope.
		scope
	  ]! !

!RBMethodNode methodsFor: 'semantics' stamp: 'ajh 2/26/2003 15:46'!
scope: aSemMethodScope

	scope _ aSemMethodScope! !

!RBMethodNode methodsFor: 'semantics' stamp: 'ajh 3/16/2003 13:40'!
verifyIn: classOrScope
	"Look up vars in classOrScope.  My tree will be annotated with bindings to SemScopes and SemVars."

	SemanticChecker new
		scope: classOrScope parseScope;
		visitNode: self! !

!RBMethodNode methodsFor: 'translate' stamp: 'ajh 3/10/2003 20:21'!
generate: trailer

	^ self generateIR compiledMethodWith: trailer! !

!RBMethodNode methodsFor: 'translate' stamp: 'ajh 3/10/2003 20:20'!
generateIR

	^ ir _ SemanticTranslator new
		visitNode: self;
		ir! !

!RBMethodNode methodsFor: 'translate' stamp: 'ajh 3/10/2003 20:20'!
ir

	^ ir ifNil: [self generateIR]! !

!RBMethodNode methodsFor: 'translate' stamp: 'ajh 3/3/2003 12:07'!
privIR: irMethod

	ir _ irMethod! !

!RBMethodNode methodsFor: 'debugging' stamp: 'ajh 2/27/2003 23:58'!
sourceMap
	"Return a mapping from bytecode pcs to source code ranges"

	^ self ir sourceMap! !

!RBMethodNode methodsFor: 'debugging' stamp: 'ajh 3/17/2003 11:11'!
tempNames
	"All temp names in context order"

	^ self ir tempKeys collect: [:semVar | semVar asString]! !

!RBMethodNode methodsFor: 'C translation' stamp: 'ajh 3/17/2003 01:11'!
asTranslationMethodOfClass: aClass

	^ TNodeTranslator new
		tMethodClass: aClass;
		visitNode: self! !


!RBProgramNodeVisitor methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/26/2003 18:34'!
acceptDoItNode: aDoItNode 
	self visitNode: aDoItNode body! !

!RBProgramNodeVisitor methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/3/2003 12:43'!
acceptPseudoNode: aVariableNode! !


!RBFormatter methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/19/2003 14:13'!
acceptArrayNode: anArrayNode

	anArrayNode start ifNil: [anArrayNode leftBrace: self fullPosition + 1].
	codeStream nextPutAll: '{ '.
	self indent: 1 while: [
		self indent.
		self formatStatementsFor: anArrayNode.
	].
	codeStream nextPutAll: '}'.
	anArrayNode stop ifNil: [anArrayNode rightBrace: self fullPosition].
! !

!RBFormatter methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/13/2003 15:09'!
acceptAssignmentNode: anAssignmentNode 
	self indent: 2
		while: 
			[self visitNode: anAssignmentNode variable.
			codeStream space; nextPutAll: self class assignmentOperator; space.
			self visitNode: anAssignmentNode value]! !

!RBFormatter methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/19/2003 15:50'!
acceptBlockNode: aBlockNode 
	| seqNode multiline formattedBody formatter start |
	seqNode := aBlockNode body.
	formatter := (self copy) lineStart: 0;
				yourself.
	start _ self fullPosition.
	formattedBody := formatter format: seqNode.
	multiline := self lineLength + formattedBody size > self maxLineSize
				or: [formatter isMultiLine].
	multiline ifTrue: [self indent].
	codeStream nextPut: $[.
	aBlockNode left ifNil: [aBlockNode left: self fullPosition].
	aBlockNode arguments do: 
			[:each | 
			codeStream nextPut: $:.
			self visitNode: each.
			codeStream nextPut: $ ].
	aBlockNode arguments isEmpty ifFalse: 
			[codeStream nextPutAll: '| '.
			multiline ifTrue: [self indent]].
	aBlockNode right ifNil: [
		"assume if block had no position seqNode had no position also"
		seqNode adjustPositionsAfter: start by: self fullPosition - start.
	].
	codeStream nextPutAll: formattedBody.
	codeStream nextPut: $].
	aBlockNode right ifNil: [aBlockNode right: self fullPosition].
! !

!RBFormatter methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/19/2003 22:11'!
acceptLiteralNode: aLiteralNode

	| start |
	start _ self fullPosition + 1.
	s _ codeStream position + 1.
	self formatLiteral: aLiteralNode value.
	aLiteralNode token start ifNil: [
		aLiteralNode token
			start: start;
			value: (codeStream originalContents copyFrom: s to: codeStream position)
	].
! !

!RBFormatter methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/24/2003 13:15'!
acceptPseudoNode: aPseudoNode

	aPseudoNode isLabel ifTrue: [
		codeStream nextPut: $L.
		aPseudoNode destination printOn: codeStream.
		^ self].
	aPseudoNode isGoto ifTrue: [
		codeStream nextPut: $G.
		aPseudoNode destination printOn: codeStream.
		^ self].
	aPseudoNode isIf ifTrue: [
		codeStream nextPutAll: 'If '.
		codeStream nextPut: (aPseudoNode boolean ifTrue: [$t] ifFalse: [$f]).
		codeStream space.
		aPseudoNode destination printOn: codeStream.
		codeStream space.
		aPseudoNode otherwise printOn: codeStream.
		^ self].
! !

!RBFormatter methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/19/2003 14:13'!
acceptReturnNode: aReturnNode

	aReturnNode start ifNil: [aReturnNode return: self fullPosition + 1].
	codeStream nextPut: $^; space.
	self visitNode: aReturnNode value! !

!RBFormatter methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/19/2003 14:13'!
acceptVariableNode: aVariableNode

	aVariableNode token start ifNil: [
		aVariableNode token start: self fullPosition + 1].
	codeStream nextPutAll: aVariableNode name! !

!RBFormatter methodsFor: 'private-formatting' stamp: 'ajh 3/15/2003 01:22'!
formatLiteral: aValue 
	| isArray |
	(isArray := aValue class == Array) | (aValue class == ByteArray) ifTrue: 
			[codeStream nextPutAll: (isArray ifTrue: ['#('] ifFalse: ['#[']).
			self for: aValue
				do: [:each | self formatLiteral: each]
				separatedBy: [codeStream nextPut: $ ].
			codeStream nextPut: (isArray ifTrue: [$)] ifFalse: [$]]).
			^self].
	aValue isSymbol ifTrue: 
			[self formatSymbol: aValue.
			^self].
	aValue class == Character ifTrue: 
			[codeStream nextPut: $$;
				nextPut: aValue.
			^self].
	aValue isVariableBinding ifTrue: [
		codeStream nextPutAll: '##'; nextPutAll: aValue key.
		^ self].
	aValue storeOn: codeStream! !

!RBFormatter methodsFor: 'private-formatting' stamp: 'ajh 3/19/2003 16:03'!
formatMessage: aMessageNode cascade: cascadeBoolean 
	| selectorParts arguments multiLine formattedArgs indentFirst firstArgLength length start needsPos argStarts |
	selectorParts := aMessageNode selectorParts.
	needsPos _ selectorParts first start isNil.
	arguments := aMessageNode arguments.
	formattedArgs := OrderedCollection new.
	multiLine := aMessageNode selector numArgs > self maximumArgumentsPerLine.
	length := aMessageNode selector size + arguments size + 1.
	firstArgLength := 0.
	start _ self fullPosition.
	self indentWhile: 
			[1 to: arguments size
				do: 
					[:i | 
					| formatter string |
					formatter := (self copy) 
								lineStart: (selectorParts at: i) length negated;
								yourself.
					string := formatter format: (arguments at: i).
					formattedArgs add: string.
					i == 1 ifTrue: [firstArgLength := formatter firstLineLength].
					length := length + string size.
					multiLine := multiLine or: [formatter isMultiLine]]].
	multiLine := multiLine or: [length + self lineLength > self maxLineSize].
	indentFirst := cascadeBoolean not and: 
					[multiLine and: 
							[(self startMessageSendOnNewLine: aMessageNode) or: 
									[self lineLength + selectorParts first length + 2 + firstArgLength 
										> self maxLineSize]]].
	indentFirst ifTrue: [self indent].
	argStarts _ self 
		formatMessageSelector: selectorParts
		withArguments: formattedArgs
		multiline: multiLine.
	needsPos ifTrue: [
		arguments with: argStarts do: [:node :pos |
			node adjustPositionsAfter: start by: pos - start].
	].
! !

!RBFormatter methodsFor: 'private-formatting' stamp: 'ajh 3/19/2003 16:06'!
formatMessageSelector: selectorParts withArguments: formattedArgs multiline: multiLine

	| selectorPart argStarts |
	argStarts _ Array new: formattedArgs size.
	formattedArgs isEmpty 
		ifTrue: [
			selectorParts first start ifNil: [
				selectorParts first start: self fullPosition + 1].
			codeStream nextPutAll: selectorParts first value]
		ifFalse: 
			[1 to: formattedArgs size
				do: 
					[:i | 
					i ~~ 1 & multiLine not ifTrue: [codeStream nextPut: $ ].
					selectorPart _ selectorParts at: i.
					selectorPart start ifNil: [selectorPart start: self fullPosition + 1].
					codeStream 
						nextPutAll: selectorPart value;
						nextPut: $ .
					argStarts at: i put: self fullPosition.
					codeStream nextPutAll: (formattedArgs at: i).
					(multiLine and: [i < formattedArgs size]) ifTrue: [self indent]]].
	^ argStarts! !

!RBFormatter methodsFor: 'private-formatting' stamp: 'ajh 3/19/2003 14:15'!
formatMethodPatternFor: aMethodNode 
	| selectorParts arguments |
	selectorParts := aMethodNode selectorParts.
	arguments := aMethodNode arguments.
	arguments isEmpty
		ifTrue: [
			selectorParts first start ifNil: [selectorParts first start: self fullPosition + 1].
			codeStream nextPutAll: selectorParts first value]
		ifFalse: 
			[selectorParts with: arguments
				do: 
					[:selector :arg | 
					selector start ifNil: [selector start: self fullPosition + 1].
					codeStream nextPutAll: selector value;
						nextPut: $ .
					self visitArgument: arg.
					codeStream nextPut: $ ]]! !

!RBFormatter methodsFor: 'initialize-release' stamp: 'ajh 3/19/2003 14:03'!
initialize
	super initialize.
	codeStream := WriteStream on: (String new: 60).
	tabs := 0.
	lineStart := 0.
	positionDelta _ 0.
! !

!RBFormatter methodsFor: 'copying' stamp: 'ajh 3/19/2003 14:03'!
postCopy
	super postCopy.
	positionDelta _ positionDelta + codeStream position.
	lineStart := self lineLength negated.
	codeStream := WriteStream on: (String new: 60).
	firstLineLength := nil! !

!RBFormatter methodsFor: 'position fill' stamp: 'ajh 3/19/2003 14:12'!
fullPosition

	^ positionDelta + codeStream position! !


!RBFormatter class methodsFor: 'as yet unclassified' stamp: 'ajh 3/13/2003 18:55'!
assignmentOperator

	^ RBParser isSqueak
		ifTrue: ['_']
		ifFalse: [':=']! !


!RBPseudoNode methodsFor: 'visitor' stamp: 'ajh 3/17/2003 00:25'!
acceptVisitor: aProgramNodeVisitor
	^ aProgramNodeVisitor acceptPseudoNode: self! !

!RBPseudoNode methodsFor: 'testing' stamp: 'ajh 3/3/2003 18:41'!
isPseudo

	^ true! !

!RBPseudoNode methodsFor: 'accessing' stamp: 'ajh 3/20/2003 17:20'!
mapInstr

	^ mapInstr! !

!RBPseudoNode methodsFor: 'accessing' stamp: 'ajh 3/20/2003 17:19'!
mapInstr: irInstr

	mapInstr _ irInstr! !


!RBPseudoDupNode methodsFor: 'testing' stamp: 'ajh 3/2/2003 23:24'!
isDup

	^ true! !


!RBPseudoGotoNode methodsFor: 'testing' stamp: 'ajh 3/2/2003 23:23'!
isGoto

	^ true! !

!RBPseudoGotoNode methodsFor: 'testing' stamp: 'ajh 3/20/2003 19:05'!
isRet
	"is return"

	^ self destination = #return! !

!RBPseudoGotoNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/3/2003 21:20'!
destination

	^ destination! !

!RBPseudoGotoNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/9/2003 13:44'!
destination: seqNum

	destination _ seqNum! !

!RBPseudoGotoNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/20/2003 01:16'!
forValue
	"true if sequence before me is for value, false if for effect"

	^ forValue and: [self isRet not]! !

!RBPseudoGotoNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/4/2003 22:18'!
forValue: boolean
	"true if sequence before me is for value, false if for effect"

	forValue _ boolean! !


!RBPseudoIfNode methodsFor: 'testing' stamp: 'ajh 3/1/2003 20:13'!
isIf

	^ true! !

!RBPseudoIfNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/3/2003 21:20'!
boolean

	^ boolean! !

!RBPseudoIfNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/1/2003 23:08'!
boolean: bool

	boolean _ bool! !

!RBPseudoIfNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/3/2003 21:20'!
destination

	^ destination! !

!RBPseudoIfNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/9/2003 13:45'!
destination: seqNum

	destination _ seqNum! !

!RBPseudoIfNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/3/2003 21:20'!
otherwise

	^ otherwise! !

!RBPseudoIfNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/2/2003 23:19'!
otherwise: instructionSequence

	otherwise _ instructionSequence! !


!RBPseudoLabelNode methodsFor: 'testing' stamp: 'ajh 3/2/2003 23:23'!
isLabel

	^ true! !

!RBPseudoLabelNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/3/2003 21:21'!
destination

	^ destination! !

!RBPseudoLabelNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/9/2003 13:45'!
destination: seqNum

	destination _ seqNum! !


!RBPseudoPopNode methodsFor: 'testing' stamp: 'ajh 3/2/2003 23:23'!
isPop

	^ true! !


!RBPseudoSendNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/23/2003 22:23'!
isPseudoSend

	^ true! !

!RBPseudoSendNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/23/2003 22:23'!
selector

	^ selector! !

!RBPseudoSendNode methodsFor: 'as yet unclassified' stamp: 'ajh 3/23/2003 22:23'!
selector: aSelector

	selector _ aSelector! !


!RBReturnNode methodsFor: 'replacing' stamp: 'ajh 3/19/2003 16:16'!
adjustPositionsAfter: sourcePos by: delta
	"Slide token positions after sourcePos by delta"

	self return: return + delta.
	super adjustPositionsAfter: sourcePos by: delta.
! !

!RBReturnNode methodsFor: 'initialize-release' stamp: 'ajh 3/19/2003 13:51'!
return: returnInteger
	return := returnInteger! !

!RBReturnNode methodsFor: 'semantics' stamp: 'ajh 2/26/2003 14:48'!
homeBinding

	^  homeBinding! !

!RBReturnNode methodsFor: 'semantics' stamp: 'ajh 2/26/2003 14:40'!
homeBinding: aSemVar

	homeBinding _ aSemVar! !


!RBSequenceNode methodsFor: 'accessing' stamp: 'ajh 3/16/2003 18:24'!
stop

	^ ((periods isNil or: [periods isEmpty])
		ifTrue: [0]
		ifFalse: [periods last])
		max: (statements isEmpty
				ifTrue: [0]
				ifFalse: [statements last stop])! !


!RBVariableNode methodsFor: 'accessing' stamp: 'ajh 3/13/2003 15:17'!
token

	^ token! !

!RBVariableNode methodsFor: 'semantics' stamp: 'ajh 2/25/2003 00:50'!
binding

	^ binding! !

!RBVariableNode methodsFor: 'semantics' stamp: 'ajh 2/25/2003 23:41'!
binding: aSemVar

	binding _ aSemVar! !

!RBVariableNode methodsFor: 'replacing' stamp: 'ajh 3/13/2003 16:13'!
adjustPositionsAfter: sourcePos by: delta
	"Slide token positions after sourcePos by delta"

	token start > sourcePos ifTrue: [
		token start: token start + delta]! !

!RBVariableNode methodsFor: 'replacing' stamp: 'ajh 3/17/2003 13:12'!
changeToken: newToken

	self root adjustPositionsAfter: token stop by: newToken stop - token stop.
	token _ newToken.
! !


!Scanner class methodsFor: 'testing' stamp: 'ajh 3/24/2003 14:10'!
isLiteralSymbol: aSymbol 
	"Test whether a symbol can be stored as # followed by its characters.  
	Symbols created internally with asSymbol may not have this property, 
	e.g. '3' asSymbol."
	| i ascii type |
	i _ aSymbol size.
	i = 0 ifTrue: [^ false].
	ascii _ (aSymbol at: 1) asciiValue.
	"TypeTable should have been origined at 0 rather than 1 ..."
	ascii = 0 ifTrue: [^ false].
	type _ TypeTable at: ascii.
	(type == #xBinary or: [type == #verticalBar]) ifTrue: 
			[[i > 1]
				whileTrue: 
					[ascii _ (aSymbol at: i) asciiValue.
					ascii = 0 ifTrue: [^ false].
					type _ TypeTable at: ascii.
					(type == #xBinary or: [type == #verticalBar]) ifFalse: [^ false].
					i _ i - 1].
			^ true].
	type == #xLetter ifTrue: 
			[[i > 1]
				whileTrue: 
					[ascii _ (aSymbol at: i) asciiValue.
					ascii = 0 ifTrue: [^ false].
					type _ TypeTable at: ascii.
					(type == #xLetter or: [type == #xDigit or: [type == #xColon]])
						ifFalse: [^ false].
					i _ i - 1].
			^ true].
	^ false! !


!SemScope methodsFor: 'levels' stamp: 'ajh 3/18/2003 19:38'!
newBlockScope

	^ SemContextScope new
		outerScope: self;
		asBlockScope! !

!SemScope methodsFor: 'levels' stamp: 'ajh 3/18/2003 15:29'!
newMethodScope

	^ SemContextScope new
		outerScope: self;
		asMethodScope! !

!SemScope methodsFor: 'levels' stamp: 'ajh 2/26/2003 13:44'!
outerScope

	^ outerScope! !

!SemScope methodsFor: 'levels' stamp: 'ajh 3/20/2003 11:27'!
scopeLevel

	outerScope ifNil: [^ 0].
	^ outerScope scopeLevel + 1! !

!SemScope methodsFor: 'adding' stamp: 'ajh 2/25/2003 22:44'!
addSelector: string

	^ string asSymbol! !

!SemScope methodsFor: 'lookup' stamp: 'ajh 2/27/2003 00:40'!
classEncoding

	^ self outerScope classEncoding! !

!SemScope methodsFor: 'lookup' stamp: 'ajh 2/25/2003 22:41'!
lookupSelector: name

	Symbol hasInterned: name ifTrue: [ :sym | ^ sym].
	^ nil! !

!SemScope methodsFor: 'lookup' stamp: 'ajh 3/11/2003 20:19'!
lookupVar: name
	"subclass responsibility"

	^ self outerScope lookupVar: name! !

!SemScope methodsFor: 'lookup' stamp: 'ajh 2/25/2003 22:40'!
possibleSelectorsFor: string

	^ Symbol possibleSelectorsFor: string! !

!SemScope methodsFor: 'lookup' stamp: 'ajh 2/25/2003 22:39'!
possibleVarsFor: string

	| list |
	list _ self possibleVarsFor: string continued: nil.
	^ string correctAgainst: nil continuedFrom: list
! !

!SemScope methodsFor: 'lookup' stamp: 'ajh 3/11/2003 20:17'!
possibleVarsFor: name continued: listOrNil
	"subclass responsibility"

	^ self outerScope possibleVarsFor: name continued: listOrNil! !

!SemScope methodsFor: 'lookup' stamp: 'ajh 3/25/2003 23:38'!
rawVar: name
	"subclass responsibility"

	^ self outerScope rawVar: name! !

!SemScope methodsFor: 'initializing' stamp: 'ajh 2/25/2003 21:27'!
initialize
! !

!SemScope methodsFor: 'initializing' stamp: 'ajh 2/26/2003 13:43'!
outerScope: aSemScope

	outerScope _ aSemScope! !

!SemScope methodsFor: 'initializing' stamp: 'ajh 2/26/2003 20:01'!
parseScope

	^ self! !

!SemScope methodsFor: 'old protocol' stamp: 'ajh 2/25/2003 20:09'!
requestor: editor
	"Old protocol, do nothing"
! !


!SemClassScope methodsFor: 'initializing' stamp: 'ajh 2/25/2003 20:10'!
class: aBehavior

	class _ aBehavior! !

!SemClassScope methodsFor: 'levels' stamp: 'ajh 3/18/2003 15:53'!
newMethodScope

	| scope |
	scope _ super newMethodScope.
	scope fieldVarNames: class allInstVarNames startingAt: 1.
	^ scope! !

!SemClassScope methodsFor: 'lookup' stamp: 'ajh 2/25/2003 20:10'!
classEncoding

	^ class! !

!SemClassScope methodsFor: 'lookup' stamp: 'ajh 3/18/2003 15:41'!
lookupVar: name
	"Return a SemVar for my pool var with this name.  Return nil if none found"

	class scopeHas: name asSymbol ifTrue:
		[:assoc | ^ SemPoolVar newFor: assoc].
	^ nil
! !

!SemClassScope methodsFor: 'lookup' stamp: 'ajh 3/18/2003 15:42'!
possibleVarsFor: name continued: listOrNil
	"Return my pool var names that are close to name"

	name first isUppercase ifFalse: [^ listOrNil].
	^ class possibleVariablesFor: name continuedFrom: listOrNil! !

!SemClassScope methodsFor: 'lookup' stamp: 'ajh 3/25/2003 23:38'!
rawVar: name
	"Return a SemVar for my pool var with this name.  Return nil if none found"

	class scopeHas: name asSymbol ifTrue:
		[:assoc | ^ SemPoolVar newFor: assoc].
	^ nil
! !

!SemClassScope methodsFor: 'printing' stamp: 'ajh 2/25/2003 20:10'!
printOn: stream

	class printOn: stream.
	stream nextPutAll: ' parseScope'.
! !


!SemContextScope methodsFor: 'levels' stamp: 'ajh 3/18/2003 14:09'!
popScope
	"Propogate free var usages to their outer vars, then return outer scope"

	self fieldVars do: [:v | v markOuterUsage].
	^ self outerScope! !

!SemContextScope methodsFor: 'adding' stamp: 'ajh 3/19/2003 10:52'!
addTemp: name

	^ vars at: name put: (
		SemTempVar new
			name: name;
			index: self numTemps + 1;
			scope: self
	  )! !

!SemContextScope methodsFor: 'lookup' stamp: 'ajh 5/9/2003 14:41'!
lookupVar: name
	"Return the SemVar with this name.  If found in outer scope and self is not inlined then create captured var in self that references it.  If none found return nil."

	| nm |
	nm _ name = 'super' ifTrue: ['self'] ifFalse: [name].
	^ vars at: nm ifAbsent: [
		| var |
		var _ self outerScope lookupVar: nm.
		(var notNil and: [isInlined not]) ifTrue: [
			var _ var asCapturedVarIn: self.
			vars at: nm put: var.
		].
		var
	  ]! !

!SemContextScope methodsFor: 'lookup' stamp: 'ajh 3/18/2003 13:18'!
possibleVarsFor: name continued: listOrNil
	"Return my var names that are close in spelling to name"

	| list |
	list _ name first isLowercase
		ifTrue: [name correctAgainstDictionary: vars continuedFrom: listOrNil]
		ifFalse: [listOrNil].
	^ self outerScope possibleVarsFor: name continued: list! !

!SemContextScope methodsFor: 'lookup' stamp: 'ajh 3/25/2003 23:38'!
rawVar: name
	"Return the SemVar with this name without wrapping in captured var or adding to local dictionary"

	| nm |
	nm _ name = 'super' ifTrue: ['self'] ifFalse: [name].
	^ vars at: nm ifAbsent: [self outerScope rawVar: nm]! !

!SemContextScope methodsFor: 'accessing' stamp: 'ajh 3/18/2003 14:08'!
capturedVars

	^ self fieldVars collect: [:var | var outerVar]! !

!SemContextScope methodsFor: 'accessing' stamp: 'ajh 3/18/2003 17:05'!
fieldVars

	^ (vars select: [:var | var isFieldVar and: [var receiverVar == self receiverVar]])
		asSortedCollection: [:x :y | x index <= y index]! !

!SemContextScope methodsFor: 'accessing' stamp: 'ajh 5/9/2003 14:48'!
isBlockScope

	^ self thisContextVar name == #thisContext  "see asBlockScope"! !

!SemContextScope methodsFor: 'accessing' stamp: 'ajh 5/28/2003 01:10'!
isInlined

	^ isInlined! !

!SemContextScope methodsFor: 'accessing' stamp: 'ajh 5/28/2003 01:21'!
nonInlinedScope

	^ isInlined
		ifTrue: [self outerScope nonInlinedScope]
		ifFalse: [self]! !

!SemContextScope methodsFor: 'accessing' stamp: 'ajh 3/18/2003 19:25'!
numFields

	| max |
	max _ 0.
	vars do: [:var |
		(var isFieldVar and: [var receiverVar == self receiverVar]) ifTrue: [
			var index > max ifTrue: [
				max _ var index]].
	].
	^ self isBlockScope
		ifTrue: [max max: BlockClosure instSize]
		ifFalse: [max]! !

!SemContextScope methodsFor: 'accessing' stamp: 'ajh 3/18/2003 14:29'!
numTemps

	| max |
	max _ 0.
	vars do: [:var |
		var isTemp ifTrue: [
			var index > max ifTrue: [
				max _ var index]].
	].
	^ max! !

!SemContextScope methodsFor: 'accessing' stamp: 'ajh 5/9/2003 14:49'!
receiverVar

	^ isInlined
		ifTrue: [self outerScope receiverVar]
		ifFalse: [receiverVar]! !

!SemContextScope methodsFor: 'accessing' stamp: 'ajh 3/19/2003 01:57'!
tempVars

	^ (vars select: [:var | var isTemp and: [var index > 0]])
		asSortedCollection: [:x :y | x index <= y index]! !

!SemContextScope methodsFor: 'accessing' stamp: 'ajh 5/9/2003 14:49'!
thisContextVar

	^ isInlined
		ifTrue: [self outerScope thisContextVar]
		ifFalse: [thisContextVar]! !

!SemContextScope methodsFor: 'initializing' stamp: 'ajh 3/21/2003 00:53'!
asBlockScope

	thisContextVar name: #thisContext.  "see isBlockScope"
	vars at: 'thisContext' put: thisContextVar.
	receiverVar name: 'this closure'.
	vars at: 'this closure' put: receiverVar.
! !

!SemContextScope methodsFor: 'initializing' stamp: 'ajh 3/21/2003 00:52'!
asMethodScope

	thisContextVar name: 'home context'.
	vars at: 'thisContext' put: thisContextVar.
	vars at: 'home context' put: thisContextVar.
	receiverVar name: 'self'.
	vars at: 'self' put: receiverVar.
	vars at: 'super' put: receiverVar.
! !

!SemContextScope methodsFor: 'initializing' stamp: 'ajh 3/18/2003 19:26'!
fieldVarNames: names startingAt: index

	| i |
	i _ index.
	names do: [:name |
		(vars includesKey: name) ifTrue: [self errorNameClash: name].
		vars at: name put: (SemFieldVar new
			name: name;
			receiverVar: self receiverVar;
			index: i).
		i _ i + 1.
	].
! !

!SemContextScope methodsFor: 'initializing' stamp: 'ajh 3/24/2003 21:04'!
initClosureReceiver

	receiverVar name: 'thisClosure'.
	vars at: 'thisClosure' put: receiverVar.
	vars at: 'home context' put: thisContextVar.
! !

!SemContextScope methodsFor: 'initializing' stamp: 'ajh 3/24/2003 21:04'!
initContextReceiver

	receiverVar name: 'thisContext'.
	vars at: 'thisContext' put: receiverVar.
	vars at: 'home context' put: thisContextVar.
! !

!SemContextScope methodsFor: 'initializing' stamp: 'ajh 5/9/2003 14:34'!
initialize

	isInlined _ false.
	vars _ Dictionary new.
	thisContextVar _ SemTempVar new
		name: 'this context';
		index: -1;
		scope: self;
		markArg.
	receiverVar _ SemTempVar new
		name: 'this receiver';
		index: 0;
		scope: self;
		markArg.
! !

!SemContextScope methodsFor: 'initializing' stamp: 'ajh 5/9/2003 14:50'!
markInlined

	isInlined _ true.
	receiverVar _ nil.
	thisContextVar _ nil.
! !

!SemContextScope methodsFor: 'decompiling' stamp: 'ajh 3/18/2003 15:33'!
capturedVars: outerVars

	outerVars do: [:var |
		(vars includesKey: var name) ifTrue: [self errorNameClash: var].
		vars at: var name put: (var asCapturedVarIn: self).
	].
! !

!SemContextScope methodsFor: 'decompiling' stamp: 'ajh 3/18/2003 14:23'!
receiverVarAt: instVarIndex

	^ vars detect: [:var | var isFieldVar and: [var index = instVarIndex]] ifNone: [
		| name |
		name _ 'f', instVarIndex asString.
		vars at: name put: 
			(SemFieldVar new
				name: name;
				receiverVar: self receiverVar;
				index: instVarIndex)
	  ]! !

!SemContextScope methodsFor: 'decompiling' stamp: 'ajh 3/20/2003 11:33'!
tempVarAt: tempIndex

	tempIndex = -1 ifTrue: [^ self thisContextVar].
	tempIndex = 0 ifTrue: [^ self receiverVar].
	^ vars detect: [:var | var isTemp and: [var index = tempIndex]] ifNone: [
		| name |
		name _ String streamContents: [:str |
			self scopeLevel timesRepeat: [str nextPut: $t].
			tempIndex printOn: str].
		vars at: name put: 
			(SemTempVar new
				name: name;
				index: tempIndex;
				scope: self)
	  ]! !

!SemContextScope methodsFor: 'printing' stamp: 'ajh 3/20/2003 11:29'!
printOn: stream

	super printOn: stream.
	stream space.
	self scopeLevel printOn: stream.
! !


!SemNilScope methodsFor: 'lookup' stamp: 'ajh 2/26/2003 13:42'!
classEncoding
	"subclass responsibility, default is nil"

	^ nil! !

!SemNilScope methodsFor: 'lookup' stamp: 'ajh 3/11/2003 16:11'!
lookupVar: name
	"subclass responsibility, default is to look up global vars"

	ProtoObject scopeHas: name asSymbol ifTrue:
		[:assoc | ^ SemPoolVar newFor: assoc].
	^ nil
! !

!SemNilScope methodsFor: 'lookup' stamp: 'ajh 3/12/2003 12:10'!
possibleVarsFor: name continued: listOrNil
	"subclass responsibility, default is to look up global vars"

	name first isLowercase ifTrue: [^ listOrNil].
	^ ProtoObject possibleVariablesFor: name continuedFrom: listOrNil! !

!SemNilScope methodsFor: 'lookup' stamp: 'ajh 3/25/2003 23:38'!
rawVar: name
	"subclass responsibility, default is to look up global vars"

	ProtoObject scopeHas: name asSymbol ifTrue:
		[:assoc | ^ SemPoolVar newFor: assoc].
	^ nil
! !


!SemScope class methodsFor: 'as yet unclassified' stamp: 'ajh 2/25/2003 21:26'!
new

	^ super new initialize! !


!SemNilScope class methodsFor: 'as yet unclassified' stamp: 'ajh 2/26/2003 13:42'!
new

	^ super new initialize! !


!SemVar methodsFor: 'initializing' stamp: 'ajh 3/18/2003 12:38'!
asCapturedVarIn: blockScope

	^ self! !

!SemVar methodsFor: 'initializing' stamp: 'ajh 3/18/2003 12:11'!
initialize

	usage _ #none! !

!SemVar methodsFor: 'initializing' stamp: 'ajh 2/25/2003 21:29'!
name: string

	name _ string! !

!SemVar methodsFor: 'accessing' stamp: 'ajh 3/16/2003 20:08'!
asString

	^ self name! !

!SemVar methodsFor: 'accessing' stamp: 'ajh 3/18/2003 12:13'!
isFieldVar

	^ false! !

!SemVar methodsFor: 'accessing' stamp: 'ajh 3/18/2003 11:13'!
isTemp

	^ false! !

!SemVar methodsFor: 'accessing' stamp: 'ajh 3/12/2003 13:40'!
isUnused

	^ false! !

!SemVar methodsFor: 'accessing' stamp: 'ajh 2/25/2003 21:56'!
name

	^ name! !

!SemVar methodsFor: 'accessing' stamp: 'ajh 2/27/2003 00:38'!
printOn: stream

	stream nextPutAll: self name! !

!SemVar methodsFor: 'emitting' stamp: 'ajh 2/25/2003 00:26'!
emitStore: methodBuilder

	self subclassResponsibility! !

!SemVar methodsFor: 'emitting' stamp: 'ajh 2/25/2003 18:23'!
emitValue: methodBuilder

	self subclassResponsibility! !

!SemVar methodsFor: 'read/write usage' stamp: 'ajh 3/18/2003 12:15'!
markRead

	self markUse: #read! !

!SemVar methodsFor: 'read/write usage' stamp: 'ajh 3/18/2003 12:15'!
markUse: usageSymbol

	usage _ UsageAutomaton state: usage transition: usageSymbol! !

!SemVar methodsFor: 'read/write usage' stamp: 'ajh 3/18/2003 12:15'!
markWrite

	self markUse: #write! !


!SemFieldVar methodsFor: 'initializing' stamp: 'ajh 3/18/2003 12:20'!
index: int

	index _ int! !

!SemFieldVar methodsFor: 'initializing' stamp: 'ajh 3/18/2003 12:18'!
outerVar: outerTempOrFieldVar
	"When self is a free var captured from outerTempOrFieldVar. otherwise outerVar remains nil"

	outerVar _ outerTempOrFieldVar! !

!SemFieldVar methodsFor: 'initializing' stamp: 'ajh 3/18/2003 12:21'!
receiverVar: aSemVar

	receiverVar _ aSemVar! !

!SemFieldVar methodsFor: 'accessing' stamp: 'ajh 3/19/2003 00:56'!
asCapturedVarIn: blockScope

	^ self originalTempVar ifNil: [
		SemFieldVar new
			name: self name;
			receiverVar: ((blockScope lookupVar: self receiverVar name) ifNil: [self errorNotFound]);
			index: self index;
			outerVar: self
	  ] ifNotNil: [
		SemFieldVar new
			name: self name;
			receiverVar: blockScope receiverVar;
			index: blockScope numFields + 1;
			outerVar: self
	  ]! !

!SemFieldVar methodsFor: 'accessing' stamp: 'ajh 3/18/2003 12:20'!
index

	^ index! !

!SemFieldVar methodsFor: 'accessing' stamp: 'ajh 3/18/2003 12:13'!
isFieldVar

	^ true! !

!SemFieldVar methodsFor: 'accessing' stamp: 'ajh 3/18/2003 12:20'!
originalTempVar

	^ outerVar ifNotNil: [outerVar originalTempVar]! !

!SemFieldVar methodsFor: 'accessing' stamp: 'ajh 3/18/2003 12:20'!
outerVar

	^ outerVar! !

!SemFieldVar methodsFor: 'accessing' stamp: 'ajh 3/18/2003 12:21'!
receiverVar

	^ receiverVar! !

!SemFieldVar methodsFor: 'read/write usage' stamp: 'ajh 3/18/2003 13:03'!
closureUsage
	"Return #readBlock, #writeBlock, or #indirect"

	(#(read readBlock) identityIncludes: usage) ifTrue: [^ #readBlock].
	(#(write wrRdBlock) identityIncludes: usage) ifTrue: [^ #writeBlock].
	(#(writeBlock indirect)  identityIncludes: usage) ifTrue: [^ #indirect].
	self error: usage printString, ', unexpected closureVar useState'
! !

!SemFieldVar methodsFor: 'read/write usage' stamp: 'ajh 3/18/2003 12:19'!
escapes

	^ usage == #indirect or: [outerVar notNil and: [outerVar escapes]]! !

!SemFieldVar methodsFor: 'read/write usage' stamp: 'ajh 3/18/2003 13:03'!
isUnused

	^ usage == #none and: [outerVar notNil and: [outerVar isUnused]]! !

!SemFieldVar methodsFor: 'read/write usage' stamp: 'ajh 3/19/2003 00:40'!
markEscape

	usage _ #indirect! !

!SemFieldVar methodsFor: 'read/write usage' stamp: 'ajh 3/18/2003 13:03'!
markOuterUsage

	outerVar markUse: self closureUsage! !

!SemFieldVar methodsFor: 'read/write usage' stamp: 'ajh 3/19/2003 01:39'!
markRead

	self originalTempVar
		ifNil: [self receiverVar markRead]
		ifNotNil: [super markRead]! !

!SemFieldVar methodsFor: 'read/write usage' stamp: 'ajh 3/19/2003 01:39'!
markWrite

	self originalTempVar
		ifNil: [self receiverVar markRead]
		ifNotNil: [super markWrite]! !

!SemFieldVar methodsFor: 'emitting' stamp: 'ajh 3/18/2003 12:53'!
emitStorageValue: methodBuilder

	self receiverVar emitValue: methodBuilder.
	methodBuilder getField: self index.
! !

!SemFieldVar methodsFor: 'emitting' stamp: 'ajh 3/18/2003 12:53'!
emitStore: methodBuilder

	self receiverVar emitValue: methodBuilder.
	self escapes ifTrue: [
		methodBuilder getField: self index.
		methodBuilder send: #storeInTempHolder:.
	] ifFalse: [
		methodBuilder setField: self index.
	].
! !

!SemFieldVar methodsFor: 'emitting' stamp: 'ajh 3/18/2003 12:54'!
emitValue: methodBuilder

	self receiverVar emitValue: methodBuilder.
	methodBuilder getField: self index.
	self escapes ifTrue: [methodBuilder send: #tempHolderValue].
! !


!SemPoolVar methodsFor: 'initializing' stamp: 'ajh 2/26/2003 17:31'!
assoc: anAssociation

	assoc _ anAssociation! !

!SemPoolVar methodsFor: 'emitting' stamp: 'ajh 2/26/2003 17:31'!
emitStore: methodBuilder

	methodBuilder pushLiteral: assoc.
	methodBuilder send: #setInPool:.
! !

!SemPoolVar methodsFor: 'emitting' stamp: 'ajh 2/26/2003 17:31'!
emitValue: methodBuilder

	methodBuilder pushLiteral: assoc.
	methodBuilder send: #poolValue.
! !


!SemTempVar methodsFor: 'initializing' stamp: 'ajh 3/18/2003 11:29'!
index: n

	index _ n! !

!SemTempVar methodsFor: 'initializing' stamp: 'ajh 3/18/2003 11:30'!
scope: aSemClosureScope

	scope _ aSemClosureScope! !

!SemTempVar methodsFor: 'accessing' stamp: 'ajh 3/18/2003 12:39'!
asCapturedVarIn: blockScope

	^ SemFieldVar new
		name: self name;
		receiverVar: blockScope receiverVar;
		index: blockScope numFields + 1;
		outerVar: self
! !

!SemTempVar methodsFor: 'accessing' stamp: 'ajh 3/18/2003 11:30'!
index

	^ index! !

!SemTempVar methodsFor: 'accessing' stamp: 'ajh 2/26/2003 15:19'!
isTemp

	^ true! !

!SemTempVar methodsFor: 'accessing' stamp: 'ajh 3/17/2003 10:23'!
originalTempVar

	^ self! !

!SemTempVar methodsFor: 'accessing' stamp: 'ajh 3/18/2003 11:30'!
scope

	^ scope! !

!SemTempVar methodsFor: 'emitting' stamp: 'ajh 3/13/2003 04:07'!
emitArgInit: methodBuilder

	self escapes ifTrue: [
		methodBuilder
			pushTemp: self;
			send: #wrapInTempHolder;
			storeTemp: self;
			popTop
	].
! !

!SemTempVar methodsFor: 'emitting' stamp: 'ajh 2/26/2003 17:41'!
emitStorageValue: method
	"Push value to be captured by embedded block"

	method pushTemp: self.
	self = self scope thisContextVar ifTrue: [
		"Push contextTag to be captured"
		method send: #contextTag].
! !

!SemTempVar methodsFor: 'emitting' stamp: 'ajh 2/26/2003 17:23'!
emitStore: methodBuilder

	self escapes
		ifTrue: [methodBuilder pushTemp: self.
				methodBuilder send: #storeInTempHolder:]
		ifFalse: [methodBuilder storeTemp: self].
! !

!SemTempVar methodsFor: 'emitting' stamp: 'ajh 3/13/2003 04:07'!
emitTempInit: methodBuilder

	self escapes ifTrue: [
		methodBuilder
			pushLiteral: nil;
			send: #wrapInTempHolder;
			storeTemp: self;
			popTop
	].
! !

!SemTempVar methodsFor: 'emitting' stamp: 'ajh 2/26/2003 17:25'!
emitValue: methodBuilder

	methodBuilder pushTemp: self.
	self escapes ifTrue: [methodBuilder send: #tempHolderValue].
! !

!SemTempVar methodsFor: 'read/write usage' stamp: 'ajh 3/19/2003 00:40'!
escapes
	"If a block captures me and assigns to me then my storage escapes from the context allowing it to exist after the context with the block"

	^ usage == #indirect! !

!SemTempVar methodsFor: 'read/write usage' stamp: 'ajh 3/13/2003 02:04'!
isUnused

	^ usage == #none! !

!SemTempVar methodsFor: 'read/write usage' stamp: 'ajh 3/12/2003 13:02'!
markArg

	usage _ #arg! !

!SemTempVar methodsFor: 'read/write usage' stamp: 'ajh 3/18/2003 12:15'!
markEscape

	usage _ #indirect! !


!SemVar class methodsFor: 'as yet unclassified' stamp: 'ajh 3/18/2003 12:12'!
initialize

	self initializeAutomata.
! !

!SemVar class methodsFor: 'as yet unclassified' stamp: 'ajh 3/18/2003 12:11'!
initializeAutomata
	"SemVar initializeAutomata"

UsageAutomaton _ FiniteAutomaton new

actions:			#(read		write		readBlock	writeBlock	indirect)
transitions: 
#("from state"	"to states"
 (none			(read		write		readBlock	writeBlock	indirect))
 (arg			(read		write		readBlock	writeBlock	indirect))
 (read			(read		write		readBlock	writeBlock	indirect))
 (write			(write		write		wrRdBlock	writeBlock	indirect))
 (readBlock		(readBlock	indirect		readBlock	indirect		indirect))
 (wrRdBlock		(wrRdBlock	indirect		wrRdBlock	indirect		indirect))
 (writeBlock		(indirect	indirect		indirect		indirect		indirect))
 (indirect		(indirect	indirect		indirect		indirect		indirect))
).
! !

!SemVar class methodsFor: 'as yet unclassified' stamp: 'ajh 3/18/2003 12:11'!
new

	^ super new initialize! !


!SemPoolVar class methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 16:11'!
newFor: assoc

	^ self new name: assoc poolName; assoc: assoc! !


!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/25/2003 19:43'!
acceptArrayNode: anArrayNode 
	anArrayNode statements do: [:each | self visitNode: each]! !

!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/11/2003 18:21'!
acceptAssignmentNode: anAssignmentNode

	| var |
	self visitNode: anAssignmentNode value.
	var _ (scope lookupVar: anAssignmentNode variable name)
		ifNil: [self undeclaredVariable: anAssignmentNode variable].
	var markWrite.
	anAssignmentNode variable binding: var.
! !

!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 5/9/2003 14:56'!
acceptBlockNode: aBlockNode

	scope _ scope newBlockScope.
	aBlockNode scope: scope.
	aBlockNode isInlined ifTrue: [scope markInlined].
	aBlockNode arguments do: [:node | (self declareVariableNode: node) markArg].
	self visitNode: aBlockNode body.
	scope _ scope popScope.
! !

!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/25/2003 22:15'!
acceptCascadeNode: aCascadeNode 
	aCascadeNode messages do: [:each | self visitNode: each]! !

!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 15:34'!
acceptDoItNode: aDoItNode

	aDoItNode scope: scope.
	self visitNode: aDoItNode body.
! !

!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/25/2003 19:34'!
acceptLiteralNode: aLiteralNode! !

!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/11/2003 22:51'!
acceptMessageNode: aMessageNode

	(aMessageNode isCascaded not or: [aMessageNode isFirstCascaded]) 
		ifTrue: [self visitNode: aMessageNode receiver].
	aMessageNode arguments do: [:each | self visitNode: each].
	"aMessageNode binding:"
		((scope lookupSelector: aMessageNode selectorString)
			ifNil: [self undeclaredSelector: aMessageNode])! !

!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/19/2003 21:40'!
acceptMethodNode: aMethodNode

	scope _ scope newMethodScope.
	aMethodNode scope: scope.
	aMethodNode arguments do: [:node | (self declareVariableNode: node) markArg].
	aMethodNode primitiveNode: (PrimitiveParser new
		startPosition: aMethodNode primitiveStartPosition;
		parse: aMethodNode primitiveText).
	self visitNode: aMethodNode body.
	scope _ scope outerScope.
! !

!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/26/2003 14:52'!
acceptReturnNode: aReturnNode

	| var |
	self visitNode: aReturnNode value.
	var _ scope lookupVar: 'home context'.
	var markRead.
	aReturnNode homeBinding: var.
! !

!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/12/2003 14:09'!
acceptSequenceNode: aSequenceNode

	aSequenceNode temporaries do: [:node | self declareVariableNode: node].
	aSequenceNode statements do: [:each | self visitNode: each].
	aSequenceNode temporaries do: [:node |
		node binding isUnused ifTrue: [self unusedVariable: node]].
! !

!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/12/2003 13:40'!
acceptVariableNode: aVariableNode

	| var |
	var _ (scope lookupVar: aVariableNode name)
		ifNil: [self undeclaredVariable: aVariableNode].
	var isUnused
		ifTrue: [self uninitializedVariable: aVariableNode].
	var markRead.
	aVariableNode binding: var.
! !

!SemanticChecker methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/25/2003 23:39'!
declareVariableNode: aVariableNode

	| semVar |
	(semVar _ scope rawVar: aVariableNode name)
		ifNotNil: [self variable: aVariableNode shadows: semVar].
	semVar _ scope addTemp: aVariableNode name.
	aVariableNode binding: semVar.
	^ semVar! !

!SemanticChecker methodsFor: 'initialize-release' stamp: 'ajh 2/27/2003 12:51'!
initialize

	scope _ SemNilScope new.  "in case never initialized"! !

!SemanticChecker methodsFor: 'initialize-release' stamp: 'ajh 2/26/2003 19:58'!
scope: aSemScope

	scope _ aSemScope! !

!SemanticChecker methodsFor: 'error handling' stamp: 'ajh 3/11/2003 22:46'!
undeclaredSelector: messageNode

	^ UndeclaredSelectorError new
		messageNode: messageNode;
		signal! !

!SemanticChecker methodsFor: 'error handling' stamp: 'ajh 3/11/2003 12:06'!
undeclaredVariable: variableNode

	^ UndeclaredVariableError new
		variableNode: variableNode;
		signal! !

!SemanticChecker methodsFor: 'error handling' stamp: 'ajh 3/12/2003 13:35'!
uninitializedVariable: variableNode

	^ UninitializedVariableError new
		variableNode: variableNode;
		signal! !

!SemanticChecker methodsFor: 'error handling' stamp: 'ajh 3/12/2003 14:10'!
unusedVariable: variableNode

	^ UnusedVariableError new
		variableNode: variableNode;
		signal! !

!SemanticChecker methodsFor: 'error handling' stamp: 'ajh 3/19/2003 13:33'!
variable: variableNode shadows: semVar

	^ ShadowVariableError new
		variableNode: variableNode;
		shadowedVar: semVar;
		signal! !


!SemanticError methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 14:28'!
correctIn: compiler

	self subclassResponsibility! !

!SemanticError methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 14:54'!
isResumable

	^ true! !


!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/10/2003 14:53'!
acceptArrayNode: anArrayNode 

	| elementNodes |
	elementNodes _ anArrayNode children.
	elementNodes size <= 4 ifTrue: [
		"Short form: Array braceWith: a with: b ..."
		methodBuilder pushLiteral: Array holder; send: #poolValue.
		elementNodes do: [:node | valueTranslator visitNode: node].
		methodBuilder send: (#(braceWithNone braceWith: braceWith:with: braceWith:with:with: braceWith:with:with:with:) at: elementNodes size + 1).
	] ifFalse: [
		"Long form: (Array braceStream: N) nextPut: a; nextPut: b; ...; braceArray"
		methodBuilder pushLiteral: Array holder; send: #poolValue.
		methodBuilder pushLiteral: elementNodes size.
		methodBuilder send: #braceStream:.
		elementNodes do: [:node |
			methodBuilder pushDup.
			valueTranslator visitNode: node.
			methodBuilder send: #nextPut:.
			methodBuilder popTop.
		].
		methodBuilder send: #braceArray.
	].
! !

!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/27/2003 14:12'!
acceptAssignmentNode: anAssignmentNode 

	valueTranslator visitNode: anAssignmentNode value.
	anAssignmentNode variable binding emitStore: methodBuilder.
! !

!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/25/2003 00:51'!
acceptBlockNode: aBlockNode 
! !

!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/10/2003 14:53'!
acceptCascadeNode: aCascadeNode 

	valueTranslator visitNode: aCascadeNode receiver.
	aCascadeNode messages allButLastDo: [:node |
		methodBuilder pushDup.
		effectTranslator visitNode: node.
	].
	valueTranslator visitNode: aCascadeNode messages last.
! !

!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 10:06'!
acceptDoItNode: aDoItNode 

	methodBuilder thisContext: aDoItNode scope thisContextVar.
	methodBuilder rargs: {aDoItNode scope receiverVar}.
	valueTranslator visitNode: aDoItNode body.
	aDoItNode body lastIsReturn ifFalse: [
		methodBuilder returnTop].
! !

!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/24/2003 20:06'!
acceptLiteralNode: aLiteralNode! !

!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/25/2003 13:23'!
acceptMessageNode: aMessageNode 

	aMessageNode isInlineIf ifTrue: [^ self emitIfNode: aMessageNode].
	aMessageNode isInlineIfNil ifTrue: [^ self emitIfNilNode: aMessageNode].
	aMessageNode isInlineAndOr ifTrue: [^ self emitAndOrNode: aMessageNode].
	aMessageNode isInlineWhile ifTrue: [^ self emitWhileNode: aMessageNode].
	aMessageNode isInlineToDo ifTrue: [^ self emitToDoNode: aMessageNode].
	aMessageNode isInlineCase ifTrue: [^ self emitCaseNode: aMessageNode].
	^ self emitMessageNode: aMessageNode! !

!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 10:06'!
acceptMethodNode: aMethodNode 

	| vars |
	methodBuilder thisContext: aMethodNode scope thisContextVar.
	vars _ aMethodNode arguments collect: [:node | node binding].
	methodBuilder rargs: (vars copyWithFirst: aMethodNode scope receiverVar).
	vars do: [:var | var emitArgInit: methodBuilder].
	methodBuilder primitiveNode: aMethodNode primitiveNode.
	effectTranslator visitNode: aMethodNode body.
	aMethodNode body lastIsReturn ifFalse: [
		methodBuilder pushReceiver; returnTop].
! !

!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/27/2003 01:44'!
acceptReturnNode: aReturnNode

	valueTranslator visitNode: aReturnNode value.
	aReturnNode homeBinding isTemp ifTrue: [
		methodBuilder returnTop.
	] ifFalse: [
		aReturnNode homeBinding emitValue: methodBuilder.
		methodBuilder remoteReturn.
	].
! !

!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/13/2003 04:11'!
acceptSequenceNode: aSequenceNode 

	| vars |
	vars _ aSequenceNode temporaries collect: [:n | n binding].
	methodBuilder addTemps: vars.
	vars do: [:var | var emitTempInit: methodBuilder].
	aSequenceNode statements allButLastDo: [:n | effectTranslator visitNode: n].
	aSequenceNode statements notEmpty ifTrue: [self visitNode: aSequenceNode statements last].
! !

!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/24/2003 20:06'!
acceptVariableNode: aVariableNode! !

!SemanticTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 10:07'!
translateBlockNode: aBlockNode 

	| vars |
	methodBuilder thisContext: aBlockNode scope thisContextVar.
	vars _ aBlockNode arguments collect: [:node | node binding].
	methodBuilder rargs: (vars copyWithFirst: aBlockNode scope receiverVar).
	vars do: [:var | var emitArgInit: methodBuilder].
	valueTranslator visitNode: aBlockNode body.
	aBlockNode body lastIsReturn ifFalse: [
		methodBuilder returnTop].
! !

!SemanticTranslator methodsFor: 'inline messages' stamp: 'ajh 3/13/2003 00:21'!
emitAndOrNode: aMessageNode

	valueTranslator visitNode: aMessageNode receiver.
	methodBuilder jumpAheadTo: #else if: (aMessageNode selector == #or:).
	self visitNode: aMessageNode arguments first body.
	self == valueTranslator ifTrue: [
		methodBuilder jumpAheadTo: #end].
	methodBuilder jumpAheadTarget: #else.
	self == valueTranslator ifTrue: [
		methodBuilder pushLiteral: (aMessageNode selector == #or:).
		methodBuilder jumpAheadTarget: #end.
	].
! !

!SemanticTranslator methodsFor: 'inline messages' stamp: 'ajh 3/18/2003 11:34'!
emitCaseNode: aMessageNode

	| cases assocMessageNode s |
	cases _ aMessageNode arguments first statements.
	valueTranslator visitNode: aMessageNode receiver.
	1 to: cases size - 1 do: [:i |
		methodBuilder pushDup.
		assocMessageNode _ cases at: i.
		valueTranslator visitNode: assocMessageNode receiver body.
		methodBuilder send: #=.
		methodBuilder jumpAheadTo: #next if: false.
		methodBuilder popTop.
		self visitNode: assocMessageNode arguments first body.
		methodBuilder jumpAheadTo: #end.
		methodBuilder jumpAheadTarget: #next.
	].
	aMessageNode arguments size = 2 ifTrue: [
		"last case with otherwise"
		assocMessageNode _ cases last.
		valueTranslator visitNode: assocMessageNode receiver body.
		methodBuilder send: #=.
		methodBuilder jumpAheadTo: #next if: false.
		self visitNode: assocMessageNode arguments first body.
		methodBuilder jumpAheadTo: #end.
		methodBuilder jumpAheadTarget: #next.
		self visitNode: aMessageNode arguments last body.
	] ifFalse: [
		"last case without otherwise"
		methodBuilder pushDup.
		assocMessageNode _ cases last.
		valueTranslator visitNode: assocMessageNode receiver body.
		methodBuilder send: #=.
		methodBuilder jumpAheadTo: #next if: false.
		methodBuilder popTop.
		self visitNode: assocMessageNode arguments first body.
		methodBuilder jumpAheadTo: #end.
		methodBuilder jumpAheadTarget: #next.
		methodBuilder send: #caseError.
		aMessageNode lastIsReturn ifTrue: [
			(s _ aMessageNode owningScope) isBlockScope ifTrue: [
				(s lookupVar: 'home context') emitValue: methodBuilder.
				methodBuilder remoteReturn.
			] ifFalse: [
				methodBuilder returnTop.
			]
		] ifFalse: [
			self == effectTranslator ifTrue: [methodBuilder popTop].
		].
	].

	"there has to be a one-to-one correspondence between every jump and target"	
	1 to: cases size do: [:i | methodBuilder jumpAheadTarget: #end].
! !

!SemanticTranslator methodsFor: 'inline messages' stamp: 'ajh 3/10/2003 14:56'!
emitIfNilNode: aMessageNode

	| args selector |
	valueTranslator visitNode: aMessageNode receiver.
	args _ aMessageNode arguments.
	(selector _ aMessageNode selector) caseOf: {
		[#ifNil:] -> [self == valueTranslator ifTrue: [methodBuilder pushDup]].
		[#ifNil:ifNotNil:] -> [args last arguments isEmpty ifFalse: [args first arguments first binding emitStore: methodBuilder]].
		[#ifNotNil:] -> [args first arguments isEmpty ifFalse: [args first arguments first binding emitStore: methodBuilder]].
		[#ifNotNil:ifNil:] -> [args first arguments isEmpty ifFalse: [args first arguments first binding emitStore: methodBuilder]]
	}.
	methodBuilder pushLiteral: nil.
	methodBuilder send: #==.
	methodBuilder jumpAheadTo: #else if: (selector beginsWith: #ifNotNil:).
	(self == valueTranslator and: [selector == #ifNil:])
		ifTrue: [methodBuilder popTop].
	self visitNode: args first body.
	(args size > 1 or: [self == valueTranslator and: [selector == #ifNotNil:]])
		ifTrue: [methodBuilder jumpAheadTo: #end].
	methodBuilder jumpAheadTarget: #else.
	(args size > 1 or: [self == valueTranslator and: [selector == #ifNotNil:]]) ifTrue: [
		args size > 1
			ifTrue: [self visitNode: args last body]
			ifFalse: [methodBuilder pushLiteral: nil].
		methodBuilder jumpAheadTarget: #end.
	].
! !

!SemanticTranslator methodsFor: 'inline messages' stamp: 'ajh 2/27/2003 14:32'!
emitIfNode: aMessageNode

	| args |
	valueTranslator visitNode: aMessageNode receiver.
	methodBuilder jumpAheadTo: #else if: (aMessageNode selector beginsWith: #ifFalse:).
	args _ aMessageNode arguments.
	self visitNode: args first body.
	(args size > 1 or: [self == valueTranslator]) ifTrue: [
		methodBuilder jumpAheadTo: #end].
	methodBuilder jumpAheadTarget: #else.
	(args size > 1 or: [self == valueTranslator]) ifTrue: [
		args size > 1
			ifTrue: [self visitNode: args last body]
			ifFalse: [methodBuilder pushLiteral: nil].
		methodBuilder jumpAheadTarget: #end.
	].
! !

!SemanticTranslator methodsFor: 'inline messages' stamp: 'ajh 3/13/2003 04:20'!
emitMessageNode: aMessageNode

	| r |
	aMessageNode isCascaded ifFalse: [
		valueTranslator visitNode: aMessageNode receiver].
	aMessageNode arguments do: [:each | 
		valueTranslator visitNode: each].
	r _ aMessageNode receiver.
	(r isVariable and: [r name = 'super' and: [r binding name = 'self']])
		ifTrue: [methodBuilder send: aMessageNode selector toSuperOf: aMessageNode owningScope classEncoding]
		ifFalse: [methodBuilder send: aMessageNode selector].
! !

!SemanticTranslator methodsFor: 'inline messages' stamp: 'ajh 3/17/2003 08:59'!
emitToDoNode: aMessageNode

	| limit step block iterator limitEmit |
	limit _ aMessageNode arguments first.
	step _ 1.
	aMessageNode arguments size = 3 ifTrue: [
		"to:by:do:"
		step _ aMessageNode arguments second.
		step isLiteral ifFalse: [self error: 'should not have been inlined'].
		step _ step value.
	].
	block _ aMessageNode arguments last.

	"push start. allocate and initialize iterator"
	valueTranslator visitNode: aMessageNode receiver.
	iterator _ block arguments first binding.
	methodBuilder addTemp: iterator.
	methodBuilder storeTemp: iterator.

	"push limit, store in a temp if it's an expression or a changing variable"
	valueTranslator visitNode: limit.
	limitEmit _ [valueTranslator visitNode: limit].
	(limit isLiteral or: 
	 [limit isVariable and:
	  [ParseTreeSearcher new
		matches: limit name , ' := `@object' do: [:n :a | false];
		executeTree: block initialAnswer: true]]
	) ifFalse: [
		limit _ {iterator. #limit}.  "must be unique throughout method"
		methodBuilder addTemp: limit.
		methodBuilder storeTemp: limit.
		limitEmit _ [methodBuilder pushTemp: limit].
	].

	"loop"
	methodBuilder jumpBackTarget: #start.
	methodBuilder send: (step > 0 ifTrue: [#<=] ifFalse: [#>=]).
	methodBuilder jumpAheadTo: #done if: false.
	effectTranslator visitNode: block body.
	methodBuilder pushTemp: iterator.
	methodBuilder pushLiteral: step.
	methodBuilder send: #+.
	methodBuilder storeTemp: iterator.
	limitEmit value.
	methodBuilder jumpBackTo: #start.
	methodBuilder jumpAheadTarget: #done.
! !

!SemanticTranslator methodsFor: 'inline messages' stamp: 'ajh 2/27/2003 14:33'!
emitWhileNode: aMessageNode

	methodBuilder jumpBackTarget: #begin.
	valueTranslator visitNode: aMessageNode receiver body.
	aMessageNode selector caseOf: {
		[#whileTrue:]	-> [methodBuilder jumpAheadTo: #end if: false].
		[#whileTrue]	-> [methodBuilder jumpAheadTo: #end if: false].
		[#whileFalse:]	-> [methodBuilder jumpAheadTo: #end if: true].
		[#whileFalse]	-> [methodBuilder jumpAheadTo: #end if: true]
	}.
	aMessageNode arguments notEmpty ifTrue: [
		effectTranslator visitNode: aMessageNode arguments first body].
	methodBuilder jumpBackTo: #begin.
	methodBuilder jumpAheadTarget: #end.
! !

!SemanticTranslator methodsFor: 'visiting' stamp: 'ajh 3/2/2003 03:31'!
visitNode: aNode

	methodBuilder mapToNode: aNode.
	super visitNode: aNode.
	methodBuilder popMap.
! !

!SemanticTranslator methodsFor: 'initialize' stamp: 'ajh 3/10/2003 20:15'!
initialize

	methodBuilder _ IRBuilder new.
	effectTranslator _ self as: SemanticTranslatorForEffect.
	valueTranslator _ self as: SemanticTranslatorForValue.
	effectTranslator instVarNamed: #effectTranslator put: effectTranslator.
	effectTranslator instVarNamed: #valueTranslator put: valueTranslator.
	valueTranslator instVarNamed: #valueTranslator put: valueTranslator.
! !

!SemanticTranslator methodsFor: 'accessing' stamp: 'ajh 3/10/2003 17:59'!
ir

	^ methodBuilder ir! !


!SemanticTranslatorForEffect methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/10/2003 14:56'!
acceptArrayNode: anArrayNode 

	super acceptArrayNode: anArrayNode.
	methodBuilder popTop.
! !

!SemanticTranslatorForEffect methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/10/2003 14:56'!
acceptAssignmentNode: anAssignmentNode 

	super acceptAssignmentNode: anAssignmentNode.
	methodBuilder popTop.
! !

!SemanticTranslatorForEffect methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/10/2003 14:56'!
acceptCascadeNode: aCascadeNode 

	super acceptCascadeNode: aCascadeNode.
	methodBuilder popTop.
! !

!SemanticTranslatorForEffect methodsFor: 'emit messages' stamp: 'ajh 3/10/2003 14:57'!
emitMessageNode: aMessageNode 

	super emitMessageNode: aMessageNode.
	methodBuilder popTop.
! !


!SemanticTranslatorForValue methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/11/2003 01:25'!
acceptBlockNode: aBlockNode 

	| blockMethod capturedVars n |
	blockMethod _ aBlockNode generate: #(0).
	capturedVars _ aBlockNode scope capturedVars.
	n _ capturedVars size.
	n = 0 ifTrue: [
		"create block at compile time"
		methodBuilder pushLiteral: blockMethod createBlock.
		^ self
	].
	"create block at run time"
	methodBuilder pushLiteral: blockMethod.
	n < 16 ifTrue: [
		capturedVars do: [:var | var emitStorageValue: methodBuilder].
		methodBuilder send: (
			(String streamContents: [:s |
				s nextPutAll: #createBlock.
				1 to: n do: [:i | s nextPutAll: 't:'].
	  		]) asSymbol).
		^ self
	].
	"Long form: (Array braceStream: N) nextPut: a; nextPut: b; ...; braceArray"
	methodBuilder pushLiteral: Array holder; send: #poolValue.
	methodBuilder pushLiteral: n.
	methodBuilder send: #braceStream:.
	capturedVars do: [:var |
		methodBuilder pushDup.
		var emitStorageValue: methodBuilder.
		methodBuilder send: #nextPut:.
		methodBuilder popTop.
	].
	methodBuilder send: #braceArray.
	methodBuilder send: #createBlockWithVars:.
! !

!SemanticTranslatorForValue methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/27/2003 18:18'!
acceptLiteralNode: aLiteralNode

	methodBuilder pushLiteral: aLiteralNode value.
! !

!SemanticTranslatorForValue methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/25/2003 18:21'!
acceptSequenceNode: aSequenceNode 

	super acceptSequenceNode: aSequenceNode.
	aSequenceNode statements isEmpty ifTrue: [
		methodBuilder pushLiteral: nil].
! !

!SemanticTranslatorForValue methodsFor: 'visitor-double dispatching' stamp: 'ajh 2/27/2003 18:18'!
acceptVariableNode: aVariableNode

	aVariableNode binding emitValue: methodBuilder.
! !

!SemanticTranslatorForValue methodsFor: 'emit messages' stamp: 'ajh 2/25/2003 16:50'!
emitToDoNode: aMessageNode

	super emitToDoNode: aMessageNode.
	methodBuilder pushLiteral: nil.
! !

!SemanticTranslatorForValue methodsFor: 'emit messages' stamp: 'ajh 2/25/2003 16:49'!
emitWhileNode: aMessageNode

	super emitWhileNode: aMessageNode.
	methodBuilder pushLiteral: nil.
! !


!ShadowVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/19/2003 16:42'!
correctIn: compiler

	self resume: (compiler variable: self variableNode shadows: self shadowedVar)! !

!ShadowVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/19/2003 13:34'!
defaultAction
	"allow variable to shadow previous/inst var of the same name"

	^ nil! !

!ShadowVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/19/2003 13:32'!
shadowedVar

	^ shadowedVar! !

!ShadowVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/19/2003 13:32'!
shadowedVar: semVar

	shadowedVar _ semVar! !

!ShadowVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/19/2003 13:19'!
variableNode

	^ variableNode! !

!ShadowVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/19/2003 13:24'!
variableNode: aVariableNode

	variableNode _ aVariableNode.
	messageText _ 'Temp shadows: ', aVariableNode name.
! !


!SmaCCScanner methodsFor: 'initialize-release' stamp: 'ajh 3/17/2003 12:05'!
on: aStream 
	stream := aStream.
	start := stream position + 1.! !

!SmaCCScanner methodsFor: 'private' stamp: 'ajh 3/17/2003 12:04'!
resetScanner
	start := stream position + 1.
	outputStream reset.
	lastOutputStreamMatchPosition := 0! !

!SmaCCScanner methodsFor: 'private' stamp: 'ajh 3/17/2003 12:05'!
scannerError
	(stream atEnd and: [start == (stream position + 1)]) 
		ifTrue: 
			[returnMatchBlock value: (SmaCCToken 
						value: ''
						start: start
						id: (Array with: self emptySymbolTokenId))].
	stream position: start - 1.
	returnMatchBlock value: (SmaCCToken 
				value: (String with: stream next)
				start: start
				id: #(0))! !


!SmaCCToken methodsFor: 'accessing' stamp: 'ajh 2/27/2003 17:55'!
length

	^ value size! !

!SmaCCToken methodsFor: 'accessing' stamp: 'ajh 3/11/2003 23:05'!
sourceInterval

	^ self start to: self stop! !

!SmaCCToken methodsFor: 'accessing' stamp: 'ajh 3/17/2003 12:03'!
start

	^ start! !

!SmaCCToken methodsFor: 'accessing' stamp: 'ajh 3/17/2003 12:03'!
startPosition
	^ start! !

!SmaCCToken methodsFor: 'accessing' stamp: 'ajh 3/17/2003 12:03'!
stop

	^ start ifNotNil: [start + value size - 1]! !

!SmaCCToken methodsFor: 'accessing' stamp: 'ajh 3/17/2003 12:03'!
stopPosition
	^ start + value size - 1! !

!SmaCCToken methodsFor: 'initialize-release' stamp: 'ajh 3/13/2003 14:04'!
start: startPositionInteger

	start _ startPositionInteger
! !

!SmaCCToken methodsFor: 'initialize-release' stamp: 'ajh 3/13/2003 15:25'!
value: anObject

	value _ anObject! !


!SmaCCToken class methodsFor: 'instance creation' stamp: 'ajh 3/13/2003 14:04'!
value: aString

	^ self value: aString start: nil id: nil! !

!SmaCCToken class methodsFor: 'instance creation' stamp: 'ajh 3/11/2003 23:28'!
value: aString start: anInteger

	^ self value: aString start: anInteger id: nil! !


!SqueakParser methodsFor: 'generated-tables' stamp: 'ajh 3/16/2003 23:56'!
reduceTable
	^#(
#(33 0 #array)
#(33 2 #add:)
#(34 2 #method:)
#(34 3 #methodPrim:)
#(34 4 #methodTempsPrim:)
#(35 0 #reduceActionForOptionalXXXXX1:)
#(35 1 #reduceActionForOptionalXXXXX2:)
#(36 1 #first:)
#(36 1 #first:)
#(36 1 #first:)
#(37 2 #secondIn:)
#(37 3 #add3:)
#(38 5 #blockWithArgs:)
#(38 3 #blockNoArgs:)
#(38 3 #blockArgs:)
#(38 7 #blockWithTemps:)
#(39 3 #assignment:)
#(40 1 #first:)
#(40 3 #cascade:)
#(41 3 #second:)
#(41 1 #first:)
#(41 1 #first:)
#(41 1 #first:)
#(41 1 #first:)
#(42 3 #array:)
#(43 1 #litTrue:)
#(43 1 #litFalse:)
#(43 1 #litNil:)
#(43 1 #litNumber:)
#(43 1 #litNumber:)
#(43 1 #litChar:)
#(43 1 #litString:)
#(43 2 #litStringSymbol:)
#(43 2 #litSymbol:)
#(43 2 #litSymbol:)
#(43 2 #litSymbol:)
#(43 2 #litSymbol:)
#(43 2 #litSymbol:)
#(43 4 #litArray:)
#(43 4 #litArray:)
#(44 0 #array)
#(44 2 #first:)
#(44 5 #returnAdd:)
#(44 3 #return:)
#(45 1 #first:)
#(45 1 #first:)
#(45 1 #first:)
#(46 1 #first:)
#(46 1 #first:)
#(46 1 #first:)
#(47 1 #reduceFor:)
#(48 2 #messageSend:)
#(48 2 #messageSend:)
#(48 2 #messageSend:)
#(49 2 #messageSend:)
#(49 2 #messageSend:)
#(50 1 #unaryMessage:)
#(51 2 #messagePart:)
#(52 2 #messagePart:)
#(52 3 #addMessagePart:)
#(53 1 #first:)
#(53 1 #first:)
#(53 1 #first:)
#(54 1 #first:)
#(54 1 #first:)
#(55 1 #array)
#(55 3 #second:)
#(56 2 #messagePart:)
#(56 3 #addMessagePart:)
#(57 3 #primitiveMessage:)
#(58 1 #unaryMessage:)
#(58 2 #messagePart:)
#(58 1 #first:)
#(59 1 #sequence:)
#(59 2 #sequenceWithTemps:)
#(60 0 #byteStream)
#(60 2 #byteStreamPut:)
#(61 0 #stream)
#(61 2 #streamPut:)
#(62 1 #value:)
#(62 1 #valueSymbol:)
#(62 1 #valueSymbol:)
#(62 1 #valueSymbol:)
#(62 1 #valueSymbol:)
#(62 1 #valueSymbol:)
#(62 3 #contents2:)
#(62 3 #contents2:)
#(62 1 #valueSymbol:)
#(63 0 #reduceFor:)
#(63 2 #reduceFor:)
#(63 2 #reduceFor:)
#(63 2 #reduceFor:)
#(63 2 #reduceFor:)
#(63 2 #reduceFor:)
#(63 2 #reduceFor:)
#(63 2 #reduceFor:)
#(63 2 #reduceFor:)
#(63 2 #reduceFor:)
#(63 2 #reduceFor:)
#(66 2 #messageSend:)
#(66 2 #messageSend:)
#(66 2 #messageSend:)
#(67 1 #variable:)
#(68 1 #firstIn:)
#(68 3 #add3:)
)! !

!SqueakParser methodsFor: 'generated-tables' stamp: 'ajh 3/16/2003 23:56'!
transitionTable
	^#(
#(3 17 22 21 23 25 25 29 34 33 56 37 58)
#(3 41 1 45 2 49 3 53 4 57 8 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 101 36 105 38 109 39 113 40 117 41 121 42 125 43 129 44 133 45 137 48 141 49 145 55 149 59 166 64 153 66 157 67 161 68)
#(3 17 22 21 23 25 25 33 56 165 58)
#(2 286 1 2 3 4 6 8 11 12 13 14 16 19 20 21 22 30 64)
#(3 93 22 169 67)
#(3 93 22 173 67)
#(2 0 64)
#(3 294 1 294 2 294 3 294 4 294 6 294 8 294 11 294 12 294 13 294 14 294 16 294 19 294 20 294 21 294 22 177 23 294 30 294 64)
#(3 41 1 45 2 49 3 53 4 181 6 57 8 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 101 36 105 38 109 39 113 40 117 41 121 42 125 43 129 44 133 45 137 48 141 49 185 55 189 57 193 59 166 64 153 66 157 67 161 68)
#(3 45 2 53 4 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 197 36 105 38 109 39 113 40 117 41 121 42 125 43 133 45 137 48 141 49 153 66 157 67)
#(3 41 1 45 2 49 3 53 4 166 5 57 8 201 10 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 101 36 205 37 105 38 109 39 113 40 117 41 121 42 125 43 129 44 133 45 137 48 141 49 145 55 209 59 153 66 157 67 161 68)
#(3 6 3 6 22 213 33)
#(3 217 2 221 14 225 21 229 22 233 23 237 24 241 25 245 27)
#(2 266 1 2 4 5 6 11 12 13 14 16 19 20 21 22 30 64)
#(2 110 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 114 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 106 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(3 45 2 53 4 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 249 36 105 38 109 39 113 40 117 41 121 42 125 43 133 45 137 48 141 49 153 66 157 67)
#(3 41 1 45 2 53 4 61 11 65 12 69 13 73 14 166 15 77 16 81 19 85 20 89 21 93 22 97 30 101 36 105 38 109 39 113 40 117 41 121 42 125 43 253 44 133 45 137 48 141 49 153 66 157 67 161 68)
#(2 118 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 122 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 130 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 414 1 2 3 4 5 6 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 25 26 30 64)
#(2 126 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 418 5 9 15 64)
#(2 90 5 9 15 17 18 22 23 25 64)
#(2 34 5 9 15 17 64)
#(3 38 5 38 9 38 15 38 17 257 18 38 64)
#(3 42 5 42 9 42 15 42 17 261 22 265 23 269 25 273 50 277 51 281 52 42 64)
#(2 86 5 9 15 17 18 22 23 25 64)
#(2 94 5 9 15 17 18 22 23 25 64)
#(2 298 5 64)
#(2 74 5 9 15 17 18 64)
#(3 186 5 186 9 186 15 186 17 186 18 265 23 269 25 285 51 289 52 186 64)
#(3 190 5 190 9 190 15 190 17 190 18 261 22 265 23 269 25 293 50 297 51 301 52 190 64)
#(3 41 1 45 2 53 4 166 5 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 101 36 105 38 109 39 113 40 117 41 121 42 125 43 305 44 133 45 137 48 141 49 166 64 153 66 157 67 161 68)
#(2 0 64)
#(2 182 5 9 15 17 18 64)
#(3 98 5 98 9 98 15 98 17 98 22 98 23 98 25 309 26 98 64)
#(3 26 5 313 9 26 15 317 35 26 64)
#(2 0 64)
#(2 274 1 2 3 4 6 8 11 12 13 14 16 19 20 21 22 23 30 64)
#(2 290 1 2 3 4 6 8 11 12 13 14 16 19 20 21 22 30 64)
#(3 93 22 321 67)
#(3 358 4 358 7 358 9 358 11 358 12 358 13 358 14 358 15 358 16 358 17 358 18 358 19 358 20 358 21 358 22 358 23 358 25 358 30 325 63)
#(3 41 1 45 2 53 4 181 6 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 101 36 105 38 109 39 113 40 117 41 121 42 125 43 305 44 133 45 137 48 141 49 329 57 166 64 153 66 157 67 161 68)
#(3 41 1 45 2 49 3 53 4 57 8 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 101 36 105 38 109 39 113 40 117 41 121 42 125 43 129 44 133 45 137 48 141 49 145 55 333 59 166 64 153 66 157 67 161 68)
#(2 14 64)
#(3 26 5 337 9 26 15 341 35 26 64)
#(3 93 22 345 67)
#(3 349 3 353 5 357 8 361 10)
#(2 365 5)
#(3 369 3 93 22 373 67)
#(3 306 5 306 19 377 60)
#(3 314 2 314 4 314 11 314 12 314 13 314 14 314 17 314 19 314 20 314 21 314 22 314 23 314 24 314 25 314 27 314 30 314 32 381 61)
#(2 134 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 138 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 146 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 150 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 142 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 154 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 385 17)
#(2 389 15)
#(3 261 22 265 23 269 25 393 46 397 50 401 51 405 52)
#(2 230 5 9 15 17 18 22 23 25 64)
#(3 45 2 53 4 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 105 38 409 41 121 42 125 43 413 48 417 49 421 53 425 67)
#(3 45 2 53 4 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 105 38 429 41 121 42 125 43 433 49 437 54 425 67)
#(2 226 5 9 15 17 18 22 23 25 64)
#(2 218 5 9 15 17 18 23 25 64)
#(3 410 5 410 9 410 15 410 17 410 18 441 23 410 64)
#(2 210 5 9 15 17 18 23 25 64)
#(3 402 5 402 9 402 15 402 17 402 18 441 23 402 64)
#(2 222 5 9 15 17 18 22 23 25 64)
#(2 214 5 9 15 17 18 23 25 64)
#(3 406 5 406 9 406 15 406 17 406 18 441 23 406 64)
#(2 302 5 64)
#(3 45 2 53 4 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 445 36 105 38 109 39 113 40 117 41 121 42 125 43 133 45 137 48 141 49 153 66 157 67)
#(3 449 1 45 2 53 4 30 5 61 11 65 12 69 13 73 14 30 15 77 16 81 19 85 20 89 21 93 22 97 30 453 36 105 38 109 39 113 40 117 41 121 42 125 43 133 45 137 48 141 49 30 64 153 66 157 67)
#(2 170 5 15 64)
#(2 278 1 2 3 4 6 8 11 12 13 14 16 19 20 21 22 23 30 64)
#(3 53 4 457 7 461 9 61 11 65 12 69 13 465 14 469 15 473 16 477 17 481 18 81 19 85 20 89 21 485 22 489 23 493 25 97 30 497 43)
#(3 41 1 45 2 53 4 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 101 36 105 38 109 39 113 40 117 41 121 42 125 43 501 44 133 45 137 48 141 49 166 64 153 66 157 67 161 68)
#(2 18 64)
#(2 30 5 15 64)
#(2 178 5 15 64)
#(2 46 3 5 8 10)
#(3 41 1 45 2 49 3 53 4 166 5 57 8 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 101 36 105 38 109 39 113 40 117 41 121 42 125 43 129 44 133 45 137 48 141 49 145 55 505 59 153 66 157 67 161 68)
#(2 62 5 9 15 17 18 22 23 25 64)
#(3 6 3 6 22 509 33)
#(3 93 22 513 67)
#(2 58 5 9 15 17 18 22 23 25 64)
#(2 270 1 2 4 5 6 11 12 13 14 16 19 20 21 22 30 64)
#(2 10 3 22)
#(3 517 5 521 19)
#(3 525 2 53 4 61 11 65 12 69 13 529 14 533 17 81 19 85 20 89 21 537 22 541 23 545 24 549 25 553 27 97 30 557 32 561 43 565 62)
#(2 82 5 9 15 17 18 22 23 25 64)
#(2 102 5 9 15 17 18 22 23 25 64)
#(2 78 5 9 15 17 18 64)
#(2 194 5 9 15 17 18 64)
#(2 198 5 9 15 17 18 64)
#(3 202 5 202 9 202 15 202 17 202 18 441 23 202 64)
#(3 254 5 254 9 254 15 254 17 254 18 261 22 254 23 269 25 273 50 277 51 254 64)
#(3 246 5 246 9 246 15 246 17 246 18 246 23 269 25 285 51 246 64)
#(3 250 5 250 9 250 15 250 17 250 18 261 22 250 23 269 25 293 50 297 51 250 64)
#(2 238 5 9 15 17 18 23 64)
#(2 98 5 9 15 17 18 22 23 25 64)
#(3 262 5 262 9 262 15 262 17 262 18 261 22 262 23 262 25 273 50 262 64)
#(3 258 5 258 9 258 15 258 17 258 18 261 22 258 23 258 25 293 50 258 64)
#(2 234 5 9 15 17 18 23 25 64)
#(3 45 2 53 4 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 105 38 409 41 121 42 125 43 413 48 417 49 569 53 425 67)
#(2 70 5 9 15 17 64)
#(3 45 2 53 4 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 573 36 105 38 109 39 113 40 117 41 121 42 125 43 133 45 137 48 141 49 153 66 157 67)
#(2 422 5 9 15 64)
#(2 282 1 2 3 4 8 11 12 13 14 16 19 20 21 22 30 64)
#(2 390 4 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 25 30)
#(2 370 4 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 25 30)
#(2 382 4 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 25 30)
#(2 378 4 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 25 30)
#(2 374 4 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 25 30)
#(2 386 4 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 25 30)
#(2 394 4 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 25 30)
#(2 362 4 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 25 30)
#(2 398 4 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 25 30)
#(2 366 4 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 25 30)
#(2 22 64)
#(2 577 5)
#(3 581 3 93 22 373 67)
#(2 50 3 5 8 10)
#(2 158 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 310 5 19)
#(3 306 5 306 19 585 60)
#(3 314 2 314 4 314 11 314 12 314 13 314 14 314 17 314 19 314 20 314 21 314 22 314 23 314 24 314 25 314 27 314 30 314 32 589 61)
#(2 162 2 4 5 7 9 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 27 30 32 64)
#(2 326 2 4 11 12 13 14 17 19 20 21 22 23 24 25 27 30 32)
#(2 334 2 4 11 12 13 14 17 19 20 21 22 23 24 25 27 30 32)
#(2 338 2 4 11 12 13 14 17 19 20 21 22 23 24 25 27 30 32)
#(2 330 2 4 11 12 13 14 17 19 20 21 22 23 24 25 27 30 32)
#(2 342 2 4 11 12 13 14 17 19 20 21 22 23 24 25 27 30 32)
#(2 354 2 4 11 12 13 14 17 19 20 21 22 23 24 25 27 30 32)
#(2 322 2 4 11 12 13 14 17 19 20 21 22 23 24 25 27 30 32)
#(2 318 2 4 11 12 13 14 17 19 20 21 22 23 24 25 27 30 32)
#(2 242 5 9 15 17 18 23 64)
#(3 26 5 337 9 26 15 593 35 26 64)
#(2 54 5 9 15 17 18 22 23 25 64)
#(3 41 1 45 2 53 4 166 5 61 11 65 12 69 13 73 14 77 16 81 19 85 20 89 21 93 22 97 30 101 36 105 38 109 39 113 40 117 41 121 42 125 43 597 44 133 45 137 48 141 49 153 66 157 67 161 68)
#(3 601 5 521 19)
#(3 525 2 53 4 61 11 65 12 69 13 529 14 605 17 81 19 85 20 89 21 537 22 541 23 545 24 549 25 553 27 97 30 557 32 561 43 565 62)
#(2 174 5 15 64)
#(2 609 5)
#(2 350 2 4 11 12 13 14 17 19 20 21 22 23 24 25 27 30 32)
#(2 346 2 4 11 12 13 14 17 19 20 21 22 23 24 25 27 30 32)
#(2 66 5 9 15 17 18 22 23 25 64)
)! !

!SqueakParser methodsFor: 'generated-reduction actions' stamp: 'ajh 3/16/2003 23:56'!
reduceActionForOptionalXXXXX1: nodes 
	^nil! !

!SqueakParser methodsFor: 'generated-reduction actions' stamp: 'ajh 3/16/2003 23:56'!
reduceActionForOptionalXXXXX2: nodes 
	^nodes at: 1! !

!SqueakParser methodsFor: 'private' stamp: 'ajh 3/17/2003 12:07'!
actionForCurrentToken
	| ids action |
	ids := currentToken id.
	1 to: ids size do: [:i | 
		action := self actionFor: (ids at: i).
		(action bitAnd: self actionMask) = self errorAction ifFalse: [^ action].
		(ids at: i) = scanner negativeNumberId ifTrue: [
			currentToken := SmaCCToken 
				value: '-'
				start: currentToken startPosition
				id: (Array with: scanner binarySymbolId).
			scanner position: currentToken stopPosition.
			^ self actionForCurrentToken]
	].
	^ self errorAction! !

!SqueakParser methodsFor: 'private' stamp: 'ajh 3/6/2003 00:02'!
nodeStack

	^ nodeStack! !

!SqueakParser methodsFor: 'private' stamp: 'ajh 3/6/2003 00:19'!
scanner

	^ scanner! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:01'!
add3: nodes

	^ nodes first copyWith: nodes third! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:59'!
add: nodes

	^ nodes first copyWith: nodes second! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:54'!
addMessagePart: nodes

	^ {nodes first first copyWith: nodes second.
	   nodes first second copyWith: nodes third}! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:58'!
array

	^ #()! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:30'!
array: nodes

	^ RBArrayNode
		leftBrace: nodes first start
		rightBrace: nodes third stop
		statements: nodes second! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:07'!
assignment: nodes

	^ RBAssignmentNode
		variable: nodes first
		value: nodes third! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:04'!
blockArgs: nodes

	^ (RBBlockNode
		arguments: nodes second
		body: (RBSequenceNode statements: #()))
		left: nodes first start;
		right: nodes third stop! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:03'!
blockNoArgs: nodes

	^ (RBBlockNode
		body: nodes second)
		left: nodes first start;
		right: nodes third stop! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:02'!
blockWithArgs: nodes

	^ (RBBlockNode
		arguments: nodes second
		body: nodes fourth)
		left: nodes first start;
		right: nodes fifth stop! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:06'!
blockWithTemps: nodes

	^ (RBBlockNode
		arguments: nodes second
		body: (RBSequenceNode
			temporaries: nodes fourth
			statements: nodes sixth))
		left: nodes first start;
		right: nodes seventh stop! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:23'!
byteStream

	^ ByteArray new writeStream! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:25'!
byteStreamPut: nodes

	^ nodes first
		nextPut: nodes second value asNumber;
		yourself! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:12'!
cascade: nodes

	^ (nodes first isKindOf: RBMessageNode)
		ifTrue: [RBCascadeNode messages:
			{nodes first.
			 RBMessageNode new
				receiver: nodes first receiver
				selectorParts: nodes third first
				arguments: nodes third last}]
		ifFalse: [RBCascadeNode messages:
			(nodes first messages copyWith:
				(RBMessageNode new
					receiver: nodes first messages first receiver
					selectorParts: nodes third first
					arguments: nodes third last))]! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:27'!
contents2: nodes

	^ nodes second contents! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:53'!
first: nodes

	^ nodes first! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:01'!
firstIn: nodes

	^ {nodes first}! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/17/2003 12:10'!
litArray: nodes

	^ RBLiteralNode
		literalToken: (SmaCCToken
			value: (String new: (nodes fourth stop - nodes second start + 1))
			start: nodes second start)
		value: nodes third contents! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:16'!
litChar: nodes

	^ RBLiteralNode
		literalToken: nodes first
		value: (nodes first value at: 2)! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:14'!
litFalse: nodes

	^ RBLiteralNode
		literalToken: nodes first
		value: false! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:14'!
litNil: nodes

	^ RBLiteralNode
		literalToken: nodes first
		value: nil! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 13:42'!
litNumber: nodes

	| str num |
	str _ nodes first value readStream.
	num _ Number readFrom: str.
	str atEnd ifFalse: [
		currentToken _ nil.
		scanner position: scanner position - (str originalContents size - str position) + 1.
		self reportErrorMessage: 'Digit out of range'].
	^ RBLiteralNode
		literalToken: nodes first
		value: num! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:18'!
litString: nodes

	| s |
	s _ nodes first value.
	^ RBLiteralNode
		literalToken: nodes first
		value: ((s copyFrom: 2 to: s size - 1)
				copyReplaceAll: '''''' with: '''')! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:19'!
litStringSymbol: nodes

	| s |
	s _ nodes second value.
	^ RBLiteralNode
		literalToken: nodes second
		value: ((s copyFrom: 2 to: s size - 1)
				copyReplaceAll: '''''' with: '''') asSymbol! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:20'!
litSymbol: nodes

	^ RBLiteralNode
		literalToken: nodes second
		value: nodes second value asSymbol! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:14'!
litTrue: nodes

	^ RBLiteralNode
		literalToken: nodes first
		value: true! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:52'!
messagePart: nodes

	^ {{nodes first}. {nodes second}}! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:13'!
messageSend: nodes

	^ RBMessageNode new
		receiver: nodes first
		selectorParts: nodes second first
		arguments: nodes second last! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:48'!
method: nodes

	^ RBMethodNode new
		selectorParts: nodes first first
		arguments: nodes first last;
		body: nodes second! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:49'!
methodPrim: nodes

	^ RBMethodNode new
		selectorParts: nodes first first
		arguments: nodes first last;
		body: nodes third;
		tags: nodes second! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:50'!
methodTempsPrim: nodes

	^ RBMethodNode new
		selectorParts: nodes first first
		arguments: nodes first last;
		body: (RBSequenceNode
			temporaries: nodes second
			statements: nodes fourth);
		tags: nodes third! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:55'!
primitiveMessage: nodes

	^ {nodes first start to: nodes third stop}! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/19/2003 13:49'!
return: nodes

	^ {RBReturnNode
		return: nodes first start
		value: nodes second}! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/19/2003 13:49'!
returnAdd: nodes

	^ nodes first copyWith: (RBReturnNode
		return: nodes third start
		value: nodes fourth)! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:58'!
second: nodes

	^ nodes second! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:06'!
secondIn: nodes

	^ {nodes second}! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:57'!
sequence: nodes

	^ RBSequenceNode
		statements: nodes first! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:58'!
sequenceWithTemps: nodes

	^ RBSequenceNode
		temporaries: nodes first
		statements: nodes second! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:23'!
stream

	^ Array new writeStream! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:26'!
streamPut: nodes

	^ nodes first
		nextPut: nodes second;
		yourself! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/14/2003 23:52'!
unaryMessage: nodes

	^ {{nodes first}. #()}! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:27'!
value: nodes

	^ nodes first value! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:27'!
valueSymbol: nodes

	^ nodes first value asSymbol! !

!SqueakParser methodsFor: 'reduction actions' stamp: 'ajh 3/15/2003 00:13'!
variable: nodes

	^ RBVariableNode new
		identifierToken: nodes first! !


!SqueakParser class methodsFor: 'generated-starting states' stamp: 'ajh 3/16/2003 23:56'!
startingStateForMethod
	^1! !

!SqueakParser class methodsFor: 'generated-starting states' stamp: 'ajh 3/16/2003 23:56'!
startingStateForMethodPattern
	^3! !

!SqueakParser class methodsFor: 'generated-starting states' stamp: 'ajh 3/16/2003 23:56'!
startingStateForSequence
	^2! !

!SqueakParser class methodsFor: 'generated-accessing' stamp: 'ajh 3/16/2003 23:56'!
scannerClass
	^SqueakScanner! !

!SqueakParser class methodsFor: 'generated-comments' stamp: 'ajh 3/16/2003 23:56'!
parserDefinitionComment

	"%id <name> <negativeNumber> <binarySymbol>;
%start Sequence MethodPattern;

Method:
	MethodPattern Sequence						{#method:}
|	MethodPattern Primitive Sequence			{#methodPrim:}
|	MethodPattern Temporaries Primitive Statements {#methodTempsPrim:};

MethodPattern:
	<name>										{#unaryMessage:}
|	<binarySymbol> Variable						{#messagePart:}
|	KeywordMethodPattern						{#first:};

KeywordMethodPattern:
	<keyword> Variable							{#messagePart:}
|	KeywordMethodPattern <keyword> Variable	{#addMessagePart:};

Primitive:
	""<"" PrimitiveMessage "">""						{#primitiveMessage:};

Sequence:
	Statements									{#sequence:}
|	Temporaries Statements						{#sequenceWithTemps:};

Temporaries:
	""||""											{#array}
|	""|"" TemporaryVariables ""|""					{#second:};

TemporaryVariables:
												{#array}
|	TemporaryVariables Variable				{#add:};

Statements:
												{#array}
|	StatementList "".""?							{#first:}
|	StatementList ""."" ""^"" Expression "".""?			{#returnAdd:}
|	""^"" Expression "".""?							{#return:};

StatementList:
	Expression									{#firstIn:}
|	StatementList ""."" Expression					{#add3:};

Block:
	""["" BlockArgs ""|"" Sequence ""]""					{#blockWithArgs:}
|	""["" Sequence ""]""								{#blockNoArgs:}
|	""["" BlockArgs ""]""								{#blockArgs:}
|	""["" BlockArgs ""||"" TemporaryVariables ""|"" Statements ""]"" {#blockWithTemps:};

BlockArgs:
	"":"" Variable									{#secondIn:}
|	BlockArgs "":"" Variable						{#add3:};

Expression:
	Assignment									{#first:}
|	Cascade										{#first:}
|	Primary									{#first:};

Primary:
	""("" Expression "")""								{#second:}
|	Array										{#first:}
|	Block										{#first:}
|	Literal										{#first:}
|	Variable									{#first:};

Assignment:
	Variable <assignment> Expression				{#assignment:};

Cascade:
	MessageSend								{#first:}
|	Cascade "";"" Message							{#cascade:};

MessageSend:
	KeywordMessageSend						{#first:}
|	BinaryMessageSend							{#first:}
|	UnaryMessageSend							{#first:};

Message:
	UnaryMessage								{#first:}
|	BinaryMessage								{#first:}
|	KeywordMessage							{#first:};

KeywordMessageSend:
	BinaryMessageSend KeywordMessage			{#messageSend:}
|	UnaryMessageSend KeywordMessage			{#messageSend:}
|	Primary KeywordMessage					{#messageSend:};

KeywordMessage:
	<keyword> KeywordArgument				{#messagePart:}
|	KeywordMessage <keyword> KeywordArgument {#addMessagePart:};

KeywordArgument:
	BinaryMessageSend							{#first:}
|	UnaryMessageSend							{#first:}
|	Primary									{#first:};

BinaryMessageSend:
	BinaryMessageSend BinaryMessage			{#messageSend:}
|	UnaryMessageSend BinaryMessage			{#messageSend:}
|	Primary BinaryMessage						{#messageSend:};

BinaryMessage :
	<binarySymbol> BinaryArgument				{#messagePart:};

BinaryArgument:
	UnaryMessageSend							{#first:}
|	Primary									{#first:};

UnaryMessageSend :
	UnaryMessageSend UnaryMessage			{#messageSend:}
|	Primary UnaryMessage						{#messageSend:};

UnaryMessage :
	<name>										{#unaryMessage:};

Array:
	""{"" Statements ""}""								{#array:};

Variable: 
	<name>										{#variable:};

Literal:
	""true""										{#litTrue:}
|	""false""										{#litFalse:}
|	""nil""										{#litNil:}
|	<number>									{#litNumber:}
|	<negativeNumber>							{#litNumber:} 
|	<character>									{#litChar:}
|	<string>										{#litString:}
|	""#"" <string>									{#litStringSymbol:}
|	""#"" <name>									{#litSymbol:}
|	""#"" <binarySymbol>							{#litSymbol:}
|	""#"" <keyword>								{#litSymbol:}
|	""#"" <multikeyword>							{#litSymbol:}
|	""#"" <alternateKeyword>						{#litSymbol:}
|	""#"" ""["" ByteArray ""]""							{#litArray:}
|	""#"" ""("" LiteralArray "")""						{#litArray:};

ByteArray:
												{#byteStream}
|	ByteArray <number>							{#byteStreamPut:};

LiteralArray:
												{#stream}
|	LiteralArray ArrayLiteral					{#streamPut:};

ArrayLiteral:
	Literal										{#value:}
|	<name>										{#valueSymbol:}
|	<binarySymbol>								{#valueSymbol:}
|	<keyword>									{#valueSymbol:}
|	<multikeyword>								{#valueSymbol:}
|	<alternateKeyword>							{#valueSymbol:}
|	""("" LiteralArray "")""							{#contents2:}
|	""["" ByteArray ""]""							{#contents2:}
|	<anyChar>									{#valueSymbol:};

PrimitiveMessage:
	| PrimitiveMessage <keyword>
	| PrimitiveMessage Literal
	| PrimitiveMessage ""(""
	| PrimitiveMessage "")""
	| PrimitiveMessage ""{""
	| PrimitiveMessage ""}""
	| PrimitiveMessage "";""
	| PrimitiveMessage "".""
	| PrimitiveMessage <name>
	| PrimitiveMessage <binarySymbol>;
"! !

!SqueakParser class methodsFor: 'parsing' stamp: 'ajh 2/27/2003 16:40'!
parseDoIt: stringOrStream

	| sequence |
	sequence _ self
		parseStream: stringOrStream readStream
		startingAt: self startingStateForSequence.
	^ (RBDoItNode body: sequence)
		source: stringOrStream contents! !

!SqueakParser class methodsFor: 'parsing' stamp: 'ajh 2/27/2003 16:40'!
parseMethod: stringOrStream

	^ (self
		parseStream: stringOrStream readStream
		startingAt: self startingStateForMethod)
		source: stringOrStream contents! !

!SqueakParser class methodsFor: 'parsing' stamp: 'ajh 3/6/2003 00:50'!
parseMethodPattern: stringOrStream

	| array comments |
	array _
		[self
			parseStream: stringOrStream readStream
			startingAt: self startingStateForMethodPattern
		] on: SmaCCParserError do: [:ex | | parser |
			parser _ ex tag.
			comments _ parser scanner comments.
			parser nodeStack size = 1 ifFalse: [ex pass].
			parser nodeStack top
		].
	^ RBMethodNode new
		selectorParts: array first arguments: array last;
		comments: comments! !


!SqueakScanner methodsFor: 'generated-tokens' stamp: 'ajh 3/16/2003 23:56'!
binarySymbolId
	^25! !

!SqueakScanner methodsFor: 'generated-tokens' stamp: 'ajh 3/16/2003 23:56'!
emptySymbolTokenId
	^64! !

!SqueakScanner methodsFor: 'generated-tokens' stamp: 'ajh 3/16/2003 23:56'!
errorTokenId
	^65! !

!SqueakScanner methodsFor: 'generated-tokens' stamp: 'ajh 3/16/2003 23:56'!
nameId
	^22! !

!SqueakScanner methodsFor: 'generated-tokens' stamp: 'ajh 3/16/2003 23:56'!
negativeNumberId
	^20! !

!SqueakScanner methodsFor: 'generated-scanner' stamp: 'ajh 3/16/2003 23:56'!
scan1
	self step.
	currentCharacter isDigit 
		ifTrue: 
			[
			[self recordMatch: #(19).
			self step.
			currentCharacter isDigit] 
					whileTrue.
			^self reportLastMatch].
	currentCharacter == $- 
		ifTrue: 
			[self step.
			currentCharacter isDigit 
				ifTrue: 
					[
					[self recordMatch: #(19).
					self step.
					currentCharacter isDigit] 
							whileTrue.
					^self reportLastMatch].
			^self reportLastMatch].
	^self reportLastMatch! !

!SqueakScanner methodsFor: 'generated-scanner' stamp: 'ajh 3/16/2003 23:56'!
scan2
	self step.
	currentCharacter isDigit 
		ifTrue: 
			[
			[self recordMatch: #(20).
			self step.
			currentCharacter isDigit] 
					whileTrue.
			^self reportLastMatch].
	currentCharacter == $- 
		ifTrue: 
			[self step.
			currentCharacter isDigit 
				ifTrue: 
					[
					[self recordMatch: #(20).
					self step.
					currentCharacter isDigit] 
							whileTrue.
					^self reportLastMatch].
			^self reportLastMatch].
	^self reportLastMatch! !

!SqueakScanner methodsFor: 'generated-scanner' stamp: 'ajh 3/16/2003 23:56'!
scan3
	
	[self step.
	currentCharacter ~~ $'] whileTrue.
	^self scan4! !

!SqueakScanner methodsFor: 'generated-scanner' stamp: 'ajh 3/16/2003 23:56'!
scan4
	self recordMatch: #(21).
	self step.
	currentCharacter == $' ifTrue: [^self scan3].
	^self reportLastMatch! !

!SqueakScanner methodsFor: 'generated-scanner' stamp: 'ajh 3/16/2003 23:56'!
scan5
	
	[self step.
	currentCharacter isAlphaNumeric] whileTrue.
	currentCharacter == $: 
		ifTrue: 
			[self recordMatch: #(24).
			self step.
			currentCharacter isLetter ifTrue: [^self scan5].
			^self reportLastMatch].
	^self reportLastMatch! !

!SqueakScanner methodsFor: 'generated-scanner' stamp: 'ajh 3/16/2003 23:56'!
scan6
	
	[self step.
	currentCharacter isAlphaNumeric] whileTrue.
	currentCharacter == $: 
		ifTrue: 
			[self recordMatch: #(27).
			self step.
			currentCharacter isLetter ifTrue: [^self scan6].
			^self reportLastMatch].
	^self reportLastMatch! !

!SqueakScanner methodsFor: 'generated-scanner' stamp: 'ajh 3/16/2003 23:56'!
scan7
	self step.
	currentCharacter isDigit 
		ifTrue: 
			[
			[self recordMatch: #(19).
			self step.
			currentCharacter isDigit] 
					whileTrue.
			currentCharacter == $e ifTrue: [^self scan1].
			^self reportLastMatch].
	^self reportLastMatch! !

!SqueakScanner methodsFor: 'generated-scanner' stamp: 'ajh 3/16/2003 23:56'!
scan8
	self step.
	(currentCharacter isUppercase or: [currentCharacter between: $0 and: $9]) 
		ifTrue: 
			[
			[self recordMatch: #(19).
			self step.
			currentCharacter isUppercase or: [currentCharacter between: $0 and: $9]] 
					whileTrue.
			currentCharacter == $. 
				ifTrue: 
					[self step.
					(currentCharacter isUppercase or: [currentCharacter between: $0 and: $9]) 
						ifTrue: 
							[
							[self recordMatch: #(19).
							self step.
							currentCharacter isUppercase or: [currentCharacter between: $0 and: $9]] 
									whileTrue.
							currentCharacter == $e ifTrue: [^self scan1].
							^self reportLastMatch].
					^self reportLastMatch].
			currentCharacter == $e ifTrue: [^self scan1].
			^self reportLastMatch].
	^self reportLastMatch! !

!SqueakScanner methodsFor: 'generated-scanner' stamp: 'ajh 3/16/2003 23:56'!
scan9
	self recordMatch: #(23).
	self step.
	currentCharacter isLetter ifTrue: [^self scan5].
	currentCharacter == $= 
		ifTrue: [^self recordAndReportMatch: #variableAssignment].
	^self reportLastMatch! !

!SqueakScanner methodsFor: 'generated-scanner' stamp: 'ajh 3/16/2003 23:56'!
scanForToken
	self step.
	(currentCharacter <= $ or: 
			[(currentCharacter between: $ and: $) 
				or: [currentCharacter == $` or: [currentCharacter >= $]]]) 
		ifTrue: [^self recordAndReportMatch: #(32)].
	currentCharacter isLetter 
		ifTrue: 
			[self recordMatch: #(22 32).
			self step.
			currentCharacter isAlphaNumeric 
				ifTrue: 
					[
					[self recordMatch: #(22).
					self step.
					currentCharacter isAlphaNumeric] 
							whileTrue.
					currentCharacter == $: ifTrue: [^self scan9].
					^self reportLastMatch].
			currentCharacter == $: ifTrue: [^self scan9].
			^self reportLastMatch].
	(currentCharacter == $!! or: 
			[(currentCharacter between: $% and: $&) or: 
					[(currentCharacter between: $* and: $,) or: 
							[currentCharacter == $/ or: 
									[currentCharacter == $= or: 
											[(currentCharacter between: $? and: $@) 
												or: [currentCharacter == $\ or: [currentCharacter == $~]]]]]]]) 
		ifTrue: 
			[self recordMatch: #(25 32).
			self step.
			(currentCharacter == $!! or: 
					[(currentCharacter between: $% and: $&) or: 
							[(currentCharacter between: $* and: $,) or: 
									[currentCharacter == $/ or: 
											[(currentCharacter between: $< and: $@) or: 
													[currentCharacter == $\ 
														or: [currentCharacter == $| or: [currentCharacter == $~]]]]]]]) 
				ifTrue: 
					[
					[self recordMatch: #(25).
					self step.
					currentCharacter == $!! or: 
							[(currentCharacter between: $% and: $&) or: 
									[(currentCharacter between: $* and: $,) or: 
											[currentCharacter == $/ or: 
													[(currentCharacter between: $< and: $@) or: 
															[currentCharacter == $\ 
																or: [currentCharacter == $| or: [currentCharacter == $~]]]]]]]] 
							whileTrue.
					^self reportLastMatch].
			^self reportLastMatch].
	currentCharacter isDigit 
		ifTrue: 
			[self recordMatch: #(19 32).
			self step.
			currentCharacter isDigit 
				ifTrue: 
					[
					[self recordMatch: #(19).
					self step.
					currentCharacter isDigit] 
							whileTrue.
					currentCharacter == $. ifTrue: [^self scan7].
					currentCharacter == $e ifTrue: [^self scan1].
					currentCharacter == $r ifTrue: [^self scan8].
					^self reportLastMatch].
			currentCharacter == $. ifTrue: [^self scan7].
			currentCharacter == $e ifTrue: [^self scan1].
			currentCharacter == $r ifTrue: [^self scan8].
			^self reportLastMatch].
	(currentCharacter isSeparator or: [currentCharacter == $]) 
		ifTrue: 
			[self recordMatch: #whitespace.
			self step.
			(currentCharacter isSeparator or: [currentCharacter == $]) 
				ifTrue: 
					[
					[self recordMatch: #whitespace.
					self step.
					currentCharacter isSeparator or: [currentCharacter == $]] 
							whileTrue.
					^self reportLastMatch].
			^self reportLastMatch].
	currentCharacter == $" 
		ifTrue: 
			[self recordMatch: #(32).
			self step.
			currentCharacter ~~ $" 
				ifTrue: 
					[
					[self step.
					currentCharacter ~~ $"] whileTrue.
					^self recordAndReportMatch: #comment].
			currentCharacter == $" ifTrue: [^self recordAndReportMatch: #comment].
			^self reportLastMatch].
	currentCharacter == $# ifTrue: [^self recordAndReportMatch: #(4 32)].
	currentCharacter == $$ 
		ifTrue: 
			[self recordMatch: #(32).
			self step.
			^self recordAndReportMatch: #(30)].
	currentCharacter == $' 
		ifTrue: 
			[self recordMatch: #(32).
			self step.
			currentCharacter ~~ $' ifTrue: [^self scan3].
			currentCharacter == $' ifTrue: [^self scan4].
			^self reportLastMatch].
	currentCharacter == $( ifTrue: [^self recordAndReportMatch: #(14 32)].
	currentCharacter == $) ifTrue: [^self recordAndReportMatch: #(17 32)].
	currentCharacter == $- 
		ifTrue: 
			[self recordMatch: #(25 32).
			self step.
			(currentCharacter == $!! or: 
					[(currentCharacter between: $% and: $&) or: 
							[(currentCharacter between: $* and: $,) or: 
									[currentCharacter == $/ or: 
											[(currentCharacter between: $< and: $@) or: 
													[currentCharacter == $\ 
														or: [currentCharacter == $| or: [currentCharacter == $~]]]]]]]) 
				ifTrue: 
					[
					[self recordMatch: #(25).
					self step.
					currentCharacter == $!! or: 
							[(currentCharacter between: $% and: $&) or: 
									[(currentCharacter between: $* and: $,) or: 
											[currentCharacter == $/ or: 
													[(currentCharacter between: $< and: $@) or: 
															[currentCharacter == $\ 
																or: [currentCharacter == $| or: [currentCharacter == $~]]]]]]]] 
							whileTrue.
					^self reportLastMatch].
			currentCharacter isDigit 
				ifTrue: 
					[
					[self recordMatch: #(20).
					self step.
					currentCharacter isDigit] 
							whileTrue.
					currentCharacter == $. 
						ifTrue: 
							[self step.
							currentCharacter isDigit 
								ifTrue: 
									[
									[self recordMatch: #(20).
									self step.
									currentCharacter isDigit] 
											whileTrue.
									currentCharacter == $e ifTrue: [^self scan2].
									^self reportLastMatch].
							^self reportLastMatch].
					currentCharacter == $e ifTrue: [^self scan2].
					currentCharacter == $r 
						ifTrue: 
							[self step.
							(currentCharacter isUppercase or: [currentCharacter between: $0 and: $9]) 
								ifTrue: 
									[
									[self recordMatch: #(20).
									self step.
									currentCharacter isUppercase or: [currentCharacter between: $0 and: $9]] 
											whileTrue.
									currentCharacter == $. 
										ifTrue: 
											[self step.
											(currentCharacter isUppercase or: [currentCharacter between: $0 and: $9]) 
												ifTrue: 
													[
													[self recordMatch: #(20).
													self step.
													currentCharacter isUppercase or: [currentCharacter between: $0 and: $9]] 
															whileTrue.
													currentCharacter == $e ifTrue: [^self scan2].
													^self reportLastMatch].
											^self reportLastMatch].
									currentCharacter == $e ifTrue: [^self scan2].
									^self reportLastMatch].
							^self reportLastMatch].
					^self reportLastMatch].
			^self reportLastMatch].
	currentCharacter == $. ifTrue: [^self recordAndReportMatch: #(9 32)].
	currentCharacter == $: 
		ifTrue: 
			[self recordMatch: #(10 32).
			self step.
			currentCharacter isLetter ifTrue: [^self scan6].
			currentCharacter == $= ifTrue: [^self recordAndReportMatch: #(26)].
			^self reportLastMatch].
	currentCharacter == $; ifTrue: [^self recordAndReportMatch: #(18 32)].
	currentCharacter == $< 
		ifTrue: 
			[self recordMatch: #(6 25 32).
			self step.
			(currentCharacter == $!! or: 
					[(currentCharacter between: $% and: $&) or: 
							[(currentCharacter between: $* and: $,) or: 
									[currentCharacter == $/ or: 
											[(currentCharacter between: $< and: $@) or: 
													[currentCharacter == $\ 
														or: [currentCharacter == $| or: [currentCharacter == $~]]]]]]]) 
				ifTrue: 
					[
					[self recordMatch: #(25).
					self step.
					currentCharacter == $!! or: 
							[(currentCharacter between: $% and: $&) or: 
									[(currentCharacter between: $* and: $,) or: 
											[currentCharacter == $/ or: 
													[(currentCharacter between: $< and: $@) or: 
															[currentCharacter == $\ 
																or: [currentCharacter == $| or: [currentCharacter == $~]]]]]]]] 
							whileTrue.
					^self reportLastMatch].
			^self reportLastMatch].
	currentCharacter == $> 
		ifTrue: 
			[self recordMatch: #(7 25 32).
			self step.
			(currentCharacter == $!! or: 
					[(currentCharacter between: $% and: $&) or: 
							[(currentCharacter between: $* and: $,) or: 
									[currentCharacter == $/ or: 
											[(currentCharacter between: $< and: $@) or: 
													[currentCharacter == $\ 
														or: [currentCharacter == $| or: [currentCharacter == $~]]]]]]]) 
				ifTrue: 
					[
					[self recordMatch: #(25).
					self step.
					currentCharacter == $!! or: 
							[(currentCharacter between: $% and: $&) or: 
									[(currentCharacter between: $* and: $,) or: 
											[currentCharacter == $/ or: 
													[(currentCharacter between: $< and: $@) or: 
															[currentCharacter == $\ 
																or: [currentCharacter == $| or: [currentCharacter == $~]]]]]]]] 
							whileTrue.
					^self reportLastMatch].
			^self reportLastMatch].
	currentCharacter == $[ ifTrue: [^self recordAndReportMatch: #(2 32)].
	currentCharacter == $] ifTrue: [^self recordAndReportMatch: #(5 32)].
	currentCharacter == $^ ifTrue: [^self recordAndReportMatch: #(1 32)].
	currentCharacter == $_ ifTrue: [^self recordAndReportMatch: #(26 32)].
	currentCharacter == ${ ifTrue: [^self recordAndReportMatch: #(16 32)].
	currentCharacter == $| 
		ifTrue: 
			[self recordMatch: #(3 25 32).
			self step.
			(currentCharacter == $!! or: 
					[(currentCharacter between: $% and: $&) or: 
							[(currentCharacter between: $* and: $,) or: 
									[currentCharacter == $/ or: 
											[(currentCharacter between: $< and: $@) 
												or: [currentCharacter == $\ or: [currentCharacter == $~]]]]]]) 
				ifTrue: 
					[
					[self recordMatch: #(25).
					self step.
					currentCharacter == $!! or: 
							[(currentCharacter between: $% and: $&) or: 
									[(currentCharacter between: $* and: $,) or: 
											[currentCharacter == $/ or: 
													[(currentCharacter between: $< and: $@) or: 
															[currentCharacter == $\ 
																or: [currentCharacter == $| or: [currentCharacter == $~]]]]]]]] 
							whileTrue.
					^self reportLastMatch].
			currentCharacter == $| 
				ifTrue: 
					[self recordMatch: #(8 25).
					self step.
					(currentCharacter == $!! or: 
							[(currentCharacter between: $% and: $&) or: 
									[(currentCharacter between: $* and: $,) or: 
											[currentCharacter == $/ or: 
													[(currentCharacter between: $< and: $@) or: 
															[currentCharacter == $\ 
																or: [currentCharacter == $| or: [currentCharacter == $~]]]]]]]) 
						ifTrue: 
							[
							[self recordMatch: #(25).
							self step.
							currentCharacter == $!! or: 
									[(currentCharacter between: $% and: $&) or: 
											[(currentCharacter between: $* and: $,) or: 
													[currentCharacter == $/ or: 
															[(currentCharacter between: $< and: $@) or: 
																	[currentCharacter == $\ 
																		or: [currentCharacter == $| or: [currentCharacter == $~]]]]]]]] 
									whileTrue.
							^self reportLastMatch].
					^self reportLastMatch].
			^self reportLastMatch].
	currentCharacter == $} ifTrue: [^self recordAndReportMatch: #(15 32)].
	^self reportLastMatch! !

!SqueakScanner methodsFor: 'token handling' stamp: 'ajh 3/6/2003 00:15'!
comment

	comments add: outputStream contents.
	super comment.
! !

!SqueakScanner methodsFor: 'token handling' stamp: 'ajh 3/15/2003 12:16'!
variableAssignment

	| token |
	outputStream skip: -2.
	stream skip: -2.
	token := SmaCCToken 
				value: outputStream contents
				start: start
				id: (Array with: self nameId).
	outputStream reset.
	matchActions := nil.
	returnMatchBlock value: token.
! !

!SqueakScanner methodsFor: 'private' stamp: 'ajh 3/6/2003 00:16'!
comments

	^ comments! !

!SqueakScanner methodsFor: 'private' stamp: 'ajh 3/6/2003 00:13'!
initialize

	super initialize.
	comments _ OrderedCollection new.
! !


!SqueakScanner class methodsFor: 'generated-initialization' stamp: 'ajh 3/16/2003 23:56'!
initializeKeywordMap
	keywordMap _ Dictionary new.
	#(#(22 'false' 11) #(22 'nil' 12) #(22 'true' 13)) do: 
			[:each | 
			(keywordMap at: each first ifAbsentPut: [Dictionary new]) at: (each at: 2)
				put: each last].
	^keywordMap! !

!SqueakScanner class methodsFor: 'generated-comments' stamp: 'ajh 3/16/2003 23:56'!
scannerDefinitionComment

	"<decimalNumber>:	[0-9]+ (\. [0-9]+)? ;
<radixNumber>:		[0-9]+ r [0-9A-Z]+ (\. [0-9A-Z]+)? ;
<exponentNumber>:	(<decimalNumber> | <radixNumber>) e \-? [0-9]+ ;
<number>:			<decimalNumber> | <radixNumber> | <exponentNumber> ;
<negativeNumber>:	\- <number> ;
<string>:				\'  [^\']* \' (\' [^\']* \')* ;
<name>:				[a-zA-Z] [a-zA-Z0-9]* ;
<keyword>:			<name> \: ;
<multikeyword>:		<name> \: (<name> \: )+ ;
<binarySymbol>:		[\~\-\!!\@\%\&\*\+\=\\\|\?\/\>\<\,] [\~\!!\@\%\&\*\+\=\\\|\?\/\>\<\,]* ;
<assignment>:		\: = | \_ ;
<alternateKeyword>:	\: <name> \: (<name> \:)* ;
<whitespace>:		\s+ ;
<comment>:			\"" [^\""]* \"" ;
<character>:			\$ . ;
<variableAssignment>:	<name> \: \= ;

<anyChar>:			. ; # For VW literal arrays that handle #(;) -> #(#';')
"! !


!StackCount methodsFor: 'initialize' stamp: 'ajh 3/13/2003 01:48'!
startAt: pos

	start _ position _ length _ pos! !

!StackCount methodsFor: 'affecting' stamp: 'ajh 3/13/2003 01:32'!
pop

	^ self pop: 1! !

!StackCount methodsFor: 'affecting' stamp: 'ajh 3/13/2003 01:32'!
pop: n

	(position _ position - n) "< 0 
		ifTrue: [self error: 'Parse stack underflow']"! !

!StackCount methodsFor: 'affecting' stamp: 'ajh 3/13/2003 01:32'!
push

	^ self push: 1! !

!StackCount methodsFor: 'affecting' stamp: 'ajh 3/13/2003 01:32'!
push: n

	(position _ position + n) > length 
		ifTrue: [length _ position]! !

!StackCount methodsFor: 'results' stamp: 'ajh 3/13/2003 01:32'!
length

	^length! !

!StackCount methodsFor: 'results' stamp: 'ajh 3/13/2003 18:37'!
linkTo: stackOrNil

	stackOrNil ifNil: [^ self class startAt: self position].
	^ self position = stackOrNil start
		ifTrue: [stackOrNil]
		ifFalse: [self errorStackOutOfSync: stackOrNil]! !

!StackCount methodsFor: 'results' stamp: 'ajh 3/13/2003 01:32'!
position

	^position! !

!StackCount methodsFor: 'results' stamp: 'ajh 3/13/2003 01:32'!
size

	^length! !

!StackCount methodsFor: 'results' stamp: 'ajh 3/13/2003 01:36'!
start

	^ start! !

!StackCount methodsFor: 'printing' stamp: 'ajh 3/13/2003 01:38'!
printOn: aStream
	
	super printOn: aStream.
	aStream
		nextPutAll: ' start '; print: start;
		nextPutAll: ' stop '; print: position;
		nextPutAll: ' max '; print: length.
! !

!StackCount methodsFor: 'comparing' stamp: 'ajh 3/13/2003 01:39'!
= other

	^ self class == other class 
	  and: [start = other start
	  and: [position = other position
	  and: [length = other size]]]! !

!StackCount methodsFor: 'comparing' stamp: 'ajh 3/13/2003 01:51'!
hash

	^ position hash bitXor: (length hash bitXor: start hash)! !


!StackCount class methodsFor: 'as yet unclassified' stamp: 'ajh 3/13/2003 01:49'!
new

	^ super new startAt: 0! !

!StackCount class methodsFor: 'as yet unclassified' stamp: 'ajh 3/13/2003 01:49'!
startAt: pos

	^ super new startAt: pos! !


!String methodsFor: 'private' stamp: 'ajh 3/18/2003 13:16'!
correctAgainstEnumerator: wordBlock continuedFrom: oldCollection
	"The guts of correction, instead of a wordList, there is a block that should take another block and enumerate over some list with it."

	| choices scoreMin results score maxChoices |
	scoreMin _ self size // 2 min: 3.
	maxChoices _ 10.
	oldCollection isNil
		ifTrue: [ choices _ SortedCollection sortBlock: [ :x :y | x value > y value ] ]
		ifFalse: [ choices _ oldCollection ].
	wordBlock isNil
		ifTrue:
			[ results _ OrderedCollection new.
			1 to: (maxChoices min: choices size) do: [ :i | results add: (choices at: i) key ] ]
		ifFalse:
			[ wordBlock value: [ :word |
				((score _ self alike: word) >= scoreMin
				 and: [(choices anySatisfy: [:assoc | assoc key = word]) not])
				ifTrue: [ choices add: (Association key: word value: score).
					     (choices size >= maxChoices) ifTrue: [ scoreMin _ (choices at: maxChoices) value] ] ].
			results _ choices ].
	^ results! !


!SystemDictionary methodsFor: 'housekeeping' stamp: 'ajh 3/15/2003 16:17'!
macroBenchmark1    "Smalltalk macroBenchmark1"
	"Decompiles and prettyPrints the source for every method in the system (or less depending on the *FILTER*, and then compiles that source and verifies that it generates (and decompiles to) identical code.  This currently fails in a number of places because some different patterns (esp involving conditionals where the first branch returns) decompile the same.  Because it never installs the new method, it should not cause any flusing of the method cache."
	 | methodNode oldMethod newMethod badOnes oldCodeString n classes |
	classes _ Smalltalk allClasses select: [:c | c name < 'B3'].
	badOnes _ OrderedCollection new.
'Decompiling and recompiling...'
displayProgressAt: Sensor cursorPoint
from: 0 to: (classes detectSum: [:c | c selectors size])
during: [:bar | n _ 0.
	classes do:
		[:cls | 
		"Transcript cr; show: cls name."
		cls selectors do:
			[:selector | bar value: (n _ n+1).
			oldMethod _ cls compiledMethodAt: selector.
			oldCodeString _ (oldMethod decompilerClass new
								decompile: selector in: cls method: oldMethod)
							decompileString.
			methodNode _ cls compilerClass new
						compile: oldCodeString
						in: cls notifying: nil ifFail: [].
			newMethod _ methodNode generate: #(0 0 0 0).
			oldCodeString = (newMethod decompilerClass new
								decompile: selector in: cls method: newMethod)
							decompileString ifFalse: [badOnes add: cls name , ' ' , selector]]].
].
	^ badOnes size! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'ajh 2/10/2003 14:37'!
recompileAll 
	"Recompile all classes"

	| n |
	Smalltalk forgetDoIts.
	n _ self classNames size.
	'Recompiling ', n printString, ' classes'
		displayProgressAt: Sensor cursorPoint from: 0 to: n
		during: [:bar |
			n _ 0.
			self allClassesDo: [:class |
				Transcript show: class name; cr.
				class compileAll.
				bar value: (n _ n + 1)]
		].

	"Smalltalk recompileAll"
! !

!SystemDictionary methodsFor: 'housekeeping' stamp: 'ajh 3/15/2003 16:17'!
testDecompiler    "Smalltalk testDecompiler"
	"Decompiles the source for every method in the system, and then compiles that source and verifies that it generates (and decompiles to) identical code.  This currently fails in a number of places because some different patterns (esp involving conditionals where the first branch returns) decompile the same."
	 | methodNode oldMethod newMethod badOnes oldCodeString n |
	badOnes _ OrderedCollection new.
	Smalltalk forgetDoIts.
'Decompiling all classes...'
displayProgressAt: Sensor cursorPoint
from: 0 to: CompiledMethod instanceCount
during: [:bar | n _ 0.
	Smalltalk allBehaviorsDo:
		[:cls | 
		"Transcript cr; show: cls name."
		cls selectors do:
			[:selector | (n _ n+1) \\ 100 = 0 ifTrue: [bar value: n].
			oldMethod _ cls compiledMethodAt: selector.
			oldCodeString _ (oldMethod decompilerClass new
								decompile: selector in: cls method: oldMethod)
							decompileString.
			methodNode _ cls compilerClass new
						compile: oldCodeString
						in: cls notifying: nil ifFail: [].
			newMethod _ methodNode generate: #(0 0 0 0).
			oldCodeString = (newMethod decompilerClass new
								decompile: selector in: cls method: newMethod)
							decompileString ifFalse: [Transcript cr; show: '***' , cls name , ' ' , selector.
											badOnes add: cls name , ' ' , selector]]].
].
	Smalltalk browseMessageList: badOnes asSortedCollection name: 'Decompiler Discrepancies'! !


!TMethod methodsFor: 'initialization' stamp: 'ajh 1/21/2003 13:10'!
setSelector2: sel args: argList locals: localList block: tBlockNode primitive: aNumber
	"Initialize this method using the given information."

	selector _ sel.
	returnType _ 'int'. 	 "assume return type is int for now"
	args _ argList asOrderedCollection.
	locals _ localList asOrderedCollection.
	declarations _ Dictionary new.
	primitive _ aNumber.
	parseTree _ tBlockNode.
	labels _ OrderedCollection new.
	complete _ false.  "set to true when all possible inlining has been done"
	export _ self extractExportDirective.
	static _ self extractStaticDirective.
	self removeFinalSelfReturn.
	self recordDeclarations.
! !

!TMethod methodsFor: 'transformations' stamp: 'ajh 3/17/2003 08:31'!
removeFinalSelfReturn
	"The Smalltalk parser automatically adds the statement '^self' to the end of methods without explicit returns. This method removes such statements, since the generated code has no notion of 'self' anyway."

	| stmtList lastStmt |
	stmtList _ parseTree statements asOrderedCollection.
	stmtList isEmpty ifTrue: [^ self].
	lastStmt _ stmtList last.

	((lastStmt isReturn) and:
	 [(lastStmt expression isVariable) and:
	 ['self' = lastStmt expression name]]) ifTrue: [
		stmtList removeLast.
		parseTree setStatements: stmtList.
	].! !

!TMethod methodsFor: 'inlining' stamp: 'ajh 3/17/2003 08:40'!
statementsListsForInlining
	"Answer a collection of statement list nodes that are candidates for inlining. Currently, we cannot inline into the argument blocks of and: and or: messages."

	| stmtLists |
	stmtLists _ OrderedCollection new: 10.
	parseTree nodesDo: [ :node | 
		node isStmtList ifTrue: [ stmtLists add: node ].
	].
	parseTree nodesDo: [ :node | 
		node isSend ifTrue: [
			((node selector = #and:) or: [node selector = #or:]) ifTrue: [
				"Note: the PP 2.3 compiler produces two arg nodes for these selectors"
				stmtLists remove: node args first ifAbsent: [].
				stmtLists remove: node args last ifAbsent: [].
			].
			((node selector = #ifTrue:) or: [node selector = #ifFalse:]) ifTrue: [
				stmtLists remove: node receiver ifAbsent: [].
			].
			((node selector = #ifTrue:ifFalse:) or: [node selector = #ifFalse:ifTrue:]) ifTrue: [
				stmtLists remove: node receiver ifAbsent: [].
			].
			((node selector = #whileFalse:) or: [node selector = #whileTrue:]) ifTrue: [
				"Allow inlining if it is a [...] whileTrue/whileFalse.
				This is identified by having more than one statement in the 
				receiver block in which case the C code wouldn't work anyways"
				node receiver statements size = 1
					ifTrue:[stmtLists remove: node receiver ifAbsent: []].
			].
			(node selector = #to:do:) ifTrue: [
				stmtLists remove: node receiver ifAbsent: [].
				stmtLists remove: node args first ifAbsent: [].
			].
			(node selector = #to:by:do:) ifTrue: [
				stmtLists remove: node receiver ifAbsent: [].
				stmtLists remove: node args first ifAbsent: [].
				stmtLists remove: (node args at: 2) ifAbsent: [].
			].
		].
		node isCaseStmt ifTrue: [
			"don't inline cases"
			node cases do: [: case | stmtLists remove: case ifAbsent: [] ].
		].
	].
	^stmtLists! !


!TNodeTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 00:29'!
acceptArrayNode: anArrayNode 

	^ self unexpected: anArrayNode! !

!TNodeTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 00:27'!
acceptAssignmentNode: anAssignmentNode 

	^ TAssignmentNode new
		setVariable: (self visitNode: anAssignmentNode variable)
		expression: (self visitNode: anAssignmentNode value);
		comment: anAssignmentNode comment! !

!TNodeTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 00:35'!
acceptBlockNode: aBlockNode 

	| node |
	node _ self visitNode: aBlockNode body.
	node setArguments: (aBlockNode arguments collect: [:arg | arg name]).
	^ node! !

!TNodeTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 00:38'!
acceptCascadeNode: aCascadeNode 

	^ TStmtListNode new
		setArguments: #()
		statements: (aCascadeNode messages collect: [:msg | self visitNode: msg]);
		comment: aCascadeNode comment! !

!TNodeTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 00:39'!
acceptDoItNode: aDoItNode 

	^ self unexpected: aDoItNode! !

!TNodeTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 00:40'!
acceptLiteralNode: aLiteralNode

	^ TConstantNode new setValue: aLiteralNode value! !

!TNodeTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 00:43'!
acceptMessageNode: aMessageNode 

	^ TSendNode new
		setSelector: aMessageNode selector
		receiver: (self visitNode: aMessageNode receiver)
		arguments: (aMessageNode arguments collect: [:arg | self visitNode: arg])! !

!TNodeTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 01:09'!
acceptMethodNode: aMethodNode 

	^ tMethodClass new
		setSelector2: aMethodNode selector
		args: aMethodNode argumentNames
		locals: aMethodNode body allDefinedVariables
		block: (self visitNode: aMethodNode body)
		primitive: aMethodNode primitiveNode num;
		comment: aMethodNode comment
! !

!TNodeTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 00:55'!
acceptReturnNode: aReturnNode

	^ TReturnNode new 
		setExpression: (self visitNode: aReturnNode value);
		comment: aReturnNode comment! !

!TNodeTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 00:58'!
acceptSequenceNode: aSequenceNode 

	| statementList |
	statementList _ OrderedCollection new.
	aSequenceNode statements do: [ :s | | newS |
		newS _ self visitNode: s.
		newS isStmtList ifTrue: [
			"inline the statement list returned when a CascadeNode is translated"
			statementList addAll: newS statements.
		] ifFalse: [
			statementList add: newS.
		].
	].
	^ TStmtListNode new
		setArguments: #()
		statements: statementList;
		comment: aSequenceNode comment! !

!TNodeTranslator methodsFor: 'visitor-double dispatching' stamp: 'ajh 3/17/2003 00:58'!
acceptVariableNode: aVariableNode

	^ TVariableNode new setName: aVariableNode name
! !

!TNodeTranslator methodsFor: 'initialize' stamp: 'ajh 3/17/2003 01:09'!
tMethodClass: aTMethodClass

	tMethodClass _ aTMethodClass! !


!TestCodeGenerator methodsFor: 'coercing' stamp: 'ajh 2/7/2003 21:27'!
ccgTVarBlock: anInteger

	| ctxtString |
	ctxtString _ Preferences compileBlocksAsClosures
		ifTrue: ['thisContext']  "thisContext refers to captured context not eval context"
		ifFalse: ['(thisContext tempAt: 1)'].  "thisContext refers to eval context"
	^ [:expr | ctxtString, ' tempAt: ', anInteger asString, ' put: (', expr, ')']! !


!TestTMethod methodsFor: 'initializing' stamp: 'ajh 1/21/2003 13:11'!
setSelector2: sel args: argList locals: localList block: tBlockNode primitive: aNumber
	"Initialize this method using the given information."

	selector _ sel.
	returnType _ 'int'. 	 "assume return type is int for now"
	args _ argList asOrderedCollection.
	locals _ localList asOrderedCollection.
	declarations _ Dictionary new.
	primitive _ aNumber.
	parseTree _ tBlockNode.
	labels _ OrderedCollection new.
	complete _ false.  "set to true when all possible inlining has been done"
	export _ self extractExportDirective.
	static _ self extractStaticDirective.
	isPrimitive _ false.  "set to true only if you find a primtive direction."
	suppressingFailureGuards _ self extractSuppressFailureGuardDirective.
	self recordDeclarations.
	self extractPrimitiveDirectives.
! !


!Text methodsFor: 'emphasis' stamp: 'ajh 3/17/2003 09:19'!
makeSelectorBold
	"For formatting Smalltalk source code, set the emphasis of that portion of 
	the receiver's string that parses as a message selector to be bold."

	| parser |
	string size = 0 ifTrue: [^ self].
	i _ 0.
	[(string at: (i _ i + 1)) isSeparator] whileTrue.
	(string at: i) = $[ ifTrue: [^ self].  "block, no selector"
	(parser _ Compiler parserClass new) parseSelector: string.
	self makeBoldFrom: 1 to: (parser endOfLastToken min: string size)! !


!UndeclaredSelectorError methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 14:28'!
correctIn: compiler

	self resume: (compiler correctSelector: self messageNode)! !

!UndeclaredSelectorError methodsFor: 'as yet unclassified' stamp: 'ajh 3/24/2003 22:30'!
defaultAction
	"create new selector"

	^ messageNode selectorString asSymbol! !

!UndeclaredSelectorError methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 22:43'!
messageNode

	^ messageNode! !

!UndeclaredSelectorError methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 23:37'!
messageNode: aMessageNode

	messageNode _ aMessageNode.
	messageText _ 'Unknown selector: ', aMessageNode selectorString.
! !


!UndeclaredVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 14:28'!
correctIn: compiler

	self resume: (compiler correctVariable: self variableNode)! !

!UndeclaredVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 12:05'!
variableNode

	^ variableNode! !

!UndeclaredVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/11/2003 23:37'!
variableNode: aVariableNode

	variableNode _ aVariableNode.
	messageText _ 'Undeclared variable: ', aVariableNode name.
! !


!UndefinedObject methodsFor: 'class hierarchy' stamp: 'ajh 2/26/2003 20:01'!
parseScope

	^ SemNilScope new! !

!UndefinedObject methodsFor: 'class hierarchy' stamp: 'ajh 1/21/2003 13:03'!
parserClass

	^ Compiler parserClass! !


!UninitializedVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 14:28'!
correctIn: compiler

	self resume: (compiler queryUninitializedTemp: self variableNode)! !

!UninitializedVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 13:37'!
defaultAction
	"unitialized temp will be initialized to nil, so ignore this error if not handled"

	^ nil! !

!UninitializedVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 13:25'!
variableNode

	^ variableNode! !

!UninitializedVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 14:08'!
variableNode: aVariableNode

	variableNode _ aVariableNode.
	messageText _ 'Uninitialized temp: ', aVariableNode name.
! !


!UnusedVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 14:28'!
correctIn: compiler

	self resume: (compiler removeUnusedTemp: self variableNode)! !

!UnusedVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 14:07'!
defaultAction
	"unused temp will be ignored, so ignore this error if not handled"

	^ nil! !

!UnusedVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 14:06'!
variableNode

	^ variableNode! !

!UnusedVariableError methodsFor: 'as yet unclassified' stamp: 'ajh 3/12/2003 14:07'!
variableNode: aVariableNode

	variableNode _ aVariableNode.
	messageText _ 'Unused temp: ', aVariableNode name.
! !

SemVar initialize!
RBProgramNode subclass: #RBMethodNode
	instanceVariableNames: 'selector selectorParts body source arguments tags scope primitiveNode ir '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-Parser'!
RBLiteralNode initialize!
RBValueNode subclass: #RBBlockNode
	instanceVariableNames: 'left right colons body arguments bar scope ir '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Refactory-Parser'!
RBArrayNode removeSelector: #lastIsReturn!
Compiler initialize!
BytecodeGenerator initialize!
